@@ -1,5 +1,5 @@
 /*
- * $Id: SND_OpenALDevice.cpp,v 1.25 2006/01/29 21:29:14 blendix Exp $
+ * $Id: SND_OpenALDevice.cpp,v 1.26 2006/12/13 02:54:59 hos Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -43,9 +43,15 @@
 
 #include "SND_Utils.h"
 
+#ifdef APPLE_FRAMEWORK_FIX
+#include <al.h>
+#include <alc.h>
+#include <alut.h>
+#else
 #include <AL/al.h>
 #include <AL/alc.h>
 #include <AL/alut.h>
+#endif
 
 #include <stdio.h>
 #include <stdlib.h>
