@@ -36,7 +36,6 @@
 #include "BOP_Mesh.h"
 #include "BOP_Face2Face.h"
 #include "BOP_Merge.h"
-#include "BOP_MaterialContainer.h"
 #include "BOP_Chrono.h"
 
 //#define DEBUG
@@ -50,18 +49,14 @@ BOP_Face3* BOP_createFace(BOP_Mesh* mesh,
 						  BOP_Index vertex1, 
 						  BOP_Index vertex2, 
 						  BOP_Index vertex3, 
-						  BOP_Index idxFace);
+						  BOP_Index origFace);
 void BOP_addMesh(BOP_Mesh*                     mesh,
 				 BOP_Faces*                    meshFacesId,
-				 BOP_MaterialContainer*        materials,
-				 CSG_MeshPropertyDescriptor    props,
 				 CSG_FaceIteratorDescriptor&   face_it,
 				 CSG_VertexIteratorDescriptor& vertex_it,
 				 bool                          invert);
-BSP_CSGMesh* BOP_newEmptyMesh(CSG_MeshPropertyDescriptor props);  
+BSP_CSGMesh* BOP_newEmptyMesh();
 BSP_CSGMesh* BOP_exportMesh(BOP_Mesh*                  inputMesh, 
-							BOP_MaterialContainer*     materials, 
-							CSG_MeshPropertyDescriptor props,
 							bool                       invert);
 void BOP_meshFilter(BOP_Mesh* meshC, BOP_Faces* faces, BOP_BSPTree* bsp);
 void BOP_simplifiedMeshFilter(BOP_Mesh* meshC, BOP_Faces* faces, BOP_BSPTree* bsp, bool inverted);
@@ -70,27 +65,20 @@ void BOP_meshClassify(BOP_Mesh* meshC, BOP_Faces* faces, BOP_BSPTree* bsp);
 /**
  * Performs a generic booleam operation, the entry point for external modules.
  * @param opType Boolean operation type BOP_INTERSECTION, BOP_UNION, BOP_DIFFERENCE
- * @param outputProps Output mesh properties
  * @param outputMesh Output mesh, the final result (the object C)
- * @param obAProps Object A properties
  * @param obAFaces Object A faces list
  * @param obAVertices Object A vertices list
- * @param obBProps Object B properties
  * @param obBFaces Object B faces list
  * @param obBVertices Object B vertices list
  * @param interpFunc Interpolating function
  * @return operation state: BOP_OK, BOP_NO_SOLID, BOP_ERROR
  */
 BoolOpState BOP_performBooleanOperation(BoolOpType                    opType,
-										CSG_MeshPropertyDescriptor    outputProps,
 										BSP_CSGMesh**                 outputMesh,
-										CSG_MeshPropertyDescriptor    obAProps,
 										CSG_FaceIteratorDescriptor    obAFaces,
 										CSG_VertexIteratorDescriptor  obAVertices,
-										CSG_MeshPropertyDescriptor    obBProps,
 										CSG_FaceIteratorDescriptor    obBFaces,
-										CSG_VertexIteratorDescriptor  obBVertices,
-										CSG_InterpolateUserFaceVertexDataFunc interpFunc)
+										CSG_VertexIteratorDescriptor  obBVertices)
 {
 	#ifdef DEBUG
 	cout << "BEGIN BOP_performBooleanOperation" << endl;
@@ -111,15 +99,11 @@ BoolOpState BOP_performBooleanOperation(BoolOpType                    opType,
 	// Build C-mesh, the output mesh
 	BOP_Mesh meshC;
 
-	// Prepare the material container
-	BOP_MaterialContainer materials;
-	materials.setInterpFunc(interpFunc);
-
 	// Add A-mesh into C-mesh
-	BOP_addMesh(&meshC, &meshAFacesId, &materials, obAProps, obAFaces, obAVertices, invertMeshA);
+	BOP_addMesh(&meshC, &meshAFacesId, obAFaces, obAVertices, invertMeshA);
 
 	// Add B-mesh into C-mesh
-	BOP_addMesh(&meshC, &meshBFacesId, &materials, obBProps, obBFaces, obBVertices, invertMeshB);
+	BOP_addMesh(&meshC, &meshBFacesId, obBFaces, obBVertices, invertMeshB);
 
 	// for now, allow operations on non-manifold (non-solid) meshes
 #if 0
@@ -132,7 +116,7 @@ BoolOpState BOP_performBooleanOperation(BoolOpType                    opType,
 												invertMeshA, invertMeshB);
 
 	// Invert the output mesh if is required
-	*outputMesh = BOP_exportMesh(&meshC, &materials, outputProps, invertMeshC);
+	*outputMesh = BOP_exportMesh(&meshC, invertMeshC);
 
 	#ifdef DEBUG
 	cout << "END BOP_performBooleanOperation" << endl;
@@ -324,37 +308,33 @@ void BOP_meshClassify(BOP_Mesh* meshC, BOP_Faces* faces, BOP_BSPTree* bsp)
  * @param vertex1 first vertex of the new face
  * @param vertex2 second vertex of the new face
  * @param vertex3 third vertex of the new face
- * @param idxFace identifier of the new face
+ * @param origFace identifier of the new face
  * @return new the new face
  */
 BOP_Face3 *BOP_createFace3(BOP_Mesh* mesh, 
 						   BOP_Index       vertex1, 
 						   BOP_Index       vertex2, 
 						   BOP_Index       vertex3, 
-						   BOP_Index       idxFace)
+						   BOP_Index       origFace)
 {
 	MT_Point3 p1 = mesh->getVertex(vertex1)->getPoint();
 	MT_Point3 p2 = mesh->getVertex(vertex2)->getPoint();
 	MT_Point3 p3 = mesh->getVertex(vertex3)->getPoint();
 	MT_Plane3 plane(p1,p2,p3);
 
-	return new BOP_Face3(vertex1, vertex2, vertex3, plane, idxFace);
+	return new BOP_Face3(vertex1, vertex2, vertex3, plane, origFace);
 }
 
 /**
  * Adds mesh information into destination mesh.
  * @param mesh input/output mesh, destination for the new mesh data
  * @param meshFacesId output mesh faces, contains an added faces list
- * @param materials used to store material data
- * @param props Properties of the input mesh data
  * @param face_it faces iterator
  * @param vertex_it vertices iterator
  * @param inverted if TRUE adding inverted faces, non-inverted otherwise
  */
 void BOP_addMesh(BOP_Mesh*                     mesh,
 				 BOP_Faces*                    meshFacesId,
-				 BOP_MaterialContainer*        materials,
-				 CSG_MeshPropertyDescriptor    props,
 				 CSG_FaceIteratorDescriptor&   face_it,
 				 CSG_VertexIteratorDescriptor& vertex_it,
 				 bool                          invert)
@@ -374,46 +354,12 @@ void BOP_addMesh(BOP_Mesh*                     mesh,
 	
 	// now for the polygons.
 	// we may need to decalare some memory for user defined face properties.
-	unsigned int sizeFace = props.user_data_size;
-	if (sizeFace) {
-		face.user_face_data = new char[sizeFace];
-	} 
-	else {
-		face.user_face_data = NULL;
-	}
 
-	unsigned int sizeVertex = props.user_face_vertex_data_size;
-	if (sizeVertex) {
-		char * fv_data2 = NULL;
-		fv_data2 = new char[4 * sizeVertex];
-		
-		face.user_face_vertex_data[0] = fv_data2;
-		face.user_face_vertex_data[1] = fv_data2 + sizeVertex;
-		face.user_face_vertex_data[2] = fv_data2 + 2*sizeVertex;
-		face.user_face_vertex_data[3] = fv_data2 + 3*sizeVertex;
-	} 
-	else {
-		face.user_face_vertex_data[0] = NULL;
-		face.user_face_vertex_data[1] = NULL;
-		face.user_face_vertex_data[2] = NULL;
-		face.user_face_vertex_data[3] = NULL;
-	}
-
-	unsigned int idFaceMaterial;
-	BOP_Material faceMaterial(sizeFace,sizeVertex);
-	BOP_Material* materialHandler;
 	BOP_Face3 *newface;
 	
 	while (!face_it.Done(face_it.it)) {
 		face_it.Fill(face_it.it,&face);
 
-		faceMaterial.setFaceMaterial((char *)face.user_face_data);
-		faceMaterial.setFaceVertexMaterial((char *)face.user_face_vertex_data[0]);
-		faceMaterial.setOriginalFace(mesh->getNumFaces());
-		faceMaterial.setIsQuad(face.vertex_number == 4);
-		idFaceMaterial = materials->addMaterial(faceMaterial);
-		materialHandler = materials->getMaterial(idFaceMaterial);
-
 		// Let's not rely on quads being coplanar - especially if they 
 		// are coming out of that soup of code from blender...
 		if (face.vertex_number == 4){
@@ -423,40 +369,32 @@ void BOP_addMesh(BOP_Mesh*                     mesh,
 										  face.vertex_index[2] + vtxIndexOffset,
 										  face.vertex_index[0] + vtxIndexOffset,
 										  face.vertex_index[3] + vtxIndexOffset,
-										  idFaceMaterial);
+										  face.orig_face);
 				meshFacesId->push_back(newface);
 				mesh->addFace(newface);
-				materialHandler->setOriginalFaceVertex(newface->getVertex(0),2);
-				materialHandler->setOriginalFaceVertex(newface->getVertex(1),0);
-				materialHandler->setOriginalFaceVertex(newface->getVertex(2),3);
 				newface = BOP_createFace3(mesh,
 										  face.vertex_index[2] + vtxIndexOffset,
 										  face.vertex_index[1] + vtxIndexOffset,
 										  face.vertex_index[0] + vtxIndexOffset,
-										  idFaceMaterial);
+										  face.orig_face);
 				meshFacesId->push_back(newface);
 				mesh->addFace(newface);
-				materialHandler->setOriginalFaceVertex(newface->getVertex(1),1);
 			}
 			else {
 				newface = BOP_createFace3(mesh,
 										 face.vertex_index[0] + vtxIndexOffset,
 										 face.vertex_index[2] + vtxIndexOffset,
 										 face.vertex_index[3] + vtxIndexOffset,
-										 idFaceMaterial);
+										 face.orig_face);
 				meshFacesId->push_back(newface);
 				mesh->addFace(newface);
-				materialHandler->setOriginalFaceVertex(newface->getVertex(0),0);
-				materialHandler->setOriginalFaceVertex(newface->getVertex(1),2);
-				materialHandler->setOriginalFaceVertex(newface->getVertex(2),3);
 				newface = BOP_createFace3(mesh,
 										 face.vertex_index[0] + vtxIndexOffset,
 										 face.vertex_index[1] + vtxIndexOffset,
 										 face.vertex_index[2] + vtxIndexOffset,
-										 idFaceMaterial);
+										 face.orig_face);
 				meshFacesId->push_back(newface);
 				mesh->addFace(newface);
-				materialHandler->setOriginalFaceVertex(newface->getVertex(1),1);
 			}
 		}
 		else {
@@ -466,54 +404,37 @@ void BOP_addMesh(BOP_Mesh*                     mesh,
 										  face.vertex_index[2] + vtxIndexOffset,
 										  face.vertex_index[1] + vtxIndexOffset,
 										  face.vertex_index[0] + vtxIndexOffset,
-										  idFaceMaterial);
+										  face.orig_face);
 				meshFacesId->push_back(newface);
 				mesh->addFace(newface);
-				materialHandler->setOriginalFaceVertex(newface->getVertex(0),2);
-				materialHandler->setOriginalFaceVertex(newface->getVertex(1),1);
-				materialHandler->setOriginalFaceVertex(newface->getVertex(2),0);
 			}
 			else {
 				newface = BOP_createFace3(mesh,
 										  face.vertex_index[0] + vtxIndexOffset,
 										  face.vertex_index[1] + vtxIndexOffset,
 										  face.vertex_index[2] + vtxIndexOffset,
-										  idFaceMaterial);
+										  face.orig_face);
 				meshFacesId->push_back(newface);
 				mesh->addFace(newface);
-				materialHandler->setOriginalFaceVertex(newface->getVertex(0),0);
-				materialHandler->setOriginalFaceVertex(newface->getVertex(1),1);
-				materialHandler->setOriginalFaceVertex(newface->getVertex(2),2);
 			}
 		}
 		
 		face_it.Step(face_it.it);
 	}
-	
-	// delete temporal material data
-	if (face.user_face_data)
-		delete[] static_cast<char *>(face.user_face_data);
-	if (face.user_face_vertex_data)
-		delete[] static_cast<char *>(face.user_face_vertex_data[0]);
 }
 
 /**
  * Returns an empty mesh with the specified properties.
- * @param props Output mesh properties
  * @return a new empty mesh
  */
-BSP_CSGMesh* BOP_newEmptyMesh(CSG_MeshPropertyDescriptor props)
+BSP_CSGMesh* BOP_newEmptyMesh()
 {
 	BSP_CSGMesh* mesh = BSP_CSGMesh::New();
 	if (mesh == NULL) return mesh;
 
 	vector<BSP_MVertex>* vertices = new vector<BSP_MVertex>;
-	BSP_CSGUserData* faceData     = new BSP_CSGUserData(props.user_data_size);
-	BSP_CSGUserData* faceVtxData  = new BSP_CSGUserData(props.user_face_vertex_data_size);
 	
 	mesh->SetVertices(vertices);
-	mesh->SetFaceData(faceData);
-	mesh->SetFaceVertexData(faceVtxData);
 
 	return mesh;
 }
@@ -521,22 +442,16 @@ BSP_CSGMesh* BOP_newEmptyMesh(CSG_MeshPropertyDescriptor props)
 /**
  * Exports a BOP_Mesh to a BSP_CSGMesh.
  * @param mesh Input mesh
- * @param materials used to reconstruct original faces materials
- * @param props Properties of output mesh
  * @param invert if TRUE export with inverted faces, no inverted otherwise
  * @return the corresponding new BSP_CSGMesh
  */
 BSP_CSGMesh* BOP_exportMesh(BOP_Mesh*                  mesh, 
-							BOP_MaterialContainer*     materials, 
-							CSG_MeshPropertyDescriptor props,
 							bool                       invert)
 {
-	BSP_CSGMesh* outputMesh = BOP_newEmptyMesh(props);
+	BSP_CSGMesh* outputMesh = BOP_newEmptyMesh();
+
+	if (outputMesh == NULL) return NULL;
 
-	// User data handlers
-	BSP_CSGUserData* outputFaceVtxData = &(outputMesh->FaceVertexData());
-	BSP_CSGUserData* outputFaceData    = &(outputMesh->FaceData());
-  
 	// vtx index dictionary, to translate indeces from input to output.
 	map<int,unsigned int> dic;
 	map<int,unsigned int>::iterator itDic;
@@ -547,9 +462,6 @@ BSP_CSGMesh* BOP_exportMesh(BOP_Mesh*                  mesh,
 	BOP_Faces faces = mesh->getFaces();
 	BOP_Vertexs vertexs = mesh->getVertexs();
 
-	// Reserve temporal memory
-	char* tmpFaceVtxData = new char[props.user_face_vertex_data_size];
-			
 	for (BOP_IT_Faces face = faces.begin(); face != faces.end(); face++) {
 		if ((*face)->getTAG()!=BROKEN){    
 			// Add output face
@@ -558,11 +470,8 @@ BSP_CSGMesh* BOP_exportMesh(BOP_Mesh*                  mesh,
 
 			// Copy face
 			outFace.m_verts.clear();
-			outFace.m_fv_data.clear();
 			outFace.m_plane = (*face)->getPlane();
-
-			// Copy face user data from input mesh
-			outputFaceData->Duplicate(materials->getFaceMaterial((*face)->getOriginalFace()));
+			outFace.m_orig_face = (*face)->getOriginalFace();
 
 			// invert face if is required
 			if (invert) (*face)->invert();
@@ -585,22 +494,11 @@ BSP_CSGMesh* BOP_exportMesh(BOP_Mesh*                  mesh,
 					// The vertex is added
 					outVtxId = BSP_VertexInd(itDic->second);
 				}
-				
-				// Add vertex to output face
-				outFace.m_verts.push_back(outVtxId);
 
-				// Add face vertex user data
-				char* faceVtxData = materials->getFaceVertexMaterial(mesh,
-																	(*face)->getOriginalFace(),
-																	(mesh->getVertex(idVertex))->getPoint(),
-																	tmpFaceVtxData);
-				BSP_UserFVInd userFVInd(outputFaceVtxData->Duplicate((void*) faceVtxData));
-				outFace.m_fv_data.push_back(userFVInd);
+				outFace.m_verts.push_back(outVtxId);
 			}
 		}
 	}
-	// free temporal memory
-	delete[] tmpFaceVtxData;
 	
 	// Build the mesh edges using topological informtion
 	outputMesh->BuildEdges();
