@@ -1,5 +1,5 @@
 /**
- * $Id: GHOST_C-api.h,v 1.6 2003/12/26 20:12:40 rwenzlaff Exp $
+ * $Id: GHOST_C-api.h,v 1.7 2006/08/03 12:22:59 broken Exp $
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
  * This program is free software; you can redistribute it and/or
@@ -607,6 +607,13 @@ extern GHOST_TSuccess GHOST_ActivateWindowDrawingContext(GHOST_WindowHandle wind
  */
 extern GHOST_TSuccess GHOST_InvalidateWindow(GHOST_WindowHandle windowhandle);
 
+/**
+ * Returns the status of the tablet
+ * @param windowhandle The handle to the window
+ * @return Status of tablet
+ */
+extern const GHOST_TabletData *GHOST_GetTabletData(GHOST_WindowHandle windowhandle);
+
 /**
  * Access to rectangle width.
  * @param rectanglehandle The handle to the rectangle
@@ -751,7 +758,6 @@ extern void GHOST_SetRectangleCenter(GHOST_RectangleHandle rectanglehandle,
  */
 extern GHOST_TSuccess GHOST_ClipRectangle(GHOST_RectangleHandle rectanglehandle,
 										  GHOST_RectangleHandle anotherrectanglehandle);
-
 #ifdef __cplusplus
 }
 #endif
