@@ -1,5 +1,5 @@
 /**
- * $Id: GHOST_WindowCarbon.cpp,v 1.17 2006/06/19 04:04:31 broken Exp $
+ * $Id: GHOST_WindowCarbon.cpp,v 1.20 2006/12/07 16:04:12 harkyman Exp $
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
  * This program is free software; you can redistribute it and/or
@@ -31,7 +31,7 @@
 
 /**
 
- * $Id: GHOST_WindowCarbon.cpp,v 1.17 2006/06/19 04:04:31 broken Exp $
+ * $Id: GHOST_WindowCarbon.cpp,v 1.20 2006/12/07 16:04:12 harkyman Exp $
  * Copyright (C) 2001 NaN Technologies B.V.
  * @author	Maarten Gribnau
  * @date	May 10, 2001
@@ -185,6 +185,8 @@ GHOST_WindowCarbon::GHOST_WindowCarbon(
         setDrawingContextType(GHOST_kDrawingContextTypeOpenGL);;installDrawingContext(GHOST_kDrawingContextTypeOpenGL);
         updateDrawingContext();
         activateDrawingContext();        
+
+	m_tablet.Active = 0;
     }
 }
 
@@ -401,9 +403,19 @@ GHOST_TSuccess GHOST_WindowCarbon::setOrder(GHOST_TWindowOrder order)
     return GHOST_kSuccess;
 }
 
+/*#define  WAIT_FOR_VSYNC 1*/
+#ifdef WAIT_FOR_VSYNC
+#include <OpenGL/OpenGL.h>
+#endif
 
 GHOST_TSuccess GHOST_WindowCarbon::swapBuffers()
 {
+#ifdef WAIT_FOR_VSYNC
+/* wait for vsync, to avoid tearing artifacts */
+long VBL = 1;
+CGLSetParameter(CGLGetCurrentContext(), kCGLCPSwapInterval, &VBL);
+#endif
+
     GHOST_TSuccess succeeded = GHOST_kSuccess;
     if (m_drawingContextType == GHOST_kDrawingContextTypeOpenGL) {
         if (m_aglCtx) {
