@@ -1,5 +1,5 @@
 /**
- * $Id: GHOST_WindowX11.cpp,v 1.17 2006/11/11 22:47:41 blendix Exp $
+ * $Id: GHOST_WindowX11.cpp,v 1.19 2007/04/15 06:02:12 campbellbarton Exp $
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
  * This program is free software; you can redistribute it and/or
@@ -36,6 +36,7 @@
 
 // For standard X11 cursors
 #include <X11/cursorfont.h>
+#include <X11/Xatom.h>
 
 // For obscure full screen mode stuuf
 // lifted verbatim from blut.
@@ -65,11 +66,11 @@ GHOST_WindowX11(
 	const bool stereoVisual
 ) :
 	GHOST_Window(title,left,top,width,height,state,type,stereoVisual),
-	m_display(display),
-	m_valid_setup (false),
-	m_system (system),
-	m_invalid_window(false),
 	m_context(NULL),
+	m_display(display),
+	m_system (system),
+	m_valid_setup (false),
+	m_invalid_window(false),
 	m_empty_cursor(None),
 	m_custom_cursor(None)
 {
@@ -163,9 +164,28 @@ GHOST_WindowX11(
 			XChangeProperty(m_display, m_window,
 				atom, atom, 32,
 				PropModeReplace, (unsigned char *) &hints, 4);
-		}		
-	}
+		}
+	} else if (state == GHOST_kWindowStateMaximized) {
+		// With this, xprop should report the following just after launch
+		// _NET_WM_STATE(ATOM) = _NET_WM_STATE_MAXIMIZED_VERT, _NET_WM_STATE_MAXIMIZED_HORZ
+		// After demaximization the right side is empty, though (maybe not the most correct then?)
+		Atom state, atomh, atomv;
 
+		state = XInternAtom(m_display, "_NET_WM_STATE", False);
+		atomh = XInternAtom(m_display, "_NET_WM_STATE_MAXIMIZED_HORZ", False);
+		atomv = XInternAtom(m_display, "_NET_WM_STATE_MAXIMIZED_VERT", False);
+		if (state == None ) {
+			GHOST_PRINT("Atom _NET_WM_STATE requested but not avaliable nor created.\n");
+		} else {
+			XChangeProperty(m_display, m_window,
+				state, XA_ATOM, 32,
+				PropModeAppend, (unsigned char *) &atomh, 1);
+			XChangeProperty(m_display, m_window,
+				state, XA_ATOM, 32,
+				PropModeAppend, (unsigned char *) &atomv, 1);
+		}
+ 	}
+	
 	// Create some hints for the window manager on how
 	// we want this window treated.	
 
@@ -476,7 +496,7 @@ setState(
 ){
 	//TODO
 
-	if (state == getState()) {
+        if (state == (int)getState()) {
 		return GHOST_kSuccess;
 	} else {
 		return GHOST_kFailure;
