@@ -1,5 +1,5 @@
 /**
- * $Id: mallocn.c,v 1.21 2006/07/22 17:04:18 ton Exp $
+ * $Id: mallocn.c,v 1.24 2006/11/27 13:59:55 ton Exp $
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
  * This program is free software; you can redistribute it and/or
@@ -31,7 +31,7 @@
 
 /**
 
- * $Id: mallocn.c,v 1.21 2006/07/22 17:04:18 ton Exp $
+ * $Id: mallocn.c,v 1.24 2006/11/27 13:59:55 ton Exp $
  * Copyright (C) 2001 NaN Technologies B.V.
  * Guarded memory allocation, and boundary-write detection.
  */
@@ -117,6 +117,10 @@ volatile unsigned long mem_in_use= 0, mmap_in_use= 0;
 volatile static struct localListBase _membase;
 volatile static struct localListBase *membase = &_membase;
 static void (*error_callback)(char *) = NULL;
+static void (*thread_lock_callback)(void) = NULL;
+static void (*thread_unlock_callback)(void) = NULL;
+
+static int malloc_debug_memset= 0;
 
 #ifdef malloc
 #undef malloc
@@ -147,6 +151,18 @@ static void print_error(const char *str, ...)
 	if (error_callback) error_callback(buf);
 }
 
+static void mem_lock_thread()
+{
+	if (thread_lock_callback)
+		thread_lock_callback();
+}
+
+static void mem_unlock_thread()
+{
+	if (thread_unlock_callback)
+		thread_unlock_callback();
+}
+
 int MEM_check_memory_integrity()
 {
 	const char* err_val = NULL;
@@ -167,6 +183,16 @@ void MEM_set_error_callback(void (*func)(char *))
 	error_callback = func;
 }
 
+void MEM_set_lock_callback(void (*lock)(void), void (*unlock)(void))
+{
+	thread_lock_callback = lock;
+	thread_unlock_callback = unlock;
+}
+
+void MEM_set_memory_debug(void)
+{
+	malloc_debug_memset= 1;
+}
 
 int MEM_allocN_len(void *vmemh)
 {
@@ -191,6 +217,9 @@ void *MEM_dupallocN(void *vmemh)
 			newp= MEM_mapallocN(memh->len, "dupli_mapalloc");
 		else
 			newp= MEM_mallocN(memh->len, "dupli_alloc");
+
+		if (newp == NULL) return NULL;
+
 		memcpy(newp, vmemh, memh->len);
 	}
 
@@ -222,14 +251,20 @@ void *MEM_mallocN(unsigned int len, const char *str)
 {
 	MemHead *memh;
 
+	mem_lock_thread();
+
 	len = (len + 3 ) & ~3; 	/* allocate in units of 4 */
 	
 	memh= (MemHead *)malloc(len+sizeof(MemHead)+sizeof(MemTail));
 
 	if(memh) {
 		make_memhead_header(memh, len, str);
+		mem_unlock_thread();
+		if(malloc_debug_memset && len)
+			memset(memh+1, 255, len);
 		return (++memh);
 	}
+	mem_unlock_thread();
 	print_error("Malloc returns nill: len=%d in %s, total %u\n",len, str, mem_in_use);
 	return NULL;
 }
@@ -238,14 +273,18 @@ void *MEM_callocN(unsigned int len, const char *str)
 {
 	MemHead *memh;
 
+	mem_lock_thread();
+
 	len = (len + 3 ) & ~3; 	/* allocate in units of 4 */
 
 	memh= (MemHead *)calloc(len+sizeof(MemHead)+sizeof(MemTail),1);
 
 	if(memh) {
 		make_memhead_header(memh, len, str);
+		mem_unlock_thread();
 		return (++memh);
 	}
+	mem_unlock_thread();
 	print_error("Calloc returns nill: len=%d in %s, total %u\n",len, str, mem_in_use);
 	return 0;
 }
@@ -257,6 +296,8 @@ void *MEM_mapallocN(unsigned int len, const char *str)
 	return MEM_callocN(len, str);
 #else
 	MemHead *memh;
+
+	mem_lock_thread();
 	
 	len = (len + 3 ) & ~3; 	/* allocate in units of 4 */
 	
@@ -280,13 +321,14 @@ void *MEM_mapallocN(unsigned int len, const char *str)
 		make_memhead_header(memh, len, str);
 		memh->mmap= 1;
 		mmap_in_use += len;
+		mem_unlock_thread();
 		return (++memh);
 	}
 	else {
+		mem_unlock_thread();
 		print_error("Mapalloc returns nill, fallback to regular malloc: len=%d in %s, total %u\n",len, str, mmap_in_use);
 		return MEM_callocN(len, str);
 	}
-	return NULL;
 #endif
 }
 
@@ -295,6 +337,8 @@ void MEM_printmemlist()
 {
 	MemHead *membl;
 
+	mem_lock_thread();
+
 	membl = membase->first;
 	if (membl) membl = MEMNEXT(membl);
 	while(membl) {
@@ -303,6 +347,8 @@ void MEM_printmemlist()
 			membl= MEMNEXT(membl->next);
 		else break;
 	}
+
+	mem_unlock_thread();
 }
 
 short MEM_freeN(void *vmemh)		/* anders compileertie niet meer */
@@ -337,6 +383,8 @@ short MEM_freeN(void *vmemh)		/* anders compileertie niet meer */
 		return(-1);
 	}
 
+	mem_lock_thread();
+
 	if ((memh->tag1 == MEMTAG1) && (memh->tag2 == MEMTAG2) && ((memh->len & 0x3) == 0)) {
 		memt = (MemTail *)(((char *) memh) + sizeof(MemHead) + memh->len);
 		if (memt->tag3 == MEMTAG3){
@@ -346,6 +394,8 @@ short MEM_freeN(void *vmemh)		/* anders compileertie niet meer */
 			memt->tag3 = MEMFREE;
 			/* after tags !!! */
 			rem_memblock(memh);
+
+			mem_unlock_thread();
 			
 			return(0);
 		}
@@ -365,6 +415,8 @@ short MEM_freeN(void *vmemh)		/* anders compileertie niet meer */
 	totblock--;
 	/* here a DUMP should happen */
 
+	mem_unlock_thread();
+
 	return(error);
 }
 
@@ -404,10 +456,11 @@ static void remlink(volatile localListBase *listbase, void *vlink)
 static void rem_memblock(MemHead *memh)
 {
     remlink(membase,&memh->next);
-    if (memh->prev){
-        if (memh->next) MEMNEXT(memh->prev)->nextname = 
-MEMNEXT(memh->next)->name;
-        else MEMNEXT(memh->prev)->nextname = 0;
+    if (memh->prev) {
+        if (memh->next) 
+			MEMNEXT(memh->prev)->nextname = MEMNEXT(memh->next)->name;
+        else 
+			MEMNEXT(memh->prev)->nextname = NULL;
     }
 
     totblock--;
@@ -422,8 +475,11 @@ MEMNEXT(memh->next)->name;
         if (munmap(memh, memh->len + sizeof(MemHead) + sizeof(MemTail)))
             printf("Couldn't unmap memory %s\n", memh->name);
     }
-    else   
+    else {
+		if(malloc_debug_memset && memh->len)
+			memset(memh+1, 255, memh->len);
         free(memh);
+	}
 #endif
 }
 
