@@ -1,7 +1,7 @@
 /**
  * blenlib/BKE_anim.h (mar-2001 nzc);
  *	
- * $Id: BKE_anim.h,v 1.7 2006/02/06 22:11:49 ton Exp $ 
+ * $Id: BKE_anim.h,v 1.8 2006/11/14 20:41:37 ton Exp $ 
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -44,7 +44,7 @@ typedef struct DupliObject {
 	struct DupliObject *next, *prev;
 	struct Object *ob;
 	unsigned int origlay;
-	int index;
+	int index, no_draw;
 	float mat[4][4], omat[4][4];
 } DupliObject;
 
