@@ -4,7 +4,7 @@
  * Stuff that definitely does not belong in the kernel! These will all
  * have to be removed in order to restore sanity.
  *
- * $Id: BKE_bad_level_calls.h,v 1.35 2007/01/24 01:34:04 nicholasbishop Exp $ 
+ * $Id: BKE_bad_level_calls.h,v 1.37 2007/04/04 13:18:27 campbellbarton Exp $ 
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -96,7 +96,7 @@ void make_editMesh(void);
 struct EditMesh;
 void free_editMesh(struct EditMesh *);
 void free_editArmature(void);
-void docentre_new(void);
+void docenter_new(void);
 int saveover(char *str);
 
 /* image.c */
@@ -201,8 +201,8 @@ struct Multires;
 struct MultiresLevel;
 struct MultiresLevel *multires_level_n(struct Multires *mr, int n);
 void multires_free(struct Multires *mr);
-void multires_set_level(void *ob, void *me_v);
-void multires_update_levels(struct Mesh *me);
+void multires_set_level(struct Object *ob, struct Mesh *me, const int render);
+void multires_update_levels(struct Mesh *me, const int render);
 void multires_calc_level_maps(struct MultiresLevel *lvl);
 struct Multires *multires_copy(struct Multires *orig);
 /* sculptmode.c */
