@@ -1,7 +1,7 @@
 /**
  * blenlib/BKE_curve.h (mar-2001 nzc)
  *	
- * $Id: BKE_curve.h,v 1.12 2006/08/13 08:31:58 broken Exp $ 
+ * $Id: BKE_curve.h,v 1.14 2007/03/27 11:37:53 campbellbarton Exp $ 
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -48,9 +48,10 @@ struct BevList;
 
 void unlink_curve( struct Curve *cu);
 void free_curve( struct Curve *cu);
-struct Curve *add_curve(int type);
+struct Curve *add_curve(char *name, int type);
 struct Curve *copy_curve( struct Curve *cu);
 void make_local_curve( struct Curve *cu);
+short curve_type( struct Curve *cu);
 void test_curve_type( struct Object *ob);
 void tex_space_curve( struct Curve *cu);
 int count_curveverts( struct ListBase *nurb);
