@@ -1,5 +1,5 @@
 /*
-* $Id: BKE_customdata.h,v 1.14 2007/01/29 15:10:54 artificer Exp $
+* $Id: BKE_customdata.h,v 1.16 2007/05/02 00:01:22 campbellbarton Exp $
 *
 * ***** BEGIN GPL LICENSE BLOCK *****
 *
@@ -34,7 +34,7 @@
 
 struct CustomData;
 struct CustomDataLayer;
-typedef int CustomDataMask;
+typedef long CustomDataMask;
 
 extern const CustomDataMask CD_MASK_BAREMESH;
 extern const CustomDataMask CD_MASK_MESH;
@@ -184,6 +184,7 @@ void *CustomData_get_layer_named(const struct CustomData *data, int type,
 int CustomData_get_layer_index(const struct CustomData *data, int type);
 int CustomData_get_named_layer_index(const struct CustomData *data, int type, char *name);
 int CustomData_get_active_layer_index(const struct CustomData *data, int type);
+int CustomData_get_render_layer_index(const struct CustomData *data, int type);
 
 /* copies the data from source to the data element at index in the first
  * layer of type
@@ -204,6 +205,7 @@ void *CustomData_set_layer_n(const struct CustomData *data, int type, int n, voi
 
 /* sets the nth layer of type as active */
 void CustomData_set_layer_active(struct CustomData *data, int type, int n);
+void CustomData_set_layer_render(struct CustomData *data, int type, int n);
 
 /* adds flag to the layer flags */
 void CustomData_set_layer_flag(struct CustomData *data, int type, int flag);
