@@ -1,6 +1,6 @@
 /* display list (or rather multi purpose list) stuff */
 /* 
-	$Id: BKE_displist.h,v 1.44 2006/06/21 15:14:43 blendix Exp $
+	$Id: BKE_displist.h,v 1.47 2006/12/05 17:42:00 artificer Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -35,6 +35,9 @@
 #ifndef BKE_DISPLIST_H
 #define BKE_DISPLIST_H
 
+#include "DNA_customdata_types.h"
+#include "BKE_customdata.h"
+
 /* dl->type */
 #define DL_POLY                 0
 #define DL_SEGM                 1
@@ -82,53 +85,19 @@ struct ListBase;
 struct Material;
 struct Bone;
 struct Mesh;
-struct TFace;
-struct EditMesh;
-struct EditEdge;
-struct EditFace;
 
-typedef struct DispListMesh DispListMesh;
-struct DispListMesh {
-	int totvert, totedge, totface;
-	struct MVert *mvert;
-	struct MEdge *medge;
-	struct MCol *mcol;
-	struct MFace *mface;
-	struct TFace *tface;
-	float *nors; // facenormals
 
-	int dontFreeVerts, dontFreeNors, dontFreeOther;
-};
-
-void displistmesh_free(DispListMesh *dlm);
-
-void displistmesh_to_mesh(DispListMesh *dlm, struct Mesh *me);
-
-DispListMesh *displistmesh_copy(DispListMesh *odlm);
-DispListMesh *displistmesh_copyShared(DispListMesh *odlm);
-
-/*
- * All the different DispList.type's use the
- * data in the displist structure in fairly
- * different ways which can be rather confusing, 
- * the best thing to do would be to make a structure
- * for each displaylist type that has the fields
- * needed w/ proper names, and then make the actual
- * DispList structure a typed union.
- *   - zr
- */
-
-/* needs splitting! */
+/* used for curves, nurbs, mball, importing */
 typedef struct DispList {
     struct DispList *next, *prev;
     short type, flag;
     int parts, nr;
-    short col, rt;              /* rt wordt gebruikt door initrenderNurbs */
+    short col, rt;              /* rt used by initrenderNurbs */
 	float *verts, *nors;
 	int *index;
 	unsigned int *col1, *col2;
 	int charidx;
-	int pad;
+	int totindex;				/* indexed array drawing surfaces */
 
 	unsigned int *bevelSplitFlag;
 } DispList;
@@ -141,7 +110,7 @@ extern void addnormalsDispList(struct Object *ob, struct ListBase *lb);
 extern void count_displist(struct ListBase *lb, int *totvert, int *totface);
 extern void freedisplist(struct ListBase *lb);
 extern int displist_has_faces(struct ListBase *lb);
-extern void makeDispListMesh(struct Object *ob);
+extern void makeDerivedMesh(struct Object *ob, CustomDataMask dataMask);
 extern void makeDispListSurf(struct Object *ob, struct ListBase *dispbase, int forRender);
 extern void makeDispListCurveTypes(struct Object *ob, int forOrco);
 extern void makeDispListMBall(struct Object *ob);
@@ -154,8 +123,5 @@ void filldisplist(struct ListBase *dispbase, struct ListBase *to);
 
 void fastshade_free_render(void);
 
-
-void displistmesh_add_edges(DispListMesh *dlm);
-
 #endif
 
