@@ -1,7 +1,7 @@
 /**
  * blenlib/BKE_image.h (mar-2001 nzc)
  *	
- * $Id: BKE_image.h,v 1.16 2006/12/20 17:57:41 ton Exp $ 
+ * $Id: BKE_image.h,v 1.17 2007/05/03 15:10:43 bebraw Exp $ 
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -112,7 +112,7 @@ struct ImBuf *BKE_image_get_ibuf(struct Image *ima, struct ImageUser *iuser);
 struct Image *BKE_add_image_file(const char *name);
 
 /* adds image, adds ibuf, generates color or pattern */
-struct Image *BKE_add_image_size(int width, int height, char *name, short uvtestgrid);
+struct Image *BKE_add_image_size(int width, int height, char *name, short uvtestgrid, float color[4]);
 
 /* for reload, refresh, pack */
 void BKE_image_signal(struct Image *ima, struct ImageUser *iuser, int signal);
