@@ -1,5 +1,5 @@
 /**
- * $Id: BKE_lattice.h,v 1.18 2006/10/31 15:51:56 ton Exp $
+ * $Id: BKE_lattice.h,v 1.19 2007/03/11 16:25:13 campbellbarton Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -43,7 +43,7 @@ extern struct Lattice *editLatt;
 
 
 void resizelattice(struct Lattice *lt, int u, int v, int w, struct Object *ltOb);
-struct Lattice *add_lattice(void);
+struct Lattice *add_lattice(char *name);
 struct Lattice *copy_lattice(struct Lattice *lt);
 void free_lattice(struct Lattice *lt);
 void make_local_lattice(struct Lattice *lt);
