@@ -3,7 +3,7 @@
  *	
  * Library
  *
- * $Id: BKE_library.h,v 1.10 2006/12/20 17:57:41 ton Exp $ 
+ * $Id: BKE_library.h,v 1.14 2007/04/20 18:48:29 campbellbarton Exp $ 
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -47,6 +47,7 @@ void *copy_libblock(void *rt);
 void id_lib_extern(struct ID *id);
 void id_us_plus(struct ID *id);
 
+int check_for_dupid(struct ListBase *lb, struct ID *id, char *name);
 int new_id(struct ListBase *lb, struct ID *id, const char *name);
 
 struct ListBase *wich_libbase(struct Main *mainlib, short type);
@@ -61,7 +62,7 @@ void free_main(struct Main *mainvar);
 void splitIDname(char *name, char *left, int *nr);
 void rename_id(struct ID *id, char *name);
 void test_idbutton(char *name);
-void all_local(struct Library *lib);
+void all_local(struct Library *lib, int untagged_only);
 struct ID *find_id(char *type, char *name);
 void clear_id_newpoins(void);
 
@@ -69,5 +70,8 @@ void IDnames_to_pupstring(char **str, char *title, char *extraops, struct ListBa
 void IMAnames_to_pupstring(char **str, char *title, char *extraops, struct ListBase *lb, struct ID *link, short *nr);
 void IPOnames_to_pupstring(char **str, char *title, char *extraops, struct ListBase *lb, struct ID* link, short *nr, int blocktype);
 
+void flag_listbase_ids(ListBase *lb, short flag, short value);
+void flag_all_listbases_ids(short flag, short value);
+
 #endif
 
