@@ -1,4 +1,4 @@
-/* $Id: BKE_subsurf.h,v 1.18 2005/04/04 03:38:21 zuster Exp $ 
+/* $Id: BKE_subsurf.h,v 1.28 2006/11/20 04:27:48 blendix Exp $ 
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -35,9 +35,13 @@ struct Mesh;
 struct Object;
 struct DerivedMesh;
 struct EditMesh;
+struct SubsurfModifierData;
 
-struct DerivedMesh *subsurf_make_derived_from_editmesh(struct EditMesh *em, int subdivLevels, short type, struct DerivedMesh *oldDerived);
-struct DerivedMesh *subsurf_make_derived_from_mesh(struct Mesh *me, int subdivLevels);
+struct DerivedMesh *subsurf_make_derived_from_derived(
+                        struct DerivedMesh *dm,
+                        struct SubsurfModifierData *smd,
+                        int useRenderParams, float (*vertCos)[3],
+                        int isFinalCalc, int editMode);
 
 void subsurf_calculate_limit_positions(Mesh *me, float (*positions_r)[3]);
 
