@@ -1,7 +1,7 @@
 /**
  * blenlib/BKE_text.h (mar-2001 nzc)
  *	
- * $Id: BKE_text.h,v 1.12 2006/03/05 19:50:13 themyers Exp $ 
+ * $Id: BKE_text.h,v 1.13 2007/03/11 16:25:13 campbellbarton Exp $ 
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -45,7 +45,7 @@ struct SpaceText;
 void			free_text		(struct Text *text);
 void 			txt_set_undostate	(int u);
 int 			txt_get_undostate	(void);
-struct Text*	add_empty_text	(void);
+struct Text*	add_empty_text	(char *name);
 int	            reopen_text		(struct Text *text);
 struct Text*	add_text		(char *file); 
 struct Text*	copy_text		(struct Text *ta);
