@@ -1,6 +1,6 @@
 
 /**
- * $Id: stubs.c,v 1.69 2007/01/24 01:34:05 nicholasbishop Exp $
+ * $Id: stubs.c,v 1.71 2007/04/04 13:18:27 campbellbarton Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -142,7 +142,7 @@ Material defmaterial;
 void load_editMesh(void){}
 void make_editMesh(void){}
 void free_editMesh(struct EditMesh *em){}
-void docentre_new(void){}
+void docenter_new(void){}
 int saveover(char *str){ return 0;}
 
 /* image.c */
@@ -294,8 +294,8 @@ void post_server_add(void) {}
  /* Multires/sculpt stubs */
 struct MultiresLevel *multires_level_n(struct Multires *mr, int n) {return NULL;}
 void multires_free(struct Multires *mr) {}
-void multires_set_level(void *ob, void *me_v) {}
-void multires_update_levels(struct Mesh *me) {}
+void multires_set_level(struct Object *ob, struct Mesh *me, const int render) {}
+void multires_update_levels(struct Mesh *me, const int render) {}
 void multires_calc_level_maps(struct MultiresLevel *lvl) {}
 struct Multires *multires_copy(struct Multires *orig) {return NULL;}
 void sculptmode_init(struct Scene *sce) {}
