@@ -1,5 +1,5 @@
 /**
- * $Id: action.c,v 1.64 2007/01/19 10:05:17 joeedh Exp $
+ * $Id: action.c,v 1.67 2007/05/09 16:38:04 ton Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -360,7 +360,7 @@ bActionChannel *get_action_channel(bAction *act, const char *name)
 {
 	bActionChannel *chan;
 	
-	if (!act)
+	if (!act || !name)
 		return NULL;
 	
 	for (chan = act->chanbase.first; chan; chan=chan->next){
@@ -735,6 +735,11 @@ void copy_pose_result(bPose *to, bPose *from)
 		if(pchanto) {
 			Mat4CpyMat4(pchanto->pose_mat, pchanfrom->pose_mat);
 			Mat4CpyMat4(pchanto->chan_mat, pchanfrom->chan_mat);
+			/* used for local constraints */
+			VECCOPY(pchanto->loc, pchanfrom->loc);
+			QUATCOPY(pchanto->quat, pchanfrom->quat);
+			VECCOPY(pchanto->size, pchanfrom->size);
+			
 			VECCOPY(pchanto->pose_head, pchanfrom->pose_head);
 			VECCOPY(pchanto->pose_tail, pchanfrom->pose_tail);
 			pchanto->flag= pchanfrom->flag;
@@ -982,20 +987,22 @@ static void cyclic_offs_bone(Object *ob, bPose *pose, bActionStrip *strip, float
 				/* bring it into armature space */
 				VecSubf(min, max, min);
 				bone= get_named_bone(ob->data, strip->offs_bone);	/* weak */
-				Mat4Mul3Vecfl(bone->arm_mat, min);
-				
-				/* dominant motion, cyclic_offset was cleared in rest_pose */
-				if (strip->flag & (ACTSTRIP_CYCLIC_USEX | ACTSTRIP_CYCLIC_USEY | ACTSTRIP_CYCLIC_USEZ)) {
-					if (strip->flag & ACTSTRIP_CYCLIC_USEX) pose->cyclic_offset[0]= time*min[0];
-					if (strip->flag & ACTSTRIP_CYCLIC_USEY) pose->cyclic_offset[1]= time*min[1];
-					if (strip->flag & ACTSTRIP_CYCLIC_USEZ) pose->cyclic_offset[2]= time*min[2];
-				} else {
-					if( fabs(min[0]) >= fabs(min[1]) && fabs(min[0]) >= fabs(min[2]))
-						pose->cyclic_offset[0]= time*min[0];
-					else if( fabs(min[1]) >= fabs(min[0]) && fabs(min[1]) >= fabs(min[2]))
-						pose->cyclic_offset[1]= time*min[1];
-					else
-						pose->cyclic_offset[2]= time*min[2];
+				if(bone) {
+					Mat4Mul3Vecfl(bone->arm_mat, min);
+					
+					/* dominant motion, cyclic_offset was cleared in rest_pose */
+					if (strip->flag & (ACTSTRIP_CYCLIC_USEX | ACTSTRIP_CYCLIC_USEY | ACTSTRIP_CYCLIC_USEZ)) {
+						if (strip->flag & ACTSTRIP_CYCLIC_USEX) pose->cyclic_offset[0]= time*min[0];
+						if (strip->flag & ACTSTRIP_CYCLIC_USEY) pose->cyclic_offset[1]= time*min[1];
+						if (strip->flag & ACTSTRIP_CYCLIC_USEZ) pose->cyclic_offset[2]= time*min[2];
+					} else {
+						if( fabs(min[0]) >= fabs(min[1]) && fabs(min[0]) >= fabs(min[2]))
+							pose->cyclic_offset[0]= time*min[0];
+						else if( fabs(min[1]) >= fabs(min[0]) && fabs(min[1]) >= fabs(min[2]))
+							pose->cyclic_offset[1]= time*min[1];
+						else
+							pose->cyclic_offset[2]= time*min[2];
+					}
 				}
 			}
 		}
