@@ -1,5 +1,5 @@
 /**
- * $Id: armature.c,v 1.98 2007/02/05 04:52:39 aligorith Exp $
+ * $Id: armature.c,v 1.103 2007/05/05 09:31:01 aligorith Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -82,11 +82,11 @@ bArmature *get_armature(Object *ob)
 	else return NULL;
 }
 
-bArmature *add_armature()
+bArmature *add_armature(char *name)
 {
 	bArmature *arm;
 	
-	arm= alloc_libblock (&G.main->armature, ID_AR, "Armature");
+	arm= alloc_libblock (&G.main->armature, ID_AR, name);
 	arm->deformflag = ARM_DEF_VGROUP|ARM_DEF_ENVELOPE;
 	arm->layer= 1;
 	return arm;
@@ -438,7 +438,7 @@ Mat4 *b_bone_spline_setup(bPoseChannel *pchan)
 		Mat4MulVecfl(imat, h1);
 		/* if previous bone is B-bone too, use average handle direction */
 		if(prev->bone->segments>1) h1[1]-= length;
-		Normalise(h1);
+		Normalize(h1);
 		VecMulf(h1, -hlength1);
 	}
 	else {
@@ -466,7 +466,7 @@ Mat4 *b_bone_spline_setup(bPoseChannel *pchan)
 		roll= atan2(mat3[2][0], mat3[2][2]);
 		
 		/* and only now negate handle */
-		Normalise(h2);
+		Normalize(h2);
 		VecMulf(h2, -hlength2);
 		
 	}
@@ -548,7 +548,7 @@ float distfactor_to_bone (float vec[3], float b1[3], float b2[3], float rad1, fl
 	float hsqr, a, l, rad;
 	
 	VecSubf (bdelta, b2, b1);
-	l = Normalise (bdelta);
+	l = Normalize (bdelta);
 	
 	VecSubf (pdelta, vec, b1);
 	
@@ -732,7 +732,7 @@ void armature_deform_verts(Object *armOb, Object *target, DerivedMesh *dm,
 
 		if(use_dverts || armature_def_nr >= 0) {
 			if(dm) dvert = dm->getVertData(dm, i, CD_MDEFORMVERT);
-			else if(i < target_totvert) dvert = dverts + i;
+			else if(dverts && i < target_totvert) dvert = dverts + i;
 			else dvert = NULL;
 		} else
 			dvert = NULL;
@@ -941,18 +941,18 @@ void armature_mat_pose_to_delta(float delta_mat[][4], float pose_mat[][4], float
      contain only a rotation, and no scaling. */ 
 void mat3_to_vec_roll(float mat[][3], float *vec, float *roll) 
 {
-     if (vec)
-         VecCopyf(vec, mat[1]);
+    if (vec)
+        VecCopyf(vec, mat[1]);
 
-     if (roll) {
-         float vecmat[3][3], vecmatinv[3][3], rollmat[3][3];
+    if (roll) {
+        float vecmat[3][3], vecmatinv[3][3], rollmat[3][3];
 
-         vec_roll_to_mat3(mat[1], 0.0f, vecmat);
-         Mat3Inv(vecmatinv, vecmat);
-         Mat3MulMat3(rollmat, vecmatinv, mat);
+        vec_roll_to_mat3(mat[1], 0.0f, vecmat);
+        Mat3Inv(vecmatinv, vecmat);
+        Mat3MulMat3(rollmat, vecmatinv, mat);
 
-         *roll= atan2(rollmat[2][0], rollmat[2][2]);
-     }
+        *roll= atan2(rollmat[2][0], rollmat[2][2]);
+    }
 }
 
 /*	Calculates the rest matrix of a bone based
@@ -964,14 +964,14 @@ void vec_roll_to_mat3(float *vec, float roll, float mat[][3])
 	float	rMatrix[3][3], bMatrix[3][3];
 	
 	VECCOPY (nor, vec);
-	Normalise (nor);
+	Normalize (nor);
 	
 	/*	Find Axis & Amount for bone matrix*/
 	Crossf (axis,target,nor);
 
 	if (Inpf(axis,axis) > 0.0000000000001) {
 		/* if nor is *not* a multiple of target ... */
-		Normalise (axis);
+		Normalize (axis);
 		
 		theta= NormalizedVecAngle2(target, nor);
 		
@@ -1614,12 +1614,37 @@ static void do_local_constraint(bPoseChannel *pchan, bConstraint *con)
 			if(data->tar && data->subtarget[0]) {
 				bPoseChannel *pchant= get_pose_channel(data->tar->pose, data->subtarget);
 				if(pchant) {
+					float loc[3];
+					
+					/* copy location of tip of bone? */
+					if (data->flag & LOCLIKE_TIP) {
+						float mat[4][4], tmat[4][4];
+						
+						Mat4One(tmat);
+						VECCOPY(tmat[3], pchant->pose_tail);
+						
+						armature_mat_pose_to_delta(mat, tmat, pchant->bone->arm_mat);
+						VECCOPY(loc, mat[3]);
+					}
+					else
+						VECCOPY(loc, pchant->loc);
+					
+					/* do offsets? */
+					if (data->flag & LOCLIKE_OFFSET)
+						VecAddf(loc, loc, pchan->loc);
+					
 					if (data->flag & LOCLIKE_X)
-						pchan->loc[0]= FloatLerpf(pchant->loc[0], pchan->loc[0], fac);
+						pchan->loc[0]= FloatLerpf(loc[0], pchan->loc[0], fac);
 					if (data->flag & LOCLIKE_Y)
-						pchan->loc[1]= FloatLerpf(pchant->loc[1], pchan->loc[1], fac);
+						pchan->loc[1]= FloatLerpf(loc[1], pchan->loc[1], fac);
 					if (data->flag & LOCLIKE_Z)
-						pchan->loc[2]= FloatLerpf(pchant->loc[2], pchan->loc[2], fac);
+						pchan->loc[2]= FloatLerpf(loc[2], pchan->loc[2], fac);
+					if (data->flag & LOCLIKE_X_INVERT)
+						pchan->loc[0]= FloatLerpf(pchant->loc[0], pchan->loc[0], -fac);
+					if (data->flag & LOCLIKE_Y_INVERT)
+						pchan->loc[1]= FloatLerpf(pchant->loc[1], pchan->loc[1], -fac);
+					if (data->flag & LOCLIKE_Z_INVERT)
+						pchan->loc[2]= FloatLerpf(pchant->loc[2], pchan->loc[2], -fac);
 				}
 			}
 		}
@@ -1631,14 +1656,20 @@ static void do_local_constraint(bPoseChannel *pchan, bConstraint *con)
 				bPoseChannel *pchant= get_pose_channel(data->tar->pose, data->subtarget);
 				if(pchant) {
 					if(data->flag != (ROTLIKE_X|ROTLIKE_Y|ROTLIKE_Z)) {
-						float eul[3], eult[3], euln[3], fac= con->enforce;
+						float eul[3], eult[3], euln[3];
+						float fac= con->enforce;
 						
 						QuatToEul(pchan->quat, eul);
 						QuatToEul(pchant->quat, eult);
 						VECCOPY(euln, eul);
+						
 						if(data->flag & ROTLIKE_X) euln[0]= FloatLerpf(eult[0], eul[0], fac); 
 						if(data->flag & ROTLIKE_Y) euln[1]= FloatLerpf(eult[1], eul[1], fac);
 						if(data->flag & ROTLIKE_Z) euln[2]= FloatLerpf(eult[2], eul[2], fac);
+						if(data->flag & ROTLIKE_X_INVERT) euln[0]= FloatLerpf(eult[0], eul[0], -fac); 
+						if(data->flag & ROTLIKE_Y_INVERT) euln[1]= FloatLerpf(eult[1], eul[1], -fac);
+						if(data->flag & ROTLIKE_Z_INVERT) euln[2]= FloatLerpf(eult[2], eul[2], -fac);
+						
 						compatible_eul(eul, euln);
 						EulToQuat(euln, pchan->quat);
 					}
@@ -1791,7 +1822,9 @@ static void do_strip_modifiers(Object *armob, Bone *bone, bPoseChannel *pchan)
 				continue;
 			
 			for(amod= strip->modifiers.first; amod; amod= amod->next) {
-				if(amod->type==ACTSTRIP_MOD_DEFORM) {
+				switch (amod->type) {
+				case ACTSTRIP_MOD_DEFORM:
+				{
 					/* validate first */
 					if(amod->ob && amod->ob->type==OB_CURVE && amod->channel[0]) {
 						
@@ -1805,6 +1838,62 @@ static void do_strip_modifiers(Object *armob, Bone *bone, bPoseChannel *pchan)
 						}
 					}
 				}
+					break;
+				case ACTSTRIP_MOD_NOISE:	
+				{
+					if( strcmp(pchan->name, amod->channel)==0 ) {
+						float nor[3], loc[3], ofs;
+						float eul[3], size[3], eulo[3], sizeo[3];
+						
+						/* calculate turbulance */
+						ofs = amod->turbul / 200.0f;
+						
+						/* make a copy of starting conditions */
+						VECCOPY(loc, pchan->pose_mat[3]);
+						Mat4ToEul(pchan->pose_mat, eul);
+						Mat4ToSize(pchan->pose_mat, size);
+						VECCOPY(eulo, eul);
+						VECCOPY(sizeo, size);
+						
+						/* apply noise to each set of channels */
+						if (amod->channels & 4) {
+							/* for scaling */
+							nor[0] = BLI_gNoise(amod->noisesize, size[0]+ofs, size[1], size[2], 0, 0) - ofs;
+							nor[1] = BLI_gNoise(amod->noisesize, size[0], size[1]+ofs, size[2], 0, 0) - ofs;	
+							nor[2] = BLI_gNoise(amod->noisesize, size[0], size[1], size[2]+ofs, 0, 0) - ofs;
+							VecAddf(size, size, nor);
+							
+							if (sizeo[0] != 0)
+								VecMulf(pchan->pose_mat[0], size[0] / sizeo[0]);
+							if (sizeo[1] != 0)
+								VecMulf(pchan->pose_mat[1], size[1] / sizeo[1]);
+							if (sizeo[2] != 0)
+								VecMulf(pchan->pose_mat[2], size[2] / sizeo[2]);
+						}
+						if (amod->channels & 2) {
+							/* for rotation */
+							nor[0] = BLI_gNoise(amod->noisesize, eul[0]+ofs, eul[1], eul[2], 0, 0) - ofs;
+							nor[1] = BLI_gNoise(amod->noisesize, eul[0], eul[1]+ofs, eul[2], 0, 0) - ofs;	
+							nor[2] = BLI_gNoise(amod->noisesize, eul[0], eul[1], eul[2]+ofs, 0, 0) - ofs;
+							
+							compatible_eul(nor, eulo);
+							VecAddf(eul, eul, nor);
+							compatible_eul(eul, eulo);
+							
+							LocEulSizeToMat4(pchan->pose_mat, loc, eul, size);
+						}
+						if (amod->channels & 1) {
+							/* for location */
+							nor[0] = BLI_gNoise(amod->noisesize, loc[0]+ofs, loc[1], loc[2], 0, 0) - ofs;
+							nor[1] = BLI_gNoise(amod->noisesize, loc[0], loc[1]+ofs, loc[2], 0, 0) - ofs;	
+							nor[2] = BLI_gNoise(amod->noisesize, loc[0], loc[1], loc[2]+ofs, 0, 0) - ofs;
+							
+							VecAddf(pchan->pose_mat[3], loc, nor);
+						}
+					}
+				}
+					break;
+				}
 			}
 		}
 	}
