@@ -1,5 +1,5 @@
 /**
- * $Id: brush.c,v 1.9 2007/02/09 15:08:53 blendix Exp $
+ * $Id: brush.c,v 1.11 2007/04/04 13:18:28 campbellbarton Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -364,7 +364,7 @@ void brush_sample_tex(Brush *brush, float *xy, float *rgba)
 void brush_imbuf_new(Brush *brush, short flt, short texfall, int size, ImBuf **outbuf)
 {
 	ImBuf *ibuf;
-	float xy[2], dist, rgba[3], *dstf;
+	float xy[2], dist, rgba[4], *dstf;
 	int x, y, rowbytes, xoff, yoff, imbflag;
 	char *dst, crgb[3];
 
@@ -780,7 +780,7 @@ int brush_painter_paint(BrushPainter *painter, BrushFunc func, float *pos, doubl
 		/* setup starting time, direction vector and accumulated time */
 		starttime= painter->accumtime;
 		Vec2Subf(dmousepos, pos, painter->lastmousepos);
-		len= Normalise2(dmousepos);
+		len= Normalize2(dmousepos);
 		painter->accumtime += curtime - painter->lasttime;
 
 		/* do paint op over unpainted time distance */
@@ -815,7 +815,7 @@ int brush_painter_paint(BrushPainter *painter, BrushFunc func, float *pos, doubl
 		/* setup starting distance, direction vector and accumulated distance */
 		startdistance= painter->accumdistance;
 		Vec2Subf(dmousepos, pos, painter->lastmousepos);
-		len= Normalise2(dmousepos);
+		len= Normalize2(dmousepos);
 		painter->accumdistance += len;
 
 		/* do paint op over unpainted distance */
