@@ -1,5 +1,5 @@
 /*
-* $Id: cdderivedmesh.c,v 1.10 2007/01/08 23:47:03 blendix Exp $
+* $Id: cdderivedmesh.c,v 1.13 2007/04/29 13:39:45 campbellbarton Exp $
 *
 * ***** BEGIN GPL LICENSE BLOCK *****
 *
@@ -177,14 +177,14 @@ static void cdDM_drawVerts(DerivedMesh *dm)
 static void cdDM_drawUVEdges(DerivedMesh *dm)
 {
 	CDDerivedMesh *cddm = (CDDerivedMesh*) dm;
-	MTFace *tf = DM_get_face_data_layer(dm, CD_MTFACE);
 	MFace *mf = cddm->mface;
+	MTFace *tf = DM_get_face_data_layer(dm, CD_MTFACE);
 	int i;
 
-	if(tf) {
+	if(mf) {
 		glBegin(GL_LINES);
-		for(i = 0; i < dm->numFaceData; i++, tf++, mf++) {
-			if(!(tf->flag&TF_HIDE)) {
+		for(i = 0; i < dm->numFaceData; i++, mf++, tf++) {
+			if(!(mf->flag&ME_HIDE)) {
 				glVertex2fv(tf->uv[0]);
 				glVertex2fv(tf->uv[1]);
 
@@ -399,65 +399,67 @@ static void cdDM_drawFacesTex_common(DerivedMesh *dm,
 		else {
 			if(index) {
 				orig = *index++;
-				if(orig == ORIGINDEX_NONE) continue;
-				flag = drawParamsMapped(userData, orig);
+				if(orig == ORIGINDEX_NONE)		{ if(nors) nors += 3; continue; }
+				if(drawParamsMapped) flag = drawParamsMapped(userData, orig);
+				else	{ if(nors) nors += 3; continue; }
 			}
 			else
-				flag = drawParamsMapped(userData, i);
+				if(drawParamsMapped) flag = drawParamsMapped(userData, i);
+				else	{ if(nors) nors += 3; continue; }
 		}
+		
+		if(flag != 0) { /* if the flag is 0 it means the face is hidden or invisible */
+			if (flag==1 && mcol)
+				cp= (unsigned char*) &mcol[i*4];
 
-		if(flag == 0)
-			continue;
-		else if (flag==1 && mcol)
-			cp= (unsigned char*) &mcol[i*4];
-
-		if(!(mf->flag&ME_SMOOTH)) {
-			if (nors) {
-				glNormal3fv(nors);
-			}
-			else {
-				/* TODO make this better (cache facenormals as layer?) */
-				float nor[3];
-				if(mf->v4) {
-					CalcNormFloat4(mv[mf->v1].co, mv[mf->v2].co,
-								   mv[mf->v3].co, mv[mf->v4].co,
-								   nor);
-				} else {
-					CalcNormFloat(mv[mf->v1].co, mv[mf->v2].co,
-								  mv[mf->v3].co, nor);
+			if(!(mf->flag&ME_SMOOTH)) {
+				if (nors) {
+					glNormal3fv(nors);
+				}
+				else {
+					/* TODO make this better (cache facenormals as layer?) */
+					float nor[3];
+					if(mf->v4) {
+						CalcNormFloat4(mv[mf->v1].co, mv[mf->v2].co,
+									   mv[mf->v3].co, mv[mf->v4].co,
+									   nor);
+					} else {
+						CalcNormFloat(mv[mf->v1].co, mv[mf->v2].co,
+									  mv[mf->v3].co, nor);
+					}
+					glNormal3fv(nor);
 				}
-				glNormal3fv(nor);
 			}
-		}
 
-		glBegin(mf->v4?GL_QUADS:GL_TRIANGLES);
-		if(tf) glTexCoord2fv(tf[i].uv[0]);
-		if(cp) glColor3ub(cp[3], cp[2], cp[1]);
-		mvert = &mv[mf->v1];
-		if(mf->flag&ME_SMOOTH) glNormal3sv(mvert->no);
-		glVertex3fv(mvert->co);
-			
-		if(tf) glTexCoord2fv(tf[i].uv[1]);
-		if(cp) glColor3ub(cp[7], cp[6], cp[5]);
-		mvert = &mv[mf->v2];
-		if(mf->flag&ME_SMOOTH) glNormal3sv(mvert->no);
-		glVertex3fv(mvert->co);
-
-		if(tf) glTexCoord2fv(tf[i].uv[2]);
-		if(cp) glColor3ub(cp[11], cp[10], cp[9]);
-		mvert = &mv[mf->v3];
-		if(mf->flag&ME_SMOOTH) glNormal3sv(mvert->no);
-		glVertex3fv(mvert->co);
-
-		if(mf->v4) {
-			if(tf) glTexCoord2fv(tf[i].uv[3]);
-			if(cp) glColor3ub(cp[15], cp[14], cp[13]);
-			mvert = &mv[mf->v4];
+			glBegin(mf->v4?GL_QUADS:GL_TRIANGLES);
+			if(tf) glTexCoord2fv(tf[i].uv[0]);
+			if(cp) glColor3ub(cp[3], cp[2], cp[1]);
+			mvert = &mv[mf->v1];
+			if(mf->flag&ME_SMOOTH) glNormal3sv(mvert->no);
+			glVertex3fv(mvert->co);
+				
+			if(tf) glTexCoord2fv(tf[i].uv[1]);
+			if(cp) glColor3ub(cp[7], cp[6], cp[5]);
+			mvert = &mv[mf->v2];
 			if(mf->flag&ME_SMOOTH) glNormal3sv(mvert->no);
 			glVertex3fv(mvert->co);
-		}
-		glEnd();
 
+			if(tf) glTexCoord2fv(tf[i].uv[2]);
+			if(cp) glColor3ub(cp[11], cp[10], cp[9]);
+			mvert = &mv[mf->v3];
+			if(mf->flag&ME_SMOOTH) glNormal3sv(mvert->no);
+			glVertex3fv(mvert->co);
+
+			if(mf->v4) {
+				if(tf) glTexCoord2fv(tf[i].uv[3]);
+				if(cp) glColor3ub(cp[15], cp[14], cp[13]);
+				mvert = &mv[mf->v4];
+				if(mf->flag&ME_SMOOTH) glNormal3sv(mvert->no);
+				glVertex3fv(mvert->co);
+			}
+			glEnd();
+		}
+		
 		if(nors) nors += 3;
 	}
 }
@@ -481,7 +483,8 @@ static void cdDM_drawMappedFaces(DerivedMesh *dm, int (*setDrawOptions)(void *us
 
 		if(index) {
 			orig = *index++;
-			if(setDrawOptions && orig == ORIGINDEX_NONE) continue;
+			if(setDrawOptions && orig == ORIGINDEX_NONE)
+				{ if(nors) nors += 3; continue; }
 		}
 		else
 			orig = i;
@@ -1002,9 +1005,9 @@ void CDDM_calc_normals(DerivedMesh *dm)
 	for(i = 0; i < numVerts; i++, mv++) {
 		float *no = temp_nors[i];
 		
-		if (Normalise(no) == 0.0) {
+		if (Normalize(no) == 0.0) {
 			VECCOPY(no, mv->co);
-			Normalise(no);
+			Normalize(no);
 		}
 
 		mv->no[0] = (short)(no[0] * 32767.0);
