@@ -1,5 +1,5 @@
 /**
- * $Id: constraint.c,v 1.61 2007/01/14 23:42:06 theeth Exp $
+ * $Id: constraint.c,v 1.68 2007/04/11 12:44:03 ton Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -127,10 +127,6 @@ void relink_constraints (struct ListBase *list)
 				data = con->data;
 				
 				ID_NEW(data->tar);
-			}
-				break;
-			case CONSTRAINT_TYPE_NULL:
-			{
 			}
 				break;
 			case CONSTRAINT_TYPE_TRACKTO:
@@ -213,22 +209,12 @@ void relink_constraints (struct ListBase *list)
 				ID_NEW(data->tar);
 			}
 				break;
-			case CONSTRAINT_TYPE_LOCLIMIT:
-			{
-				bLocLimitConstraint *data;
-				data = con->data;
-			}
-				break;
-			case CONSTRAINT_TYPE_ROTLIMIT:
-			{
-				bRotLimitConstraint *data;
-				data = con->data;
-			}
-				break;
-			case CONSTRAINT_TYPE_SIZELIMIT:
-			{
-				bSizeLimitConstraint *data;
+			case CONSTRAINT_TYPE_CLAMPTO:
+			{	
+				bClampToConstraint *data;
 				data = con->data;
+				
+				ID_NEW(data->tar);
 			}
 				break;
 		}
@@ -355,6 +341,13 @@ char constraint_has_target (bConstraint *con)
 				return 1;
 		}
 		break;
+	case CONSTRAINT_TYPE_CLAMPTO:
+		{
+			bClampToConstraint *data = con->data;
+			if (data->tar)
+				return 1;
+		}
+			break;
 	}
 	// Unknown types or CONSTRAINT_TYPE_NULL or no target
 	return 0;
@@ -444,6 +437,13 @@ Object *get_constraint_target(bConstraint *con, char **subtarget)
 			return data->tar;
 		}
 		break;
+	case CONSTRAINT_TYPE_CLAMPTO:
+		{
+			bClampToConstraint *data = con->data;
+			*subtarget= NULL;
+			return data->tar;
+		}
+		break;
 	default:
 		*subtarget= NULL;
 		break;
@@ -533,6 +533,12 @@ void set_constraint_target(bConstraint *con, Object *ob, char *subtarget)
 			if(subtarget) BLI_strncpy(data->subtarget, subtarget, 32);
 		}
 			break;
+		case CONSTRAINT_TYPE_CLAMPTO: 
+		{
+			bClampToConstraint *data = con->data;
+			data->tar= ob;
+		}
+			break;
 	}
 }
 
@@ -544,6 +550,12 @@ void unique_constraint_name (bConstraint *con, ListBase *list)
 	int exists = 0;
 	bConstraint *curcon;
 	
+	/* See if we are given an empty string */
+	if (con->name[0] == '\0') {
+		/* give it default name first */
+		strcpy (con->name, "Const");
+	}
+	
 	/* See if we even need to do this */
 	for (curcon = list->first; curcon; curcon=curcon->next){
 		if (curcon!=con){
@@ -706,16 +718,6 @@ void *new_constraint_data (short type)
 		{
 			bLocLimitConstraint *data;
 			data = MEM_callocN(sizeof(bLocLimitConstraint), "LocLimitConstraint");
-			
-			data->flag = 0;
-			data->flag2 = 0;
-			data->xmin = 0.0f;
-			data->xmax = 0.0f;
-			data->ymin = 0.0f;
-			data->ymax = 0.0f;
-			data->zmin = 0.0f;
-			data->zmax = 0.0f;
-			
 			result = data;
 		}
 		break;
@@ -723,15 +725,6 @@ void *new_constraint_data (short type)
 		{
 			bRotLimitConstraint *data;
 			data = MEM_callocN(sizeof(bRotLimitConstraint), "RotLimitConstraint");
-			
-			data->flag = 0;
-			data->xmin = 0.0f;
-			data->xmax = 0.0f;
-			data->ymin = 0.0f;
-			data->ymax = 0.0f;
-			data->zmin = 0.0f;
-			data->zmax = 0.0f;
-			
 			result = data;
 		}
 		break;
@@ -739,15 +732,6 @@ void *new_constraint_data (short type)
 		{
 			bSizeLimitConstraint *data;
 			data = MEM_callocN(sizeof(bSizeLimitConstraint), "SizeLimitConstraint");
-			
-			data->flag = 0;
-			data->xmin = 0.0f;
-			data->xmax = 0.0f;
-			data->ymin = 0.0f;
-			data->ymax = 0.0f;
-			data->zmin = 0.0f;
-			data->zmax = 0.0f;
-			
 			result = data;
 		}
 		break;
@@ -783,6 +767,13 @@ void *new_constraint_data (short type)
 			result = data;
 		}
 		break;
+	case CONSTRAINT_TYPE_CLAMPTO:
+		{
+			bClampToConstraint *data;
+			data = MEM_callocN(sizeof(bClampToConstraint), "ClampToConstraint");
+			result = data;
+		}
+		break;
   
    	default:
 		result = NULL;
@@ -935,7 +926,7 @@ static void vectomat(float *vec, float *target_up, short axis, short upflag, sho
 	int right_index;
 
 	VecCopyf(n, vec);
-	if(Normalise(n) == 0.0) { 
+	if(Normalize(n) == 0.0) { 
 		n[0] = 0.0;
 		n[1] = 0.0;
 		n[2] = 1.0;
@@ -963,15 +954,15 @@ static void vectomat(float *vec, float *target_up, short axis, short upflag, sho
 	VecSubf(proj, u, proj); /* then onto the plane */
 	/* proj specifies the transformation of the up axis */
 
-	if(Normalise(proj) == 0.0) { /* degenerate projection */
+	if(Normalize(proj) == 0.0) { /* degenerate projection */
 		proj[0] = 0.0;
 		proj[1] = 1.0;
 		proj[2] = 0.0;
 	}
 
-	/* normalised cross product of n and proj specifies transformation of the right axis */
+	/* Normalized cross product of n and proj specifies transformation of the right axis */
 	Crossf(right, proj, n);
-	Normalise(right);
+	Normalize(right);
 
 	if(axis != upflag) {
 		right_index = 3 - axis - upflag;
@@ -1107,9 +1098,32 @@ short get_constraint_target_matrix (bConstraint *con, short ownertype, void* own
 	case CONSTRAINT_TYPE_LOCLIKE:
 		{
 			bLocateLikeConstraint *data = (bLocateLikeConstraint*)con->data;
-
-			if (data->tar){
-				constraint_target_to_mat4(data->tar, data->subtarget, mat, size);
+			Object *ob= data->tar;
+			
+			if (data->tar) {
+				if (strlen(data->subtarget)) {
+					bPoseChannel *pchan;
+					float tmat[4][4];
+					float bsize[3]={1, 1, 1};
+					
+					pchan = get_pose_channel(ob->pose, data->subtarget);
+					if (pchan) {
+						Mat4CpyMat4(tmat, pchan->pose_mat);
+						
+						if (data->flag & LOCLIKE_TIP) 
+							VECCOPY(tmat[3], pchan->pose_tail);
+							
+						Mat4MulMat4 (mat, tmat, ob->obmat);
+					}
+					else 
+						Mat4CpyMat4 (mat, ob->obmat);
+					
+					VECCOPY(size, bsize); // what's this hack for? 
+				}
+				else {
+					Mat4CpyMat4 (mat, ob->obmat);
+					VECCOPY(size, data->tar->size); // what's this hack for? 
+				}
 				valid=1;
 			}
 			else
@@ -1241,7 +1255,7 @@ short get_constraint_target_matrix (bConstraint *con, short ownertype, void* own
 						if(data->followflag){
 							quat= vectoquat(dir, (short) data->trackflag, (short) data->upflag);
 
-							Normalise(dir);
+							Normalize(dir);
 							q[0]= (float)cos(0.5*vec[3]);
 							x1= (float)sin(0.5*vec[3]);
 							q[1]= -x1*dir[0];
@@ -1276,6 +1290,24 @@ short get_constraint_target_matrix (bConstraint *con, short ownertype, void* own
 				Mat4One (mat);
 		}
 		break;
+	case CONSTRAINT_TYPE_CLAMPTO:
+		{
+			bClampToConstraint *data;
+			data = (bClampToConstraint*)con->data;
+
+			if (data->tar) {
+				Curve *cu= data->tar->data;
+				
+				/* note; when creating constraints that follow path, the curve gets the CU_PATH set now,
+					currently for paths to work it needs to go through the bevlist/displist system (ton) */
+				
+				if(cu->path==NULL || cu->path->data==NULL) /* only happens on reload file, but violates depsgraph still... fix! */
+					makeDispListCurveTypes(data->tar, 0);
+			}
+			
+			Mat4One (mat);
+		}
+		break;
 
 	default:
 		Mat4One(mat);
@@ -1285,7 +1317,6 @@ short get_constraint_target_matrix (bConstraint *con, short ownertype, void* own
 	return valid;
 }
 
-
 /* only called during solve_constraints */
 /* bone constraints create a fake object to work on, then ob is a workob */
 /* if ownerdata is set, it's the posechannel */
@@ -1299,7 +1330,11 @@ void evaluate_constraint (bConstraint *constraint, Object *ob, short ownertype,
 
 	Mat4One (M_identity);
 	
-	switch (constraint->type){
+	switch (constraint->type) {
+	case CONSTRAINT_TYPE_NULL:
+	case CONSTRAINT_TYPE_KINEMATIC: /* removed */
+		break;
+	
 	case CONSTRAINT_TYPE_ACTION:
 		{
 			float temp[4][4];
@@ -1314,15 +1349,31 @@ void evaluate_constraint (bConstraint *constraint, Object *ob, short ownertype,
 	case CONSTRAINT_TYPE_LOCLIKE:
 		{
 			bLocateLikeConstraint *data;
+			float offset[3] = {0.0f, 0.0f, 0.0f};
 
 			data = constraint->data;
 			
-			if (data->flag & LOCLIKE_X)
+			if (data->flag & LOCLIKE_OFFSET)
+				VECCOPY(offset, ob->obmat[3]);
+			
+			if (data->flag & LOCLIKE_X) {
 				ob->obmat[3][0] = targetmat[3][0];
-			if (data->flag & LOCLIKE_Y)
+				
+				if(data->flag & LOCLIKE_X_INVERT) ob->obmat[3][0] *= -1;
+				ob->obmat[3][0] += offset[0];
+			}
+			if (data->flag & LOCLIKE_Y) {
 				ob->obmat[3][1] = targetmat[3][1];
-			if (data->flag & LOCLIKE_Z)
+				
+				if(data->flag & LOCLIKE_Y_INVERT) ob->obmat[3][1] *= -1;
+				ob->obmat[3][1] += offset[1];
+			}
+			if (data->flag & LOCLIKE_Z) {
 				ob->obmat[3][2] = targetmat[3][2];
+				
+				if(data->flag & LOCLIKE_Z_INVERT) ob->obmat[3][2] *= -1;
+				ob->obmat[3][2] += offset[2];
+			}
 		}
 		break;
 	case CONSTRAINT_TYPE_ROTLIKE:
@@ -1339,14 +1390,27 @@ void evaluate_constraint (bConstraint *constraint, Object *ob, short ownertype,
 			
 			Mat4ToEul(targetmat, eul);
 			Mat4ToEul(ob->obmat, obeul);
-			
+				
 			if(data->flag != (ROTLIKE_X|ROTLIKE_Y|ROTLIKE_Z)) {
-				if(!(data->flag & ROTLIKE_X)) eul[0]= obeul[0];
-				if(!(data->flag & ROTLIKE_Y)) eul[1]= obeul[1];
-				if(!(data->flag & ROTLIKE_Z)) eul[2]= obeul[2];
+				if(!(data->flag & ROTLIKE_X)) {
+					eul[0]= obeul[0];
+				}
+				if(!(data->flag & ROTLIKE_Y)) {
+					eul[1]= obeul[1];
+				}
+				if(!(data->flag & ROTLIKE_Z)) {
+					eul[2]= obeul[2];
+				}
 				compatible_eul(eul, obeul);
 			}
-
+			
+			if((data->flag & ROTLIKE_X) && (data->flag & ROTLIKE_X_INVERT))
+				eul[0]*=-1;
+			if((data->flag & ROTLIKE_Y) && (data->flag & ROTLIKE_Y_INVERT))
+				eul[1]*=-1;
+			if((data->flag & ROTLIKE_Z) && (data->flag & ROTLIKE_Z_INVERT))
+				eul[2]*=-1;
+			
 			LocEulSizeToMat4(ob->obmat, loc, eul, size);
 		}
 		break;
@@ -1368,10 +1432,6 @@ void evaluate_constraint (bConstraint *constraint, Object *ob, short ownertype,
  				VecMulf(ob->obmat[2], size[2] / obsize[2]);
   		}
   		break;
-	case CONSTRAINT_TYPE_NULL:
-		{
-		}
-		break;
 	case CONSTRAINT_TYPE_MINMAX:
 		{
 			float val1, val2;
@@ -1491,11 +1551,6 @@ void evaluate_constraint (bConstraint *constraint, Object *ob, short ownertype,
 			}
 		}
 		break;
-	case CONSTRAINT_TYPE_KINEMATIC:
-		{
-			/* removed */
-		}
-		break;
 	case CONSTRAINT_TYPE_LOCKTRACK:
 		{
 			bLockTrackConstraint *data;
@@ -1524,13 +1579,13 @@ void evaluate_constraint (bConstraint *constraint, Object *ob, short ownertype,
 						/* Projection of Vector on the plane */
 						Projf(vec2, vec, ob->obmat[0]);
 						VecSubf(totmat[1], vec, vec2);
-						Normalise(totmat[1]);
+						Normalize(totmat[1]);
 
 						/* the x axis is fixed*/
 						totmat[0][0] = ob->obmat[0][0];
 						totmat[0][1] = ob->obmat[0][1];
 						totmat[0][2] = ob->obmat[0][2];
-						Normalise(totmat[0]);
+						Normalize(totmat[0]);
 				
 						/* the z axis gets mapped onto
 						a third orthogonal vector */
@@ -1542,13 +1597,13 @@ void evaluate_constraint (bConstraint *constraint, Object *ob, short ownertype,
 						/* Projection of Vector on the plane */
 						Projf(vec2, vec, ob->obmat[0]);
 						VecSubf(totmat[2], vec, vec2);
-						Normalise(totmat[2]);
+						Normalize(totmat[2]);
 
 						/* the x axis is fixed*/
 						totmat[0][0] = ob->obmat[0][0];
 						totmat[0][1] = ob->obmat[0][1];
 						totmat[0][2] = ob->obmat[0][2];
-						Normalise(totmat[0]);
+						Normalize(totmat[0]);
 				
 						/* the z axis gets mapped onto
 						a third orthogonal vector */
@@ -1560,14 +1615,14 @@ void evaluate_constraint (bConstraint *constraint, Object *ob, short ownertype,
 						/* Projection of Vector on the plane */
 						Projf(vec2, vec, ob->obmat[0]);
 						VecSubf(totmat[1], vec, vec2);
-						Normalise(totmat[1]);
+						Normalize(totmat[1]);
 						VecMulf(totmat[1],-1);
 
 						/* the x axis is fixed*/
 						totmat[0][0] = ob->obmat[0][0];
 						totmat[0][1] = ob->obmat[0][1];
 						totmat[0][2] = ob->obmat[0][2];
-						Normalise(totmat[0]);
+						Normalize(totmat[0]);
 				
 						/* the z axis gets mapped onto
 						a third orthogonal vector */
@@ -1579,14 +1634,14 @@ void evaluate_constraint (bConstraint *constraint, Object *ob, short ownertype,
 						/* Projection of Vector on the plane */
 						Projf(vec2, vec, ob->obmat[0]);
 						VecSubf(totmat[2], vec, vec2);
-						Normalise(totmat[2]);
+						Normalize(totmat[2]);
 						VecMulf(totmat[2],-1);
 
 						/* the x axis is fixed*/
 						totmat[0][0] = ob->obmat[0][0];
 						totmat[0][1] = ob->obmat[0][1];
 						totmat[0][2] = ob->obmat[0][2];
-						Normalise(totmat[0]);
+						Normalize(totmat[0]);
 				
 						/* the z axis gets mapped onto
 						a third orthogonal vector */
@@ -1611,13 +1666,13 @@ void evaluate_constraint (bConstraint *constraint, Object *ob, short ownertype,
 						/* Projection of Vector on the plane */
 						Projf(vec2, vec, ob->obmat[1]);
 						VecSubf(totmat[0], vec, vec2);
-						Normalise(totmat[0]);
+						Normalize(totmat[0]);
 
 						/* the y axis is fixed*/
 						totmat[1][0] = ob->obmat[1][0];
 						totmat[1][1] = ob->obmat[1][1];
 						totmat[1][2] = ob->obmat[1][2];
-						Normalise(totmat[1]);
+						Normalize(totmat[1]);
 						
 						/* the z axis gets mapped onto
 						a third orthogonal vector */
@@ -1629,13 +1684,13 @@ void evaluate_constraint (bConstraint *constraint, Object *ob, short ownertype,
 						/* Projection of Vector on the plane */
 						Projf(vec2, vec, ob->obmat[1]);
 						VecSubf(totmat[2], vec, vec2);
-						Normalise(totmat[2]);
+						Normalize(totmat[2]);
 
 						/* the y axis is fixed*/
 						totmat[1][0] = ob->obmat[1][0];
 						totmat[1][1] = ob->obmat[1][1];
 						totmat[1][2] = ob->obmat[1][2];
-						Normalise(totmat[1]);
+						Normalize(totmat[1]);
 						
 						/* the z axis gets mapped onto
 						a third orthogonal vector */
@@ -1647,14 +1702,14 @@ void evaluate_constraint (bConstraint *constraint, Object *ob, short ownertype,
 						/* Projection of Vector on the plane */
 						Projf(vec2, vec, ob->obmat[1]);
 						VecSubf(totmat[0], vec, vec2);
-						Normalise(totmat[0]);
+						Normalize(totmat[0]);
 						VecMulf(totmat[0],-1);
 
 						/* the y axis is fixed*/
 						totmat[1][0] = ob->obmat[1][0];
 						totmat[1][1] = ob->obmat[1][1];
 						totmat[1][2] = ob->obmat[1][2];
-						Normalise(totmat[1]);
+						Normalize(totmat[1]);
 						
 						/* the z axis gets mapped onto
 						a third orthogonal vector */
@@ -1666,14 +1721,14 @@ void evaluate_constraint (bConstraint *constraint, Object *ob, short ownertype,
 						/* Projection of Vector on the plane */
 						Projf(vec2, vec, ob->obmat[1]);
 						VecSubf(totmat[2], vec, vec2);
-						Normalise(totmat[2]);
+						Normalize(totmat[2]);
 						VecMulf(totmat[2],-1);
 
 						/* the y axis is fixed*/
 						totmat[1][0] = ob->obmat[1][0];
 						totmat[1][1] = ob->obmat[1][1];
 						totmat[1][2] = ob->obmat[1][2];
-						Normalise(totmat[1]);
+						Normalize(totmat[1]);
 						
 						/* the z axis gets mapped onto
 						a third orthogonal vector */
@@ -1698,13 +1753,13 @@ void evaluate_constraint (bConstraint *constraint, Object *ob, short ownertype,
 						/* Projection of Vector on the plane */
 						Projf(vec2, vec, ob->obmat[2]);
 						VecSubf(totmat[0], vec, vec2);
-						Normalise(totmat[0]);
+						Normalize(totmat[0]);
 
 						/* the z axis is fixed*/
 						totmat[2][0] = ob->obmat[2][0];
 						totmat[2][1] = ob->obmat[2][1];
 						totmat[2][2] = ob->obmat[2][2];
-						Normalise(totmat[2]);
+						Normalize(totmat[2]);
 						
 						/* the x axis gets mapped onto
 						a third orthogonal vector */
@@ -1716,13 +1771,13 @@ void evaluate_constraint (bConstraint *constraint, Object *ob, short ownertype,
 						/* Projection of Vector on the plane */
 						Projf(vec2, vec, ob->obmat[2]);
 						VecSubf(totmat[1], vec, vec2);
-						Normalise(totmat[1]);
+						Normalize(totmat[1]);
 
 						/* the z axis is fixed*/
 						totmat[2][0] = ob->obmat[2][0];
 						totmat[2][1] = ob->obmat[2][1];
 						totmat[2][2] = ob->obmat[2][2];
-						Normalise(totmat[2]);
+						Normalize(totmat[2]);
 						
 						/* the x axis gets mapped onto
 						a third orthogonal vector */
@@ -1734,14 +1789,14 @@ void evaluate_constraint (bConstraint *constraint, Object *ob, short ownertype,
 						/* Projection of Vector on the plane */
 						Projf(vec2, vec, ob->obmat[2]);
 						VecSubf(totmat[0], vec, vec2);
-						Normalise(totmat[0]);
+						Normalize(totmat[0]);
 						VecMulf(totmat[0],-1);
 
 						/* the z axis is fixed*/
 						totmat[2][0] = ob->obmat[2][0];
 						totmat[2][1] = ob->obmat[2][1];
 						totmat[2][2] = ob->obmat[2][2];
-						Normalise(totmat[2]);
+						Normalize(totmat[2]);
 						
 						/* the x axis gets mapped onto
 						a third orthogonal vector */
@@ -1753,14 +1808,14 @@ void evaluate_constraint (bConstraint *constraint, Object *ob, short ownertype,
 						/* Projection of Vector on the plane */
 						Projf(vec2, vec, ob->obmat[2]);
 						VecSubf(totmat[1], vec, vec2);
-						Normalise(totmat[1]);
+						Normalize(totmat[1]);
 						VecMulf(totmat[1],-1);
 
 						/* the z axis is fixed*/
 						totmat[2][0] = ob->obmat[2][0];
 						totmat[2][1] = ob->obmat[2][1];
 						totmat[2][2] = ob->obmat[2][2];
-						Normalise(totmat[2]);
+						Normalize(totmat[2]);
 						
 						/* the x axis gets mapped onto
 						a third orthogonal vector */
@@ -1789,9 +1844,9 @@ void evaluate_constraint (bConstraint *constraint, Object *ob, short ownertype,
 				tmpmat[0][0] = ob->obmat[0][0];tmpmat[0][1] = ob->obmat[0][1];tmpmat[0][2] = ob->obmat[0][2];
 				tmpmat[1][0] = ob->obmat[1][0];tmpmat[1][1] = ob->obmat[1][1];tmpmat[1][2] = ob->obmat[1][2];
 				tmpmat[2][0] = ob->obmat[2][0];tmpmat[2][1] = ob->obmat[2][1];tmpmat[2][2] = ob->obmat[2][2];
-				Normalise(tmpmat[0]);
-				Normalise(tmpmat[1]);
-				Normalise(tmpmat[2]);
+				Normalize(tmpmat[0]);
+				Normalize(tmpmat[1]);
+				Normalize(tmpmat[2]);
 				Mat3Inv(invmat,tmpmat);
 				Mat3MulMat3(tmpmat,totmat,invmat);
 				totmat[0][0] = tmpmat[0][0];totmat[0][1] = tmpmat[0][1];totmat[0][2] = tmpmat[0][2];
@@ -1861,20 +1916,20 @@ void evaluate_constraint (bConstraint *constraint, Object *ob, short ownertype,
                 xx[0] = ob->obmat[0][0];
                 xx[1] = ob->obmat[0][1];
                 xx[2] = ob->obmat[0][2];
-                Normalise(xx);
+                Normalize(xx);
 
                 /* store Z orientation before destroying obmat */
                 zz[0] = ob->obmat[2][0];
                 zz[1] = ob->obmat[2][1];
                 zz[2] = ob->obmat[2][2];
-                Normalise(zz);
+                Normalize(zz);
 
 				VecSubf(vec, ob->obmat[3], targetmat[3]);
 				vec[0] /= size[0];
 				vec[1] /= size[1];
 				vec[2] /= size[2];
 
-				dist = Normalise(vec);
+				dist = Normalize(vec);
                 //dist = VecLenf( ob->obmat[3], targetmat[3]);
 
                 if (data->orglength == 0)  data->orglength = dist;
@@ -1916,7 +1971,7 @@ void evaluate_constraint (bConstraint *constraint, Object *ob, short ownertype,
                 ob->obmat[2][2]=size[2]*scale[2];
                 
                 VecSubf(vec, ob->obmat[3], targetmat[3]);
-                Normalise(vec);
+                Normalize(vec);
                 /* new Y aligns  object target connection*/
                 totmat[1][0] = -vec[0];
                 totmat[1][1] = -vec[1];
@@ -1926,7 +1981,7 @@ void evaluate_constraint (bConstraint *constraint, Object *ob, short ownertype,
                     /* build new Z vector */
                     /* othogonal to "new Y" "old X! plane */
                     Crossf(orth, vec, xx);
-                    Normalise(orth);
+                    Normalize(orth);
                     
                     /* new Z*/
                     totmat[2][0] = orth[0];
@@ -1935,7 +1990,7 @@ void evaluate_constraint (bConstraint *constraint, Object *ob, short ownertype,
                     
                     /* we decided to keep X plane*/
                     Crossf(xx,orth, vec);
-                    Normalise(xx);
+                    Normalize(xx);
                     totmat[0][0] = xx[0];
                     totmat[0][1] = xx[1];
                     totmat[0][2] = xx[2];
@@ -1944,7 +1999,7 @@ void evaluate_constraint (bConstraint *constraint, Object *ob, short ownertype,
                     /* build new X vector */
                     /* othogonal to "new Y" "old Z! plane */
                     Crossf(orth, vec, zz);
-                    Normalise(orth);
+                    Normalize(orth);
                     
                     /* new X*/
                     totmat[0][0] = -orth[0];
@@ -1953,7 +2008,7 @@ void evaluate_constraint (bConstraint *constraint, Object *ob, short ownertype,
                     
                     /* we decided to keep Z */
                     Crossf(zz,orth, vec);
-                    Normalise(zz);
+                    Normalize(zz);
                     totmat[2][0] = zz[0];
                     totmat[2][1] = zz[1];
                     totmat[2][2] = zz[2];
@@ -1975,7 +2030,7 @@ void evaluate_constraint (bConstraint *constraint, Object *ob, short ownertype,
 			data = constraint->data;
 			
 			/* limit location relative to origin or parent   */
-			if (data->flag2 & LIMIT_NOPARENT) {
+			if ((data->flag2 & LIMIT_NOPARENT) && ob->parent) {
 				/* limiting relative to parent */
 				float parmat[4][4]; /* matrix of parent */
 				float objLoc[3], parLoc[3]; /* location of object, and location of parent */
@@ -2177,7 +2232,77 @@ void evaluate_constraint (bConstraint *constraint, Object *ob, short ownertype,
 
 
         }
-        break;		
+        break;	
+	case CONSTRAINT_TYPE_CLAMPTO:
+		{
+			bClampToConstraint *data;
+			Curve *cu;
+			float obmat[4][4], targetMatrix[4][4], ownLoc[3];
+			float curveMin[3], curveMax[3];
+			
+			data = constraint->data;
+			
+			/* prevent crash if user deletes curve */
+			if ((data->tar == NULL) || (data->tar->type != OB_CURVE) ) 
+				return;
+			else
+				cu= data->tar->data;
+			
+			Mat4CpyMat4(obmat, ob->obmat);
+			Mat4One(targetMatrix);
+			VECCOPY(ownLoc, obmat[3]);
+			
+			INIT_MINMAX(curveMin, curveMax)
+			minmax_object(data->tar, curveMin, curveMax);
+			
+			/* get targetmatrix */
+			if(cu->path && cu->path->data) {
+				float vec[4], dir[3], totmat[4][4];
+				float curvetime;
+				short clamp_axis;
+				
+				/* find best position on curve */
+				/* 1. determine which axis to sample on? */
+				if (data->flag==CLAMPTO_AUTO) {
+					float size[3];
+					VecSubf(size, curveMax, curveMin);
+					
+					/* find axis along which the bounding box has the greatest
+					 * extent. Otherwise, default to the x-axis, as that is quite
+					 * frequently used.
+					 */
+					if ((size[2]>size[0]) && (size[2]>size[1]))
+						clamp_axis= CLAMPTO_Z;
+					else if ((size[1]>size[0]) && (size[1]>size[2]))
+						clamp_axis= CLAMPTO_Y;
+					else
+						clamp_axis = CLAMPTO_X;
+				}
+				else 
+					clamp_axis= data->flag;
+					
+				/* 2. determine position relative to curve on a 0-1 scale */
+				if (clamp_axis > 0) clamp_axis--;
+				if (ownLoc[clamp_axis] <= curveMin[clamp_axis])
+					curvetime = 0.0;
+				else if (ownLoc[clamp_axis] >= curveMax[clamp_axis])
+					curvetime = 1.0;
+				else
+					curvetime = (ownLoc[clamp_axis] - curveMin[clamp_axis]) / (curveMax[clamp_axis] - curveMin[clamp_axis]); // umm
+				
+				/* 3. position on curve */
+				if(where_on_path(data->tar, curvetime, vec, dir) ) {
+					Mat4One(totmat);
+					VECCOPY(totmat[3], vec);
+					
+					Mat4MulSerie(targetMatrix, data->tar->obmat, totmat, NULL, NULL, NULL, NULL, NULL, NULL);
+				}
+			}
+			
+			/* obtain final object position */
+			VECCOPY(ob->obmat[3], targetMatrix[3]);
+		}
+		break;
 	default:
 		printf ("Error: Unknown constraint type\n");
 		break;
