@@ -2,7 +2,7 @@
 /*  curve.c 
  * 
  *  
- * $Id: curve.c,v 1.41 2007/01/22 23:11:18 broken Exp $
+ * $Id: curve.c,v 1.44 2007/04/04 13:18:28 campbellbarton Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -112,16 +112,11 @@ void free_curve(Curve *cu)
 	if(cu->tb) MEM_freeN(cu->tb);
 }
 
-Curve *add_curve(int type)
+Curve *add_curve(char *name, int type)
 {
 	Curve *cu;
-	char *str;
-	
-	if(type==OB_CURVE) str= "Curve";
-	else if(type==OB_SURF) str= "Surf";
-	else str= "Text";
 
-	cu= alloc_libblock(&G.main->curve, ID_CU, str);
+	cu= alloc_libblock(&G.main->curve, ID_CU, name);
 	
 	cu->size[0]= cu->size[1]= cu->size[2]= 1.0;
 	cu->flag= CU_FRONT+CU_BACK;
@@ -231,28 +226,24 @@ void make_local_curve(Curve *cu)
 	}
 }
 
-
-void test_curve_type(Object *ob)
+short curve_type(Curve *cu)
 {
 	Nurb *nu;
-	Curve *cu;
-	
-	cu= ob->data;
 	if(cu->vfont) {
-		ob->type= OB_FONT;
-		return;
+		return OB_FONT;
 	}
-	else {
-		nu= cu->nurb.first;
-		while(nu) {
-			if(nu->pntsv>1) {
-				ob->type= OB_SURF;
-				return;
-			}
-			nu= nu->next;
+	for (nu= cu->nurb.first; nu; nu= nu->next) {
+		if(nu->pntsv>1) {
+			return OB_SURF;
 		}
 	}
-	ob->type= OB_CURVE;
+	
+	return OB_CURVE;
+}
+
+void test_curve_type(Object *ob)
+{	
+	ob->type = curve_type(ob->data);
 }
 
 void tex_space_curve(Curve *cu)
@@ -1787,7 +1778,7 @@ void makeBevelList(Object *ob)
 				
 				quat= vectoquat(vec, 5, 1);
 				
-				Normalise(vec);
+				Normalize(vec);
 				q[0]= (float)cos(0.5*bevp1->alfa);
 				x1= (float)sin(0.5*bevp1->alfa);
 				q[1]= x1*vec[0];
@@ -1817,7 +1808,7 @@ void makeBevelList(Object *ob)
 					vec[1]= bevp2->y - bevp0->y;
 					vec[2]= bevp2->z - bevp0->z;
 					
-					Normalise(vec);
+					Normalize(vec);
 
 					quat= vectoquat(vec, 5, 1);
 					
@@ -2116,8 +2107,8 @@ void calchandleNurb(BezTriple *bezt, BezTriple *prev, BezTriple *next, int mode)
 				
 				VecSubf(h1, p2-3, p2);
 				VecSubf(h2, p2, p2+3);
-				len1= Normalise(h1);
-				len2= Normalise(h2);
+				len1= Normalize(h1);
+				len2= Normalize(h2);
 				
 				vz= INPR(h1, h2);
 				
