@@ -1,7 +1,7 @@
 /*  displist.c
  * 
  * 
- * $Id: displist.c,v 1.153 2007/02/14 14:42:48 ton Exp $
+ * $Id: displist.c,v 1.156 2007/05/02 01:40:27 campbellbarton Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -240,7 +240,7 @@ void addnormalsDispList(Object *ob, ListBase *lb)
 				a= dl->parts*dl->nr;
 				v1= ndata;
 				while(a--) {
-					Normalise(v1);
+					Normalize(v1);
 					v1+= 3;
 				}
 			}
@@ -279,6 +279,7 @@ void count_displist(ListBase *lb, int *totvert, int *totface)
 /* ***************************** shade displist. note colors now are in rgb(a) order ******************** */
 
 /* create default shade input... save cpu cycles with ugly global */
+/* XXXX bad code warning: local ShadeInput initialize... */
 static ShadeInput shi;
 static void init_fastshade_shadeinput(void)
 {
@@ -286,6 +287,7 @@ static void init_fastshade_shadeinput(void)
 	shi.lay= G.scene->lay;
 	shi.view[2]= -1.0f;
 	shi.passflag= SCE_PASS_COMBINED;
+	shi.combinedflag= -1;
 }
 
 static Render *fastshade_get_render(void)
@@ -332,7 +334,7 @@ static void fastshade_customdata(CustomData *fdata, int a, int j, Material *ma)
 		layer= &fdata->layers[index];
 		
 		if(needuv && layer->type == CD_MTFACE && shi.totuv < MAX_MTFACE) {
-			n= fastshade_customdata_layer_num(shi.totuv, layer->active);
+			n= fastshade_customdata_layer_num(shi.totuv, layer->active_rnd);
 			mtface= &((MTFace*)layer->data)[a];
 
 			shi.uv[shi.totuv].uv[0]= 2.0f*mtface->uv[j][0]-1.0f;
@@ -343,7 +345,7 @@ static void fastshade_customdata(CustomData *fdata, int a, int j, Material *ma)
 			shi.totuv++;
 		}
 		else if(layer->type == CD_MCOL && shi.totcol < MAX_MCOL) {
-			n= fastshade_customdata_layer_num(shi.totcol, layer->active);
+			n= fastshade_customdata_layer_num(shi.totcol, layer->active_rnd);
 			vertcol= (char*)&((MCol*)layer->data)[a*4 + j];
 
 			shi.col[shi.totcol].col[0]= ((float)vertcol[3])/255.0f;
@@ -525,7 +527,7 @@ static void mesh_create_shadedColors(Render *re, Object *ob, int onlyForMesh, un
 		vn[0]= imat[0][0]*xn+imat[0][1]*yn+imat[0][2]*zn;
 		vn[1]= imat[1][0]*xn+imat[1][1]*yn+imat[1][2]*zn;
 		vn[2]= imat[2][0]*xn+imat[2][1]*yn+imat[2][2]*zn;
-		Normalise(vn);
+		Normalize(vn);
 	}		
 
 	for (i=0; i<totface; i++) {
@@ -555,7 +557,7 @@ static void mesh_create_shadedColors(Render *re, Object *ob, int onlyForMesh, un
 		n1[0]= imat[0][0]*nor[0]+imat[0][1]*nor[1]+imat[0][2]*nor[2];
 		n1[1]= imat[1][0]*nor[0]+imat[1][1]*nor[1]+imat[1][2]*nor[2];
 		n1[2]= imat[2][0]*nor[0]+imat[2][1]*nor[1]+imat[2][2]*nor[2];
-		Normalise(n1);
+		Normalize(n1);
 
 		for (j=0; j<nverts; j++) {
 			MVert *mv= &mvert[vidx[j]];
@@ -666,7 +668,7 @@ void shadeDispList(Base *base)
 						n1[0]= imat[0][0]*dl->nors[0]+imat[0][1]*dl->nors[1]+imat[0][2]*dl->nors[2];
 						n1[1]= imat[1][0]*dl->nors[0]+imat[1][1]*dl->nors[1]+imat[1][2]*dl->nors[2];
 						n1[2]= imat[2][0]*dl->nors[0]+imat[2][1]*dl->nors[1]+imat[2][2]*dl->nors[2];
-						Normalise(n1);
+						Normalize(n1);
 						
 						fp= dl->verts;
 						
@@ -694,7 +696,7 @@ void shadeDispList(Base *base)
 							n1[0]= imat[0][0]*nor[0]+imat[0][1]*nor[1]+imat[0][2]*nor[2];
 							n1[1]= imat[1][0]*nor[0]+imat[1][1]*nor[1]+imat[1][2]*nor[2];
 							n1[2]= imat[2][0]*nor[0]+imat[2][1]*nor[1]+imat[2][2]*nor[2];
-							Normalise(n1);
+							Normalize(n1);
 				
 							fastshade(vec, n1, fp, ma, (char *)col1, NULL);
 							
@@ -732,7 +734,7 @@ void shadeDispList(Base *base)
 							n1[0]= imat[0][0]*nor[0]+imat[0][1]*nor[1]+imat[0][2]*nor[2];
 							n1[1]= imat[1][0]*nor[0]+imat[1][1]*nor[1]+imat[1][2]*nor[2];
 							n1[2]= imat[2][0]*nor[0]+imat[2][1]*nor[1]+imat[2][2]*nor[2];
-							Normalise(n1);
+							Normalize(n1);
 						
 							fastshade(vec, n1, fp, ma, (char *)col1, NULL);
 							
