@@ -1,5 +1,5 @@
 /* 
- * $Id: group.c,v 1.7 2006/11/14 17:16:15 ton Exp $
+ * $Id: group.c,v 1.9 2007/03/29 10:09:44 ton Exp $
  *
  * ***** BEGIN GPL LICENSE BLOCK *****
  *
@@ -105,11 +105,11 @@ void unlink_group(Group *group)
 	group->id.us= 0;
 }
 
-Group *add_group()
+Group *add_group(char *name)
 {
 	Group *group;
 	
-	group = alloc_libblock(&G.main->group, ID_GR, "Group");
+	group = alloc_libblock(&G.main->group, ID_GR, name);
 	group->layer= (1<<20)-1;
 	return group;
 }
@@ -231,6 +231,8 @@ static void group_replaces_nla(Object *parent, Object *target, char mode)
 
 /* puts all group members in local timing system, after this call
 you can draw everything, leaves tags in objects to signal it needs further updating */
+
+/* note: does not work for derivedmesh and render... it recreates all again in convertblender.c */
 void group_handle_recalc_and_update(Object *parent, Group *group)
 {
 	GroupObject *go;
