@@ -1,6 +1,6 @@
 /* ipo.c
  * 
- * $Id: ipo.c,v 1.48 2007/01/15 12:44:45 ton Exp $
+ * $Id: ipo.c,v 1.50 2007/05/02 16:45:13 jesterking Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -287,6 +287,20 @@ Ipo *copy_ipo(Ipo *ipo)
 	return ipon;
 }
 
+/* uses id->newid to match pointers with other copied data */
+void ipo_idnew(Ipo *ipo)
+{
+	if(ipo) {
+		IpoCurve *icu;
+		
+		for(icu= ipo->curve.first; icu; icu= icu->next) {
+			if(icu->driver) {
+				ID_NEW(icu->driver->ob);
+			}
+		}
+	}
+}
+
 void make_local_obipo(Ipo *ipo)
 {
 	Object *ob;
@@ -1467,7 +1481,11 @@ void *get_ipo_poin(ID *id, IpoCurve *icu, int *type)
 		/* yafray: aperture & focal distance params */
 		switch(icu->adrcode) {
 		case CAM_LENS:
-			poin= &(ca->lens); break;
+			if(ca->type==CAM_ORTHO)
+				poin= &(ca->ortho_scale);
+			else
+				poin= &(ca->lens); 
+			break;
 		case CAM_STA:
 			poin= &(ca->clipsta); break;
 		case CAM_END:
