@@ -2,7 +2,7 @@
 /*  key.c      
  *  
  * 
- * $Id: key.c,v 1.21 2006/04/17 17:35:19 ton Exp $
+ * $Id: key.c,v 1.25 2007/01/12 04:47:38 nicholasbishop Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -63,6 +63,8 @@
 
 #include "BLI_blenlib.h"
 
+#include "blendef.h"
+
 #ifdef HAVE_CONFIG_H
 #include <config.h>
 #endif
@@ -185,27 +187,72 @@ void make_local_key(Key *key)
 	make_local_ipo(key->ipo);
 }
 
+/*
+ * Sort shape keys and Ipo curves after a change.  This assumes that at most
+ * one key was moved, which is a valid assumption for the places it's
+ * currently being called.
+ */
 
 void sort_keys(Key *key)
 {
 	KeyBlock *kb;
-	int doit=1;
-	
-	while(doit) {
-		doit= 0;
-		
-		for(kb= key->block.first; kb; kb= kb->next) {
-			if(kb->next) {
-				if(kb->pos > kb->next->pos) {
-					BLI_remlink(&key->block, kb);
-					
-					BLI_insertlink(&key->block, kb->next, kb);
-					
-					doit= 1;
+	short i, adrcode;
+	IpoCurve *icu = NULL;
+	KeyBlock *kb2;
+
+	/* locate the key which is out of position */ 
+	for( kb= key->block.first; kb; kb= kb->next )
+		if( kb->next && kb->pos > kb->next->pos )
+			break;
+
+	/* if we find a key, move it */
+	if( kb ) {
+		kb = kb->next; /* next key is the out-of-order one */
+		BLI_remlink(&key->block, kb);
+
+		/* find the right location and insert before */
+		for( kb2=key->block.first; kb2; kb2= kb2->next ) {
+			if( kb2->pos > kb->pos ) {
+				BLI_insertlink(&key->block, kb2->prev, kb);
+				break;
+			}
+		}
+
+		/* if more than one Ipo curve, see if this key had a curve */
+
+		if(key->ipo && key->ipo->curve.first != key->ipo->curve.last ) {
+			for(icu= key->ipo->curve.first; icu; icu= icu->next) {
+				/* if we find the curve, remove it and reinsert in the 
+				 right place */
+				if(icu->adrcode==kb->adrcode) {
+					IpoCurve *icu2;
+					BLI_remlink(&key->ipo->curve, icu);
+					for(icu2= key->ipo->curve.first; icu2; icu2= icu2->next) {
+						if(icu2->adrcode >= kb2->adrcode) {
+							BLI_insertlink(&key->ipo->curve, icu2->prev, icu);
+							break;
+						}
+					}
 					break;
 				}
 			}
-		}	
+		}
+
+		/* kb points at the moved key, icu at the moved ipo (if it exists).
+		 * go back now and renumber adrcodes */
+
+		/* first new code */
+		adrcode = kb2->adrcode;
+		for( i = kb->adrcode - adrcode; i >= 0; --i, ++adrcode ) {
+			/* if the next ipo curve matches the current key, renumber it */
+			if(icu && icu->adrcode == kb->adrcode ) {
+				icu->adrcode = adrcode;
+				icu = icu->next;
+			}
+			/* renumber the shape key */
+			kb->adrcode = adrcode;
+			kb = kb->next;
+		}
 	}
 
 	/* new rule; first key is refkey, this to match drawing channels... */
@@ -903,13 +950,28 @@ static void do_key(int start, int end, int tot, char *poin, Key *key, KeyBlock *
 	}
 }
 
-static float *get_weights_array(Object *ob, Mesh *me, char *vgroup)
+static float *get_weights_array(Object *ob, char *vgroup)
 {
 	bDeformGroup *curdef;
-	int index= 0;
+	MDeformVert *dvert= NULL;
+	int totvert= 0, index= 0;
 	
+	/* no vgroup string set? */
 	if(vgroup[0]==0) return NULL;
-	if(me->dvert==NULL) return NULL;
+	
+	/* gather dvert and totvert */
+	if(ob->type==OB_MESH) {
+		Mesh *me= ob->data;
+		dvert= me->dvert;
+		totvert= me->totvert;
+	}
+	else if(ob->type==OB_LATTICE) {
+		Lattice *lt= ob->data;
+		dvert= lt->dvert;
+		totvert= lt->pntsu*lt->pntsv*lt->pntsw;
+	}
+	
+	if(dvert==NULL) return NULL;
 	
 	/* find the group (weak loop-in-loop) */
 	for (curdef = ob->defbase.first; curdef; curdef=curdef->next, index++)
@@ -917,13 +979,12 @@ static float *get_weights_array(Object *ob, Mesh *me, char *vgroup)
 			break;
 
 	if(curdef) {
-		MDeformVert *dvert= me->dvert;
 		float *weights;
 		int i, j;
 		
-		weights= MEM_callocN(me->totvert*sizeof(float), "weights");
+		weights= MEM_callocN(totvert*sizeof(float), "weights");
 		
-		for (i=0; i < me->totvert; i++, dvert++) {
+		for (i=0; i < totvert; i++, dvert++) {
 			for(j=0; j<dvert->totweight; j++) {
 				if (dvert->dw[j].def_nr == index) {
 					weights[i]= dvert->dw[j].weight;
@@ -989,7 +1050,7 @@ static int do_mesh_key(Object *ob, Mesh *me)
 			KeyBlock *kb;
 			
 			for(kb= me->key->block.first; kb; kb= kb->next)
-				kb->weights= get_weights_array(ob, me, kb->vgroup);
+				kb->weights= get_weights_array(ob, kb->vgroup);
 
 			do_rel_key(0, me->totvert, me->totvert, (char *)me->mvert->co, me->key, 0);
 			
@@ -1164,7 +1225,7 @@ static int do_curve_key(Curve *cu)
 	return 1;
 }
 
-static int do_latt_key(Lattice *lt)
+static int do_latt_key(Object *ob, Lattice *lt)
 {
 	KeyBlock *k[4];
 	float delta, cfra, ctime, t[4];
@@ -1203,7 +1264,17 @@ static int do_latt_key(Lattice *lt)
 		ctime= bsystem_time(NULL, 0, (float)G.scene->r.cfra, 0.0);
 	
 		if(lt->key->type==KEY_RELATIVE) {
+			KeyBlock *kb;
+			
+			for(kb= lt->key->block.first; kb; kb= kb->next)
+				kb->weights= get_weights_array(ob, kb->vgroup);
+			
 			do_rel_key(0, tot, tot, (char *)lt->def->vec, lt->key, 0);
+			
+			for(kb= lt->key->block.first; kb; kb= kb->next) {
+				if(kb->weights) MEM_freeN(kb->weights);
+				kb->weights= NULL;
+			}
 		}
 		else {
 			if(calc_ipo_spec(lt->key->ipo, KEY_SPEED, &ctime)==0) {
@@ -1233,7 +1304,7 @@ int do_ob_key(Object *ob)
 	
 	if(key==NULL)
 		return 0;
-	
+		
 	if(ob->shapeflag & (OB_SHAPE_LOCK|OB_SHAPE_TEMPLOCK)) {
 		KeyBlock *kb= BLI_findlink(&key->block, ob->shapenr-1);
 
@@ -1244,7 +1315,7 @@ int do_ob_key(Object *ob)
 		
 		if(ob->type==OB_MESH) {
 			Mesh *me= ob->data;
-			float *weights= get_weights_array(ob, me, kb->vgroup);
+			float *weights= get_weights_array(ob, kb->vgroup);
 
 			cp_key(0, me->totvert, me->totvert, (char *)me->mvert->co, key, kb, weights, 0);
 			
@@ -1252,9 +1323,12 @@ int do_ob_key(Object *ob)
 		}
 		else if(ob->type==OB_LATTICE) {
 			Lattice *lt= ob->data;
+			float *weights= get_weights_array(ob, kb->vgroup);
 			int tot= lt->pntsu*lt->pntsv*lt->pntsw;
 			
-			cp_key(0, tot, tot, (char *)lt->def->vec, key, kb, NULL, 0);
+			cp_key(0, tot, tot, (char *)lt->def->vec, key, kb, weights, 0);
+			
+			if(weights) MEM_freeN(weights);
 		}
 		else if ELEM(ob->type, OB_CURVE, OB_SURF) {
 			Curve *cu= ob->data;
@@ -1275,7 +1349,7 @@ int do_ob_key(Object *ob)
 		if(ob->type==OB_MESH) return do_mesh_key(ob, ob->data);
 		else if(ob->type==OB_CURVE) return do_curve_key( ob->data);
 		else if(ob->type==OB_SURF) return do_curve_key( ob->data);
-		else if(ob->type==OB_LATTICE) return do_latt_key( ob->data);
+		else if(ob->type==OB_LATTICE) return do_latt_key(ob, ob->data);
 	}
 	
 	return 0;
