@@ -2,7 +2,7 @@
  * lattice.c
  *
  *
- * $Id: lattice.c,v 1.38 2006/11/20 04:27:48 blendix Exp $
+ * $Id: lattice.c,v 1.40 2007/04/04 13:18:28 campbellbarton Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -203,11 +203,11 @@ void resizelattice(Lattice *lt, int uNew, int vNew, int wNew, Object *ltOb)
 	MEM_freeN(vertexCos);
 }
 
-Lattice *add_lattice()
+Lattice *add_lattice(char *name)
 {
 	Lattice *lt;
 	
-	lt= alloc_libblock(&G.main->latt, ID_LT, "Lattice");
+	lt= alloc_libblock(&G.main->latt, ID_LT, name);
 	
 	lt->flag= LT_GRID;
 	
@@ -585,7 +585,7 @@ static float *calc_curve_deform(Object *par, float *co, short axis, CurveDeform
 		
 		/* the tilt */
 		if(loc[3]!=0.0) {
-			Normalise(dir);
+			Normalize(dir);
 			q[0]= (float)cos(0.5*loc[3]);
 			fac= (float)sin(0.5*loc[3]);
 			q[1]= -fac*dir[0];
