@@ -1,5 +1,5 @@
 /** 
- * $Id: library.c,v 1.28 2007/01/25 11:59:29 ton Exp $
+ * $Id: library.c,v 1.35 2007/05/06 21:42:43 khughes Exp $
  * 
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -198,6 +198,28 @@ ListBase *wich_libbase(Main *mainlib, short type)
 	return 0;
 }
 
+/* Flag all ids in listbase */
+void flag_listbase_ids(ListBase *lb, short flag, short value)
+{
+	ID *id;
+	if (value) {
+		for(id= lb->first; id; id= id->next) id->flag |= flag;
+	} else {
+		flag = ~flag;
+		for(id= lb->first; id; id= id->next) id->flag &= flag;
+	}
+}
+
+/* Flag all ids in listbase */
+void flag_all_listbases_ids(short flag, short value)
+{
+	ListBase *lbarray[MAX_LIBARRAY];
+	int a;
+	a= set_listbasepointers(G.main, lbarray);
+	while(a--)	flag_listbase_ids(lbarray[a], flag, value);
+}
+
+
 /* note: MAX_LIBARRAY define should match this code */
 int set_listbasepointers(Main *main, ListBase **lb)
 {
@@ -341,7 +363,7 @@ static ID *alloc_libblock_notest(short type)
 	return id;
 }
 
-// used everywhere in blenkernel and text.c
+/* used everywhere in blenkernel and text.c */
 void *alloc_libblock(ListBase *lb, short type, const char *name)
 {
 	ID *id= NULL;
@@ -365,7 +387,7 @@ void *alloc_libblock(ListBase *lb, short type, const char *name)
 /* from blendef: */
 #define GS(a)	(*((short *)(a)))
 
-// used everywhere in blenkernel and text.c
+/* used everywhere in blenkernel and text.c */
 void *copy_libblock(void *rt)
 {
 	ID *idn, *id;
@@ -396,7 +418,7 @@ static void free_library(Library *lib)
     /* no freeing needed for libraries yet */
 }
 
-// used in headerbuttons.c image.c mesh.c screen.c sound.c and library.c
+/* used in headerbuttons.c image.c mesh.c screen.c sound.c and library.c */
 void free_libblock(ListBase *lb, void *idv)
 {
 	ID *id= idv;
@@ -672,8 +694,8 @@ static void IPOnames_to_dyn_pupstring(DynStr *pupds, ListBase *lb, ID *link, sho
 	}
 }
 
-// used by headerbuttons.c buttons.c editobject.c editseq.c
-// if nr==NULL no MAX_IDPUP, this for non-header browsing
+/* used by headerbuttons.c buttons.c editobject.c editseq.c */
+/* if nr==NULL no MAX_IDPUP, this for non-header browsing */
 void IDnames_to_pupstring(char **str, char *title, char *extraops, ListBase *lb, ID *link, short *nr)
 {
 	DynStr *pupds= BLI_dynstr_new();
@@ -718,7 +740,7 @@ void IMAnames_to_pupstring(char **str, char *title, char *extraops, ListBase *lb
 }
 
 
-// only used by headerbuttons.c
+/* only used by headerbuttons.c */
 void IPOnames_to_pupstring(char **str, char *title, char *extraops, ListBase *lb, ID *link, short *nr, int blocktype)
 {
 	DynStr *pupds= BLI_dynstr_new();
@@ -740,7 +762,7 @@ void IPOnames_to_pupstring(char **str, char *title, char *extraops, ListBase *lb
 	BLI_dynstr_free(pupds);
 }
 
-// used by buttons.c library.c mball.c
+/* used by buttons.c library.c mball.c */
 void splitIDname(char *name, char *left, int *nr)
 {
 	int a;
@@ -788,101 +810,134 @@ static void sort_alpha_id(ListBase *lb, ID *id)
 	
 }
 
-int new_id(ListBase *lb, ID *id, const char *tname)
-/* only for local blocks: external en indirect blocks already have a unique ID */
-/* return 1: created a new name */
+/* 
+ * Check to see if an ID name is already used, and find a new one if so.
+ * Return 1 if created a new name (returned in name).
+ *
+ * Normally the ID that's being check is already in the ListBase, so ID *id
+ * points at the new entry.  The Python Library module needs to know what
+ * the name of a datablock will be before it is appended; in this case ID *id
+ * id is NULL;
+ */
+
+int check_for_dupid(ListBase *lb, ID *id, char *name)
 {
 	ID *idtest;
-	int nr= 0, nrtest, maxtest=32, a;
-	char aname[32], *name, left[32], leftest[32], in_use[32];
+	int nr= 0, nrtest, a;
+	const int maxtest=32;
+	char left[32], leftest[32], in_use[32];
 	
-	/* - split name
-	 * - search
-	 */
+	/* make sure input name is terminated properly */
+	if( strlen(name) > 21 ) name[21]= 0;
 
-	if(id->lib) return 0;
+	while (1) {
 
-	if(tname==0) name= id->name+2;
-	else {
-		/* tname can be const */
-		strncpy(aname, tname, 21);
-		name= aname;
-		
-		if( strlen(name) > 21 ) name[21]= 0;
-	}
-
-	if(lb==0) lb= wich_libbase(G.main, GS(id->name));
-
-	/* phase 1: id already exists? */
-	idtest= lb->first;
-	while(idtest) {
-	
-		if(id!=idtest && idtest->lib==0) {
-			
-			/* do not test alphabetic! */
-			/* optimized */
-			if( idtest->name[2] == name[0] ) {
-				if(strcmp(name, idtest->name+2)==0) break;
+		/* phase 1: id already exists? */
+		for( idtest = lb->first; idtest; idtest = idtest->next ) {
+				/* if idtest is not a lib */ 
+			if( id != idtest && idtest->lib == NULL ) {
+				/* do not test alphabetic! */
+				/* optimized */
+				if( idtest->name[2] == name[0] ) {
+					if(strcmp(name, idtest->name+2)==0) break;
+				}
 			}
 		}
-		
-		idtest= idtest->next;
-	}	
 
-	/* if there is no double return */
-	if(idtest==0) {
-		strncpy(id->name+2, name, 21);
-		return 0;
-	}
-	
-	memset(in_use, 0, maxtest);
+		/* if there is no double, done */
+		if( idtest == NULL ) return 0;
 
-	splitIDname(name, left, &nr);
-	if(nr>999 && strlen(left)>16) left[16]= 0;
-	else if(strlen(left)>17) left[17]= 0;
+		/* we have a dup; need to make a new name */
+		/* quick check so we can reuse one of first 32 ids if vacant */
+		memset(in_use, 0, maxtest);
 
+		/* get name portion, number portion ("name.number") */
+		splitIDname( name, left, &nr);
 
-	idtest= lb->first;
-	while(idtest) {
-	
-		if(id!=idtest && idtest->lib==0) {
-			
-			splitIDname(idtest->name+2, leftest, &nrtest);
-			if(strcmp(left, leftest)==0) {
-				
-				if(nrtest<maxtest) in_use[nrtest]= 1;
-				if(nr <= nrtest) nr= nrtest+1;
+		/* if new name will be too long, truncate it */
+		if(nr>999 && strlen(left)>16) left[16]= 0;
+		else if(strlen(left)>17) left[17]= 0;
+
+		for( idtest = lb->first; idtest; idtest = idtest->next ) {
+			if( id != idtest && idtest->lib == NULL ) {
+				splitIDname(idtest->name+2, leftest, &nrtest);
+				/* if base names match... */
+				/* optimized */
+				if( *left == *leftest && strcmp(left, leftest)==0 ) {
+					if(nrtest < maxtest)
+						in_use[nrtest]= 1;	/* mark as used */
+					if(nr <= nrtest)
+						nr= nrtest+1;		/* track largest unused */
+				}
 			}
 		}
-		
-		idtest= idtest->next;
-	}
-	
-	for(a=0; a<maxtest; a++) {
-		if(a>=nr) break;
-		if( in_use[a]==0 ) {
-			nr= a;
-			break;
-		}
-	}
-	
-	if(nr==0) strncpy(id->name+2, left, 21);
-	else {
-		if (nr >= 1000 && strlen(left) > 16) {
-			// this would overflow name buffer
-			left[16]= 0;
-			return (new_id(lb, id, left));
-		}
-		/* this format specifier is from hell... */
-		sprintf(id->name+2, "%s.%.3d", left, nr);
-	}
-	
-	sort_alpha_id(lb, id);	
 
-	return 1;
+		/* decide which value of nr to use */
+		for(a=0; a<maxtest; a++) {
+			if(a>=nr) break;	/* stop when we've check up to biggest */
+			if( in_use[a]==0 ) { /* found an unused value */
+				nr = a;
+				break;
+			}
+		}
+
+		/* if non-numbered name was not in use, reuse it */
+		if(nr==0) strcpy( name, left );
+		else {
+			if(nr > 999 && strlen(left) > 16) {
+				/* this would overflow name buffer */
+				left[16] = 0;
+				strcpy( name, left );
+				continue;
+			}
+			/* this format specifier is from hell... */
+			sprintf(name, "%s.%.3d", left, nr);
+		}
+		return 1;
+	}
 }
 
-// next to indirect usage in read/writefile also in editobject.c scene.c
+/*
+ * Only for local blocks: external en indirect blocks already have a
+ * unique ID.
+ *
+ * return 1: created a new name
+ */
+
+int new_id(ListBase *lb, ID *id, const char *tname)
+{
+	int result;
+	char name[22];
+	
+	/* if library, don't rename */
+	if(id->lib) return 0;
+
+	/* if no libdata given, look up based on ID */
+	if(lb==NULL) lb= wich_libbase(G.main, GS(id->name));
+
+	if(tname==0) 	/* if no name given, use name of current ID */
+		strncpy(name, id->name+2, 21);
+	else /* else make a copy (tname args can be const) */
+		strncpy(name, tname, 21);
+
+	if( strlen(name) > 21 ) name[21]= 0;
+
+	result = check_for_dupid( lb, id, name );
+	strcpy( id->name+2, name );
+
+	/* This was in 2.43 and previous releases
+	 * however all data in blender should be sorted, not just duplicate names
+	 * sorting should not hurt, but noting just incause it alters the way other
+	 * functions work, so sort every time */
+	/* if( result )
+		sort_alpha_id(lb, id);*/
+	
+	sort_alpha_id(lb, id);
+	
+	return result;
+}
+
+/* next to indirect usage in read/writefile also in editobject.c scene.c */
 void clear_id_newpoins()
 {
 	ListBase *lbarray[MAX_LIBARRAY];
@@ -949,7 +1004,7 @@ static void lib_indirect_test_id(ID *id)
 
 
 /* if lib!=NULL, only all from lib local */
-void all_local(Library *lib)
+void all_local(Library *lib, int untagged_only)
 {
 	ListBase *lbarray[MAX_LIBARRAY], tempbase={0, 0};
 	ID *id, *idn;
@@ -963,7 +1018,14 @@ void all_local(Library *lib)
 			id->newid= NULL;
 			idn= id->next;		/* id is possibly being inserted again */
 			
-			if(id->flag & (LIB_EXTERN|LIB_INDIRECT|LIB_NEW)) {
+			/* The check on the second line (LIB_APPEND_TAG) is done so its
+			 * possible to tag data you dont want to be made local, used for
+			 * appending data, so any libdata alredy linked wont become local
+			 * (very nasty to discover all your links are lost after appending)  
+			 * */
+			if(id->flag & (LIB_EXTERN|LIB_INDIRECT|LIB_NEW) &&
+			  (untagged_only==0 || !(id->flag & LIB_APPEND_TAG)))
+			{
 				if(lib==NULL || id->lib==lib) {
 					id->flag &= ~(LIB_EXTERN|LIB_INDIRECT|LIB_NEW);
 
