@@ -2,7 +2,7 @@
 /*  material.c
  *
  * 
- * $Id: material.c,v 1.55 2007/02/11 14:59:01 halley Exp $
+ * $Id: material.c,v 1.59 2007/05/03 21:37:51 blendix Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -144,6 +144,20 @@ void init_material(Material *ma)
 	ma->pr_lamp= 3;			/* two lamps, is bits */
 	ma->pr_type= MA_SPHERE;
 
+	ma->sss_radius[0]= 1.0f;
+	ma->sss_radius[1]= 1.0f;
+	ma->sss_radius[2]= 1.0f;
+	ma->sss_col[0]= 0.8f;
+	ma->sss_col[1]= 0.8f;
+	ma->sss_col[2]= 0.8f;
+	ma->sss_error= 0.05f;
+	ma->sss_scale= 0.1f;
+	ma->sss_ior= 1.3f;
+	ma->sss_colfac= 1.0f;
+	ma->sss_texfac= 0.0f;
+	ma->sss_front= 1.0f;
+	ma->sss_back= 1.0f;
+
 	ma->mode= MA_TRACEBLE|MA_SHADBUF|MA_SHADOW|MA_RADIO|MA_RAYBIAS|MA_TANGENT_STR;
 }
 
@@ -166,6 +180,8 @@ Material *copy_material(Material *ma)
 	man= copy_libblock(ma);
 	
 	id_us_plus((ID *)man->ipo);
+	id_us_plus((ID *)man->group);
+	
 	
 	for(a=0; a<MAX_MTEX; a++) {
 		if(ma->mtex[a]) {
@@ -366,7 +382,7 @@ Material ***give_matarar(Object *ob)
 		mb= ob->data;
 		return &(mb->mat);
 	}
-	return 0;
+	return NULL;
 }
 
 short *give_totcolp(Object *ob)
@@ -387,15 +403,19 @@ short *give_totcolp(Object *ob)
 		mb= ob->data;
 		return &(mb->totcol);
 	}
-	return 0;
+	return NULL;
 }
 
 Material *give_current_material(Object *ob, int act)
 {
 	Material ***matarar, *ma;
+	short *totcolp;
 	
-	if(ob==NULL) return 0;
-	if(ob->totcol==0) return 0;
+	if(ob==NULL) return NULL;
+	
+	/* if object cannot have material, totcolp==NULL */
+	totcolp= give_totcolp(ob);
+	if(totcolp==NULL || ob->totcol==0) return NULL;
 	
 	if(act>ob->totcol) act= ob->totcol;
 	else if(act<=0) act= 1;
@@ -404,7 +424,6 @@ Material *give_current_material(Object *ob, int act)
 		ma= ob->mat[act-1];
 	}
 	else {								/* in data */
-		short *totcolp= give_totcolp(ob);
 
 		/* check for inconsistancy */
 		if(*totcolp < ob->totcol)
@@ -559,10 +578,12 @@ void new_material_to_objectdata(Object *ob)
 	if(ob->totcol>=MAXMAT) return;
 	
 	ma= give_current_material(ob, ob->actcol);
-	if(ma==0) {
+	if(ma==NULL)
 		ma= add_material("Material");
-		ma->id.us= 0;
-	}
+	else
+		ma= copy_material(ma);
+	
+	ma->id.us= 0; /* eeh... */
 	
 	if(ob->actcol) {
 		if( BTST(ob->colbits, ob->actcol-1) ) {
@@ -695,6 +716,33 @@ void end_render_materials(void)
 			end_render_material(ma);
 }
 
+static int material_in_nodetree(bNodeTree *ntree, Material *mat)
+{
+	bNode *node;
+
+	for(node=ntree->nodes.first; node; node= node->next) {
+		if(node->id && GS(node->id->name)==ID_MA) {
+			if(node->id==(ID*)mat)
+				return 1;
+		}
+		else if(node->type==NODE_GROUP)
+			if(material_in_nodetree((bNodeTree*)node->id, mat))
+				return 1;
+	}
+
+	return 0;
+}
+
+int material_in_material(Material *parmat, Material *mat)
+{
+	if(parmat==mat)
+		return 1;
+	else if(parmat->nodetree && parmat->use_nodes)
+		return material_in_nodetree(parmat->nodetree, mat);
+	else
+		return 0;
+}
+	
 /* ****************** */
 
 char colname_array[125][20]= {
