@@ -3,7 +3,7 @@
  *
  *  
  * 
- * $Id: mesh.c,v 1.81 2007/02/09 15:09:55 ton Exp $
+ * $Id: mesh.c,v 1.85 2007/04/29 13:39:45 campbellbarton Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -228,11 +228,11 @@ void free_dverts(MDeformVert *dvert, int totvert)
 	MEM_freeN (dvert);
 }
 
-Mesh *add_mesh()
+Mesh *add_mesh(char *name)
 {
 	Mesh *me;
 	
-	me= alloc_libblock(&G.main->mesh, ID_ME, "Mesh");
+	me= alloc_libblock(&G.main->mesh, ID_ME, name);
 	
 	me->size[0]= me->size[1]= me->size[2]= 1.0;
 	me->smoothresh= 30;
@@ -502,7 +502,7 @@ static float *make_orco_mesh_internal(Object *ob, int render)
 		MVert *mvert = NULL;
 		
 		if(me->mr) {
-			lvl = multires_level_n(me->mr, me->mr->renderlvl);
+			lvl = multires_level_n(me->mr, me->mr->pinlvl);
 			vcos = MEM_callocN(sizeof(*vcos)*lvl->totvert, "orco mr mesh");
 			mvert = lvl->verts;
 			totvert = lvl->totvert;
@@ -852,7 +852,7 @@ void nurbs_to_mesh(Object *ob)
 	}
 
 	/* make mesh */
-	me= add_mesh();
+	me= add_mesh("Mesh");
 	me->totvert= totvert;
 	me->totface= totvlak;
 
@@ -1072,9 +1072,9 @@ void mesh_calc_normals(MVert *mverts, int numVerts, MFace *mfaces, int numFaces,
 		MVert *mv= &mverts[i];
 		float *no= tnorms[i];
 		
-		if (Normalise(no)==0.0) {
+		if (Normalize(no)==0.0) {
 			VECCOPY(no, mv->co);
-			Normalise(no);
+			Normalize(no);
 		}
 
 		mv->no[0]= (short)(no[0]*32767.0);
@@ -1150,7 +1150,7 @@ UvVertMap *make_uv_vert_map(struct MFace *mface, struct MTFace *tface, unsigned
 	mf= mface;
 	tf= tface;
 	for(a=0; a<totface; a++, mf++, tf++)
-		if(!selected || (!(tf->flag & TF_HIDE) && (tf->flag & TF_SELECT)))
+		if(!selected || (!(mf->flag & ME_HIDE) && (mf->flag & ME_FACE_SEL)))
 			totuv += (mf->v4)? 4: 3;
 		
 	if(totuv==0)
@@ -1171,7 +1171,7 @@ UvVertMap *make_uv_vert_map(struct MFace *mface, struct MTFace *tface, unsigned
 	mf= mface;
 	tf= tface;
 	for(a=0; a<totface; a++, mf++, tf++) {
-		if(!selected || (!(tf->flag & TF_HIDE) && (tf->flag & TF_SELECT))) {
+		if(!selected || (!(mf->flag & ME_HIDE) && (mf->flag & ME_FACE_SEL))) {
 			nverts= (mf->v4)? 4: 3;
 
 			for(i=0; i<nverts; i++) {
