@@ -1,5 +1,5 @@
 /**
- * $Id: node.c,v 1.69 2007/01/26 19:17:20 ton Exp $
+ * $Id: node.c,v 1.81 2007/04/13 04:22:31 scourage Exp $
  *
  * ***** BEGIN GPL LICENSE BLOCK *****
  *
@@ -56,9 +56,20 @@
 #include "MEM_guardedalloc.h"
 #include "IMB_imbuf.h"
 
+#include "RE_pipeline.h"
+#include "RE_shader_ext.h"		/* <- TexResult */
+#include "RE_render_ext.h"		/* <- ibuf_sample() */
+
+#include "CMP_node.h"
+#include "SHD_node.h"
+
 /* not very important, but the stack solver likes to know a maximum */
 #define MAX_SOCKET	64
 
+static ListBase empty_list = {NULL, NULL};
+ListBase node_all_composit = {NULL, NULL};
+ListBase node_all_shaders = {NULL, NULL};
+
 /* ************** Type stuff **********  */
 
 static bNodeType *node_get_type(bNodeTree *ntree, int type, bNodeTree *ngroup)
@@ -70,12 +81,12 @@ static bNodeType *node_get_type(bNodeTree *ntree, int type, bNodeTree *ngroup)
 		return NULL;
 	}
 	else {
-		bNodeType **typedefs= ntree->alltypes;
+		bNodeType *ntype = ntree->alltypes.first;
+		for(; ntype; ntype= ntype->next)
+			if(ntype->type==type)
+				return ntype;
 		
-		while( *typedefs && (*typedefs)->type!=type)
-			typedefs++;
-		
-		return *typedefs;
+		return NULL;
 	}
 }
 
@@ -88,7 +99,7 @@ void ntreeInitTypes(bNodeTree *ntree)
 	else if(ntree->type==NTREE_COMPOSIT)
 		ntree->alltypes= node_all_composit;
 	else {
-		ntree->alltypes= NULL;
+		ntree->alltypes= empty_list;
 		printf("Error: no type definitions for nodes\n");
 	}
 	
@@ -216,10 +227,9 @@ void ntreeVerifyTypes(bNodeTree *ntree)
 {
 	bNode *node;
 	
-	/* commented out, in linux a 2nd initialize on a Scene ntree from a lib crashes... */
-/*	if((ntree->init & NTREE_TYPE_INIT)==0) */
-		ntreeInitTypes(ntree);
-	
+	/* if((ntree->init & NTREE_TYPE_INIT)==0) */
+	ntreeInitTypes(ntree);
+
 	/* check inputs and outputs, and remove or insert them */
 	for(node= ntree->nodes.first; node; node= node->next)
 		nodeVerifyType(ntree, node);
@@ -229,6 +239,7 @@ void ntreeVerifyTypes(bNodeTree *ntree)
 /* ************** Group stuff ********** */
 
 bNodeType node_group_typeinfo= {
+	/* next,prev   */	NULL, NULL,
 	/* type code   */	NODE_GROUP,
 	/* name        */	"Group",
 	/* width+range */	120, 60, 200,
@@ -237,7 +248,11 @@ bNodeType node_group_typeinfo= {
 	/* output sock */	NULL,
 	/* storage     */	"",
 	/* execfunc    */	NULL,
-	
+	/* butfunc     */	NULL,
+	/* initfunc    */	NULL,
+	/* freestoragefunc    */	NULL,
+	/* copystoragefunc    */	NULL,
+	/* id          */	NULL
 };
 
 /* tag internal sockets */
@@ -252,7 +267,7 @@ static void group_tag_internal_sockets(bNodeTree *ngroup)
 		for(sock= node->inputs.first; sock; sock= sock->next)
 			sock->intern= sock->flag & SOCK_HIDDEN;
 		for(sock= node->outputs.first; sock; sock= sock->next)
-			sock->intern= sock->flag & SOCK_HIDDEN;
+			sock->intern= sock->flag & (SOCK_HIDDEN|SOCK_UNAVAIL);
 	}
 	/* set tag */
 	for(link= ngroup->links.first; link; link= link->next) {
@@ -337,13 +352,13 @@ void ntreeMakeOwnType(bNodeTree *ngroup)
 	}
 	
 	/* make own type struct */
-	ngroup->owntype= MEM_mallocN(sizeof(bNodeType), "group type");
-	*ngroup->owntype= node_group_typeinfo;
+	ngroup->owntype= MEM_callocN(sizeof(bNodeType), "group type");
+	*ngroup->owntype= node_group_typeinfo; /* copy data, for init */
 	
 	/* input type arrays */
 	if(totin) {
 		bNodeSocketType *stype;
-		bNodeSocketType *inputs= MEM_mallocN(sizeof(bNodeSocketType)*(totin+1), "bNodeSocketType");
+		bNodeSocketType *inputs= MEM_callocN(sizeof(bNodeSocketType)*(totin+1), "bNodeSocketType");
 		a= 0;
 		
 		for(node= ngroup->nodes.first; node; node= node->next) {
@@ -368,7 +383,7 @@ void ntreeMakeOwnType(bNodeTree *ngroup)
 	/* output type arrays */
 	if(totout) {
 		bNodeSocketType *stype;
-		bNodeSocketType *outputs= MEM_mallocN(sizeof(bNodeSocketType)*(totout+1), "bNodeSocketType");
+		bNodeSocketType *outputs= MEM_callocN(sizeof(bNodeSocketType)*(totout+1), "bNodeSocketType");
 		a= 0;
 		
 		for(node= ngroup->nodes.first; node; node= node->next) {
@@ -753,7 +768,7 @@ bNode *nodeAddNodeType(bNodeTree *ntree, int type, bNodeTree *ngroup)
 	node->type= ntype->type;
 	node->flag= NODE_SELECT|ntype->flag;
 	node->width= ntype->width;
-	node->miniwidth= 15.0f;		/* small value only, allows print of first chars */
+	node->miniwidth= 42.0f;		/* small value only, allows print of first chars */
 	
 	if(type==NODE_GROUP)
 		node->id= (ID *)ngroup;
@@ -774,134 +789,11 @@ bNode *nodeAddNodeType(bNodeTree *ntree, int type, bNodeTree *ngroup)
 	}
 	
 	/* need init handler later? */
-	if(ntree->type==NTREE_SHADER) {
-		if(type==SH_NODE_MATERIAL)
-			node->custom1= SH_NODE_MAT_DIFF|SH_NODE_MAT_SPEC;
-		else if(type==SH_NODE_VALTORGB)
-			node->storage= add_colorband(1);
-		else if(type==SH_NODE_MAPPING)
-			node->storage= add_mapping();
-		else if(type==SH_NODE_CURVE_VEC)
-			node->storage= curvemapping_add(3, -1.0f, -1.0f, 1.0f, 1.0f);
-		else if(type==SH_NODE_CURVE_RGB)
-			node->storage= curvemapping_add(4, 0.0f, 0.0f, 1.0f, 1.0f);
-		else if(type==SH_NODE_GEOMETRY)
-			node->storage= MEM_callocN(sizeof(NodeGeometry), "NodeGeometry");
-	}
-	else if(ntree->type==NTREE_COMPOSIT) {
-		if(type==CMP_NODE_VALTORGB)
-			node->storage= add_colorband(1);
-		else if(type==CMP_NODE_CURVE_VEC)
-			node->storage= curvemapping_add(3, -1.0f, -1.0f, 1.0f, 1.0f);
-		else if(type==CMP_NODE_CURVE_RGB)
-			node->storage= curvemapping_add(4, 0.0f, 0.0f, 1.0f, 1.0f);
-		else if(type==CMP_NODE_TIME) {
-			node->custom1= G.scene->r.sfra;
-			node->custom2= G.scene->r.efra;
-			node->storage= curvemapping_add(1, 0.0f, 0.0f, 1.0f, 1.0f);
-		}
-		else if(type==CMP_NODE_MAP_VALUE)
-			node->storage= add_mapping();
-		else if(type==CMP_NODE_BLUR)
-			node->storage= MEM_callocN(sizeof(NodeBlurData), "node blur data");
-		else if(type==CMP_NODE_DEFOCUS) {
-			/* qdn: defocus node */
-			NodeDefocus *nbd = MEM_callocN(sizeof(NodeDefocus), "node defocus data");
-			nbd->bktype = 0;
-			nbd->rotation = 0.f;
-			nbd->preview = 1;
-			nbd->gamco = 0;
-			nbd->samples = 16;
-			nbd->fstop = 128.f;
-			nbd->maxblur = 0;
-			nbd->bthresh = 1.f;
-			nbd->scale = 1.f;
-			nbd->no_zbuf = 1;
-			node->storage = nbd;
-		}
-		else if(type==CMP_NODE_VECBLUR) {
-			NodeBlurData *nbd= MEM_callocN(sizeof(NodeBlurData), "node blur data");
-			node->storage= nbd;
-			nbd->samples= 32;
-			nbd->fac= 1.0f;
-		}
-		else if(ELEM3(type, CMP_NODE_IMAGE, CMP_NODE_VIEWER, CMP_NODE_SPLITVIEWER)) {
-			ImageUser *iuser= MEM_callocN(sizeof(ImageUser), "node image user");
-			node->storage= iuser;
-			iuser->sfra= 1;
-			iuser->fie_ima= 2;
-			iuser->ok= 1;
-			
-			if(type==CMP_NODE_SPLITVIEWER){
-				node->custom1= 50;	/* default 50% split */
-			}
-		}
-		else if(type==CMP_NODE_HUE_SAT) {
-			NodeHueSat *nhs= MEM_callocN(sizeof(NodeHueSat), "node hue sat");
-			node->storage= nhs;
-			nhs->hue= 0.5f;
-			nhs->sat= 1.0f;
-			nhs->val= 1.0f;
-		}
-		else if(type==CMP_NODE_OUTPUT_FILE) {
-			NodeImageFile *nif= MEM_callocN(sizeof(NodeImageFile), "node image file");
-			node->storage= nif;
-			BLI_strncpy(nif->name, G.scene->r.pic, sizeof(nif->name));
-			nif->imtype= G.scene->r.imtype;
-			nif->subimtype= G.scene->r.subimtype;
-			nif->quality= G.scene->r.quality;
-			nif->sfra= G.scene->r.sfra;
-			nif->efra= G.scene->r.efra;
-		}
-		else if(type==CMP_NODE_DIFF_MATTE){
-			NodeChroma *c= MEM_callocN(sizeof(NodeChroma), "node chroma");
-			node->storage= c;
-			c->t1= 0.01f;
-			c->t2= 0.01f;
-			c->t3= 0.01f;
-			c->fsize= 0.0f;
-			c->fstrength= 0.0f;
-			node->custom1= 1; /* RGB */
-		}
-		else if(type==CMP_NODE_COLOR_SPILL){
-			NodeChroma *c= MEM_callocN(sizeof(NodeChroma), "node chroma");
-			node->storage=c;
-			c->t1= 0.0f;
-			c->t2= 0.0f;
-			c->t3= 0.0f;
-			c->fsize= 0.0f;
-			c->fstrength= 0.0f;
-			node->custom1= 2; /* green channel */
-		}
-		else if(type==CMP_NODE_CHROMA){
-			NodeChroma *c= MEM_callocN(sizeof(NodeChroma), "node chroma");
-			node->storage= c;
-			c->t1= 30.0f;
-			c->t2= 10.0f;
-			c->t3= 0.0f;
-			c->fsize= 0.0f;
-			c->fstrength= 1.0f;
-		}
-      else if(type==CMP_NODE_CHANNEL_MATTE){
-         NodeChroma *c= MEM_callocN(sizeof(NodeChroma), "node chroma");
-         node->storage=c;
-         c->t1= 0.0f;
-         c->t2= 0.0f;
-         c->t3= 0.0f;
-         c->fsize= 0.0f;
-         c->fstrength= 0.0f;
-         node->custom1= 1; /* RGB channel */
-         node->custom2= 2; /* Green Channel */
-      }
-      else if(type==CMP_NODE_LUMA_MATTE){
-         NodeChroma *c= MEM_callocN(sizeof(NodeChroma), "node chroma");
-         node->storage=c;
-         c->t1= 0.0f;
-         c->t2= 0.0f;
-      }
-	}
-	
-	return node;
+   /* got it-bob*/
+   if(ntype->initfunc!=NULL)
+      ntype->initfunc(node);
+
+   return node;
 }
 
 /* keep socket listorder identical, for copying links */
@@ -928,23 +820,8 @@ bNode *nodeCopyNode(struct bNodeTree *ntree, struct bNode *node)
 	if(nnode->id)
 		nnode->id->us++;
 	
-	if(nnode->storage) {
-		/* another candidate for handlerizing! */
-		if(ntree->type==NTREE_SHADER) {
-			if(node->type==SH_NODE_CURVE_VEC || node->type==SH_NODE_CURVE_RGB)
-				nnode->storage= curvemapping_copy(node->storage);
-			else 
-				nnode->storage= MEM_dupallocN(nnode->storage);
-		}
-		else if(ntree->type==NTREE_COMPOSIT) {
-			if(ELEM3(node->type, CMP_NODE_TIME, CMP_NODE_CURVE_VEC, CMP_NODE_CURVE_RGB))
-				nnode->storage= curvemapping_copy(node->storage);
-			else 
-				nnode->storage= MEM_dupallocN(nnode->storage);
-		}
-		else 
-			nnode->storage= MEM_dupallocN(nnode->storage);
-	}
+	if(node->typeinfo->copystoragefunc)
+		node->typeinfo->copystoragefunc(node, nnode);
 	
 	node->new_node= nnode;
 	nnode->new_node= NULL;
@@ -979,6 +856,8 @@ bNodeTree *ntreeAddTree(int type)
 {
 	bNodeTree *ntree= MEM_callocN(sizeof(bNodeTree), "new node tree");
 	ntree->type= type;
+	ntree->alltypes.first = NULL;
+	ntree->alltypes.last = NULL;
 	
 	ntreeInitTypes(ntree);
 	return ntree;
@@ -1121,22 +1000,8 @@ void nodeFreeNode(bNodeTree *ntree, bNode *node)
 			MEM_freeN(node->preview->rect);
 		MEM_freeN(node->preview);
 	}
-	if(node->storage) {
-		/* could be handlerized at some point, now only 1 exception still */
-		if(ntree->type==NTREE_SHADER) {
-			if(node->type==SH_NODE_CURVE_VEC || node->type==SH_NODE_CURVE_RGB)
-				curvemapping_free(node->storage);
-			else 
-				MEM_freeN(node->storage);
-		}
-		else if(ntree->type==NTREE_COMPOSIT) {
-			if(ELEM3(node->type, CMP_NODE_TIME, CMP_NODE_CURVE_VEC, CMP_NODE_CURVE_RGB))
-				curvemapping_free(node->storage);
-			else 
-				MEM_freeN(node->storage);
-		}
-		else 
-			MEM_freeN(node->storage);
+	if(node->typeinfo && node->typeinfo->freestoragefunc) {
+		node->typeinfo->freestoragefunc(node);
 	}
 	MEM_freeN(node);
 }
@@ -1758,6 +1623,10 @@ static void composit_begin_exec(bNodeTree *ntree, int is_group)
 	bNodeSocket *sock;
 	
 	for(node= ntree->nodes.first; node; node= node->next) {
+		
+		/* initialize needed for groups */
+		node->exec= 0;	
+		
 		if(is_group==0) {
 			for(sock= node->outputs.first; sock; sock= sock->next) {
 				bNodeStack *ns= ntree->stack[0] + sock->stack_index;
@@ -1972,11 +1841,6 @@ void ntreeExecTree(bNodeTree *ntree, void *callerdata, int thread)
 
 /* ***************************** threaded version for execute composite nodes ************* */
 
-#define NODE_PROCESSING	1
-#define NODE_READY		2
-#define NODE_FINISHED	4
-#define NODE_FREEBUFS	8
-
 /* not changing info, for thread callback */
 typedef struct ThreadData {
 	bNodeStack *stack;
@@ -2246,7 +2110,7 @@ void ntreeCompositExecTree(bNodeTree *ntree, RenderData *rd, int do_preview)
 		for(node= ntree->nodes.first; node; node= node->next) {
 			if(node->exec & NODE_READY) {
 				if((node->exec & NODE_FINISHED)==0) {
-					BLI_remove_thread(&threads, node);
+					BLI_remove_thread(&threads, node); /* this waits for running thread to finish btw */
 					node->exec |= NODE_FINISHED;
 					
 					/* freeing unused buffers */
@@ -2262,7 +2126,278 @@ void ntreeCompositExecTree(bNodeTree *ntree, RenderData *rd, int do_preview)
 	BLI_end_threads(&threads);
 	
 	ntreeEndExecTree(ntree);
+}
+
+
+/* **************** call to switch lamploop for material node ************ */
+
+void (*node_shader_lamp_loop)(struct ShadeInput *, struct ShadeResult *);
+
+void set_node_shader_lamp_loop(void (*lamp_loop_func)(ShadeInput *, ShadeResult *))
+{
+	node_shader_lamp_loop= lamp_loop_func;
+}
+
+/* clumsy checking... should do dynamic outputs once */
+static void force_hidden_passes(bNode *node, int passflag)
+{
+	bNodeSocket *sock;
 	
+	for(sock= node->outputs.first; sock; sock= sock->next)
+		sock->flag &= ~SOCK_UNAVAIL;
+	
+	sock= BLI_findlink(&node->outputs, RRES_OUT_Z);
+	if(!(passflag & SCE_PASS_Z)) sock->flag |= SOCK_UNAVAIL;
+	sock= BLI_findlink(&node->outputs, RRES_OUT_NORMAL);
+	if(!(passflag & SCE_PASS_NORMAL)) sock->flag |= SOCK_UNAVAIL;
+	sock= BLI_findlink(&node->outputs, RRES_OUT_VEC);
+	if(!(passflag & SCE_PASS_VECTOR)) sock->flag |= SOCK_UNAVAIL;
+	sock= BLI_findlink(&node->outputs, RRES_OUT_UV);
+	if(!(passflag & SCE_PASS_UV)) sock->flag |= SOCK_UNAVAIL;
+	sock= BLI_findlink(&node->outputs, RRES_OUT_RGBA);
+	if(!(passflag & SCE_PASS_RGBA)) sock->flag |= SOCK_UNAVAIL;
+	sock= BLI_findlink(&node->outputs, RRES_OUT_DIFF);
+	if(!(passflag & SCE_PASS_DIFFUSE)) sock->flag |= SOCK_UNAVAIL;
+	sock= BLI_findlink(&node->outputs, RRES_OUT_SPEC);
+	if(!(passflag & SCE_PASS_SPEC)) sock->flag |= SOCK_UNAVAIL;
+	sock= BLI_findlink(&node->outputs, RRES_OUT_SHADOW);
+	if(!(passflag & SCE_PASS_SHADOW)) sock->flag |= SOCK_UNAVAIL;
+	sock= BLI_findlink(&node->outputs, RRES_OUT_AO);
+	if(!(passflag & SCE_PASS_AO)) sock->flag |= SOCK_UNAVAIL;
+	sock= BLI_findlink(&node->outputs, RRES_OUT_REFLECT);
+	if(!(passflag & SCE_PASS_REFLECT)) sock->flag |= SOCK_UNAVAIL;
+	sock= BLI_findlink(&node->outputs, RRES_OUT_REFRACT);
+	if(!(passflag & SCE_PASS_REFRACT)) sock->flag |= SOCK_UNAVAIL;
+	sock= BLI_findlink(&node->outputs, RRES_OUT_RADIO);
+	if(!(passflag & SCE_PASS_RADIO)) sock->flag |= SOCK_UNAVAIL;
+	sock= BLI_findlink(&node->outputs, RRES_OUT_INDEXOB);
+	if(!(passflag & SCE_PASS_INDEXOB)) sock->flag |= SOCK_UNAVAIL;
+	
+}
+
+/* based on rules, force sockets hidden always */
+void ntreeCompositForceHidden(bNodeTree *ntree)
+{
+	bNode *node;
+	
+	if(ntree==NULL) return;
+	
+	for(node= ntree->nodes.first; node; node= node->next) {
+		if( node->type==CMP_NODE_R_LAYERS) {
+			Scene *sce= node->id?(Scene *)node->id:G.scene; /* G.scene is WEAK! */
+			SceneRenderLayer *srl= BLI_findlink(&sce->r.layers, node->custom1);
+			if(srl)
+				force_hidden_passes(node, srl->passflag);
+		}
+		else if( node->type==CMP_NODE_IMAGE) {
+			Image *ima= (Image *)node->id;
+			if(ima) {
+				if(ima->rr) {
+					ImageUser *iuser= node->storage;
+					RenderLayer *rl= BLI_findlink(&ima->rr->layers, iuser->layer);
+					if(rl)
+						force_hidden_passes(node, rl->passflag);
+					else
+						force_hidden_passes(node, 0);
+				}
+				else if(ima->type!=IMA_TYPE_MULTILAYER) {	/* if ->rr not yet read we keep inputs */
+					force_hidden_passes(node, RRES_OUT_Z);
+				}
+				else
+					force_hidden_passes(node, 0);
+			}
+			else
+				force_hidden_passes(node, 0);
+		}
+	}
+
+}
+
+/* called from render pipeline, to tag render input and output */
+/* need to do all scenes, to prevent errors when you re-render 1 scene */
+void ntreeCompositTagRender(Scene *curscene)
+{
+	Scene *sce;
+	
+	for(sce= G.main->scene.first; sce; sce= sce->id.next) {
+		if(sce->nodetree) {
+			bNode *node;
+			
+			for(node= sce->nodetree->nodes.first; node; node= node->next) {
+				if(node->id==(ID *)curscene || node->type==CMP_NODE_COMPOSITE)
+					NodeTagChanged(sce->nodetree, node);
+			}
+		}
+	}
+}
+
+/* tags nodes that have animation capabilities */
+int ntreeCompositTagAnimated(bNodeTree *ntree)
+{
+	bNode *node;
+	int tagged= 0;
+	
+	if(ntree==NULL) return 0;
+	
+	for(node= ntree->nodes.first; node; node= node->next) {
+		if(node->type==CMP_NODE_IMAGE) {
+			Image *ima= (Image *)node->id;
+			if(ima && ELEM(ima->source, IMA_SRC_MOVIE, IMA_SRC_SEQUENCE)) {
+				NodeTagChanged(ntree, node);
+				tagged= 1;
+			}
+		}
+		else if(node->type==CMP_NODE_TIME) {
+			NodeTagChanged(ntree, node);
+			tagged= 1;
+		}
+		else if(node->type==CMP_NODE_R_LAYERS) {
+			NodeTagChanged(ntree, node);
+			tagged= 1;
+		}
+		else if(node->type==NODE_GROUP) {
+			if( ntreeCompositTagAnimated((bNodeTree *)node->id) ) {
+				NodeTagChanged(ntree, node);
+			}
+		}
+	}
+	
+	return tagged;
+}
+
+
+/* called from image window preview */
+void ntreeCompositTagGenerators(bNodeTree *ntree)
+{
+	bNode *node;
+	
+	if(ntree==NULL) return;
+	
+	for(node= ntree->nodes.first; node; node= node->next) {
+		if( ELEM(node->type, CMP_NODE_R_LAYERS, CMP_NODE_IMAGE))
+			NodeTagChanged(ntree, node);
+	}
+}
+
+/* ************* node definition init ********** */
+
+static bNodeType *is_nodetype_registered(ListBase *typelist, int type) 
+{
+	bNodeType *ntype= typelist->first;
+	
+	for(;ntype; ntype= ntype->next )
+		if(ntype->type==type)
+			return ntype;
+	
+	return NULL;
+}
+
+/* type can be from a static array, we make copy for duplicate types (like group) */
+void nodeRegisterType(ListBase *typelist, const bNodeType *ntype) 
+{
+	bNodeType *found= is_nodetype_registered(typelist, ntype->type);
+	
+	if(found==NULL) {
+		bNodeType *ntypen= MEM_mallocN(sizeof(bNodeType), "node type");
+		*ntypen= *ntype;
+		BLI_addtail(typelist, ntypen);
+ 	}
+}
+
+static void registerCompositNodes(ListBase *ntypelist)
+{
+	nodeRegisterType(ntypelist, &node_group_typeinfo);
+	nodeRegisterType(ntypelist, &cmp_node_rlayers);
+	nodeRegisterType(ntypelist, &cmp_node_image);
+	nodeRegisterType(ntypelist, &cmp_node_texture);
+	nodeRegisterType(ntypelist, &cmp_node_value);
+	nodeRegisterType(ntypelist, &cmp_node_rgb);
+	nodeRegisterType(ntypelist, &cmp_node_curve_time);
+	
+	nodeRegisterType(ntypelist, &cmp_node_composite);
+	nodeRegisterType(ntypelist, &cmp_node_viewer);
+	nodeRegisterType(ntypelist, &cmp_node_splitviewer);
+	nodeRegisterType(ntypelist, &cmp_node_output_file);
+	
+	nodeRegisterType(ntypelist, &cmp_node_curve_rgb);
+	nodeRegisterType(ntypelist, &cmp_node_mix_rgb);
+	nodeRegisterType(ntypelist, &cmp_node_hue_sat);
+	nodeRegisterType(ntypelist, &cmp_node_alphaover);
+	nodeRegisterType(ntypelist, &cmp_node_zcombine);
+	
+	nodeRegisterType(ntypelist, &cmp_node_normal);
+	nodeRegisterType(ntypelist, &cmp_node_curve_vec);
+	nodeRegisterType(ntypelist, &cmp_node_map_value);
+	
+	nodeRegisterType(ntypelist, &cmp_node_filter);
+	nodeRegisterType(ntypelist, &cmp_node_blur);
+	nodeRegisterType(ntypelist, &cmp_node_vecblur);
+	nodeRegisterType(ntypelist, &cmp_node_dilateerode);
+	nodeRegisterType(ntypelist, &cmp_node_defocus);
+	
+	nodeRegisterType(ntypelist, &cmp_node_valtorgb);
+	nodeRegisterType(ntypelist, &cmp_node_rgbtobw);
+	nodeRegisterType(ntypelist, &cmp_node_setalpha);
+	nodeRegisterType(ntypelist, &cmp_node_idmask);
+	nodeRegisterType(ntypelist, &cmp_node_math);
+	nodeRegisterType(ntypelist, &cmp_node_seprgba);
+	nodeRegisterType(ntypelist, &cmp_node_combrgba);
+	nodeRegisterType(ntypelist, &cmp_node_sephsva);
+	nodeRegisterType(ntypelist, &cmp_node_combhsva);
+	nodeRegisterType(ntypelist, &cmp_node_sepyuva);
+	nodeRegisterType(ntypelist, &cmp_node_combyuva);
+	nodeRegisterType(ntypelist, &cmp_node_sepycca);
+	nodeRegisterType(ntypelist, &cmp_node_combycca);
+	
+	nodeRegisterType(ntypelist, &cmp_node_diff_matte);
+	nodeRegisterType(ntypelist, &cmp_node_chroma);
+	nodeRegisterType(ntypelist, &cmp_node_channel_matte);
+	nodeRegisterType(ntypelist, &cmp_node_color_spill);
+	nodeRegisterType(ntypelist, &cmp_node_luma_matte);
+	
+	nodeRegisterType(ntypelist, &cmp_node_translate);
+	nodeRegisterType(ntypelist, &cmp_node_rotate);
+	nodeRegisterType(ntypelist, &cmp_node_scale);
+	nodeRegisterType(ntypelist, &cmp_node_flip);
+	nodeRegisterType(ntypelist, &cmp_node_displace);
+	nodeRegisterType(ntypelist, &cmp_node_mapuv);
+
+	nodeRegisterType(ntypelist, &cmp_node_brightcontrast);
+	nodeRegisterType(ntypelist, &cmp_node_gamma);
+}
+
+static void registerShaderNodes(ListBase *ntypelist) 
+{
+	nodeRegisterType(ntypelist, &node_group_typeinfo);
+	nodeRegisterType(ntypelist, &sh_node_output);
+	nodeRegisterType(ntypelist, &sh_node_mix_rgb);
+	nodeRegisterType(ntypelist, &sh_node_valtorgb);
+	nodeRegisterType(ntypelist, &sh_node_rgbtobw);
+	nodeRegisterType(ntypelist, &sh_node_normal);
+	nodeRegisterType(ntypelist, &sh_node_geom);
+	nodeRegisterType(ntypelist, &sh_node_mapping);
+	nodeRegisterType(ntypelist, &sh_node_curve_vec);
+	nodeRegisterType(ntypelist, &sh_node_curve_rgb);
+	nodeRegisterType(ntypelist, &sh_node_math);
+	nodeRegisterType(ntypelist, &sh_node_vect_math);
+	nodeRegisterType(ntypelist, &sh_node_squeeze);
+	nodeRegisterType(ntypelist, &sh_node_camera);
+	nodeRegisterType(ntypelist, &sh_node_material);
+	nodeRegisterType(ntypelist, &sh_node_value);
+	nodeRegisterType(ntypelist, &sh_node_rgb);
+	nodeRegisterType(ntypelist, &sh_node_texture);
+}
+
+void init_nodesystem(void) 
+{
+	registerCompositNodes(&node_all_composit);
+	registerShaderNodes(&node_all_shaders);
+}
+
+void free_nodesystem(void) 
+{
+	BLI_freelistN(&node_all_composit);
+	BLI_freelistN(&node_all_shaders);
 }
 
 
