@@ -1,7 +1,7 @@
 /* object.c
  *
  * 
- * $Id: object.c,v 1.123 2006/12/27 10:21:33 ton Exp $
+ * $Id: object.c,v 1.129 2007/05/04 16:36:39 artificer Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -489,13 +489,14 @@ int exist_object(Object *obtest)
 	return 0;
 }
 
-void *add_camera()
+void *add_camera(char *name)
 {
 	Camera *cam;
 	
-	cam=  alloc_libblock(&G.main->camera, ID_CA, "Camera");
+	cam=  alloc_libblock(&G.main->camera, ID_CA, name);
 
 	cam->lens= 35.0f;
+	cam->angle= 49.14f;
 	cam->clipsta= 0.1f;
 	cam->clipend= 100.0f;
 	cam->drawsize= 0.5f;
@@ -574,11 +575,11 @@ void make_local_camera(Camera *cam)
 
 
 
-void *add_lamp(void)
+void *add_lamp(char *name)
 {
 	Lamp *la;
 	
-	la=  alloc_libblock(&G.main->lamp, ID_LA, "Lamp");
+	la=  alloc_libblock(&G.main->lamp, ID_LA, name);
 	
 	la->r= la->g= la->b= la->k= 1.0;
 	la->haint= la->energy= 1.0;
@@ -712,16 +713,16 @@ void *add_wave()
 static void *add_obdata_from_type(int type)
 {
 	switch (type) {
-	case OB_MESH: G.totmesh++; return add_mesh();
-	case OB_CURVE: G.totcurve++; return add_curve(OB_CURVE);
-	case OB_SURF: G.totcurve++; return add_curve(OB_SURF);
-	case OB_FONT: return add_curve(OB_FONT);
-	case OB_MBALL: return add_mball();
-	case OB_CAMERA: return add_camera();
-	case OB_LAMP: G.totlamp++; return add_lamp();
-	case OB_LATTICE: return add_lattice();
+	case OB_MESH: G.totmesh++; return add_mesh("Mesh");
+	case OB_CURVE: G.totcurve++; return add_curve("Curve", OB_CURVE);
+	case OB_SURF: G.totcurve++; return add_curve("Surf", OB_SURF);
+	case OB_FONT: return add_curve("Text", OB_FONT);
+	case OB_MBALL: return add_mball("Meta");
+	case OB_CAMERA: return add_camera("Camera");
+	case OB_LAMP: G.totlamp++; return add_lamp("Lamp");
+	case OB_LATTICE: return add_lattice("Lattice");
 	case OB_WAVE: return add_wave();
-	case OB_ARMATURE: return add_armature();
+	case OB_ARMATURE: return add_armature("Armature");
 	case OB_EMPTY: return NULL;
 	default:
 		printf("add_obdata_from_type: Internal error, bad type: %d\n", type);
@@ -833,8 +834,15 @@ Object *add_object(int type)
 void base_init_from_view3d(Base *base, View3D *v3d)
 {
 	Object *ob= base->object;
-
-	if (v3d->localview) {
+	
+	if (!v3d) {
+		/* no 3d view, this wont happen often */
+		base->lay = 1;
+		VECCOPY(ob->loc, G.scene->cursor);
+		
+		/* return now because v3d->viewquat isnt available */
+		return;
+	} else if (v3d->localview) {
 		base->lay= ob->lay= v3d->layact + v3d->lay;
 		VECCOPY(ob->loc, v3d->cursor);
 	} else {
@@ -1260,7 +1268,7 @@ static void ob_parcurve(Object *ob, Object *par, float mat[][4])
 			quat= vectoquat(dir, ob->trackflag, ob->upflag);
 			
 			/* the tilt */
-			Normalise(dir);
+			Normalize(dir);
 			q[0]= (float)cos(0.5*vec[3]);
 			x1= (float)sin(0.5*vec[3]);
 			q[1]= -x1*dir[0];
@@ -1324,15 +1332,26 @@ static void give_parvert(Object *par, int nr, float *vec)
 			}
 		}
 		else {
-			/* maybe this is against the derivedmesh philosphy, but where_is_object is called
-			   by code that is called by build_mesh... (when ob->sf!=0.0) so it can cycle eternally */
-			DerivedMesh *dm = par->derivedDeform; //mesh_get_derived_deform(par, &needsFree);
+			DerivedMesh *dm = par->derivedFinal;
 			
 			if(dm) {
-				int tot= dm->getNumVerts(dm);
-				if(tot) {
-					if(nr >= tot) nr= 0;
-					dm->getVertCo(dm, nr, vec);
+				int i, count = 0, numVerts = dm->getNumVerts(dm);
+				int *index = (int *)dm->getVertDataArray(dm, CD_ORIGINDEX);
+				float co[3];
+
+				/* get the average of all verts with (original index == nr) */
+				for(i = 0; i < numVerts; ++i, ++index) {
+					if(*index == nr) {
+						dm->getVertCo(dm, i, co);
+						VecAddf(vec, vec, co);
+						count++;
+					}
+				}
+
+				if(count > 0) {
+					VecMulf(vec, 1.0f / count);
+				} else {
+					dm->getVertCo(dm, 0, vec);
 				}
 			}
 		}
@@ -1972,6 +1991,28 @@ void minmax_object(Object *ob, float *min, float *max)
 	}
 }
 
+/* TODO - use dupli objects bounding boxes */
+void minmax_object_duplis(Object *ob, float *min, float *max)
+{
+	if ((ob->transflag & OB_DUPLI)==0) {
+		return;
+	} else {
+		ListBase *lb;
+		DupliObject *dob;
+		
+		lb= object_duplilist(G.scene, ob);
+		for(dob= lb->first; dob; dob= dob->next) {
+			if(dob->no_draw);
+			else {
+				/* should really use bound box of dup object */
+				DO_MINMAX(dob->mat[3], min, max);
+			}
+		}
+		free_object_duplilist(lb);	/* does restore */
+	}
+}
+
+
 /* proxy rule: lib_object->proxy_from == the one we borrow from, only set temporal and cleared here */
 /*           local_object->proxy      == pointer to library object, saved in files and read */
 
@@ -2052,5 +2093,3 @@ void object_handle_update(Object *ob)
 		// printf("set proxy pointer for later group stuff %s\n", ob->id.name);
 	}
 }
-
-
