@@ -1,7 +1,7 @@
 /*  scene.c
  *  
  * 
- * $Id: scene.c,v 1.42 2006/06/22 15:25:30 ton Exp $
+ * $Id: scene.c,v 1.51 2007/01/31 10:59:39 ton Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -56,6 +56,7 @@
 #include "DNA_object_types.h"
 #include "DNA_scene_types.h"
 #include "DNA_scriptlink_types.h"
+#include "DNA_texture_types.h"
 #include "DNA_userdef_types.h"
 
 #include "BKE_action.h"			
@@ -67,6 +68,7 @@
 #include "BKE_global.h"
 #include "BKE_group.h"
 #include "BKE_ipo.h"
+#include "BKE_image.h"
 #include "BKE_key.h"
 #include "BKE_library.h"
 #include "BKE_main.h"
@@ -76,6 +78,9 @@
 #include "BKE_world.h"
 #include "BKE_utildefines.h"
 
+#include "BIF_previewrender.h"
+#include "BDR_sculptmode.h"
+
 #include "BPY_extern.h"
 #include "BLI_arithb.h"
 #include "BLI_blenlib.h"
@@ -162,6 +167,8 @@ void free_scene(Scene *sce)
 		ntreeFreeTree(sce->nodetree);
 		MEM_freeN(sce->nodetree);
 	}
+
+	sculptmode_free_all(sce);
 }
 
 Scene *add_scene(char *name)
@@ -190,22 +197,19 @@ Scene *add_scene(char *name)
 	sce->r.images= 100;
 	sce->r.framelen= 1.0;
 	sce->r.frs_sec= 25;
-
-	sce->r.postgamma= 1.0;
-	sce->r.postsat= 1.0;
-	sce->r.postmul= 1.0;
 	
-	sce->r.focus= 0.9;
-	sce->r.zgamma= 1.0;
-	sce->r.zsigma= 4.0;
-	sce->r.zblur= 10.0;
-	sce->r.zmin= 0.8;
+	sce->r.bake_mode= 1;	/* prevent to include render stuff here */
+	sce->r.bake_filter= 2;
+	sce->r.bake_osa= 5;
+	sce->r.bake_flag= R_BAKE_CLEAR;
 	
 	sce->r.xplay= 640;
 	sce->r.yplay= 480;
 	sce->r.freqplay= 60;
 	sce->r.depth= 32;
-
+	
+	sce->r.threads= 1;
+	
 	sce->r.stereomode = 1;  // no stereo
 
 	sce->toolsettings = MEM_callocN(sizeof(struct ToolSettings),"Tool Settings Struct");
@@ -224,6 +228,7 @@ Scene *add_scene(char *name)
 	sce->toolsettings->uvcalc_mapdir = 1;
 	sce->toolsettings->uvcalc_mapalign = 1;
 	sce->toolsettings->unwrapper = 1;
+	sce->toolsettings->select_thresh= 0.01f;
 	
 	sce->jumpframe = 10;
 	sce->audio.mixrate = 44100;
@@ -231,9 +236,11 @@ Scene *add_scene(char *name)
 	strcpy(sce->r.backbuf, "//backbuf");
 	strcpy(sce->r.pic, U.renderdir);
 	strcpy(sce->r.ftype, "//ftype");
-	
+
 	BLI_init_rctf(&sce->r.safety, 0.1f, 0.9f, 0.1f, 0.9f);
 	sce->r.osa= 8;
+
+	sculptmode_init(sce);
 	
 	/* note; in header_info.c the scene copy happens..., if you add more to renderdata it has to be checked there */
 	scene_add_render_layer(sce);
@@ -267,29 +274,28 @@ void set_scene_bg(Scene *sce)
 	scene_check_setscene(G.scene);
 	
 	/* deselect objects (for dataselect) */
-	ob= G.main->object.first;
-	while(ob) {
+	for(ob= G.main->object.first; ob; ob= ob->id.next)
 		ob->flag &= ~(SELECT|OB_FROMGROUP);
-		ob= ob->id.next;
-	}
 
 	/* group flags again */
-	group= G.main->group.first;
-	while(group) {
+	for(group= G.main->group.first; group; group= group->id.next) {
 		go= group->gobject.first;
 		while(go) {
 			if(go->ob) go->ob->flag |= OB_FROMGROUP;
 			go= go->next;
 		}
-		group= group->id.next;
 	}
 
 	/* sort baselist */
 	DAG_scene_sort(sce);
+	
+	/* ensure dags are built for sets */
+	for(sce= sce->set; sce; sce= sce->set)
+		if(sce->theDag==NULL)
+			DAG_scene_sort(sce);
 
 	/* copy layers and flags from bases to objects */
-	base= G.scene->base.first;
-	while(base) {
+	for(base= G.scene->base.first; base; base= base->next) {
 		ob= base->object;
 		ob->lay= base->lay;
 		
@@ -298,10 +304,11 @@ void set_scene_bg(Scene *sce)
 		flag= ob->flag & (OB_FROMGROUP);
 		base->flag |= flag;
 		
+		/* not too nice... for recovering objects with lost data */
+		if(ob->pose==NULL) base->flag &= ~OB_POSEMODE;
 		ob->flag= base->flag;
 		
 		ob->ctime= -1234567.0;	/* force ipo to be calculated later */
-		base= base->next;
 	}
 	/* no full animation update, this to enable render code to work (render code calls own animation updates) */
 	
@@ -382,11 +389,14 @@ int next_object(int val, Base **base, Object **ob)
 			else {
 				if(fase!=F_DUPLI) {
 					if( (*base)->object->transflag & OB_DUPLI) {
-						
-						duplilist= object_duplilist(G.scene, (*base)->object);
-						
-						dupob= duplilist->first;
-						
+						/* groups cannot be duplicated for mballs yet, 
+						this enters eternal loop because of 
+						makeDispListMBall getting called inside of group_duplilist */
+						if((*base)->object->dup_group == NULL) {
+							duplilist= object_duplilist(G.scene, (*base)->object);
+							
+							dupob= duplilist->first;
+						}
 					}
 				}
 				/* handle dupli's */
@@ -491,35 +501,44 @@ int scene_check_setscene(Scene *sce)
 	return 1;
 }
 
-/* applies changes right away */
-void scene_update_for_newframe(Scene *sce, unsigned int lay)
+static void scene_update(Scene *sce, unsigned int lay)
 {
 	Base *base;
 	Object *ob;
 	
+	if(sce->theDag==NULL)
+		DAG_scene_sort(sce);
+	
+	DAG_scene_update_flags(sce, lay);   // only stuff that moves or needs display still
+	
+	for(base= sce->base.first; base; base= base->next) {
+		ob= base->object;
+		
+		object_handle_update(ob);   // bke_object.h
+		
+		/* only update layer when an ipo */
+		if(ob->ipo && has_ipo_code(ob->ipo, OB_LAY) ) {
+			base->lay= ob->lay;
+		}
+	}
+}
+
+/* applies changes right away, does all sets too */
+void scene_update_for_newframe(Scene *sce, unsigned int lay)
+{
+	Scene *scene= sce;
+	
 	/* object ipos are calculated in where_is_object */
 	do_all_data_ipos();
 	
 	if (G.f & G_DOSCRIPTLINKS) BPY_do_all_scripts(SCRIPT_FRAMECHANGED);
 	
-	while(sce) {
-		if(sce->theDag==NULL)
-			DAG_scene_sort(sce);
-		
-		DAG_scene_update_flags(sce, lay);   // only stuff that moves or needs display still
-		
-		for(base= sce->base.first; base; base= base->next) {
-			ob= base->object;
-			
-			object_handle_update(ob);   // bke_object.h
-			
-			/* only update layer when an ipo */
-			if(ob->ipo && has_ipo_code(ob->ipo, OB_LAY) ) {
-				base->lay= ob->lay;
-			}
-		}
-		sce= sce->set;
-	}
+	/* sets first, we allow per definition current scene to have dependencies on sets */
+	for(sce= sce->set; sce; sce= sce->set)
+		scene_update(sce, lay);
+
+	scene_update(scene, lay);
+	
 }
 
 /* return default layer, also used to patch old files */
