@@ -1,7 +1,7 @@
 /* text.c
  *
  *
- * $Id: text.c,v 1.22 2006/11/26 16:03:35 ton Exp $
+ * $Id: text.c,v 1.24 2007/03/12 06:21:57 campbellbarton Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -154,12 +154,12 @@ void free_text(Text *text)
 	if (text->compiled) BPY_free_compiled_text(text);
 }
 
-Text *add_empty_text(void) 
+Text *add_empty_text(char *name) 
 {
 	Text *ta;
 	TextLine *tmp;
 	
-	ta= alloc_libblock(&G.main->text, ID_TXT, "Text");
+	ta= alloc_libblock(&G.main->text, ID_TXT, name);
 	ta->id.us= 1;
 	
 	ta->name= NULL;
