@@ -1,7 +1,7 @@
 /* texture.c
  *
  *
- * $Id: texture.c,v 1.34 2007/01/29 20:43:27 sirdude Exp $
+ * $Id: texture.c,v 1.35 2007/04/16 10:55:59 ton Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -292,7 +292,7 @@ int do_colorband(ColorBand *coba, float in, float out[4])
 	float fac, mfac, t[4];
 	int a;
 	
-	if(coba->tot==0) return 0;
+	if(coba==NULL || coba->tot==0) return 0;
 	
 	cbd1= coba->data;
 	if(coba->tot==1) {
