@@ -2,7 +2,7 @@
 /*  world.c
  * 
  * 
- * $Id: world.c,v 1.16 2005/06/02 10:32:54 ton Exp $
+ * $Id: world.c,v 1.20 2006/04/27 04:37:19 erwin Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -55,6 +55,7 @@
 #include "BKE_world.h"
 #include "BKE_global.h"
 #include "BKE_main.h"
+#include "BKE_icons.h"
 
 #include "BPY_extern.h"
 
@@ -75,6 +76,8 @@ void free_world(World *wrld)
 		if(mtex) MEM_freeN(mtex);
 	}
 	wrld->ipo= 0;
+	BKE_icon_delete((struct ID*)wrld);
+	wrld->id.icon_id = 0;
 }
 
 
@@ -98,7 +101,7 @@ World *add_world(char *name)
 	wrld->aoenergy= 1.0;
 	wrld->aobias= 0.05;
 	
-	wrld->physicsEngine= WOPHY_SUMO;
+	wrld->physicsEngine= WOPHY_BULLET;//WOPHY_SUMO; Bullet by default
 	
 	return wrld;
 }
@@ -175,38 +178,3 @@ void make_local_world(World *wrld)
 		}
 	}
 }
-
-
-void init_render_world()
-{
-	int a;
-	char *cp;
-	
-	if(G.scene->world) {
-		R.wrld= *(G.scene->world);
-		
-		cp= (char *)&R.wrld.fastcol;
-		
-		cp[0]= 255.0*R.wrld.horr;
-		cp[1]= 255.0*R.wrld.horg;
-		cp[2]= 255.0*R.wrld.horb;
-		cp[3]= 1;
-		
-		VECCOPY(R.grvec, R.viewmat[2]);
-		Normalise(R.grvec);
-		Mat3CpyMat4(R.imat, R.viewinv);
-		
-		for(a=0; a<MAX_MTEX; a++) 
-			if(R.wrld.mtex[a] && R.wrld.mtex[a]->tex) R.wrld.skytype |= WO_SKYTEX;
-		
-		while(R.wrld.aosamp*R.wrld.aosamp < R.osa) R.wrld.aosamp++;
-	}
-	else {
-		memset(&R.wrld, 0, sizeof(World));
-		R.wrld.exp= 0.0;
-		R.wrld.range= 1.0;
-	}
-
-	R.wrld.linfac= 1.0 + pow((2.0*R.wrld.exp + 0.5), -10);
-	R.wrld.logfac= log( (R.wrld.linfac-1.0)/R.wrld.linfac )/R.wrld.range;
-}
