@@ -4,7 +4,7 @@
  * Some editing types needed in the lib (unfortunately) for
  * scanfill.c
  *
- * $Id: BLI_editVert.h,v 1.25 2006/03/27 05:19:14 briggs Exp $
+ * $Id: BLI_editVert.h,v 1.32 2007/01/27 18:11:06 nicholasbishop Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -38,7 +38,11 @@
 #ifndef BLI_EDITVERT_H
 #define BLI_EDITVERT_H
 
+#include "DNA_customdata_types.h"
+#include "DNA_mesh_types.h"
+
 struct DerivedMesh;
+struct RetopoPaintData;
 
 /* note; changing this also might affect the undo copy in editmesh.c */
 typedef struct EditVert
@@ -64,10 +68,13 @@ typedef struct EditVert
 	f1 and f2 can be used for temp data, clear them first*/
 	unsigned char f, h, f1, f2; 
 	short fast;	/* only 0 or 1, for editmesh_fastmalloc, do not store temp data here! */
-	short	totweight; /* __NLA total number of vertex weights for this vertex */
 	int hash;
-	struct MDeformWeight *dw;	/* __NLA a pointer to an array of defirm weights */
 	int keyindex; /* original index #, for restoring  key information */
+/*#ifdef WITH_VERSE*/
+	void *vvert;
+/*#endif*/
+
+	void *data;		/* custom vertex data */
 } EditVert;
 
 struct EditEdge;
@@ -94,7 +101,7 @@ typedef struct EditEdge
 		float			fp;
 	} tmp;
 	short f1, f2;	/* short, f1 is (ab)used in subdiv */
-	unsigned char f, h, dir, seam;
+	unsigned char f, h, dir, seam, sharp;
 	float crease;
 	short fast; 		/* only 0 or 1, for editmesh_fastmalloc */
 	short fgoni;		/* index for fgon, for search */
@@ -119,11 +126,14 @@ typedef struct EditFace
 		float			fp;
 	} tmp;
 	float n[3], cent[3];
-	struct TFace tf;	/* a copy of original tface. */
 	unsigned char mat_nr, flag;
 	unsigned char f, f1, h;
 	unsigned char fast;			/* only 0 or 1, for editmesh_fastmalloc */
 	unsigned char fgonf;		/* flag for fgon options */
+/*#ifdef WITH_VERSE*/
+	void *vface;
+/*#endif*/
+	void *data;		/* custom face data */
 } EditFace;
 
 
@@ -154,6 +164,18 @@ typedef struct EditMesh
 		 * to derived final, care should be taken on release.
 		 */
 	struct DerivedMesh *derivedCage, *derivedFinal;
+	/* the custom data layer mask that was last used to calculate
+	 * derivedCage and derivedFinal
+	 */
+	int lastDataMask;
+
+	struct RetopoPaintData *retopo_paint_data;
+
+	CustomData vdata, fdata;
+
+#ifdef WITH_VERSE
+	void *vnode;
+#endif
 } EditMesh;
 
 #endif
