@@ -2,7 +2,7 @@
  * @file BLI_rand.h
  * 
  * Random number functions.
- * $Id: BLI_rand.h,v 1.5 2002/12/27 13:10:12 mein Exp $
+ * $Id: BLI_rand.h,v 1.9 2005/08/25 13:11:04 ton Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -36,9 +36,28 @@
 #ifndef BLI_RAND_H
 #define BLI_RAND_H
 
+	/* RNG is just an abstract random number generator
+	 * type that avoids using globals, otherwise identical
+	 * to BLI_rand functions below.
+	 */
+struct RNG;
+typedef struct RNG RNG;
+
+struct RNG*	rng_new			(unsigned int seed);
+void		rng_free		(struct RNG* rng);
+
+void		rng_seed		(struct RNG* rng, unsigned int seed);
+int			rng_getInt		(struct RNG* rng);
+double		rng_getDouble	(struct RNG* rng);
+float		rng_getFloat	(struct RNG* rng);
+void		rng_shuffleArray(struct RNG *rng, void *data, int elemSize, int numElems);
+
 	/** Seed the random number generator */
 void	BLI_srand		(unsigned int seed);
 
+	/** Better seed for the random number generator, using noise.c hash[] */
+void	BLI_srandom		(unsigned int seed);
+
 	/** Return a pseudo-random number N where 0<=N<(2^31) */
 int		BLI_rand		(void);
 
@@ -55,15 +74,26 @@ float	BLI_frand		(void);
 	 */
 void	BLI_fillrand	(void *addr, int len);
 
-	/** Stores the BLI randum number generator state
-	 * into the buffer in @a loc_r.
+	/** Shuffle an array randomly using the given seed.
+	 * contents. This routine does not use nor modify
+	 * the state of the BLI random number generator.
 	 */
-void	BLI_storerand	(unsigned int loc_r[2]);
+void	BLI_array_randomize	(void *data, int elemSize, int numElems, unsigned int seed);
+
+
+	/** Better seed for the random number generator, using noise.c hash[] */
+	/** Allows up to 16 threads to address */
+void	BLI_thread_srandom	(int thread, unsigned int seed);
+
+	/** Return a pseudo-random number N where 0<=N<(2^31) */
+	/** Allows up to 16 threads to address */
+int		BLI_thread_rand		(int thread);
+
+	/** Return a pseudo-random number N where 0.0f<=N<1.0f */
+	/** Allows up to 16 threads to address */
+float	BLI_thread_frand	(int thread);
+
 
-	/** Retores the BLI randum number generator state
-	 * from the buffer in @a loc.
-	 */
-void	BLI_restorerand	(unsigned int loc[2]);
 
 #endif
 
