@@ -1,5 +1,5 @@
 /**
- * $Id: rand.c,v 1.10 2005/11/05 21:17:55 ton Exp $
+ * $Id: rand.c,v 1.12 2006/11/29 17:01:07 ton Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -36,6 +36,8 @@
 #include "MEM_guardedalloc.h"
 
 #include "PIL_time.h"
+
+#include "BLI_threads.h"
 #include "BLI_rand.h"
 
 #ifdef HAVE_CONFIG_H
@@ -163,14 +165,16 @@ void BLI_array_randomize(void *data, int elemSize, int numElems, unsigned int se
 }
 
 /* ********* for threaded random ************** */
-#define MAX_RNG_THREADS		16
 
-static RNG rng_tab[MAX_RNG_THREADS];
+static RNG rng_tab[BLENDER_MAX_THREADS];
 
 void BLI_thread_srandom(int thread, unsigned int seed)
 {
 	extern unsigned char hash[];	// noise.c
 	
+	if(thread >= BLENDER_MAX_THREADS)
+		thread= 0;
+	
 	rng_seed(&rng_tab[thread], seed + hash[seed & 255]);
 	seed= rng_getInt(&rng_tab[thread]);
 	rng_seed(&rng_tab[thread], seed + hash[seed & 255]);
