@@ -3,7 +3,7 @@
  * various string, file, list operations.
  *
  *
- * $Id: util.c,v 1.42 2006/12/20 17:57:42 ton Exp $
+ * $Id: util.c,v 1.50 2007/04/29 10:49:01 ton Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -40,6 +40,7 @@
 #include <ctype.h>
 #include <string.h>
 #include <stdlib.h>
+#include <stdarg.h>
 
 #include "MEM_guardedalloc.h"
 
@@ -50,6 +51,8 @@
 #include "BLI_dynamiclist.h"
 
 #include "BLI_util.h"
+#include "BKE_utildefines.h"
+
 
 #ifdef HAVE_CONFIG_H
 #include <config.h>
@@ -63,6 +66,12 @@
 
 #ifdef WIN32
 #include "BLI_winstuff.h"
+
+/* for duplicate_defgroup */
+#if !(defined vsnprintf)
+#define vsnprintf _vsnprintf
+#endif
+
 #endif
 
 
@@ -75,16 +84,6 @@
 #include <CoreFoundation/CoreFoundation.h>
 #endif
 
-
-// copies from BKE_utildefines
-#ifndef FILE_MAXDIR
-#define FILE_MAXDIR  160
-#endif
-
-#ifndef FILE_MAXFILE
-#define FILE_MAXFILE 80
-#endif
-
 /* local */
 
 static int add_win32_extension(char *name);
@@ -648,6 +647,24 @@ char *BLI_strncpy(char *dst, const char *src, int maxncpy) {
 	return dst;
 }
 
+int BLI_snprintf(char *buffer, size_t count, const char *format, ...)
+{
+	int n;
+	va_list arg;
+
+	va_start(arg, format);
+	n = vsnprintf(buffer, count, format, arg);
+	
+	if (n != -1 && n < count) {
+		buffer[n] = '\0';
+	} else {
+		buffer[count-1] = '\0';
+	}
+	
+	va_end(arg);
+	return n;
+}
+
 int BLI_streq(char *a, char *b) {
 	return (strcmp(a, b)==0);
 }
@@ -749,23 +766,36 @@ void BLI_makestringcode(const char *relfile, char *file)
 	char * p;
 	char * q;
 	char * lslash;
-	int len=0;
 	char temp[FILE_MAXDIR+FILE_MAXFILE];
 	char res[FILE_MAXDIR+FILE_MAXFILE];
-	
+
 	/* if file is already relative, bail out */
 	if(file[0]=='/' && file[1]=='/') return;
 	
 	/* also bail out if relative path is not set */
 	if (relfile[0] == 0) return;
 
-	strcpy(temp, relfile);
-	
-#ifdef WIN32
+#ifdef WIN32 
+	if (strlen(relfile) > 2 && relfile[1] != ':') {
+		char* ptemp;
+		/* fix missing volume name in relative base,
+		   can happen with old .Blog files */
+		get_default_root(temp);
+		ptemp = &temp[2];
+		if (relfile[0] != '\\' && relfile[0] != '/') {
+			ptemp++;
+		}
+		BLI_strncpy(ptemp, relfile, FILE_MAXDIR + FILE_MAXFILE-3);
+	} else {
+		BLI_strncpy(temp, relfile, FILE_MAXDIR + FILE_MAXFILE);
+	}
+
 	if (strlen(file) > 2) {
 		if ( temp[1] == ':' && file[1] == ':' && temp[0] != file[0] )
 			return;
 	}
+#else
+	BLI_strncpy(temp, relfile, FILE_MAX);
 #endif
 
 	BLI_char_switch(temp, '\\', '/');
@@ -817,7 +847,7 @@ void BLI_makestringcode(const char *relfile, char *file)
 
 int BLI_convertstringcode(char *path, const char *basepath, int framenum)
 {
-	int len, wasrelative, is_filepath;
+	int len, wasrelative;
 	char tmp[FILE_MAXDIR+FILE_MAXFILE];
 	char base[FILE_MAXDIR];
 	char vol[3] = {'\0', '\0', '\0'};
@@ -1008,7 +1038,7 @@ void BLI_make_exist(char *dir) {
 		}
 		if (a >= 0) dir[a+1] = 0;
 		else {
-			/* defaulting to first valid drive hoping it's not empty CD and DVD drives */
+			/* defaulting to drive (usually 'C:') of Windows installation */
 			get_default_root(dir);
 			break;
 		}
@@ -1141,8 +1171,10 @@ int BLI_testextensie(const char *str, const char *ext)
 void BLI_split_dirfile(const char *string, char *dir, char *file)
 {
 	int a;
+#ifdef WIN32
 	int sl;
 	short is_relative = 0;
+#endif
 
 	dir[0]= 0;
 	file[0]= 0;
@@ -1221,7 +1253,7 @@ void BLI_split_dirfile(const char *string, char *dir, char *file)
 			BLI_getwdN(dir);
 			strcat(dir,"/");
 			strcat(dir,string);
-			strcpy(string,dir);
+			strcpy((char *)string,dir);
 		}
 
 		BLI_make_exist(dir);
@@ -1249,7 +1281,24 @@ void BLI_split_dirfile(const char *string, char *dir, char *file)
 	}
 #endif
 }
- 
+
+/* simple appending of filename to dir, does not check for valid path! */
+void BLI_join_dirfile(char *string, const char *dir, const char *file)
+{
+	int sl_dir = strlen(dir);
+	BLI_strncpy(string, dir, FILE_MAX);
+	if (sl_dir > FILE_MAX-1) sl_dir = FILE_MAX-1;
+#ifdef WIN32
+	string[sl_dir] = '\\';
+#else
+	string[sl_dir] = '/';
+#endif
+	sl_dir++;
+	if (sl_dir <FILE_MAX) {
+		BLI_strncpy(string + sl_dir, file, FILE_MAX-sl_dir);
+	}
+}
+
 static int add_win32_extension(char *name)
 {
 	int retval = 0;
@@ -1459,4 +1508,51 @@ void BLI_timestr(double time, char *str)
 	str[11]=0;
 }
 
+/* ************** 64 bits magic, trick to support up to 32 gig of address space *************** */
+/*                only works for malloced pointers (8 aligned)                   */
+
+#ifdef __LP64__ 
+
+#if defined(WIN32) && !defined(FREE_WINDOWS)
+#define PMASK		0x07FFFFFFFFi64
+#else
+#define PMASK		0x07FFFFFFFFll
+#endif
+
+
+int BLI_int_from_pointer(void *poin)
+{
+	long lval= (long)poin;
+	
+	return (int)(lval>>3);
+}
+
+void *BLI_pointer_from_int(int val)
+{
+	static int firsttime= 1;
+	static long basevalue= 0;
+	
+	if(firsttime) {
+		void *poin= malloc(10000);
+		basevalue= (long)poin;
+		basevalue &= ~PMASK;
+		printf("base: %d pointer %p\n", basevalue, poin); /* debug */
+		firsttime= 0;
+		free(poin);
+	}
+	return (void *)(basevalue | (((long)val)<<3));
+}
+
+#else
+
+int BLI_int_from_pointer(void *poin)
+{
+	return (int)poin;
+}
+void *BLI_pointer_from_int(int val)
+{
+	return (void *)val;
+}
+
+#endif
 
