@@ -2,7 +2,7 @@
  *
  * Functions for struct-dna, the genetic file dot c!
  *
- * $Id: genfile.c,v 1.9 2006/03/30 22:44:33 campbellbarton Exp $
+ * $Id: genfile.c,v 1.10 2007/04/28 16:14:59 ton Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -1089,3 +1089,14 @@ void *dna_reconstruct(struct SDNA *newsdna, struct SDNA *oldsdna, char *compflag
 	return cur;
 }
 
+int dna_elem_offset(struct SDNA *sdna, char *stype, char *vartype, char *name)
+{
+	
+	int SDNAnr= dna_findstruct_nr(sdna, stype);
+	short *spo= sdna->structs[SDNAnr];
+	char *cp= find_elem(sdna, vartype, name, spo, NULL, NULL);
+	return (int)((long)cp);
+}
+
+
+
