@@ -1,5 +1,5 @@
 /**
- * $Id: readblenentry.c,v 1.15 2006/11/25 13:07:28 ton Exp $
+ * $Id: readblenentry.c,v 1.17 2007/04/28 16:14:59 ton Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -194,7 +194,7 @@ void BLO_blendhandle_print_sizes(BlendHandle *bh, void *fp)
 			buf[2]= buf[2]?buf[2]:' ';
 			buf[3]= buf[3]?buf[3]:' ';
 			
-			fprintf(fp, "['%.4s', '%s', %d, %ld ], \n", buf, name, bhead->nr, bhead->len+sizeof(BHead));
+			fprintf(fp, "['%.4s', '%s', %d, %ld ], \n", buf, name, bhead->nr, (long)bhead->len+sizeof(BHead));
 		}
 	}
 	fprintf(fp, "]\n");
@@ -208,9 +208,9 @@ LinkNode *BLO_blendhandle_get_datablock_names(BlendHandle *bh, int ofblocktype)
 
 	for (bhead= blo_firstbhead(fd); bhead; bhead= blo_nextbhead(fd, bhead)) {
 		if (bhead->code==ofblocktype) {
-			ID *id= (ID*) (bhead+1);
+			char *idname= bhead_id_name(fd, bhead);
 			
-			BLI_linklist_prepend(&names, strdup(id->name+2));
+			BLI_linklist_prepend(&names, strdup(idname+2));
 		} else if (bhead->code==ENDB)
 			break;
 	}
