@@ -2,7 +2,7 @@
  *
  * .blend file writing
  *
- * $Id: writefile.c,v 1.100 2007/02/14 11:00:05 ton Exp $
+ * $Id: writefile.c,v 1.101 2007/04/07 03:32:56 aligorith Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -744,6 +744,9 @@ static void write_constraints(WriteData *wd, ListBase *conlist)
 		case CONSTRAINT_TYPE_RIGIDBODYJOINT:
 			writestruct(wd, DATA, "bRigidBodyJointConstraint", 1, con->data);
 			break;
+		case CONSTRAINT_TYPE_CLAMPTO:
+			writestruct(wd, DATA, "bClampToConstraint", 1, con->data);
+			break;
 		default:
 			break;
 		}
