@@ -27,7 +27,7 @@
  * Contributor(s): none yet.
  *
  * ***** END GPL/BL DUAL LICENSE BLOCK *****
- * $Id: targa.c,v 1.12 2005/03/19 21:08:10 zuster Exp $
+ * $Id: targa.c,v 1.16 2006/11/07 21:24:47 sirdude Exp $
  */
 
 #ifdef WIN32
@@ -408,7 +408,8 @@ static void decodetarga(struct ImBuf *ibuf, unsigned char *mem, int psize)
 				}
 			} else{
 				if (psize & 1){
-					col = (mem[0] << 8) + mem[1];
+					cp[0] = mem[0];
+					cp[1] = mem[1];
 					mem += 2;
 				} else{
 					col = *mem++;
@@ -445,7 +446,8 @@ static void decodetarga(struct ImBuf *ibuf, unsigned char *mem, int psize)
 						}
 					} else{
 						if (psize & 1){
-							col = (mem[0] << 8) + mem[1];
+							cp[0] = mem[0];
+							cp[1] = mem[1];
 							mem += 2;
 						} else{
 							col = *mem++;
@@ -496,7 +498,8 @@ static void ldtarga(struct ImBuf * ibuf,unsigned char * mem, int psize)
 			}
 		} else{
 			if (psize & 1){
-				col = (mem[1] << 8) + mem[0];
+				cp[0] = mem[0];
+				cp[1] = mem[1];
 				mem += 2;
 			} else{
 				col = *mem++;
@@ -518,8 +521,8 @@ struct ImBuf *imb_loadtarga(unsigned char *mem, int flags)
 	
 	if (checktarga(&tga,mem) == 0) return(0);
 
-	if (flags & IB_test) ibuf = IMB_allocImBuf(tga.xsize,tga.ysize,tga.pixsize,0,0);
-	else ibuf = IMB_allocImBuf(tga.xsize,tga.ysize,(tga.pixsize + 0x7) & ~0x7,1,0);
+	if (flags & IB_test) ibuf = IMB_allocImBuf(tga.xsize,tga.ysize,tga.pixsize, 0, 0);
+	else ibuf = IMB_allocImBuf(tga.xsize,tga.ysize,(tga.pixsize + 0x7) & ~0x7, IB_rect, 0);
 
 	if (ibuf == 0) return(0);
 	ibuf->ftype = TGA;
@@ -551,7 +554,8 @@ struct ImBuf *imb_loadtarga(unsigned char *mem, int flags)
 					mem += 3;
 					break;
 				case 2:
-					col = (mem[1] << 8) + mem[0];
+					cp[1] = mem[1];
+					cp[0] = mem[0];
 					mem += 2;
 					break;
 				case 1:
@@ -599,11 +603,18 @@ struct ImBuf *imb_loadtarga(unsigned char *mem, int flags)
 	
 	if (tga.pixsize == 16 && ibuf->cmap == 0){
 		rect = ibuf->rect;
-		for (size = ibuf->x * ibuf->y; size > 0; size --){
+		for (size = ibuf->x * ibuf->y; size > 0; --size, ++rect){
 			col = *rect;
-			col = ((col & 0x1f) << 19) +  ((col & 0x3e0) << 6) + ((col & 0x7c00) >> 7) ;
-			col += (col & 0xe0e0e0) >> 5;
-			*rect++ = col + 0xff000000;
+			cp = (uchar*)rect; 
+			mem = (uchar*)&col;
+
+			cp[3] = ((mem[1] << 1) & 0xf8);
+			cp[2] = ((mem[0] & 0xe0) >> 2) + ((mem[1] & 0x03) << 6);
+			cp[1] = ((mem[0] << 3) & 0xf8);
+			cp[1] += cp[1] >> 5;
+			cp[2] += cp[2] >> 5;
+			cp[3] += cp[3] >> 5;
+			cp[0] = 0xff;
 		}
 		ibuf->depth = 24;
 	}
@@ -629,7 +640,7 @@ struct ImBuf *imb_loadtarga(unsigned char *mem, int flags)
 
 	if (ibuf) {
 		if (ibuf->rect && (flags & IB_cmap)==0) 
-			IMB_convert_rgba_to_abgr((ibuf->x+ibuf->skipx)*ibuf->y, ibuf->rect);
+			IMB_convert_rgba_to_abgr(ibuf);
 	}
 	
 	return(ibuf);
