@@ -1,5 +1,5 @@
 /**
- * $Id: BDR_drawmesh.h,v 1.8 2006/02/08 21:00:58 blendix Exp $
+ * $Id: BDR_drawmesh.h,v 1.11 2006/11/26 23:14:52 blendix Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -34,7 +34,7 @@
 #define BDR_DRAWMESH_H
 
 struct Image;
-struct TFace;
+struct MTFace;
 struct Object;
 struct Mesh;
 struct EdgeHash;
@@ -47,11 +47,6 @@ struct EdgeHash;
  */
 void set_mipmap(int mipmap);
 
-/**
- * Returns the current setting for mipmapping.
- */
-int get_mipmap(void);
-
 /**
  * Enables or disable linear mipmap setting for realtime images (textures).
  * Note that this will will destroy all texture bindings in OpenGL.
@@ -70,10 +65,16 @@ int get_linear_mipmap(void);
  */
 void clear_realtime_image_cache(void);
 
+
+void update_realtime_image(struct Image *ima, int x, int y, int w, int h);
 void free_realtime_image(struct Image *ima);
 void free_all_realtime_images(void);
 void make_repbind(struct Image *ima);
-int set_tpage(struct TFace *tface);
+int set_tpage(struct MTFace *tface);
+
+void texpaint_enable_mipmap(void);
+void texpaint_disable_mipmap(void);
+
 void draw_tface_mesh(struct Object *ob, struct Mesh *me, int dt);
 struct EdgeHash *get_tface_mesh_marked_edge_info(struct Mesh *me);
 void init_realtime_GL(void); 
