@@ -1,5 +1,5 @@
 /**
- * $Id: BIF_editarmature.h,v 1.33 2007/02/10 19:07:47 joeedh Exp $
+ * $Id: BIF_editarmature.h,v 1.34 2007/04/04 13:18:29 campbellbarton Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -76,7 +76,7 @@ void	adduplicate_armature(void);
 void	addvert_armature(void);
 void	add_primitiveArmature(int type);
 void	apply_rot_armature (struct Object *ob, float mat[3][3]);
-void 	docentre_armature (struct Object *ob, int centremode);
+void 	docenter_armature (struct Object *ob, int centermode);
 
 void	clear_armature(struct Object *ob, char mode);
 
