@@ -1,5 +1,5 @@
 /**
- * $Id: BIF_editdeform.h,v 1.11 2006/09/03 12:16:13 ton Exp $
+ * $Id: BIF_editdeform.h,v 1.13 2007/04/27 11:16:35 aligorith Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -43,10 +43,11 @@ struct MDeformVert;
 struct MDeformWeight;
 struct bDeformGroup;
 
-void unique_vertexgroup_name (struct bDeformGroup *dg, struct Object *ob);
 struct bDeformGroup *add_defgroup_name (struct Object *ob, char *name);
 void add_defgroup (struct Object *ob);
+void del_defgroup_in_object_mode ( Object *ob );
 void del_defgroup (struct Object *ob);
+void duplicate_defgroup ( struct Object *ob );
 void assign_verts_defgroup (void);
 void remove_verts_defgroup (int allverts);
 void sel_verts_defgroup (int select);
