@@ -1,5 +1,5 @@
 /**
- * $Id: BIF_editmesh.h,v 1.74 2006/11/20 04:27:49 blendix Exp $
+ * $Id: BIF_editmesh.h,v 1.76 2007/03/28 15:25:07 campbellbarton Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -67,9 +67,9 @@ extern void separate_mesh_loose(void);
 extern void separate_material(void);
 
 /* ******************* editmesh_add.c */
-extern void make_prim(int type, float imat[3][3], short tot, short seg,
-		short subdiv, float dia, float d, short ext, short fill,
-        float cent[3]);
+extern void make_prim(int type, float imat[3][3], int tot, int seg,
+		int subdiv, float dia, float d, int ext, int fill,
+        float cent[3] );
 extern void add_primitiveMesh(int type);
 extern void adduplicate_mesh(void);
 extern void add_click_mesh(void);
