@@ -1,5 +1,5 @@
 /**
- * $Id: BIF_space.h,v 1.39 2006/06/27 12:54:16 ton Exp $
+ * $Id: BIF_space.h,v 1.42 2006/12/29 04:46:46 broken Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -54,6 +54,7 @@ struct SpaceOops;
 #define VIEW3D_HANDLER_PROPERTIES	2
 #define VIEW3D_HANDLER_OBJECT		3
 #define VIEW3D_HANDLER_PREVIEW		4
+#define VIEW3D_HANDLER_MULTIRES         5
 
 /* ipo handler codes */
 #define IPO_HANDLER_PROPERTIES	20
@@ -63,6 +64,7 @@ struct SpaceOops;
 #define IMAGE_HANDLER_PAINT			31
 #define IMAGE_HANDLER_CURVES		32
 #define IMAGE_HANDLER_PREVIEW		33
+#define IMAGE_HANDLER_GAME_PROPERTIES	34
 
 /* action handler codes */
 #define ACTION_HANDLER_PROPERTIES	40
@@ -82,6 +84,7 @@ struct SpaceOops;
 #define B_CHANGE_THEME 	3306
 #define B_THEME_COPY 	3307
 #define B_THEME_PASTE 	3308
+#define B_UPDATE_THEME_ICONS 	3309
 
 #define B_RECALCLIGHT 	3310
 
