@@ -1,5 +1,5 @@
 /**
- * $Id: BSE_edit.h,v 1.9 2005/09/24 09:05:48 ton Exp $
+ * $Id: BSE_edit.h,v 1.10 2007/04/14 13:18:24 campbellbarton Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -43,7 +43,7 @@ void mergemenu(void);
 void delete_context_selected(void);
 void duplicate_context_selected(void);
 void toggle_shading(void);
-void minmax_verts(float *min, float *max);
+int minmax_verts(float *min, float *max);
 
 void snap_sel_to_grid(void);
 void snap_sel_to_curs(void);
