@@ -1,5 +1,5 @@
 /**
- * $Id: BSE_editipo.h,v 1.32 2007/01/17 05:53:44 aligorith Exp $
+ * $Id: BSE_editipo.h,v 1.34 2007/04/28 09:35:16 aligorith Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -60,6 +60,9 @@ char *getname_cam_ei(int nr);
 char *getname_snd_ei(int nr);
 char *getname_fluidsim_ei(int nr);
 
+char *getname_ipocurve(struct IpoCurve *icu);
+int geticon_ipo_blocktype(short blocktype);
+
 struct EditIpo *get_active_editipo(void);
 
 void boundbox_ipocurve(struct IpoCurve *icu);
@@ -138,7 +141,7 @@ void filter_sampledata(float *data, int sfra, int efra);
 void sampledata_to_ipocurve(float *data, int sfra, int efra, struct IpoCurve *icu);
 void ipo_record(void);    
 
-void actstrip_map_ipo_keys(struct Object *ob, struct Ipo *ipo, short restore);
+void actstrip_map_ipo_keys(struct Object *ob, struct Ipo *ipo, short restore, short only_keys);
 
 void sethandles_ipo_keys(struct Ipo *ipo, int code);
 void snap_ipo_keys(struct Ipo *ipo, short snaptype);
