@@ -1,5 +1,5 @@
 /**
- * $Id: BSE_view.h,v 1.15 2006/11/06 01:08:22 nicholasbishop Exp $
+ * $Id: BSE_view.h,v 1.18 2007/05/08 10:15:51 blendix Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -77,18 +77,18 @@ void calctrackballvecfirst(struct rcti *area, short *mval, float *vec);
 void calctrackballvec(struct rcti *area, short *mval, float *vec);
 void viewmove(int mode);
 
-int get_view3d_viewplane(int winxi, int winyi, rctf *viewplane, float *clipsta, float *clipend);
+int get_view3d_viewplane(int winxi, int winyi, rctf *viewplane, float *clipsta, float *clipend, float *pixsize);
 void setwinmatrixview3d(int winx, int winy, struct rctf *rect);
 
-void obmat_to_viewmat(struct Object *ob);
+void obmat_to_viewmat(struct Object *ob, short smooth);
 void setviewmatrixview3d(void);
 float *give_cursor(void);
 unsigned int free_localbit(void);
 void initlocalview(void);
-void centreview(void);
+void centerview(void);
 void restore_localviewdata(struct View3D *vd);
 void endlocalview(struct ScrArea *sa);
-void view3d_home(int centre);
+void view3d_home(int center);
 short view3d_opengl_select(unsigned int *buffer, unsigned int buffsize, short x1, short y1, short x2, short y2);
 void view3d_align_axis_to_vector(struct View3D *v3d, int axisidx, float vec[3]);
 
