@@ -1,5 +1,5 @@
 /**
- * $Id: butspace.h,v 1.101 2007/01/01 08:32:11 aligorith Exp $
+ * $Id: butspace.h,v 1.107 2007/05/02 00:01:23 campbellbarton Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -137,6 +137,7 @@ extern void do_ipobuts(unsigned short event);	// drawipo.c (bad! ton)
 /* butspace.c */
 void test_meshpoin_but(char *name, struct ID **idpp);
 void test_obpoin_but(char *name, struct ID **idpp);
+void test_meshobpoin_but(char *name, struct ID **idpp);
 void test_scenepoin_but(char *name, struct ID **idpp);
 void test_matpoin_but(char *name, struct ID **idpp);
 void test_scriptpoin_but(char *name, struct ID **idpp);
@@ -357,9 +358,9 @@ void curvemap_buttons(struct uiBlock *block, struct CurveMapping *cumap, char la
 #define B_SETSMOOTH		2012
 #define B_SETSOLID		2013
 #define B_AUTOTEX		2014
-#define B_DOCENTRE		2015
-#define B_DOCENTRENEW		2016
-#define B_DOCENTRECURSOR	2017
+#define B_DOCENTER		2015
+#define B_DOCENTERNEW		2016
+#define B_DOCENTERCURSOR	2017
 
 	/* 20 values! */
 #define B_OBLAY			2019
@@ -405,6 +406,8 @@ void curvemap_buttons(struct uiBlock *block, struct CurveMapping *cumap, char la
 #define B_SETTFACE			2079
 #define B_SETMCOL			2080
 #define B_JOINTRIA			2081
+#define B_SETTFACE_RND		2082
+#define B_SETMCOL_RND		2083
 
 /* *********************** */
 #define B_VGROUPBUTS		2100
@@ -417,6 +420,7 @@ void curvemap_buttons(struct uiBlock *block, struct CurveMapping *cumap, char la
 #define B_DESELVGROUP		2096
 #define B_AUTOVGROUP		2097
 #define B_LINKEDVGROUP		2098
+#define B_COPYVGROUP 2099
 
 
 
@@ -642,6 +646,9 @@ enum {
 	B_CONSTRAINT_ADD_ROTLIMIT,
 	B_CONSTRAINT_ADD_SIZELIMIT,
 	B_CONSTRAINT_ADD_RIGIDBODYJOINT,
+	B_CONSTRAINT_ADD_CHILDOF,
+	B_CONSTRAINT_ADD_PYTHON,
+	B_CONSTRAINT_ADD_CLAMPTO,
 	B_CONSTRAINT_INF
 };
 
