@@ -1,5 +1,5 @@
 /**
- * $Id: transform.h,v 1.36 2006/12/28 22:42:58 theeth Exp $
+ * $Id: transform.h,v 1.37 2007/04/04 13:18:29 campbellbarton Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -82,7 +82,7 @@ typedef struct TransCon {
     float mtx[3][3];     /* Matrix of the Constraint space                                            */
     float imtx[3][3];    /* Inverse Matrix of the Constraint space                                    */
     float pmtx[3][3];    /* Projection Constraint Matrix (same as imtx with some axis == 0)           */
-    float center[3];     /* transformation centre to define where to draw the view widget             
+    float center[3];     /* transformation center to define where to draw the view widget             
                             ALWAYS in global space. Unlike the transformation center                  */
 	short imval[2];	     /* initial mouse value for visual calculation                                */
 	                     /* the one in TransInfo is not garanty to stay the same (Rotates change it)  */
