@@ -1,5 +1,5 @@
 /*  
- * $Id: DNA_action_types.h,v 1.38 2007/01/09 22:27:26 aligorith Exp $
+ * $Id: DNA_action_types.h,v 1.40 2007/04/13 11:15:08 aligorith Exp $
  *
  * ***** BEGIN GPL LICENSE BLOCK *****
  *
@@ -95,10 +95,10 @@ typedef struct bActionChannel {
 	struct bActionChannel	*next, *prev;
 	struct Ipo				*ipo;
 	ListBase				constraintChannels;
+	
 	int		flag;
 	char	name[32];		/* Channel name */
 	int		reserved1;
-
 } bActionChannel;
 
 typedef struct bAction {
@@ -126,6 +126,9 @@ typedef struct SpaceAction {
 #define ACHAN_HILIGHTED	0x00000002
 #define ACHAN_HIDDEN	0x00000004
 #define ACHAN_PROTECTED 0x00000008
+#define ACHAN_EXPANDED 	0x00000010
+#define ACHAN_SHOWIPO	0x00000020
+#define ACHAN_SHOWCONS 	0x00000040
 #define ACHAN_MOVED     0x80000000
 
 /* SpaceAction flag */
