@@ -1,5 +1,5 @@
 /**
- * $Id: DNA_armature_types.h,v 1.28 2006/02/23 14:45:59 campbellbarton Exp $
+ * $Id: DNA_armature_types.h,v 1.32 2007/01/01 22:48:08 aligorith Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -76,8 +76,13 @@ typedef struct bArmature {
 	ListBase	chainbase;
 	int			flag;
 	int			drawtype;			
-	short		deformflag, layer;
-	short		ghostep, ghostsize;
+	short		deformflag; 
+	short		pathflag;
+	short		layer, layer_protected;		/* for buttons to work, both variables in this order together */
+	short		ghostep, ghostsize;		/*number of frames to ghosts to show, and step between them  */
+	short		ghosttype, pathsize;		/* ghost drawing options and number of frames between points of path */
+	int			ghostsf, ghostef;		/* start and end frames of ghost-drawing range */
+	int 		pathsf, pathef;			/* start and end frames of path-calculation range for all bones */	
 }bArmature;
 
 /* armature->flag */
@@ -95,6 +100,9 @@ typedef struct bArmature {
 #define		ARM_DONT_USE    0x080
 #define		ARM_MIRROR_EDIT	0x100
 #define		ARM_AUTO_IK		0x200
+			/* made option negative, for backwards compat */
+#define		ARM_NO_CUSTOM	0x400
+#define		ARM_COL_CUSTOM	0x800
 
 /* armature->drawtype */
 #define		ARM_OCTA		0
@@ -106,6 +114,14 @@ typedef struct bArmature {
 #define		ARM_DEF_VGROUP		1
 #define		ARM_DEF_ENVELOPE	2
 
+/* armature->pathflag */
+#define		ARM_PATH_FNUMS	0x001
+#define		ARM_PATH_KFRAS	0x002
+#define		ARM_PATH_HEADS	0x004
+
+/* armature->ghosttype */
+#define 	ARM_GHOST_CUR	0
+#define		ARM_GHOST_RANGE	1
 
 /* bone->flag */
 #define		BONE_SELECTED	1
