@@ -1,5 +1,5 @@
 /**
- * $Id: DNA_constraint_types.h,v 1.34 2006/12/16 05:49:05 erwin Exp $
+ * $Id: DNA_constraint_types.h,v 1.39 2007/04/07 04:21:12 aligorith Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -216,6 +216,13 @@ typedef struct bRigidBodyJointConstraint{
 	short		pad2;
 } bRigidBodyJointConstraint;
 
+/* ClampTo Constraint */
+typedef struct bClampToConstraint {
+	Object 		*tar;			/* 'target' must be a curve */
+	int			flag;			/* which axis/plane to compare owner's location on  */
+	int			pad;
+} bClampToConstraint;
+
 /* bConstraint.type */
 #define CONSTRAINT_TYPE_NULL		0
 #define CONSTRAINT_TYPE_CHILDOF		1	/* Unimplemented */
@@ -235,6 +242,7 @@ typedef struct bRigidBodyJointConstraint{
 #define CONSTRAINT_TYPE_STRETCHTO	15  /* claiming this to be mine :) is in tuhopuu bjornmose */ 
 #define CONSTRAINT_TYPE_MINMAX      16  /* floor constraint */
 #define CONSTRAINT_TYPE_RIGIDBODYJOINT 17 /* rigidbody constraint */
+#define CONSTRAINT_TYPE_CLAMPTO		18  /* clampto constraint */		
 
 /* bConstraint.flag */
 		/* expand for UI */
@@ -252,22 +260,35 @@ typedef struct bRigidBodyJointConstraint{
 #define CONSTRAINT_CHANNEL_SELECT		0x01
 #define CONSTRAINT_CHANNEL_PROTECTED 	0x02
 
+/**
+ * The flags for ROTLIKE, LOCLIKE and SIZELIKE should be kept identical
+ * (that is, same effect, different name). It simplifies the Python API access a lot.
+ */
+
 /* bRotateLikeConstraint.flag */
 #define ROTLIKE_X		0x01
 #define ROTLIKE_Y		0x02
 #define ROTLIKE_Z		0x04
+#define ROTLIKE_X_INVERT	0x10
+#define ROTLIKE_Y_INVERT	0x20
+#define ROTLIKE_Z_INVERT	0x40
+#define ROTLIKE_OFFSET	0x80
 
 /* bLocateLikeConstraint.flag */
-#define LOCLIKE_X		0x01
-#define LOCLIKE_Y		0x02
-#define LOCLIKE_Z		0x04
-#define LOCSPACE		0x08
+#define LOCLIKE_X			0x01
+#define LOCLIKE_Y			0x02
+#define LOCLIKE_Z			0x04
+#define LOCLIKE_TIP			0x08
+#define LOCLIKE_X_INVERT	0x10
+#define LOCLIKE_Y_INVERT	0x20
+#define LOCLIKE_Z_INVERT	0x40
+#define LOCLIKE_OFFSET		0x80
  
 /* bSizeLikeConstraint.flag */
 #define SIZELIKE_X		0x01
 #define SIZELIKE_Y		0x02
 #define SIZELIKE_Z		0x04
-#define SIZESPACE		0x08
+#define SIZELIKE_OFFSET 0x80
 
 /* Axis flags */
 #define LOCK_X		0x00
@@ -297,6 +318,11 @@ typedef struct bRigidBodyJointConstraint{
 #define PLANE_Y		0x01
 #define PLANE_Z		0x02
 
+#define CLAMPTO_AUTO	0
+#define CLAMPTO_X		1
+#define	CLAMPTO_Y		2
+#define CLAMPTO_Z		3
+
 /* bKinematicConstraint->flag */
 #define CONSTRAINT_IK_TIP		1
 #define CONSTRAINT_IK_ROT		2
@@ -333,4 +359,3 @@ typedef struct bRigidBodyJointConstraint{
 #define CONSTRAINT_RB_GENERIC6DOF 12
 
 #endif
-
