@@ -1,7 +1,7 @@
 /**
  * blenlib/DNA_image_types.h (mar-2001 nzc)
  *
- * $Id: DNA_image_types.h,v 1.14 2006/12/20 17:57:44 ton Exp $ 
+ * $Id: DNA_image_types.h,v 1.16 2007/03/13 12:50:35 campbellbarton Exp $ 
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -117,6 +117,8 @@ typedef struct Image {
 #define IMA_TWINANIM		2
 #define IMA_COLCYCLE		4	/* Depreciated */
 #define IMA_MIPMAP_COMPLETE 8   /* all mipmap levels in OpenGL texture set? */
+#define IMA_CLAMP_U		16 
+#define IMA_CLAMP_V		32
 
 /* ima->type and ima->source moved to BKE_image.h, for API */
 
