@@ -1,7 +1,7 @@
 /**
  * blenlib/DNA_lamp_types.h (mar-2001 nzc)
  *
- * $Id: DNA_lamp_types.h,v 1.15 2006/02/12 20:21:07 ton Exp $ 
+ * $Id: DNA_lamp_types.h,v 1.18 2006/10/15 11:50:45 ton Exp $ 
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -58,7 +58,8 @@ typedef struct Lamp {
 	
 	float clipsta, clipend, shadspotsize;
 	float bias, soft;
-	short bufsize, samp, buffers, filtertype, pad;
+	short bufsize, samp, buffers, filtertype;
+	char bufflag, buftype;
 	
 	short ray_samp, ray_sampy, ray_sampz, ray_samp_type;
 	short area_shape;
@@ -93,7 +94,7 @@ typedef struct Lamp {
 #define LA_YF_PHOTON	5
 
 /* mode */
-#define LA_SHAD			1
+#define LA_SHAD_BUF		1
 #define LA_HALO			2
 #define LA_LAYER		4
 #define LA_QUAD			8
@@ -111,6 +112,15 @@ typedef struct Lamp {
 /* Since it is used with LOCAL lamp, can't use LA_SHAD */
 #define LA_YF_SOFT		16384
 
+/* buftype, no flag */
+#define LA_SHADBUF_REGULAR		0
+#define LA_SHADBUF_IRREGULAR	1
+#define LA_SHADBUF_HALFWAY		2
+
+/* bufflag, auto clipping */
+#define LA_SHADBUF_AUTO_START	1
+#define LA_SHADBUF_AUTO_END		2
+
 /* filtertype */
 #define LA_SHADBUF_BOX		0
 #define LA_SHADBUF_TENT		1
