@@ -1,7 +1,7 @@
 /**
  * blenlib/DNA_material_types.h (mar-2001 nzc)
  *
- * $Id: DNA_material_types.h,v 1.45 2006/12/08 09:40:43 ton Exp $ 
+ * $Id: DNA_material_types.h,v 1.46 2007/05/03 21:37:52 blendix Exp $ 
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -111,6 +111,12 @@ typedef struct Material {
 	float fhdist, xyfrict;
 	short dynamode, pad2;
 
+	float sss_radius[3], sss_col[3];
+	float sss_error, sss_scale, sss_ior;
+	float sss_colfac, sss_texfac;
+	float sss_front, sss_back;
+	short sss_flag, sss_preset;
+
 	/* yafray: absorption color, dispersion parameters and material preset menu */
 	float YF_ar, YF_ag, YF_ab, YF_dscale, YF_dpwr;
 	int YF_dsmp, YF_preset, YF_djit;
@@ -268,5 +274,8 @@ typedef struct Material {
 /* pr_back */
 #define MA_DARK			1
 
+/* sss_flag */
+#define MA_DIFF_SSS		1
+
 #endif
 
