@@ -1,5 +1,5 @@
 /**
- * $Id: DNA_mesh_types.h,v 1.31 2006/04/03 19:02:21 briggs Exp $ 
+ * $Id: DNA_mesh_types.h,v 1.35 2007/02/14 13:52:29 ton Exp $ 
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -34,9 +34,9 @@
 
 #include "DNA_listBase.h"
 #include "DNA_ID.h"
+#include "DNA_customdata_types.h"
 
 struct DerivedMesh;
-struct DispListMesh;
 struct Ipo;
 struct Key;
 struct Material;
@@ -47,17 +47,8 @@ struct MCol;
 struct MSticky;
 struct Mesh;
 struct OcInfo;
-
-typedef struct TFace {
-
-	/* this one gets interpreted as a image in texture.c  */
-	void *tpage;
-
-	float uv[4][2];		/* when you change this: also do function set_correct_uv in editmesh.c, and there are more locations that use the size of this part */
-	unsigned int col[4];
-	char flag, transp;
-	short mode, tile, unwrap;
-} TFace;
+struct Multires;
+struct PartialVisibility;
 
 typedef struct Mesh {
 	ID id;
@@ -71,8 +62,8 @@ typedef struct Mesh {
 	struct Material **mat;
 
 	struct MFace *mface;
-	struct TFace *tface;
-	void *dface;
+	struct MTFace *mtface;
+	struct TFace *tface;	/* depecrated, use mtface */
 	struct MVert *mvert;
 	struct MEdge *medge;
 	struct MDeformVert *dvert;	/* __NLA */
@@ -84,9 +75,12 @@ typedef struct Mesh {
 	struct OcInfo *oc;		/* not written in file */
 	void *sumohandle;
 
+	struct CustomData vdata, edata, fdata;
+
 	int totvert, totedge, totface, totselect, pad2;
 	int texflag;
 	
+	/* texture space, copied as one block in editobject.c */
 	float loc[3];
 	float size[3];
 	float rot[3];
@@ -99,9 +93,22 @@ typedef struct Mesh {
 	short totcol;
 	short subsurftype; 
 
+	struct Multires *mr;		/* Multiresolution modeling data */
+	struct PartialVisibility *pv;
+/*ifdef WITH_VERSE*/
+	/* not written in file, pointer at geometry VerseNode */
+	void *vnode;
+/*#endif*/
 } Mesh;
 
-
+/* deprecated by MTFace, only here for file reading */
+typedef struct TFace {
+	void *tpage;
+	float uv[4][2];
+	unsigned int col[4];
+	char flag, transp;
+	short mode, tile, unwrap;
+} TFace;
 
 /* **************** MESH ********************* */
 
@@ -123,48 +130,6 @@ typedef struct Mesh {
 #define ME_CC_SUBSURF 		0
 #define ME_SIMPLE_SUBSURF 	1
 
-#define TF_DYNAMIC		1
-/* #define TF_INVISIBLE	2 */
-#define TF_TEX			4
-#define TF_SHAREDVERT	8
-#define TF_LIGHT		16
-
-#define TF_SHAREDCOL	64
-#define TF_TILES		128
-#define TF_BILLBOARD	256
-#define TF_TWOSIDE		512
-#define TF_INVISIBLE	1024
-
-#define TF_OBCOL		2048
-#define TF_BILLBOARD2		4096	/* with Z axis constraint */
-#define TF_SHADOW		8192
-#define TF_BMFONT		16384
-
-/* tface->flag: 1=select 2=active*/
-#define TF_SELECT	1
-#define TF_ACTIVE	2
-#define TF_SEL1		4
-#define TF_SEL2		8
-#define TF_SEL3		16
-#define TF_SEL4		32
-#define TF_HIDE		64
-
-/* tface->transp */
-#define TF_SOLID	0
-#define TF_ADD		1
-#define TF_ALPHA	2
-#define TF_SUB		3
-
-/* tface->unwrap */
-#define TF_DEPRECATED1	1
-#define TF_DEPRECATED2	2
-#define TF_DEPRECATED3	4
-#define TF_DEPRECATED4	8
-#define TF_PIN1		    16
-#define TF_PIN2		    32
-#define TF_PIN3	   		64
-#define TF_PIN4	    	128
-
 #define MESH_MAX_VERTS 2000000000L
 
 #endif
