@@ -1,5 +1,5 @@
 /**
- * $Id: DNA_meshdata_types.h,v 1.32 2007/01/22 08:34:53 nicholasbishop Exp $ 
+ * $Id: DNA_meshdata_types.h,v 1.34 2007/04/29 13:39:45 campbellbarton Exp $ 
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -57,7 +57,7 @@ typedef struct MDeformWeight {
 typedef struct MDeformVert {
 	struct MDeformWeight *dw;
 	int totweight;
-	int flag;	// flag only in use for weightpaint now
+	int flag;	/* flag only in use for weightpaint now */
 } MDeformVert;
 
 typedef struct MVert {
@@ -150,7 +150,7 @@ typedef struct PartialVisibility {
 #define ME_EDGEDRAW			(1<<1)
 #define ME_SEAM				(1<<2)
 #define ME_FGON				(1<<3)
-						// reserve 16 for ME_HIDE
+						/* reserve 16 for ME_HIDE */
 #define ME_EDGERENDER		(1<<5)
 #define ME_LOOSEEDGE		(1<<7)
 #define ME_SEAM_LAST		(1<<8)
@@ -183,13 +183,13 @@ typedef struct PartialVisibility {
 #define ME_FSEL 2
 
 /* mtface->flag */
-#define TF_SELECT	1
+#define TF_SELECT	1 /* use MFace hide flag (after 2.43), should be able to reuse after 2.44 */
 #define TF_ACTIVE	2
 #define TF_SEL1		4
 #define TF_SEL2		8
 #define TF_SEL3		16
 #define TF_SEL4		32
-#define TF_HIDE		64
+#define TF_HIDE		64 /* unused, same as TF_SELECT */
 
 /* mtface->mode */
 #define TF_DYNAMIC		1
