@@ -1,5 +1,5 @@
 /**
- * $Id: DNA_modifier_types.h,v 1.30 2007/01/29 16:28:11 artificer Exp $ 
+ * $Id: DNA_modifier_types.h,v 1.39 2007/04/30 19:20:43 ianwill Exp $ 
  */
 
 #ifndef DNA_MODIFIER_TYPES_H
@@ -26,6 +26,8 @@ typedef enum ModifierType {
 	eModifierType_EdgeSplit,
 	eModifierType_Displace,
 	eModifierType_UVProject,
+	eModifierType_Smooth,
+	eModifierType_Cast,
 	NUM_MODIFIER_TYPES
 } ModifierType;
 
@@ -96,6 +98,10 @@ typedef struct BuildModifierData {
 typedef struct ArrayModifierData {
 	ModifierData modifier;
 
+	/* the object with which to cap the start of the array  */
+	struct Object *start_cap;
+	/* the object with which to cap the end of the array  */
+	struct Object *end_cap;
 	/* the curve object to use for MOD_ARR_FITCURVE */
 	struct Object *curve_ob;
 	/* the object to use for object offset */
@@ -155,7 +161,12 @@ typedef struct MirrorModifierData {
 } MirrorModifierData;
 
 /* MirrorModifierData->flag */
-#define MOD_MIR_CLIPPING	1
+#define MOD_MIR_CLIPPING	1<<0
+#define MOD_MIR_MIRROR_U	1<<1
+#define MOD_MIR_MIRROR_V	1<<2
+#define MOD_MIR_AXIS_X		1<<3
+#define MOD_MIR_AXIS_Y		1<<4
+#define MOD_MIR_AXIS_Z		1<<5
 
 typedef struct EdgeSplitModifierData {
 	ModifierData modifier;
@@ -224,15 +235,74 @@ typedef struct DecimateModifierData {
 	int faceCount;
 } DecimateModifierData;
 
+/* Smooth modifier flags */
+#define MOD_SMOOTH_X (1<<1)
+#define MOD_SMOOTH_Y (1<<2)
+#define MOD_SMOOTH_Z (1<<3)
+
+typedef struct SmoothModifierData {
+	ModifierData modifier;
+	float fac;
+	char defgrp_name[32];
+	short flag, repeat;
+
+} SmoothModifierData;
+
+/* Cast modifier flags */
+#define MOD_CAST_X (1<<1)
+#define MOD_CAST_Y (1<<2)
+#define MOD_CAST_Z (1<<3)
+#define MOD_CAST_USE_OB_TRANSFORM (1<<4)
+#define MOD_CAST_SIZE_FROM_RADIUS (1<<5)
+
+/* Cast modifier projection types */
+#define MOD_CAST_TYPE_SPHERE 0
+#define MOD_CAST_TYPE_CYLINDER 1
+#define MOD_CAST_TYPE_CUBOID 2
+
+typedef struct CastModifierData {
+	ModifierData modifier;
+
+	struct Object *object;
+	float fac;
+	float radius;
+	float size;
+	char defgrp_name[32];
+	short flag, type;
+} CastModifierData;
+
+enum {
+	MOD_WAV_MAP_LOCAL,
+	MOD_WAV_MAP_GLOBAL,
+	MOD_WAV_MAP_OBJECT,
+	MOD_WAV_MAP_UV,
+};
+
+/* WaveModifierData.flag */
+#define MOD_WAVE_X      1<<1
+#define MOD_WAVE_Y      1<<2
+#define MOD_WAVE_CYCL   1<<3
+#define MOD_WAVE_NORM   1<<4
+#define MOD_WAVE_NORM_X 1<<5
+#define MOD_WAVE_NORM_Y 1<<6
+#define MOD_WAVE_NORM_Z 1<<7
+
 typedef struct WaveModifierData {
 	ModifierData modifier;
 
 	struct Object *objectcenter;
+	char defgrp_name[32];
+	struct Tex *texture;
+	struct Object *map_object;
 
 	short flag, pad;
 
 	float startx, starty, height, width;
 	float narrow, speed, damp;
+
+	int texmapping, uvlayer_tmp;
+
+	char uvlayer_name[32];
 	
 	float timeoffs, lifetime;
 } WaveModifierData;
