@@ -1,5 +1,5 @@
 /**
- * $Id: DNA_scene_types.h,v 1.106 2007/02/07 09:17:57 ton Exp $ 
+ * $Id: DNA_scene_types.h,v 1.111 2007/04/24 14:52:34 ton Exp $ 
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -162,7 +162,8 @@ typedef struct RenderData {
 	struct QuicktimeCodecData *qtcodecdata;
 	struct FFMpegCodecData ffcodecdata;
 
-	int cfra, sfra, efra;	/* fames as in 'images' */
+	int cfra, sfra, efra;	/* frames as in 'images' */
+	int psfra, pefra;		/* start+end frames of preview range */
 
 	int images, framapto;
 	short flag, threads;
@@ -170,7 +171,7 @@ typedef struct RenderData {
 	float ctime;			/* use for calcutions */
 	float framelen, blurfac;
 
-	/** For UR edge rendering: give the edges this colour */
+	/** For UR edge rendering: give the edges this color */
 	float edgeR, edgeG, edgeB;
 	
 	short fullscreen, xplay, yplay, freqplay;	/* standalone player */
@@ -334,11 +335,11 @@ typedef struct ToolSettings {
 
 	short pad2;
 	
+	/* Image Paint (8 byte aligned please!) */
+	struct ImagePaintSettings imapaint;
+	
 	/* Select Group Threshold */
 	float select_thresh;
-
-	/* Image Paint */
-	struct ImagePaintSettings imapaint;
 	
 	/* IPO-Editor */
 	float clean_thresh;
@@ -347,8 +348,11 @@ typedef struct ToolSettings {
 	char retopo_mode;
 	char retopo_paint_tool;
 	char line_div, ellipse_div, retopo_hotspot;
+
+	/* Multires */
+	char multires_subdiv_type;
 	
-	char  pad4[3];
+	char  pad4[2];
 } ToolSettings;
 
 /* Used by all brushes to store their properties, which can be directly set
@@ -373,7 +377,7 @@ typedef struct SculptData
 	struct MTex *mtex[10];
 
 	/* Settings for each brush */
-	BrushData drawbrush, smoothbrush, pinchbrush, inflatebrush, grabbrush, layerbrush;
+	BrushData drawbrush, smoothbrush, pinchbrush, inflatebrush, grabbrush, layerbrush, flattenbrush;
 	short brush_type;
 
 	/* For the Brush Shape */
@@ -605,6 +609,7 @@ typedef struct Scene {
 #define INFLATE_BRUSH 4
 #define GRAB_BRUSH 5
 #define LAYER_BRUSH 6
+#define FLATTEN_BRUSH 7
 /* SculptData.texrept */
 #define SCULPTREPT_DRAG 1
 #define SCULPTREPT_TILE 2
