@@ -1,7 +1,7 @@
 /**
  * blenlib/DNA_sequence_types.h (mar-2001 nzc)
  *
- * $Id: DNA_sequence_types.h,v 1.22 2006/03/26 21:36:42 schlaile Exp $
+ * $Id: DNA_sequence_types.h,v 1.26 2006/11/11 22:35:39 schlaile Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -49,8 +49,7 @@ typedef struct StripElem {
 	struct ImBuf *ibuf;
 	struct StripElem *se1, *se2, *se3;
 	short ok;
-	unsigned char isneeded;
-	unsigned char pad;
+	short pad;
 	int nr;
 } StripElem;
 
@@ -160,19 +159,50 @@ typedef struct GlowVars {
 	int	bNoComp;	/*	SHOW/HIDE glow buffer */
 } GlowVars;
 
+typedef struct TransformVars {
+	float ScalexIni;
+	float ScaleyIni;
+	float ScalexFin;
+	float ScaleyFin;
+	float xIni;
+	float xFin;
+	float yIni;
+	float yFin;
+	float rotIni;
+	float rotFin;
+} TransformVars;
+
+typedef struct SolidColorVars {
+	float col[3];
+	float pad;
+} SolidColorVars;
+
+typedef struct SpeedControlVars {
+	float * frameMap;
+	float globalSpeed;
+	int flags;
+	int length;
+	int pad;
+} SpeedControlVars;
+
+/* SpeedControlVars->flags */
+#define SEQ_SPEED_INTEGRATE      1
+#define SEQ_SPEED_BLEND          2
+#define SEQ_SPEED_COMPRESS_IPO_Y 4
 
 /* ***************** SEQUENCE ****************** */
 
 /* seq->flag */
-#define SEQ_LEFTSEL		2
-#define SEQ_RIGHTSEL	4
-#define SEQ_OVERLAP		8
-#define SEQ_FILTERY		16
-#define SEQ_MUTE		32
-#define SEQ_MAKE_PREMUL	64
-#define SEQ_REVERSE_FRAMES	128
-#define SEQ_IPO_FRAME_LOCKED    256
-#define SEQ_EFFECT_NOT_LOADED   512
+#define SEQ_LEFTSEL				2
+#define SEQ_RIGHTSEL			4
+#define SEQ_OVERLAP				8
+#define SEQ_FILTERY				16
+#define SEQ_MUTE				32
+#define SEQ_MAKE_PREMUL			64
+#define SEQ_REVERSE_FRAMES		128
+#define SEQ_IPO_FRAME_LOCKED	256
+#define SEQ_EFFECT_NOT_LOADED	512
+#define SEQ_FLAG_DELETE			1024
 
 /* seq->type WATCH IT: SEQ_EFFECT BIT is used to determine if this is an effect strip!!! */
 #define SEQ_IMAGE		0
@@ -195,6 +225,10 @@ typedef struct GlowVars {
 #define SEQ_PLUGIN		24
 #define SEQ_WIPE		25
 #define SEQ_GLOW		26
+#define SEQ_TRANSFORM		27
+#define SEQ_COLOR               28
+#define SEQ_SPEED               29
+
 
 #endif
 
