@@ -1,7 +1,7 @@
 /**
  * blenlib/DNA_texture_types.h (mar-2001 nzc)
  *
- * $Id: DNA_texture_types.h,v 1.11 2004/04/28 18:08:33 ton Exp $ 
+ * $Id: DNA_texture_types.h,v 1.32 2007/01/14 11:51:51 ton Exp $ 
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -35,6 +35,7 @@
 #define DNA_TEXTURE_TYPES_H
 
 #include "DNA_ID.h"
+#include "DNA_image_types.h"
 
 struct Ipo;
 struct PluginTex;
@@ -43,12 +44,14 @@ struct EnvMap;
 struct Object;
 struct Tex;
 struct Image;
+struct ImBuf;
 
 typedef struct MTex {
 
 	short texco, mapto, maptoneg, blendtype;
 	struct Object *object;
 	struct Tex *tex;
+	char uvname[32];
 	
 	char projx, projy, projz, mapping;
 	float ofs[3], size[3];
@@ -59,7 +62,7 @@ typedef struct MTex {
 	
 	float colfac, norfac, varfac;
 	float dispfac; 
-	short pad1, pad2; 
+	float warpfac; 
 	
 } MTex;
 
@@ -89,6 +92,7 @@ typedef struct PluginTex {
 	float data[32];
 
 	int (*doit)(void);
+	void (*instance_init)(void *);
 
 	/* should be void (*)(unsigned short)... patched */	
 	void (*callback)(dna_ushort_fix);
@@ -101,24 +105,26 @@ typedef struct CBData {
 	int cur;
 } CBData;
 
+/* 32 = MAXCOLORBAND */
 typedef struct ColorBand {
 	short flag, tot, cur, ipotype;
-	CBData data[16];
+	CBData data[32];
 	
 } ColorBand;
 
 typedef struct EnvMap {
 	struct Object *object;
 	struct Image *ima;		/* type ENV_LOAD */
-	struct Image *cube[6];		/* these images are dynamic, not part of the main struct */
+	struct ImBuf *cube[6];		/* these images are dynamic, not part of the main struct */
 	float imat[4][4];
+	float obimat[3][3];
 	short type, stype;
 	float clipsta, clipend;
+	float viewscale;	/* viewscale is for planar envmaps to zoom in or out */
 	unsigned int notlay;
 	short cuberes, depth;
-	short ok, lastframe;
+	int ok, lastframe;
 	short recalc, lastsize;
-	int pad1;
 } EnvMap;
 
 typedef struct Tex {
@@ -148,10 +154,17 @@ typedef struct Tex {
 	
 	float cropxmin, cropymin, cropxmax, cropymax;
 	short xrepeat, yrepeat;
-	short extend, len;
-	float checkerdist, pad;
-	short frames, offset, sfra, fie_ima;
-	float norfac, *nor;
+	short extend;
+	
+	/* variables disabled, moved to struct iuser */
+	short fie_ima;
+	int len;
+	int frames, offset, sfra;
+	
+	float checkerdist, nabla;
+	float norfac;
+	
+	struct ImageUser iuser;
 	
 	struct Ipo *ipo;
 	struct Image *ima;
@@ -159,10 +172,26 @@ typedef struct Tex {
 	struct ColorBand *coba;
 	struct EnvMap *env;
 	
-	short fradur[4][2];
 	
 } Tex;
 
+/* used for mapping node. note: rot is in degrees */
+
+typedef struct TexMapping {
+	float loc[3], rot[3], size[3];
+	int flag;
+	
+	float mat[4][4];
+	float min[3], max[3];
+	struct Object *ob;
+
+} TexMapping;
+
+/* texmap->flag */
+#define TEXMAP_CLIP_MIN	1
+#define TEXMAP_CLIP_MAX	2
+
+
 /* **************** TEX ********************* */
 
 /* type */
@@ -212,17 +241,18 @@ typedef struct Tex {
 #define TEX_INTERPOL	1
 #define TEX_USEALPHA	2
 #define TEX_MIPMAP		4
-#define TEX_FIELDS		8
 #define TEX_IMAROT		16
 #define TEX_CALCALPHA	32
-#define TEX_ANIMCYCLIC	64
-#define TEX_ANIM5		128
-#define TEX_ANTIALI		256
-#define TEX_ANTISCALE	512
-#define TEX_STD_FIELD	1024
+#define TEX_NORMALMAP	2048
+#define TEX_GAUSS_MIP	4096
 
-#define TEX_LASOPPATCH	8192
-#define TEX_MORKPATCH	16384
+/* imaflag unused, only for version check */
+#define TEX_FIELDS_		8
+#define TEX_ANIMCYCLIC_	64
+#define TEX_ANIM5_		128
+#define TEX_ANTIALI_	256
+#define TEX_ANTISCALE_	512
+#define TEX_STD_FIELD_	1024
 
 /* flag */
 #define TEX_COLORBAND		1
@@ -230,6 +260,10 @@ typedef struct Tex {
 #define TEX_NEGALPHA		4
 #define TEX_CHECKER_ODD		8
 #define TEX_CHECKER_EVEN	16
+#define TEX_PRV_ALPHA		32
+#define TEX_PRV_NOR			64
+#define TEX_REPEAT_XMIR		128
+#define TEX_REPEAT_YMIR		256
 
 /* extend (starts with 1 because of backward comp.) */
 #define TEX_EXTEND		1
@@ -242,6 +276,31 @@ typedef struct Tex {
 #define TEX_NOISESOFT	0
 #define TEX_NOISEPERL	1
 
+/* tex->noisebasis2 in texture.c - wood waveforms */
+#define TEX_SIN			0
+#define TEX_SAW			1
+#define TEX_TRI			2
+
+/* tex->stype in texture.c - wood types */
+#define TEX_BAND		0
+#define TEX_RING		1
+#define TEX_BANDNOISE	2
+#define TEX_RINGNOISE	3
+
+/* tex->stype in texture.c - marble types */
+#define TEX_SOFT		0
+#define TEX_SHARP		1
+#define TEX_SHARPER		2
+
+/* tex->stype in texture.c - blend types */
+#define TEX_LIN			0
+#define TEX_QUAD		1
+#define TEX_EASE		2
+#define TEX_DIAG		3
+#define TEX_SPHERE		4
+#define TEX_HALO		5
+#define TEX_RAD			6
+
 /* wrap */
 #define MTEX_FLAT		0
 #define MTEX_CUBE		1
@@ -253,44 +312,6 @@ typedef struct Tex {
 #define TEX_RGB		1
 #define TEX_NOR		2
 
-/* texco */
-#define TEXCO_ORCO		1
-#define TEXCO_REFL		2
-#define TEXCO_NORM		4
-#define TEXCO_GLOB		8
-#define TEXCO_UV		16
-#define TEXCO_OBJECT	32
-#define TEXCO_LAVECTOR	64
-#define TEXCO_VIEW		128
-#define TEXCO_STICKY	256
-#define TEXCO_OSA		512
-#define TEXCO_WINDOW	1024
-#define NEED_UV			2048
-
-/* mapto */
-#define MAP_COL			1
-#define MAP_NORM		2
-#define MAP_COLSPEC		4
-#define MAP_COLMIR		8
-#define MAP_VARS		(0xFFF0)
-#define MAP_REF			16
-#define MAP_SPEC		32
-#define MAP_EMIT		64
-#define MAP_ALPHA		128
-#define MAP_HAR			256
-#define MAP_XTRA		512
-#define MAP_DISPLACE	4096
-
-/* pr_type */
-#define MA_FLAT			0
-#define MA_SPHERE		1
-#define MA_CUBE			2
-
-/* pr_back */
-#define MA_DARK			1
-
-/* pr_lamp	*/
-
 /* **************** MTEX ********************* */
 
 /* proj */
@@ -304,12 +325,19 @@ typedef struct Tex {
 #define MTEX_STENCIL	2
 #define MTEX_NEGATIVE	4
 #define MTEX_ALPHAMIX	8
+#define MTEX_VIEWSPACE	16
 
 /* blendtype */
 #define MTEX_BLEND		0
 #define MTEX_MUL		1
 #define MTEX_ADD		2
 #define MTEX_SUB		3
+#define MTEX_DIV		4
+#define MTEX_DARK		5
+#define MTEX_DIFF		6
+#define MTEX_LIGHT		7
+#define MTEX_SCREEN		8
+#define MTEX_OVERLAY	9
 
 /* **************** EnvMap ********************* */
 
