@@ -1,5 +1,5 @@
 /**
- * $Id: makesdna.c,v 1.24 2006/11/26 12:23:21 ton Exp $
+ * $Id: makesdna.c,v 1.25 2007/04/24 14:52:35 ton Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -49,7 +49,7 @@
  * numbers give more output.
  * */
 
-#define DNA_VERSION_DATE "$Id: makesdna.c,v 1.24 2006/11/26 12:23:21 ton Exp $"
+#define DNA_VERSION_DATE "$Id: makesdna.c,v 1.25 2007/04/24 14:52:35 ton Exp $"
 
 #include <string.h>
 #include <stdlib.h>
@@ -195,7 +195,7 @@ int arraysize(char *astr, int len);
 /**
  * Determine how many bytes are needed for each struct.
  */ 
-int calculate_structlens(void);
+static int calculate_structlens(int);
 
 /**
  * Construct the DNA.c file
@@ -649,7 +649,7 @@ int arraysize(char *astr, int len)
 	return mul;
 }
 
-int calculate_structlens(void)
+static int calculate_structlens(int firststruct)
 {
 	int a, b, len, alphalen, unknown= nr_structs, lastunknown, structtype, type, mul, namelen;
 	short *sp, *structpoin;
@@ -712,6 +712,12 @@ int calculate_structlens(void)
 						mul= 1;
 						if( cp[namelen-1]==']') mul= arraysize(cp, namelen);
 						
+						/* struct alignment */
+						if(type >= firststruct) {
+							if(sizeof(void *)==8 && (len % 8) )
+								printf("Align struct error: %s %s\n", types[structtype],cp);
+						}
+						
 						/* 2-4 aligned/ */
 						if(typelens[type]>3 && (len % 4) ) {
 							printf("Align 4 error in struct: %s %s (add %d padding bytes)\n", types[structtype], cp, len%4);
@@ -893,7 +899,7 @@ int make_structDNA(char *baseDirectory, FILE *file)
 	}
 	if (debugSDNA) printf("\tFinished scanning %d headers.\n", i); 
 
-	if (calculate_structlens()) {
+	if (calculate_structlens(firststruct)) {
 		// error
 		return(1);
 	}
