@@ -1,5 +1,5 @@
 /* 
- * $Id: BPY_interface.c,v 1.94 2007/02/09 18:27:20 elubie Exp $
+ * $Id: BPY_interface.c,v 1.108 2007/04/18 14:40:00 campbellbarton Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -43,7 +43,11 @@
 #include "BKE_library.h"
 #include "BKE_object.h"		/* during_scriptlink() */
 #include "BKE_text.h"
+
 #include "DNA_curve_types.h" /* for struct IpoDriver */
+#include "DNA_ID.h" /* ipo driver */
+#include "DNA_object_types.h" /* ipo driver */
+
 #include "DNA_screen_types.h"
 #include "DNA_userdef_types.h"	/* for U.pythondir */
 #include "MEM_guardedalloc.h"
@@ -55,16 +59,21 @@
 #include "api2_2x/EXPP_interface.h"
 #include "api2_2x/constant.h"
 #include "api2_2x/gen_utils.h"
+#include "api2_2x/gen_library.h" /* GetPyObjectFromID */
 #include "api2_2x/BGL.h" 
 #include "api2_2x/Blender.h"
 #include "api2_2x/Camera.h"
 #include "api2_2x/Draw.h"
-#include "api2_2x/Lamp.h"
-#include "api2_2x/NMesh.h"
 #include "api2_2x/Object.h"
 #include "api2_2x/Registry.h"
-#include "api2_2x/Scene.h"
-#include "api2_2x/World.h"
+#include "api2_2x/bpy.h" /* for the new "bpy" module */
+
+/* for scriptlinks */
+#include "DNA_lamp_types.h"
+#include "DNA_camera_types.h"
+#include "DNA_world_types.h"
+#include "DNA_scene_types.h"
+#include "DNA_material_types.h"
 
 /* bpy_registryDict is declared in api2_2x/Registry.h and defined
  * in api2_2x/Registry.c
@@ -72,7 +81,7 @@
  * choose to preserve after they are executed, so user changes can be
  * restored next time the script is used.  Check the Blender.Registry module. 
  */
-//#include "api2_2x/Registry.h"
+/*#include "api2_2x/Registry.h" */
 
 /* for pydrivers (ipo drivers defined by one-line Python expressions) */
 PyObject *bpy_pydriver_Dict = NULL;
@@ -103,11 +112,13 @@ int setup_armature_weakrefs()
 	return 1;
 }
 
-/*Declares the modules and their initialization functions
-*These are TOP-LEVEL modules e.g. import `module` - there is no
-*support for packages here e.g. import `package.module` */
+/* Declares the modules and their initialization functions
+ * These are TOP-LEVEL modules e.g. import `module` - there is no
+ * support for packages here e.g. import `package.module` */
+
 static struct _inittab BPy_Inittab_Modules[] = {
 	{"Blender", M_Blender_Init},
+	{"bpy", m_bpy_init},
 	{NULL, NULL}
 };
 
@@ -115,6 +126,7 @@ static struct _inittab BPy_Inittab_Modules[] = {
 * Structure definitions	
 **************************************************************************/
 #define FILENAME_LENGTH 24
+
 typedef struct _ScriptError {
 	char filename[FILENAME_LENGTH];
 	int lineno;
@@ -170,8 +182,21 @@ void BPY_start_python( int argc, char **argv )
 	 * print an error if not found.  See init_syspath() for the
 	 * rest of our init msgs.
 	 */
-	// Py_GetVersion() returns a ptr to astatic string
-	printf( "Compiled with Python version %.3s.\n", Py_GetVersion() );
+
+	/* print Python version
+	 * Py_GetVersion() returns a ptr to a static string "9.9.9 (aaaa..." 
+	 */
+	{
+		int count = 3;  /* a nice default for major.minor.  example 2.5 */
+		const char *version = Py_GetVersion();
+		/* we know a blank is there somewhere! */
+		char *blank_ptr = strchr( version, ' '); 
+		if(blank_ptr)
+			count = blank_ptr - version;
+		
+		printf( "Compiled with Python version %.*s.\n", count, version );
+	}
+
 
 	//Initialize the TOP-LEVEL modules
 	PyImport_ExtendInittab(BPy_Inittab_Modules);
@@ -760,6 +785,8 @@ int BPY_menu_do_python( short menutype, int event )
 	case PYMENU_HELP:
 	case PYMENU_RENDER:
 	case PYMENU_WIZARDS:
+	case PYMENU_SCRIPTTEMPLATE:
+	case PYMENU_MESHFACEKEY:
 		break;
 
 	default:
@@ -1044,8 +1071,10 @@ static int bpy_pydriver_create_dict(void)
 	if (mod) {
 		PyObject *fcn = PyObject_GetAttrString(mod, "Get");
 		Py_DECREF(mod);
-		if (fcn)
+		if (fcn) {
 			PyDict_SetItemString(d, "ob", fcn);
+			Py_DECREF(fcn);
+		}
 	}
 
 	/* me(meshname) == Blender.Mesh.Get(meshname) */
@@ -1053,8 +1082,10 @@ static int bpy_pydriver_create_dict(void)
 	if (mod) {
 		PyObject *fcn = PyObject_GetAttrString(mod, "Get");
 		Py_DECREF(mod);
-		if (fcn)
+		if (fcn) {
 			PyDict_SetItemString(d, "me", fcn);
+			Py_DECREF(fcn);
+		}
 	}
 
 	/* ma(matname) == Blender.Material.Get(matname) */
@@ -1062,8 +1093,10 @@ static int bpy_pydriver_create_dict(void)
 	if (mod) {
 		PyObject *fcn = PyObject_GetAttrString(mod, "Get");
 		Py_DECREF(mod);
-		if (fcn)
+		if (fcn) {
 			PyDict_SetItemString(d, "ma", fcn);
+			Py_DECREF(fcn);
+		}
 	}
 
 	return 0;
@@ -1144,8 +1177,9 @@ struct Object **BPY_pydriver_get_objects(IpoDriver *driver)
 float BPY_pydriver_eval(IpoDriver *driver)
 {
 	char *expr = NULL;
-	PyObject *retval, *floatval;
+	PyObject *retval, *floatval, *bpy_ob = NULL;
 	float result = 0.0f; /* default return */
+	int setitem_retval;
 
 	if (!driver) return result;
 
@@ -1159,7 +1193,16 @@ float BPY_pydriver_eval(IpoDriver *driver)
 		}
 	}
 
-/* sds*/
+	if (driver->ob)
+		bpy_ob = Object_CreatePyObject(driver->ob);
+
+	if (!bpy_ob) {
+		Py_INCREF(Py_None);
+		bpy_ob = Py_None;
+	}
+
+	setitem_retval = EXPP_dict_set_item_str(bpy_pydriver_Dict, "self", bpy_ob);
+
 	if( !setup_armature_weakrefs()){
 		fprintf( stderr, "Oops - weakref dict setup\n");
 		return result;
@@ -1171,17 +1214,18 @@ float BPY_pydriver_eval(IpoDriver *driver)
 	if (retval == NULL) {
 		return pydriver_error(driver);
 	}
-	else {
-		floatval = PyNumber_Float(retval);
-		Py_DECREF(retval);
-	}
+
+	floatval = PyNumber_Float(retval);
+	Py_DECREF(retval);
 
 	if (floatval == NULL) 
 		return pydriver_error(driver);
-	else {
-		result = (float)PyFloat_AsDouble(floatval);
-		Py_DECREF(floatval);
-	}
+
+	result = (float)PyFloat_AsDouble(floatval);
+	Py_DECREF(floatval);
+
+	/* remove 'self', since this dict is also used by py buttons */
+	if (setitem_retval == 0) PyDict_DelItemString(bpy_pydriver_Dict, "self");
 
 	/* all fine, make sure the "invalid expression" flag is cleared */
 	driver->flag &= ~IPO_DRIVER_FLAG_INVALID;
@@ -1235,7 +1279,7 @@ int BPY_button_eval(char *expr, double *value)
 		}
 	}
 
-/* sds*/
+
 	if( !setup_armature_weakrefs()){
 		fprintf(stderr, "Oops - weakref dict\n");
 		return -1;
@@ -1333,27 +1377,6 @@ static ScriptLink *ID_getScriptlink( ID * id )
 	}
 }
 
-static PyObject *ID_asPyObject( ID * id )
-{
-	switch ( MAKE_ID2( id->name[0], id->name[1] ) ) {
-	case ID_OB:
-		return Object_CreatePyObject( ( Object * ) id );
-	case ID_LA:
-		return Lamp_CreatePyObject( ( Lamp * ) id );
-	case ID_CA:
-		return Camera_CreatePyObject( ( Camera * ) id );
-	case ID_MA:
-		return Material_CreatePyObject( ( Material * ) id );
-	case ID_WO:
-		return World_CreatePyObject( ( World * ) id );
-	case ID_SCE:
-		return Scene_CreatePyObject( ( Scene * ) id );
-	default:
-		Py_INCREF( Py_None );
-		return Py_None;
-	}
-}
-
 int BPY_has_onload_script( void )
 {
 	ScriptLink *slink = &G.scene->scriptlink;
@@ -1397,7 +1420,7 @@ void BPY_do_pyscript( ID * id, short event )
 		/* set globals in Blender module to identify scriptlink */
 		EXPP_dict_set_item_str( g_blenderdict, "bylink", EXPP_incr_ret_True() );
 		EXPP_dict_set_item_str( g_blenderdict, "link",
-				      ID_asPyObject( id ) );
+				      GetPyObjectFromID( id ) );
 		EXPP_dict_set_item_str( g_blenderdict, "event",
 				      PyString_FromString( event_to_name
 							   ( event ) ) );
@@ -1982,6 +2005,7 @@ void init_ourImport( void )
 
 	m = PyImport_AddModule( "__builtin__" );
 	d = PyModule_GetDict( m );
+	
 	EXPP_dict_set_item_str( d, "__import__", import );
 }
 
