@@ -1,5 +1,5 @@
 /* 
- * $Id: BPY_menus.c,v 1.37 2007/02/09 18:27:20 elubie Exp $
+ * $Id: BPY_menus.c,v 1.41 2007/04/02 09:58:01 campbellbarton Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -105,6 +105,14 @@ static int bpymenu_group_atoi( char *str )
 		return PYMENU_VERTEXPAINT;
 	else if( !strcmp( str, "UVCalculation" ) )
 		return PYMENU_UVCALCULATION;
+	else if( !strcmp( str, "Armature" ) )
+		return PYMENU_ARMATURE;
+	else if( !strcmp( str, "ScriptTemplate" ) )
+		return PYMENU_SCRIPTTEMPLATE;
+	else if( !strcmp( str, "MeshFaceKey" ) )
+		return PYMENU_MESHFACEKEY;
+	else if( !strcmp( str, "AddMesh" ) )
+		return PYMENU_ADDMESH;
 	/* "Misc" or an inexistent group name: use misc */
 	else
 		return PYMENU_MISC;
@@ -173,6 +181,18 @@ char *BPyMenu_group_itoa( short menugroup )
 	case PYMENU_UVCALCULATION:
 		return "UVCalculation";
 		break;
+	case PYMENU_ARMATURE:
+		return "Armature";
+		break;
+	case PYMENU_SCRIPTTEMPLATE:
+		return "ScriptTemplate";
+		break;
+	case PYMENU_MESHFACEKEY:
+		return "MeshFaceKey";
+		break;
+	case PYMENU_ADDMESH:
+		return "AddMesh";
+		break;
 	case PYMENU_MISC:
 		return "Misc";
 		break;
@@ -855,14 +875,8 @@ static int bpymenu_ParseDir(char *dirname, char *parentdir, int is_userdir )
 						s = de->d_name;
 						if (parentdir) {
 							/* Join parentdir and de->d_name */
-							short a = strlen(parentdir);
-							strcpy(subdir, parentdir);
-							strcpy(subdir + a+1, de->d_name);
-#ifdef WIN32
-							subdir[a] = '\\';
-#else
-							subdir[a] = '/';
-#endif
+							BLI_join_dirfile(subdir, parentdir, de->d_name);
+
 							s = subdir;
 						}
 						bpymenu_ParseFile(file, s, is_userdir);
@@ -895,7 +909,8 @@ static int bpymenu_ParseDir(char *dirname, char *parentdir, int is_userdir )
 				}
 				s = de->d_name;
 				if (parentdir) {
-					BLI_make_file_string(NULL, subdir, parentdir, de->d_name);
+					/* Join parentdir and de->d_name */
+					BLI_join_dirfile(subdir, parentdir, de->d_name);					
 					s = subdir;
 				}
 				if (bpymenu_ParseDir(path, s, is_userdir) == -1) {
