@@ -1,5 +1,5 @@
 /* 
- * $Id: BPY_menus.h,v 1.20 2006/12/12 04:18:44 campbellbarton Exp $
+ * $Id: BPY_menus.h,v 1.23 2007/04/02 09:58:01 campbellbarton Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -100,10 +100,13 @@ typedef enum {
 	PYMENU_WEIGHTPAINT,
 	PYMENU_VERTEXPAINT,
 	PYMENU_UVCALCULATION,
+	PYMENU_ARMATURE,
+	PYMENU_SCRIPTTEMPLATE,
 	PYMENU_HELP,/*Main Help menu items - prob best to leave for 'official' ones*/
 	PYMENU_HELPSYSTEM,/* Resources, troubleshooting, system tools */
 	PYMENU_HELPWEBSITES,/* Help -> Websites submenu */
-
+	PYMENU_MESHFACEKEY, /* face key in mesh editmode */
+	PYMENU_ADDMESH, /* adds mesh */
 	PYMENU_TOTAL
 } PYMENUHOOKS;
 
