@@ -1,5 +1,5 @@
 /*
- * $Id: Armature.c,v 1.53 2007/01/31 03:12:26 stiv Exp $
+ * $Id: Armature.c,v 1.61 2007/03/26 02:10:21 campbellbarton Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -42,6 +42,7 @@
 #include "Bone.h"
 #include "NLA.h"
 #include "gen_utils.h"
+#include "gen_library.h"
 
 #include "DNA_object_types.h" //This must come before BIF_editarmature.h...
 #include "BIF_editarmature.h"
@@ -511,7 +512,7 @@ static PyObject *Armature_copy(BPy_Armature *self)
 	bArmature *bl_armature;
 	bl_armature= copy_armature(self->armature);
 	bl_armature->id.us= 0;
-	py_armature= PyArmature_FromArmature( bl_armature );
+	py_armature= Armature_CreatePyObject( bl_armature );
 	return py_armature;
 }
 
@@ -610,28 +611,6 @@ AttributeError:
 			"expected a list of integers" );
 }
 
-//------------------------Armature.users (getter)
-static PyObject *Armature_getUsers( BPy_Armature * self )
-{
-	return PyInt_FromLong( self->armature->id.us );
-}
-
-
-//------------------------Armature.fakeUser (getter)
-static PyObject *Armature_getFakeUser( BPy_Armature * self )
-{
-	if (self->armature->id.flag & LIB_FAKEUSER)
-		Py_RETURN_TRUE;
-	else
-		Py_RETURN_FALSE;
-}
-//------------------------Armature.fakeUser (setter)
-static int Armature_setFakeUser( BPy_Armature * self, PyObject * value )
-{
-	return SetIdFakeUser(&self->armature->id, value);
-}
-
-
 //------------------------Armature.mirrorEdit (getter)
 static PyObject *Armature_getMirrorEdit(BPy_Armature *self, void *closure)
 {
@@ -935,31 +914,7 @@ AttributeError:
 	return EXPP_intError(PyExc_AttributeError, "%s%s", 
 		sArmatureBadArgs, "Expects True or False");
 }
-//------------------------Armature.name (getter)
-//Gets the name of the armature
-static PyObject *Armature_getName(BPy_Armature *self, void *closure)
-{
-    return PyString_FromString(self->armature->id.name +2); //*new*
-}
-//------------------------Armature.name (setter)
-//Sets the name of the armature
-static int Armature_setName(BPy_Armature *self, PyObject *value, void *closure)
-{
-	char *name;
 
-	if(value){
-		if(PyString_Check(value)){
-			name = PyString_AsString(value);
-			rename_id(&self->armature->id, name);
-			return 0; 
-		}
-	}
-	goto AttributeError;
-
-AttributeError:
-	return EXPP_intError(PyExc_AttributeError, "%s%s", 
-		sArmatureBadArgs, "Expects string");
-}
 //------------------------Armature.bones (getter)
 //Gets the name of the armature
 static PyObject *Armature_getBoneDict(BPy_Armature *self, void *closure)
@@ -991,13 +946,14 @@ static PyMethodDef BPy_Armature_methods[] = {
 		"() - Rebuilds the armature based on changes to bones since the last call to makeEditable"},
 	{"__copy__", (PyCFunction) Armature_copy, METH_NOARGS, 
 		"() - Return a copy of the armature."},
+	{"copy", (PyCFunction) Armature_copy, METH_NOARGS, 
+		"() - Return a copy of the armature."},
 	{NULL, NULL, 0, NULL}
 };
 //------------------------tp_getset
 //This contains methods for attributes that require checking
 static PyGetSetDef BPy_Armature_getset[] = {
-	{"name", (getter)Armature_getName, (setter)Armature_setName, 
-		"The armature's name", NULL},
+	GENERIC_LIB_GETSETATTR,
 	{"bones", (getter)Armature_getBoneDict, (setter)Armature_setBoneDict, 
 		"The armature's Bone dictionary", NULL},
 	{"vertexGroups", (getter)Armature_getVertexGroups, (setter)Armature_setVertexGroups, 
@@ -1024,10 +980,6 @@ static PyGetSetDef BPy_Armature_getset[] = {
 		"Adds temporal IK chains while grabbing bones", NULL},
 	{"layers", (getter)Armature_getLayers, (setter)Armature_setLayers, 
 		"List of layers for the armature", NULL},
-	{"users", (getter)Armature_getUsers, (setter)NULL, 
-		"The number of object users", NULL},
-	{"fakeUser", (getter)Armature_getFakeUser, (setter)Armature_setFakeUser, 
-		"The fake user status of this object", NULL},
 	{NULL, NULL, NULL, NULL, NULL}
 };
 //------------------------tp_new
@@ -1038,7 +990,7 @@ static PyObject *Armature_new(PyTypeObject *type, PyObject *args, PyObject *kwds
 	BPy_Armature *py_armature = NULL;
 	bArmature *bl_armature;
 
-	bl_armature = add_armature();
+	bl_armature = add_armature("Armature");
 	if(bl_armature) {
 		bl_armature->id.us = 0; // return count to 0 - add_armature() inc'd it 
 
@@ -1135,7 +1087,7 @@ PyTypeObject Armature_Type = {
 	0,								//tp_as_number
 	0,								//tp_as_sequence
 	0,								//tp_as_mapping
-	0,								//tp_hash
+	( hashfunc ) GenericLib_hash,	//tp_hash
 	0,								//tp_call
 	0,								//tp_str
 	0,								//tp_getattro
@@ -1225,7 +1177,7 @@ static PyObject *M_Armature_Get(PyObject * self, PyObject * args)
 		if(size == 0){	//GET ALL ARMATURES
 			data = G.main->armature.first; //get the first data ID from the armature library
 			while (data){
-				py_armature = PyArmature_FromArmature(data); //*new*
+				py_armature = Armature_CreatePyObject(data); //*new*
 				sprintf(buffer, "%s", ((bArmature*)data)->id.name +2);
 				if(PyDict_SetItemString(dict, buffer, py_armature) == -1){ //add to dictionary
 					EXPP_decr3(seq, dict, py_armature);
@@ -1242,7 +1194,7 @@ static PyObject *M_Armature_Get(PyObject * self, PyObject * args)
 				Py_DECREF(item);
 				data = find_id("AR", name); //get data from library
 				if (data != NULL){
-					py_armature = PyArmature_FromArmature(data); //*new*
+					py_armature = Armature_CreatePyObject(data); //*new*
 					if(PyDict_SetItemString(dict, name, py_armature) == -1){ //add to dictionary
 						EXPP_decr3(seq, dict, py_armature);
 						goto RuntimeError;
@@ -1270,7 +1222,7 @@ static PyObject *M_Armature_Get(PyObject * self, PyObject * args)
 		Py_DECREF(seq);
 		data = find_id("AR", name); //get data from library
 		if (data != NULL){
-			return PyArmature_FromArmature(data); //*new*
+			return Armature_CreatePyObject(data); //*new*
 		}else{
 			char buffer[128];
 			PyOS_snprintf( buffer, sizeof(buffer),
@@ -1296,23 +1248,19 @@ static PyObject *M_Armature_New(PyObject * self, PyObject * args)
 	char *name = "Armature";
 	struct bArmature *armature;
 	BPy_Armature *obj;
-	char buf[21];
 
 	if( !PyArg_ParseTuple( args, "|s", &name ) )
 		return EXPP_ReturnPyObjError( PyExc_TypeError,
 					      "expected nothing or a string as argument" );
 
-	armature= add_armature();
+	armature= add_armature(name);
 	armature->id.us = 0;
-	obj = (BPy_Armature *)PyArmature_FromArmature(armature); /*new*/
+	obj = (BPy_Armature *)Armature_CreatePyObject(armature); /*new*/
 
 	if( !obj )
 		return EXPP_ReturnPyObjError( PyExc_RuntimeError,
 				       "PyObject_New() failed" );	
 
-	PyOS_snprintf( buf, sizeof( buf ), "%s", name );
-	rename_id( &armature->id, buf );
-
 	obj->armature = armature;
 	return (PyObject *)obj;
 }
@@ -1345,7 +1293,7 @@ PyObject *Armature_RebuildBones(PyObject *pyarmature)
 }
 //-----------------(internal)
 //Converts a bArmature to a PyArmature
-PyObject *PyArmature_FromArmature(struct bArmature *armature)
+PyObject *Armature_CreatePyObject(struct bArmature *armature)
 {
 	BPy_Armature *py_armature = NULL;
 	PyObject *maindict = NULL, *weakref = NULL;
@@ -1384,7 +1332,7 @@ PyObject *PyArmature_FromArmature(struct bArmature *armature)
 
 RuntimeError:
 	return EXPP_objError(PyExc_RuntimeError, "%s%s%s", 
-		sModuleError, "PyArmature_FromArmature: ", "Internal Error Ocurred");
+		sModuleError, "Armature_CreatePyObject: ", "Internal Error Ocurred");
 }
 //-----------------(internal)
 //Converts a PyArmature to a bArmature
@@ -1393,13 +1341,6 @@ struct bArmature *PyArmature_AsArmature(BPy_Armature *py_armature)
 	return (py_armature->armature);
 }
 
-// This function returns true when the given PyObject
-// is of the type Sound. Otherwise it will return false
-int Armature_CheckPyObject( PyObject * pyobj )
-{
-	return ( pyobj->ob_type == &Armature_Type);
-}
-
 struct bArmature *Armature_FromPyObject( PyObject * py_obj )
 {
 	return PyArmature_AsArmature((BPy_Armature*)py_obj);
