@@ -1,5 +1,5 @@
 /* 
- * $Id: BGL.c,v 1.21 2006/09/30 14:14:48 campbellbarton Exp $
+ * $Id: BGL.c,v 1.22 2007/03/16 11:38:01 campbellbarton Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -1091,8 +1091,9 @@ PyObject *BGL_Init(void)
 {
 	PyObject *mod= Py_InitModule("Blender.BGL", BGL_methods);
 	PyObject *dict= PyModule_GetDict(mod);
-
-	buffer_Type.ob_type = &PyType_Type;
+	
+	if( PyType_Ready( &buffer_Type) < 0)
+		Py_RETURN_NONE;
 
 #define EXPP_ADDCONST(x) EXPP_dict_set_item_str(dict, #x, PyInt_FromLong(x))
 
