@@ -1,5 +1,5 @@
 /*
- * $Id: BezTriple.c,v 1.25 2006/08/13 10:13:19 campbellbarton Exp $
+ * $Id: BezTriple.c,v 1.26 2007/03/15 01:09:11 campbellbarton Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -645,16 +645,6 @@ PyObject *BezTriple_CreatePyObject( BezTriple * bzt )
 }
 
 
-/*****************************************************************************/
-/* Function:    BezTriple_CheckPyObject                                     */
-/* Description: This function returns true when the given PyObject is of the */
-/*              type BezTriple. Otherwise it will return false.             */
-/*****************************************************************************/
-int BezTriple_CheckPyObject( PyObject * pyobj )
-{
-	return ( pyobj->ob_type == &BezTriple_Type );
-}
-
 /*****************************************************************************/
 /* Function:    BezTriple_FromPyObject                                      */
 /* Description: This function returns the Blender beztriple from the given   */
