@@ -1,5 +1,5 @@
 /* 
- * $Id: Blender.c,v 1.94 2007/01/15 07:54:08 joeedh Exp $
+ * $Id: Blender.c,v 1.109 2007/04/01 03:58:42 khughes Exp $
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
  * This program is free software; you can redistribute it and/or
@@ -71,9 +71,11 @@ struct ID; /*keep me up here */
 #include "Effect.h"
 #include "Ipo.h"
 #include "Ipocurve.h"
+#include "IDProp.h"
 #include "Key.h"
 #include "Lamp.h"
 #include "Lattice.h"
+#include "Library.h"
 #include "Mathutils.h"
 #include "Geometry.h"
 #include "Mesh.h"
@@ -93,11 +95,6 @@ struct ID; /*keep me up here */
 #include "World.h"
 #include "Types.h"
 
-//for the removefakeuser hack
-#include "NLA.h" /*This must come first*/
-#include "BKE_action.h"
-
-
 /**********************************************************/
 /* Python API function prototypes for the Blender module.	*/
 /**********************************************************/
@@ -108,7 +105,6 @@ static PyObject *Blender_Quit( PyObject * self );
 static PyObject *Blender_Load( PyObject * self, PyObject * args );
 static PyObject *Blender_Save( PyObject * self, PyObject * args );
 static PyObject *Blender_Run( PyObject * self, PyObject * args );
-static PyObject *Blender_RemoveFakeuser(PyObject *self, PyObject *args);
 static PyObject *Blender_ShowHelp( PyObject * self, PyObject * args );
 static PyObject *Blender_UpdateMenus( PyObject * self);
 static PyObject *Blender_PackAll( PyObject * self);
@@ -174,10 +170,6 @@ static char Blender_Run_doc[] =
 	"(script) - Run the given Python script.\n\
 (script) - the path to a file or the name of an available Blender Text.";
 
-static char Blender_RemoveFakeuser_doc[] =
-	"(datablock) - remove the fake user from a datablock. useful for deleting actions.\n\
-(datablock) - the datablock that has a fakeuser. currently only action object accepted.";
-
 static char Blender_ShowHelp_doc[] =
 "(script) - Show help for the given Python script.\n\
   This will try to open the 'Scripts Help Browser' script, so to have\n\
@@ -213,7 +205,6 @@ static struct PyMethodDef Blender_methods[] = {
 	{"Load", Blender_Load, METH_VARARGS, Blender_Load_doc},
 	{"Save", Blender_Save, METH_VARARGS, Blender_Save_doc},
 	{"Run", Blender_Run, METH_VARARGS, Blender_Run_doc},
-	{"RemoveFakeuser", Blender_RemoveFakeuser, METH_VARARGS, Blender_RemoveFakeuser_doc},
 	{"ShowHelp", Blender_ShowHelp, METH_VARARGS, Blender_ShowHelp_doc},
 	{"CountPackedFiles", ( PyCFunction ) Blender_CountPackedFiles, METH_NOARGS, Blender_CountPackedFiles_doc},
 	{"PackAll", ( PyCFunction ) Blender_PackAll, METH_NOARGS, Blender_PackAll_doc},
@@ -311,13 +302,17 @@ static PyObject *Blender_Get( PyObject * self, PyObject * args )
 	else if( StringEqual( str, "curtime" ) )
 		ret = PyFloat_FromDouble( frame_to_float( G.scene->r.cfra ) );
 	else if( StringEqual( str, "rt" ) )
-		ret = PyInt_FromLong( frame_to_float( G.rt ) );
+		ret = PyInt_FromLong( (long)frame_to_float( G.rt ) );
 	else if( StringEqual( str, "staframe" ) )
 		ret = PyInt_FromLong( G.scene->r.sfra );
 	else if( StringEqual( str, "endframe" ) )
 		ret = PyInt_FromLong( G.scene->r.efra );
-	else if( StringEqual( str, "filename" ) )
-		ret = PyString_FromString( G.sce );
+	else if( StringEqual( str, "filename" ) ) {
+		if ( strstr(G.main->name, ".B.blend") != 0)
+			ret = PyString_FromString("");
+		else
+			ret = PyString_FromString(G.main->name);
+	}
 	else if( StringEqual( str, "homedir" ) ) {
 		char *hdir = bpy_gethome(0);
 		if( hdir && BLI_exists( hdir ))
@@ -649,7 +644,7 @@ static PyObject *Blender_Load( PyObject * self, PyObject * args )
 		 * enough here.  Note: the default file requires extra clean-up done by
 		 * BIF_read_homefile: freeing the user theme data. */
 		if( !fname || ( strstr( fname, ".B.blend" ) && is_blend_file ) )
-			BIF_read_homefile(  );
+			BIF_read_homefile(0);
 		else
 			BIF_read_file( fname );
 
@@ -846,32 +841,6 @@ static PyObject * Blender_UpdateMenus( PyObject * self )
 	Py_RETURN_NONE;
 }
 
-static PyObject *Blender_RemoveFakeuser(PyObject *self, PyObject *args)
-{
-	ID *id;
-	BPy_Action *py_thing; //lousy coder antont did not know how to accept any bpy thing with ID..
-
-	if( !PyArg_ParseTuple( args, "O!", &Action_Type, &py_thing ) )
-		return EXPP_ReturnPyObjError( PyExc_AttributeError,
-				       "expected python action type" );
-	
-	id= (ID *)py_thing->action;
-	
-	if(id) {
-		if( id->flag & LIB_FAKEUSER) {
-					id->flag -= LIB_FAKEUSER;
-					id->us--;
-				}
-		else
-			return EXPP_ReturnPyObjError( PyExc_AttributeError,
-						      "given datablock has no fakeusers");
-	} else
-		return EXPP_ReturnPyObjError( PyExc_AttributeError,
-					      "given object does not have a Blender ID");
-
-	Py_RETURN_NONE;
-}
-
 /*****************************************************************************/
 /* Function:		Blender_PackAll		 */
 /* Python equivalent:	Blender.PackAll			 */
@@ -925,9 +894,6 @@ static PyObject *Blender_UnpackModesDict( void )
 /*****************************************************************************/
 /* Function:		initBlender		 */
 /*****************************************************************************/
-extern PyTypeObject IDArray_Type;
-extern PyTypeObject IDGroup_Type;
-extern PyTypeObject IDGroup_Iter_Type;
 
 void M_Blender_Init(void)
 {
@@ -939,9 +905,9 @@ void M_Blender_Init(void)
 	blender -P somescript.py -b
 	The if below solves the segfaults that happen when python runs and
 	G.scene is NULL */
-	if(G.background && G.main->scene.first==0) {
+	if(G.background && G.main->scene.first==NULL) {
 		Scene *sce= add_scene("1");
-		//set_scene(sce); /* causes a crash */
+		/*set_scene(sce);*/ /* causes a crash */
 		G.scene= sce;
 	}
 	
@@ -950,7 +916,7 @@ void M_Blender_Init(void)
 
 	types_InitAll();	/* set all our pytypes to &PyType_Type */
 
-	// constants for packed files
+	/* constants for packed files*/
 	UnpackModes = Blender_UnpackModesDict(  );
 	if( UnpackModes )
 		PyModule_AddObject( module, "UnpackModes", UnpackModes );
@@ -996,7 +962,7 @@ void M_Blender_Init(void)
 	PyDict_SetItemString(dict, "Key", Key_Init());
 	PyDict_SetItemString(dict, "Lamp", Lamp_Init());
 	PyDict_SetItemString(dict, "Lattice", Lattice_Init());
-	PyDict_SetItemString(dict, "Library", Library_Init());
+	PyDict_SetItemString(dict, "Library", oldLibrary_Init());
 	PyDict_SetItemString(dict, "Material", Material_Init());
 	PyDict_SetItemString(dict, "Mesh", Mesh_Init()); 
 	PyDict_SetItemString(dict, "Metaball", Metaball_Init());
