@@ -1,5 +1,5 @@
 /* 
- * $Id: Camera.c,v 1.43 2007/01/22 00:04:13 campbellbarton Exp $
+ * $Id: Camera.c,v 1.58 2007/04/28 18:20:43 campbellbarton Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -37,10 +37,12 @@
 #include "BKE_object.h"
 #include "BKE_library.h"
 #include "BLI_blenlib.h"
+#include "BLI_arithb.h" /* for M_PI */
 #include "BSE_editipo.h"
 #include "BIF_space.h"
 #include "mydevice.h"
 #include "gen_utils.h"
+#include "gen_library.h"
 #include "Ipo.h"
 
 
@@ -51,6 +53,7 @@
 
 enum cam_consts {
 	EXPP_CAM_ATTR_LENS = 0,
+	EXPP_CAM_ATTR_ANGLE,
 	EXPP_CAM_ATTR_DOFDIST,
 	EXPP_CAM_ATTR_CLIPEND,
 	EXPP_CAM_ATTR_CLIPSTART,
@@ -109,8 +112,6 @@ struct PyMethodDef M_Camera_methods[] = {
 /*****************************************************************************/
 /* Python BPy_Camera methods declarations:                                   */
 /*****************************************************************************/
-static PyObject *Camera_oldgetIpo( BPy_Camera * self );
-static PyObject *Camera_oldgetName( BPy_Camera * self );
 static PyObject *Camera_oldgetType( BPy_Camera * self );
 static PyObject *Camera_oldgetMode( BPy_Camera * self );
 static PyObject *Camera_oldgetLens( BPy_Camera * self );
@@ -118,8 +119,9 @@ static PyObject *Camera_oldgetClipStart( BPy_Camera * self );
 static PyObject *Camera_oldgetClipEnd( BPy_Camera * self );
 static PyObject *Camera_oldgetDrawSize( BPy_Camera * self );
 static PyObject *Camera_oldgetScale( BPy_Camera * self );
+static PyObject *Camera_getIpo( BPy_Camera * self );
+static int Camera_setIpo( BPy_Camera * self, PyObject * value );
 static PyObject *Camera_oldsetIpo( BPy_Camera * self, PyObject * args );
-static PyObject *Camera_oldsetName( BPy_Camera * self, PyObject * args );
 static PyObject *Camera_oldsetType( BPy_Camera * self, PyObject * args );
 static PyObject *Camera_oldsetMode( BPy_Camera * self, PyObject * args );
 static PyObject *Camera_oldsetLens( BPy_Camera * self, PyObject * args );
@@ -134,17 +136,15 @@ static PyObject *Camera_clearScriptLinks( BPy_Camera * self, PyObject * args );
 static PyObject *Camera_insertIpoKey( BPy_Camera * self, PyObject * args );
 static PyObject *Camera_copy( BPy_Camera * self );
 
-Camera *GetCameraByName( char *name );
-
 
 /*****************************************************************************/
 /* Python BPy_Camera methods table:                                          */
 /*****************************************************************************/
 static PyMethodDef BPy_Camera_methods[] = {
 	/* name, method, flags, doc */
-	{"getIpo", ( PyCFunction ) Camera_oldgetIpo, METH_NOARGS,
+	{"getIpo", ( PyCFunction ) Camera_getIpo, METH_NOARGS,
 	 "() - Return Camera Data Ipo"},
-	{"getName", ( PyCFunction ) Camera_oldgetName, METH_NOARGS,
+	{"getName", ( PyCFunction ) GenericLib_getName, METH_NOARGS,
 	 "() - Return Camera Data name"},
 	{"getType", ( PyCFunction ) Camera_oldgetType, METH_NOARGS,
 	 "() - Return Camera type - 'persp':0, 'ortho':1"},
@@ -167,7 +167,7 @@ static PyMethodDef BPy_Camera_methods[] = {
 	 "() - Unlink Ipo from this Camera."},
 	 {"insertIpoKey", ( PyCFunction ) Camera_insertIpoKey, METH_VARARGS,
 	 "( Camera IPO type ) - Inserts a key into IPO"},
-	{"setName", ( PyCFunction ) Camera_oldsetName, METH_VARARGS,
+	{"setName", ( PyCFunction ) GenericLib_setName_with_method, METH_VARARGS,
 	 "(s) - Set Camera Data name"},
 	{"setType", ( PyCFunction ) Camera_oldsetType, METH_VARARGS,
 	 "(s) - Set Camera type, which can be 'persp' or 'ortho'"},
@@ -197,75 +197,25 @@ static PyMethodDef BPy_Camera_methods[] = {
 	 "([s1<,s2,s3...>]) - Delete specified scriptlinks from this camera."},
 	{"__copy__", ( PyCFunction ) Camera_copy, METH_NOARGS,
 	 "() - Return a copy of the camera."},
+	{"copy", ( PyCFunction ) Camera_copy, METH_NOARGS,
+	 "() - Return a copy of the camera."},
 	{NULL, NULL, 0, NULL}
 };
 
 /*****************************************************************************/
 /* Python Camera_Type callback function prototypes:                          */
 /*****************************************************************************/
-static void Camera_dealloc( BPy_Camera * self );
-//static int Camera_setAttr( BPy_Camera * self, char *name, PyObject * v );
 static int Camera_compare( BPy_Camera * a, BPy_Camera * b );
-//static PyObject *Camera_getAttr( BPy_Camera * self, char *name );
 static PyObject *Camera_repr( BPy_Camera * self );
 
-
-//~ /*****************************************************************************/
-//~ /* Python Camera_Type structure definition:	                             */
-//~ /*****************************************************************************/
-//~ PyTypeObject Camera_Type = {
-	//~ PyObject_HEAD_INIT( NULL ) /* required macro */ 
-	//~ NULL,	/* ob_size */
-	//~ "Blender Camera",	/* tp_name */
-	//~ sizeof( BPy_Camera ),	/* tp_basicsize */
-	//~ NULL,			/* tp_itemsize */
-	//~ /* methods */
-	//~ ( destructor ) Camera_dealloc,	/* tp_dealloc */
-	//~ NULL,			/* tp_print */
-	//~ NULL,	/* tp_getattr */
-	//~ NULL,	/* tp_setattr */
-	//~ ( cmpfunc ) Camera_compare,	/* tp_compare */
-	//~ ( reprfunc ) Camera_repr,	/* tp_repr */
-	//~ NULL,			/* tp_as_number */
-	//~ NULL,			/* tp_as_sequence */
-	//~ NULL,			/* tp_as_mapping */
-	//~ NULL,			/* tp_as_hash */
-	//~ 0, 0, 0, 0, 0, 0,
-	//~ 0,			/* tp_doc */
-	//~ 0, 0, 0, 0, 0, 0,
-	//~ BPy_Camera_methods,	/* tp_methods */
-	//~ 0,			/* tp_members */
-	//~ 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
-//~ };
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
 static PyObject *M_Camera_New( PyObject * self, PyObject * args,
 			       PyObject * kwords )
 {
 	char *type_str = "persp";	/* "persp" is type 0, "ortho" is type 1 */
-	char *name_str = "CamData";
+	char *name_str = "Camera";
 	static char *kwlist[] = { "type_str", "name_str", NULL };
 	PyObject *pycam;	/* for Camera Data object wrapper in Python */
 	Camera *blcam;		/* for actual Camera Data we create in Blender */
-	char buf[21];
 
 	/* Parse the arguments passed in by the Python interpreter */
 	if( !PyArg_ParseTupleAndKeywords( args, kwords, "|ss", kwlist,
@@ -274,7 +224,7 @@ static PyObject *M_Camera_New( PyObject * self, PyObject * args,
 		return EXPP_ReturnPyObjError( PyExc_AttributeError,
 					      "expected zero, one or two strings as arguments" );
 
-	blcam = add_camera(  );	/* first create the Camera Data in Blender */
+	blcam = add_camera( name_str );	/* first create the Camera Data in Blender */
 
 	if( blcam )		/* now create the wrapper obj in Python */
 		pycam = Camera_CreatePyObject( blcam );
@@ -300,14 +250,7 @@ static PyObject *M_Camera_New( PyObject * self, PyObject * args,
 	else
 		return EXPP_ReturnPyObjError( PyExc_AttributeError,
 					      "unknown camera type" );
-
-	if( strcmp( name_str, "CamData" ) == 0 )
-		return pycam;
-	else {			/* user gave us a name for the camera, use it */
-		PyOS_snprintf( buf, sizeof( buf ), "%s", name_str );
-		rename_id( &blcam->id, buf );	/* proper way in Blender */
-	}
-
+	
 	return pycam;
 }
 
@@ -392,25 +335,6 @@ PyObject *Camera_Init( void )
 	return submodule;
 }
 
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
 /* Three Python Camera_Type helper functions needed by the Object module: */
 
 PyObject *Camera_CreatePyObject( Camera * cam )
@@ -424,71 +348,18 @@ PyObject *Camera_CreatePyObject( Camera * cam )
 					      "couldn't create BPy_Camera PyObject" );
 
 	pycam->camera = cam;
-
 	return ( PyObject * ) pycam;
 }
 
-int Camera_CheckPyObject( PyObject * pyobj )
-{
-	return ( pyobj->ob_type == &Camera_Type );
-}
-
 Camera *Camera_FromPyObject( PyObject * pyobj )
 {
 	return ( ( BPy_Camera * ) pyobj )->camera;
 }
 
-/*****************************************************************************/
-/* Description: Returns the object with the name specified by the argument  */
-/*	name. Note that the calling function has to remove the first */
-/*	two characters of the object name. These two characters		 */
-/*	specify the type of the object (OB, ME, WO, ...)		 */
-/*	The function will return NULL when no object with the given  */
-/*	name is found.							    */
-/*****************************************************************************/
-Camera *GetCameraByName( char *name )
-{
-	Camera *cam_iter;
-
-	cam_iter = G.main->camera.first;
-	while( cam_iter ) {
-		if( StringEqual( name, GetIdName( &( cam_iter->id ) ) ) ) {
-			return ( cam_iter );
-		}
-		cam_iter = cam_iter->id.next;
-	}
-
-	/* There is no camera with the given name */
-	return ( NULL );
-}
-
 /*****************************************************************************/
 /* Python BPy_Camera methods:                                               */
 /*****************************************************************************/
 
-static PyObject *Camera_oldgetIpo( BPy_Camera * self )
-{
-	struct Ipo *ipo = self->camera->ipo;
-
-	if( !ipo )
-		Py_RETURN_NONE;
-
-	return Ipo_CreatePyObject( ipo );
-}
-
-
-static PyObject *Camera_oldgetName( BPy_Camera * self )
-{
-
-	PyObject *attr = PyString_FromString( self->camera->id.name + 2 );
-
-	if( attr )
-		return attr;
-
-	return EXPP_ReturnPyObjError( PyExc_RuntimeError,
-				      "couldn't get Camera.name attribute" );
-}
-
 static PyObject *Camera_oldgetType( BPy_Camera * self )
 {
 	PyObject *attr = PyInt_FromLong( self->camera->type );
@@ -570,36 +441,7 @@ static PyObject *Camera_oldgetDrawSize( BPy_Camera * self )
 
 static PyObject *Camera_oldsetIpo( BPy_Camera * self, PyObject * args )
 {
-	PyObject *pyipo = 0;
-	Ipo *ipo = NULL;
-	Ipo *oldipo;
-
-	if( !PyArg_ParseTuple( args, "O!", &Ipo_Type, &pyipo ) )
-		return EXPP_ReturnPyObjError( PyExc_TypeError,
-					      "expected Ipo as argument" );
-
-	ipo = Ipo_FromPyObject( pyipo );
-
-	if( !ipo )
-		return EXPP_ReturnPyObjError( PyExc_RuntimeError,
-					      "null ipo!" );
-
-	if( ipo->blocktype != ID_CA )
-		return EXPP_ReturnPyObjError( PyExc_TypeError,
-					      "this ipo is not a camera data ipo" );
-
-	oldipo = self->camera->ipo;
-	if( oldipo ) {
-		ID *id = &oldipo->id;
-		if( id->us > 0 )
-			id->us--;
-	}
-
-	id_us_plus(&ipo->id);
-
-	self->camera->ipo = ipo;
-
-	Py_RETURN_NONE;
+	return EXPP_setterWrapper( (void *)self, args, (setter)Camera_setIpo );
 }
 
 static PyObject *Camera_oldclearIpo( BPy_Camera * self )
@@ -619,19 +461,6 @@ static PyObject *Camera_oldclearIpo( BPy_Camera * self )
 	return EXPP_incr_ret_False(); /* no ipo found */
 }
 
-static PyObject *Camera_oldsetName( BPy_Camera * self, PyObject * args )
-{
-	char *name;
-
-	if( !PyArg_ParseTuple( args, "s", &name ) )
-		return EXPP_ReturnPyObjError( PyExc_TypeError,
-					      "expected string argument" );
-
-	rename_id( &self->camera->id, name );
-
-	Py_RETURN_NONE;
-}
-
 static PyObject *Camera_oldsetType( BPy_Camera * self, PyObject * args )
 {
 	char *type;
@@ -825,50 +654,6 @@ static PyObject *Camera_copy( BPy_Camera * self )
 	return pycam;
 }
 
-static void Camera_dealloc( BPy_Camera * self )
-{
-	PyObject_DEL( self );
-}
-
-static PyObject *Camera_getName( BPy_Camera * self )
-{
-	return PyString_FromString( self->camera->id.name + 2 );
-}
-
-static int Camera_setName( BPy_Camera * self, PyObject * value )
-{
-	char *name = NULL;
-	
-	name = PyString_AsString ( value );
-	if( !name )
-		return EXPP_ReturnIntError( PyExc_TypeError,
-					      "expected string argument" );
-	
-	rename_id( &self->camera->id, name);
-	return 0;
-}
-
-
-static PyObject *Camera_getUsers( BPy_Camera * self )
-{
-	return PyInt_FromLong( self->camera->id.us );
-}
-
-
-static PyObject *Camera_getFakeUser( BPy_Camera * self )
-{
-	if (self->camera->id.flag & LIB_FAKEUSER)
-		Py_RETURN_TRUE;
-	else
-		Py_RETURN_FALSE;
-}
-
-static int Camera_setFakeUser( BPy_Camera * self, PyObject * value )
-{
-	return SetIdFakeUser(&self->camera->id, value);
-}
-
-
 static PyObject *Camera_getType( BPy_Camera * self )
 {
 	if (self->camera->type == EXPP_CAM_TYPE_PERSP)
@@ -929,43 +714,7 @@ static PyObject *Camera_getIpo( BPy_Camera * self )
 
 static int Camera_setIpo( BPy_Camera * self, PyObject * value )
 {
-	Ipo *ipo = NULL;
-	Ipo *oldipo = self->camera->ipo;
-	ID *id;
-
-	/* if parameter is not None, check for valid Ipo */
-
-	if ( value != Py_None ) {
-		if ( !Ipo_CheckPyObject( value ) )
-			return EXPP_ReturnIntError( PyExc_TypeError,
-					"expected an Ipo object" );
-
-		ipo = Ipo_FromPyObject( value );
-
-		if( !ipo )
-			return EXPP_ReturnIntError( PyExc_RuntimeError,
-					"null ipo!" );
-
-		if( ipo->blocktype != ID_CA )
-			return EXPP_ReturnIntError( PyExc_TypeError,
-					"Ipo is not a camera data Ipo" );
-	}
-
-	/* if already linked to Ipo, delete link */
-
-	if ( oldipo ) {
-		id = &oldipo->id;
-		if( id->us > 0 )
-			id->us--;
-	}
-
-	/* assign new Ipo and increment user count, or set to NULL if deleting */
-
-	self->camera->ipo = ipo;
-	if ( ipo )
-		id_us_plus(&ipo->id);
-
-	return 0;
+	return GenericLib_assignData(value, (void **) &self->camera->ipo, 0, 1, ID_IP, ID_CA);
 }
 
 /*
@@ -981,6 +730,9 @@ static PyObject *getFloatAttr( BPy_Camera *self, void *type )
 	case EXPP_CAM_ATTR_LENS: 
 		param = cam->lens;
 		break;
+	case EXPP_CAM_ATTR_ANGLE: 
+		param = 360.0f * atan(16.0f/cam->lens) / M_PI;
+		break;
 	case EXPP_CAM_ATTR_DOFDIST: 
 		param = cam->YF_dofdist;
 		break;
@@ -1025,14 +777,19 @@ static int setFloatAttrClamp( BPy_Camera *self, PyObject *value, void *type )
 	float *param;
 	struct Camera *cam = self->camera;
 	float min, max;
-
-
+	int ret;
+ 
 	switch( (int)type ) {
 	case EXPP_CAM_ATTR_LENS:
 		min = 1.0;
 		max = 250.0;
 		param = &cam->lens;
 		break;
+	case EXPP_CAM_ATTR_ANGLE:
+		min = 7.323871;
+		max = 172.847331;
+		param = &cam->lens;
+		break;
 	case EXPP_CAM_ATTR_DOFDIST:
 		min = 0.0;
 		max = 5000.0;
@@ -1049,12 +806,12 @@ static int setFloatAttrClamp( BPy_Camera *self, PyObject *value, void *type )
 		param = &cam->clipend;
 		break;
 	case EXPP_CAM_ATTR_DRAWSIZE:
-		min = 0.1;
+		min = 0.1f;
 		max = 10.0;
 		param = &cam->drawsize;
 		break;
 	case EXPP_CAM_ATTR_SCALE:
-		min = 0.01;
+		min = 0.01f;
 		max = 1000.0;
 		param = &cam->ortho_scale;
 		break;
@@ -1079,7 +836,14 @@ static int setFloatAttrClamp( BPy_Camera *self, PyObject *value, void *type )
 				"undefined type in setFloatAttrClamp" );
 	}
 
-	return EXPP_setFloatClamped( value, param, min, max );
+	ret = EXPP_setFloatClamped( value, param, min, max );
+	
+	if (ret==0) {
+		if ((int)type == EXPP_CAM_ATTR_ANGLE) {
+			cam->lens = 16.0f / tan(M_PI*cam->lens/360.0f);
+		}
+	}
+	return ret;
 }
 
 
@@ -1095,8 +859,6 @@ static PyObject *getFlagAttr( BPy_Camera *self, void *type )
 		Py_RETURN_FALSE;
 }
 
-
-
 /*
  * set floating point attributes which require clamping
  */
@@ -1115,18 +877,7 @@ static int setFlagAttr( BPy_Camera *self, PyObject *value, void *type )
 /* Python attributes get/set structure:                                      */
 /*****************************************************************************/
 static PyGetSetDef BPy_Camera_getseters[] = {
-	{"name",
-	 (getter)Camera_getName, (setter)Camera_setName,
-	 "Camera name",
-	 NULL},
-	{"users",
-	 (getter)Camera_getUsers, (setter)NULL,
-	 "Number of camera users",
-	 NULL},
-	{"fakeUser",
-	 (getter)Camera_getFakeUser, (setter)Camera_setFakeUser,
-	 "Cameras fake user state",
-	 NULL},
+	GENERIC_LIB_GETSETATTR,
 	{"type",
 	 (getter)Camera_getType, (setter)Camera_setType,
 	 "camera type \"persp\" or \"ortho\"",
@@ -1139,12 +890,17 @@ static PyGetSetDef BPy_Camera_getseters[] = {
 	 (getter)Camera_getIpo, (setter)Camera_setIpo,
 	 "Cameras ipo",
 	 NULL},
-	 
+	
 	/* float settings */
 	{"lens",
 	 (getter)getFloatAttr, (setter)setFloatAttrClamp,
 	 "lens angle for perspective cameras",
 	 (void *)EXPP_CAM_ATTR_LENS},
+	{"angle",
+	 (getter)getFloatAttr, (setter)setFloatAttrClamp,
+	 "lens angle for perspective cameras",
+	 (void *)EXPP_CAM_ATTR_ANGLE},
+	 
 	{"scale",
 	 (getter)getFloatAttr, (setter)setFloatAttrClamp,
 	 "scale for ortho cameras",
@@ -1157,6 +913,14 @@ static PyGetSetDef BPy_Camera_getseters[] = {
 	 (getter)getFloatAttr, (setter)setFloatAttrClamp,
 	 "the cameras clip end",
 	 (void *)EXPP_CAM_ATTR_CLIPEND},
+	{"shiftX",
+	 (getter)getFloatAttr, (setter)setFloatAttrClamp,
+	 "the cameras X perspective shift",
+	 (void *)EXPP_CAM_ATTR_SHIFTX},
+	{"shiftY",
+	 (getter)getFloatAttr, (setter)setFloatAttrClamp,
+	 "the cameras Y perspective shift",
+	 (void *)EXPP_CAM_ATTR_SHIFTY},
 	{"dofDist",
 	 (getter)getFloatAttr, (setter)setFloatAttrClamp,
 	 "cameras dof distance",
@@ -1191,7 +955,11 @@ static PyGetSetDef BPy_Camera_getseters[] = {
 	 (getter)getFlagAttr, (setter)setFlagAttr,
 	 "toggle the passPartOut display flag",
 	 (void *)CAM_SHOWPASSEPARTOUT},
-	{NULL,NULL,NULL,NULL,NULL}  /* Sentinel */
+	{"angleToggle",
+	 (getter)getFlagAttr, (setter)setFlagAttr,
+	 "toggle the camera input unit flag",
+	 (void *)CAM_ANGLETOGGLE},
+	{NULL,NULL,NULL,NULL}  /* Sentinel */
 };
 
 
@@ -1208,7 +976,7 @@ PyTypeObject Camera_Type = {
 
 	/* Methods to implement standard operations */
 
-	( destructor ) Camera_dealloc,/* destructor tp_dealloc; */
+	NULL,						/* destructor tp_dealloc; */
 	NULL,                       /* printfunc tp_print; */
 	NULL,                       /* getattrfunc tp_getattr; */
 	NULL,                       /* setattrfunc tp_setattr; */
