@@ -1,5 +1,5 @@
 /*
- * $Id: Constraint.c,v 1.9 2006/12/02 03:48:13 erwin Exp $
+ * $Id: Constraint.c,v 1.15 2007/04/22 22:08:17 campbellbarton Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -103,6 +103,7 @@ enum constraint_constants {
 
 	EXPP_CONSTR_COPY,
 	EXPP_CONSTR_LIMIT,
+	EXPP_CONSTR_CLAMP,
 	
 	EXPP_CONSTR_LIMXMIN = LIMIT_XMIN,
 	EXPP_CONSTR_LIMXMAX = LIMIT_XMAX,
@@ -206,7 +207,6 @@ static PyMappingMethods Constraint_as_mapping = {
 /*****************************************************************************/
 /* Python Constraint_Type callback function prototypes:                      */
 /*****************************************************************************/
-static void Constraint_dealloc( BPy_Constraint * self );
 static PyObject *Constraint_repr( BPy_Constraint * self );
 static int Constraint_compare( BPy_Constraint * a, BPy_Constraint * b );
 
@@ -223,7 +223,7 @@ PyTypeObject Constraint_Type = {
 
 	/* Methods to implement standard operations */
 
-	( destructor ) Constraint_dealloc,/* destructor tp_dealloc; */
+	NULL,						/* destructor tp_dealloc; */
 	NULL,                       /* printfunc tp_print; */
 	NULL,                       /* getattrfunc tp_getattr; */
 	NULL,                       /* setattrfunc tp_setattr; */
@@ -392,10 +392,14 @@ static PyObject *Constraint_insertKey( BPy_Constraint * self, PyObject * arg )
 		return EXPP_ReturnPyObjError( PyExc_TypeError,
 				"expected a float argument" );
 
-	// constraint_active_func(ob_v, con_v);
+	/* constraint_active_func(ob_v, con_v); */
 	get_constraint_ipo_context( ob, actname );
 	icu= verify_ipocurve((ID *)ob, ID_CO, actname, con->name, CO_ENFORCE);
-
+	
+	if (!icu)
+		return EXPP_ReturnPyObjError( PyExc_RuntimeError,
+				"cannot get a curve from this IPO, may be using libdata" );
+	
 	if( ob->action )
 		insert_vert_ipo( icu, get_action_frame(ob, cfra), con->enforce);
 	else
@@ -466,9 +470,9 @@ static int kinematic_setter( BPy_Constraint *self, int type, PyObject *value )
 	case EXPP_CONSTR_CHAINLEN:
 		return EXPP_setIValueClamped( value, &con->rootbone, 0, 255, 'i' );
 	case EXPP_CONSTR_POSWEIGHT:
-		return EXPP_setFloatClamped( value, &con->weight, 0.01, 1.0 );
+		return EXPP_setFloatClamped( value, &con->weight, 0.01f, 1.0 );
 	case EXPP_CONSTR_ROTWEIGHT:
-		return EXPP_setFloatClamped( value, &con->orientweight, 0.01, 1.0 );
+		return EXPP_setFloatClamped( value, &con->orientweight, 0.01f, 1.0 );
 	case EXPP_CONSTR_ROTATE:
 		return EXPP_setBitfield( value, &con->flag, CONSTRAINT_IK_ROT, 'h' );
 	case EXPP_CONSTR_USETIP:
@@ -531,7 +535,7 @@ static int action_setter( BPy_Constraint *self, int type, PyObject *value )
 		}
 	case EXPP_CONSTR_ACTION: {
 		bAction *act = (( BPy_Action * )value)->action;
-		if( !Action_CheckPyObject( value ) )
+		if( !BPy_Action_Check( value ) )
 			return EXPP_ReturnIntError( PyExc_TypeError, 
 					"expected BPy action argument" );
 		con->act = act;
@@ -724,6 +728,40 @@ static int followpath_setter( BPy_Constraint *self, int type, PyObject *value )
 	}
 }
 
+static PyObject *clampto_getter( BPy_Constraint * self, int type )
+{
+	bClampToConstraint *con = (bClampToConstraint *)(self->con->data);
+
+	switch( type ) {
+	case EXPP_CONSTR_TARGET:
+		return Object_CreatePyObject( con->tar );
+	case EXPP_CONSTR_CLAMP:
+		return PyInt_FromLong( (long)con->flag );
+	default:
+		return EXPP_ReturnPyObjError( PyExc_KeyError, "key not found" );
+	}
+}
+
+static int clampto_setter( BPy_Constraint *self, int type, PyObject *value )
+{
+	bClampToConstraint *con = (bClampToConstraint *)(self->con->data);
+
+	switch( type ) {
+	case EXPP_CONSTR_TARGET: {
+		Object *obj = (( BPy_Object * )value)->object;
+		if( !BPy_Object_Check( value ) )
+			return EXPP_ReturnIntError( PyExc_TypeError, 
+					"expected BPy object argument" );
+		con->tar = obj;
+		return 0;
+		}
+	case EXPP_CONSTR_CLAMP:
+		return EXPP_setIValueRange( value, &con->flag,
+				CLAMPTO_AUTO, CLAMPTO_Z, 'i' );
+	default:
+		return EXPP_ReturnIntError( PyExc_KeyError, "key not found" );
+	}
+}
 static PyObject *locktrack_getter( BPy_Constraint * self, int type )
 {
 	bLockTrackConstraint *con = (bLockTrackConstraint *)(self->con->data);
@@ -877,7 +915,7 @@ static int locatelike_setter( BPy_Constraint *self, int type, PyObject *value )
 		}
 	case EXPP_CONSTR_COPY:
 		return EXPP_setIValueRange( value, &con->flag,
-				0, LOCLIKE_X | LOCLIKE_Y | LOCLIKE_Z, 'i' );
+				0, LOCLIKE_X | LOCLIKE_Y | LOCLIKE_Z | LOCLIKE_X_INVERT | LOCLIKE_Y_INVERT | LOCLIKE_Z_INVERT, 'i' );
 	case EXPP_CONSTR_LOCAL:
 		if( !get_armature( con->tar ) )
 			return EXPP_ReturnIntError( PyExc_RuntimeError,
@@ -935,7 +973,7 @@ static int rotatelike_setter( BPy_Constraint *self, int type, PyObject *value )
 		}
 	case EXPP_CONSTR_COPY:
 		return EXPP_setIValueRange( value, &con->flag,
-				0, LOCLIKE_X | LOCLIKE_Y | LOCLIKE_Z, 'i' );
+				0, LOCLIKE_X | LOCLIKE_Y | LOCLIKE_Z | LOCLIKE_X_INVERT | LOCLIKE_Y_INVERT | LOCLIKE_Z_INVERT, 'i' );
 	case EXPP_CONSTR_LOCAL:
 		if( !get_armature( con->tar ) )
 			return EXPP_ReturnIntError( PyExc_RuntimeError,
@@ -995,7 +1033,7 @@ static int sizelike_setter( BPy_Constraint *self, int type, PyObject *value )
 		}
 	case EXPP_CONSTR_COPY:
 		return EXPP_setIValueRange( value, &con->flag,
-				0, LOCLIKE_X | LOCLIKE_Y | LOCLIKE_Z, 'i' );
+				0, LOCLIKE_X | LOCLIKE_Y | LOCLIKE_Z | LOCLIKE_X_INVERT | LOCLIKE_Y_INVERT | LOCLIKE_Z_INVERT, 'i' );
 #if 0
 	case EXPP_CONSTR_LOCAL:
 		if( !get_armature( con->tar ) )
@@ -1341,6 +1379,8 @@ static PyObject *Constraint_getData( BPy_Constraint * self, PyObject * key )
 			return sizelimit_getter( self, setting );
 		case CONSTRAINT_TYPE_RIGIDBODYJOINT:
 			return rigidbody_getter( self, setting );
+		case CONSTRAINT_TYPE_CLAMPTO:
+			return clampto_getter( self, setting );
 		case CONSTRAINT_TYPE_CHILDOF:	/* Unimplemented */
 		case CONSTRAINT_TYPE_PYTHON:
 		default:
@@ -1405,6 +1445,9 @@ static int Constraint_setData( BPy_Constraint * self, PyObject * key,
 	case CONSTRAINT_TYPE_RIGIDBODYJOINT:
 		result = rigidbody_setter( self, key_int, arg);
 		break;
+	case CONSTRAINT_TYPE_CLAMPTO:
+		result = clampto_setter( self, key_int, arg);
+		break;
 	case CONSTRAINT_TYPE_NULL:
 		return EXPP_ReturnIntError( PyExc_KeyError, "key not found" );
 	case CONSTRAINT_TYPE_CHILDOF:	/* Unimplemented */
@@ -1418,17 +1461,6 @@ static int Constraint_setData( BPy_Constraint * self, PyObject * key,
 	return result;
 }
 
-/*****************************************************************************/
-/* Function:    Constraint_dealloc                                           */
-/* Description: This is a callback function for the BPy_Constraint type. It  */
-/*              destroys data when the object is deleted.                    */
-/*****************************************************************************/
-static void Constraint_dealloc( BPy_Constraint * self )
-{
-	PyObject_DEL( self );
-}
-
-
 /*****************************************************************************/
 /* Function:    Constraint_compare                                           */
 /* Description: This compares 2 constraint python types, == or != only.      */
@@ -1481,16 +1513,6 @@ PyObject *Constraint_CreatePyObject( bPoseChannel *pchan, Object *obj,
 	return ( PyObject * ) pycon;
 }
 
-/*****************************************************************************/
-/* Function:    Constraint_CheckPyObject                                     */
-/* Description: This function returns true when the given PyObject is of the */
-/*              type Constraint. Otherwise it will return false.               */
-/*****************************************************************************/
-int Constraint_CheckPyObject( PyObject * pyobj )
-{
-	return ( pyobj->ob_type == &Constraint_Type );
-}
-
 /*****************************************************************************/
 /* Function:    Constraint_FromPyObject                                      */
 /* Description: This function returns the Blender constraint from the given  */
@@ -1900,6 +1922,8 @@ static PyObject *M_Constraint_TypeDict( void )
 				PyInt_FromLong( CONSTRAINT_TYPE_SIZELIMIT ) );
 		PyConstant_Insert( d, "RIGIDBODYJOINT", 
 				PyInt_FromLong( CONSTRAINT_TYPE_RIGIDBODYJOINT ) );
+		PyConstant_Insert( d, "CLAMPTO", 
+				PyInt_FromLong( CONSTRAINT_TYPE_CLAMPTO ) );
 	}
 	return S;
 }
@@ -1979,6 +2003,21 @@ static PyObject *M_Constraint_SettingsDict( void )
 				PyInt_FromLong( LOCLIKE_Y ) );
 		PyConstant_Insert( d, "COPYZ",
 				PyInt_FromLong( LOCLIKE_Z ) );
+		PyConstant_Insert( d, "COPYXINVERT",
+				PyInt_FromLong( LOCLIKE_X_INVERT ) );
+		PyConstant_Insert( d, "COPYYINVERT",
+				PyInt_FromLong( LOCLIKE_Y_INVERT ) );
+		PyConstant_Insert( d, "COPYZINVERT",
+				PyInt_FromLong( LOCLIKE_Z_INVERT ) );
+				
+		PyConstant_Insert( d, "CLAMPAUTO",
+				PyInt_FromLong( CLAMPTO_AUTO ) );
+		PyConstant_Insert( d, "CLAMPX",
+				PyInt_FromLong( CLAMPTO_X ) );
+		PyConstant_Insert( d, "CLAMPY",
+				PyInt_FromLong( CLAMPTO_Y ) );
+		PyConstant_Insert( d, "CLAMPZ",
+				PyInt_FromLong( CLAMPTO_Z ) );
 
 		PyConstant_Insert( d, "TARGET",
 				PyInt_FromLong( EXPP_CONSTR_TARGET ) );
@@ -2042,6 +2081,8 @@ static PyObject *M_Constraint_SettingsDict( void )
 				PyInt_FromLong( EXPP_CONSTR_COPY ) );
 		PyConstant_Insert( d, "LIMIT",
 				PyInt_FromLong( EXPP_CONSTR_LIMIT ) );
+		PyConstant_Insert( d, "CLAMP",
+				PyInt_FromLong( EXPP_CONSTR_CLAMP ) );
 		
 		PyConstant_Insert( d, "LIMIT_XMIN",
 				PyInt_FromLong( EXPP_CONSTR_LIMXMIN ) );
