@@ -1,5 +1,5 @@
 /* 
- * $Id: Curve.h,v 1.19 2005/07/18 03:50:36 ascotan Exp $
+ * $Id: Curve.h,v 1.21 2007/03/15 01:09:11 campbellbarton Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -43,7 +43,7 @@ extern PyTypeObject Curve_Type;
 /* Python BPy_Curve structure definition */
 typedef struct {
 	PyObject_HEAD		/* required py macro */
-	Curve * curve;
+	Curve * curve;		/* libdata must be second */
 	/* pointer for iterator:  does not point to owned memory */
 	Nurb *iter_pointer;
 } BPy_Curve;
@@ -55,7 +55,6 @@ typedef struct {
 
 PyObject *Curve_Init( void );
 PyObject *Curve_CreatePyObject( struct Curve * curve );
-int Curve_CheckPyObject( PyObject * py_obj );
 struct Curve *Curve_FromPyObject( PyObject * py_obj );
 PyObject *Curve_update( BPy_Curve * self );
 
