@@ -1,5 +1,5 @@
 /* 
- * $Id: Draw.c,v 1.58 2007/01/06 04:10:51 campbellbarton Exp $
+ * $Id: Draw.c,v 1.67 2007/04/28 05:09:09 khughes Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -45,7 +45,9 @@
 #include "BKE_image.h"
 #include "BKE_object.h"
 #include "BKE_main.h"
+#include "BKE_utildefines.h"
 #include "BIF_gl.h"
+#include "BIF_mywindow.h"
 #include "BIF_screen.h"
 #include "BIF_space.h"
 #include "BIF_interface.h"
@@ -70,6 +72,14 @@ current image frame, some images change frame if they are a sequence */
 
 #define ButtonObject_Check(v) ((v)->ob_type == &Button_Type)
 
+#define UI_METHOD_ERRORCHECK \
+	if (check_button_event(&event) == -1)\
+		return EXPP_ReturnPyObjError( PyExc_AttributeError,\
+			"button event argument must be in the range [0, 16382]");\
+	if (callback && !PyCallable_Check(callback))\
+		return EXPP_ReturnPyObjError( PyExc_ValueError,\
+			"callback is not a python function");\
+
 /* pointer to main dictionary defined in Blender.c */
 extern PyObject *g_blenderdict;
 
@@ -98,6 +108,7 @@ static PyObject *Method_Register( PyObject * self, PyObject * args );
 static PyObject *Method_Redraw( PyObject * self, PyObject * args );
 static PyObject *Method_Draw( PyObject * self, PyObject * args );
 static PyObject *Method_Create( PyObject * self, PyObject * args );
+static PyObject *Method_UIBlock( PyObject * self, PyObject * args );
 
 static PyObject *Method_Button( PyObject * self, PyObject * args );
 static PyObject *Method_Menu( PyObject * self, PyObject * args );
@@ -110,8 +121,9 @@ static PyObject *Method_Number( PyObject * self, PyObject * args );
 static PyObject *Method_String( PyObject * self, PyObject * args );
 static PyObject *Method_GetStringWidth( PyObject * self, PyObject * args );
 static PyObject *Method_Text( PyObject * self, PyObject * args );
+static PyObject *Method_Label( PyObject * self, PyObject * args );
+/* by Campbell: */
 static PyObject *Method_PupMenu( PyObject * self, PyObject * args );
-/* next Five by Campbell: */
 static PyObject *Method_PupIntInput( PyObject * self, PyObject * args );
 static PyObject *Method_PupFloatInput( PyObject * self, PyObject * args );
 static PyObject *Method_PupStrInput( PyObject * self, PyObject * args );
@@ -125,8 +137,13 @@ static PyObject *Method_PupBlock( PyObject * self, PyObject * args );
 static uiBlock *Get_uiBlock( void );
 static void py_slider_update( void *butv, void *data2_unused );
 
+/* hack to get 1 block for the UIBlock, only ever 1 at a time */
+static uiBlock *uiblock=NULL;
+
 static char Draw_doc[] = "The Blender.Draw submodule";
 
+static char Method_UIBlock_doc[] = "(drawfunc, x,y) - Popup dialog where buttons can be drawn (expemental)";
+
 static char Method_Register_doc[] =
 	"(draw, event, button) - Register callbacks for windowing\n\n\
 (draw) A function to draw the screen, taking no arguments\n\
@@ -273,6 +290,11 @@ static char Method_Text_doc[] =
 (font) The font size: 'normal' (default), 'small' or 'tiny'.\n\n\
 This function returns the width of the drawn string.";
 
+static char Method_Label_doc[] =
+	"(text, x, y) - Draw a text label onscreen\n\n\
+(text) The text to draw\n\
+(x, y) The lower left coordinate of the lable";
+
 static char Method_PupMenu_doc[] =
 	"(string, maxrow = None) - Display a pop-up menu at the screen.\n\
 The contents of the pop-up are specified through the 'string' argument,\n\
@@ -348,6 +370,7 @@ static char Method_Exit_doc[] = "() - Exit the windowing interface";
 
 static struct PyMethodDef Draw_methods[] = {
 	MethodDef( Create ),
+	MethodDef( UIBlock ),
 	MethodDef( Button ),
 	MethodDef( Toggle ),
 	MethodDef( Menu ),
@@ -359,6 +382,7 @@ static struct PyMethodDef Draw_methods[] = {
 	MethodDef( String ),
 	MethodDef( GetStringWidth ),
 	MethodDef( Text ),
+	MethodDef( Label ),
 	MethodDef( PupMenu ),
 	MethodDef( PupIntInput ),
 	MethodDef( PupFloatInput ),
@@ -512,7 +536,7 @@ static int Button_setattr( PyObject * self, char *name, PyObject * v )
 			char *newstr;
 			unsigned int newlen;
 
-			PyString_AsStringAndSize( v, &newstr, &newlen );
+			PyString_AsStringAndSize( v, &newstr, (int *)&newlen );
 			
 			if (newlen+1> UI_MAX_DRAW_STR)
 				return EXPP_ReturnIntError( PyExc_ValueError, "Error: button string length exceeded max limit (399 chars).");
@@ -711,6 +735,85 @@ void BPY_spacescript_do_pywin_event( SpaceScript * sc, unsigned short event,
 	}
 }
 
+static void exec_but_callback(void *pyobj, void *data)
+{
+	PyObject *result;
+	PyObject * pyvalue;
+	uiBut *but = (uiBut *)data;
+	PyObject *arg = PyTuple_New( 2 );
+	PyObject *callback = (PyObject *)pyobj;
+	
+	double value = ui_get_but_val(but);
+	
+	if (callback==NULL || callback == Py_None)
+		return;
+	
+	/* Button types support
+	case MENU:	
+	case TEX:
+	case TOG:
+	case NUMSLI:
+	case NUM:
+	case COL:
+	case BUT_NORMAL:
+	case BUT */
+	switch (but->type) {
+	case TEX:
+		/*printf("TEX\n");*/
+		pyvalue = PyString_FromString( (char *)but->poin );
+		break;
+	case NUM:
+	case NUMSLI:
+	case TOG:
+	case MENU:
+		if (but->pointype==FLO) {
+			/*printf("FLO\n");*/
+			pyvalue = PyFloat_FromDouble( (float)value );
+		} else if (but->pointype==INT) {
+			/*printf("INT\n");*/
+			pyvalue = PyInt_FromLong( (int)value );
+		} else if (but->pointype==SHO) {
+			/*printf("SHO\n");*/
+			pyvalue = PyInt_FromLong( (short)value );
+		}	
+		break;
+	case COL:
+	case BUT_NORMAL:
+	{
+		float vec[3];
+		VECCOPY(vec, (float *)but->poin);
+		pyvalue = Py_BuildValue("(fff)", vec[0], vec[1], vec[2]);
+		break;
+	}
+	case BUT:
+		pyvalue = Py_None;
+		break;
+	default:
+		pyvalue = Py_None;
+		printf("Error, no button type matched.");
+	}
+	
+	if (uiblock==NULL)
+		PyTuple_SetItem( arg, 0, PyInt_FromLong(but->retval - EXPP_BUTTON_EVENTS_OFFSET) );
+	else
+		PyTuple_SetItem( arg, 0, PyInt_FromLong(but->retval) );
+	
+	PyTuple_SetItem( arg, 1, pyvalue );
+	
+	result = PyObject_CallObject( callback, arg );
+	if (!result) {
+		PyErr_Print(  );
+		error( "Python script error: check console" );
+	}
+	Py_XDECREF( result );
+}
+
+static void set_pycallback(uiBut *ubut, PyObject *callback)
+{
+	if (!callback || !PyCallable_Check(callback)) return;
+	uiButSetFunc(ubut, exec_but_callback, callback, ubut);
+}
+
 static PyObject *Method_Exit( PyObject * self, PyObject * args )
 {
 	SpaceScript *sc;
@@ -721,7 +824,7 @@ static PyObject *Method_Exit( PyObject * self, PyObject * args )
 	if( curarea->spacetype == SPACE_SCRIPT )
 		sc = curarea->spacedata.first;
 	else
-		return EXPP_incr_ret( Py_None );
+		Py_RETURN_NONE;
 
 	if( !PyArg_ParseTuple( args, "" ) )
 		return EXPP_ReturnPyObjError( PyExc_AttributeError,
@@ -734,7 +837,7 @@ static PyObject *Method_Exit( PyObject * self, PyObject * args )
 	/* remove our lock to the current namespace */
 	script->flags &= ~SCRIPT_GUI;
 
-	return EXPP_incr_ret( Py_None );
+	Py_RETURN_NONE;
 }
 
 /* Method_Register (Draw.Register) registers callbacks for drawing, events
@@ -762,7 +865,7 @@ static PyObject *Method_Register( PyObject * self, PyObject * args )
 		newbuttonc = NULL;
 
 	if( !( newdrawc || neweventc || newbuttonc ) )
-		return EXPP_incr_ret( Py_None );
+		Py_RETURN_NONE;
 
 	startspace = curarea->spacetype;
 
@@ -822,7 +925,7 @@ static PyObject *Method_Register( PyObject * self, PyObject * args )
 
 	scrarea_queue_redraw( sc->area );
 
-	return EXPP_incr_ret( Py_None );
+	Py_RETURN_NONE;
 }
 
 static PyObject *Method_Redraw( PyObject * self, PyObject * args )
@@ -838,7 +941,7 @@ static PyObject *Method_Redraw( PyObject * self, PyObject * args )
 	else
 		scrarea_queue_winredraw( curarea );
 
-	return EXPP_incr_ret( Py_None );
+	Py_RETURN_NONE;
 }
 
 static PyObject *Method_Draw( PyObject * self, PyObject * args )
@@ -846,7 +949,7 @@ static PyObject *Method_Draw( PyObject * self, PyObject * args )
 	/*@ If forced drawing is disable queue a redraw event instead */
 	if( EXPP_disable_force_draw ) {
 		scrarea_queue_winredraw( curarea );
-		return EXPP_incr_ret( Py_None );
+		Py_RETURN_NONE;
 	}
 
 	if( !PyArg_ParseTuple( args, "" ) )
@@ -857,7 +960,7 @@ static PyObject *Method_Draw( PyObject * self, PyObject * args )
 
 	screen_swapbuffers(  );
 
-	return EXPP_incr_ret( Py_None );
+	Py_RETURN_NONE;
 }
 
 static PyObject *Method_Create( PyObject * self, PyObject * args )
@@ -903,15 +1006,54 @@ static PyObject *Method_Create( PyObject * self, PyObject * args )
 	return (PyObject*) but;
 }
 
+static PyObject *Method_UIBlock( PyObject * self, PyObject * args )
+{
+	PyObject *val = NULL;
+	PyObject *result = NULL;
+	ListBase listb= {NULL, NULL};
+
+	if ( !PyArg_ParseTuple( args, "O", &val ) || !PyCallable_Check( val ) ) 
+		return EXPP_ReturnPyObjError( PyExc_AttributeError,
+					      "expected 1 python function and 2 ints" );
+
+	if (uiblock)
+		return EXPP_ReturnPyObjError( PyExc_RuntimeError,
+	      "cannot run more then 1 UIBlock at a time" );
+	
+	mywinset(G.curscreen->mainwin);
+	uiblock= uiNewBlock(&listb, "numbuts", UI_EMBOSS, UI_HELV, G.curscreen->mainwin);
+	
+	uiBlockSetFlag(uiblock, UI_BLOCK_LOOP|UI_BLOCK_REDRAW);
+	result = PyObject_CallObject( val, Py_BuildValue( "()" ) );
+
+	if (!result) {
+		PyErr_Print(  );
+		error( "Python script error: check console" );
+	} else {
+		uiBoundsBlock(uiblock, 5);
+		uiDoBlocks(&listb, 0);
+	}
+	uiFreeBlocks(&listb);
+	uiblock = NULL;
+	
+	Py_XDECREF( result );
+	Py_RETURN_NONE;
+}
+
 static uiBlock *Get_uiBlock( void )
 {
 	char butblock[32];
-
+	/* Global, used now for UIBlock */
+	if (uiblock) {
+		return uiblock;
+	}
+	/* Local */
 	sprintf( butblock, "win %d", curarea->win );
 
 	return uiGetBlock( butblock, curarea );
 }
 
+
 /* We restrict the acceptable event numbers to a proper "free" range
  * according to other spaces in Blender.
  * winqread***space() (space events callbacks) use short for events
@@ -923,37 +1065,11 @@ static int check_button_event(int *event) {
 			(*event > EXPP_BUTTON_EVENTS_MAX)) {
 		return -1;
 	}
-	*event += EXPP_BUTTON_EVENTS_OFFSET;
+	if (uiblock==NULL) /* For UIBlock we need non offset UI elements */
+		*event += EXPP_BUTTON_EVENTS_OFFSET;
 	return 0;
 }
 
-static PyObject *Method_Button( PyObject * self, PyObject * args )
-{
-	uiBlock *block;
-	char *name, *tip = NULL;
-	int event;
-	int x, y, w, h;
-
-	if( !PyArg_ParseTuple( args, "siiiii|s", &name, &event,
-			       &x, &y, &w, &h, &tip ) )
-		return EXPP_ReturnPyObjError( PyExc_TypeError,
-					      "expected a string, five ints and optionally another string as arguments" );
-
-	if (check_button_event(&event) == -1)
-		return EXPP_ReturnPyObjError( PyExc_AttributeError,
-			"button event argument must be in the range [0, 16382]");
-
-	block = Get_uiBlock(  );
-
-	if( block )
-		uiDefBut( block, BUT, event, name, (short)x, (short)y, (short)w, (short)h, 0, 0, 0, 0, 0,
-			  tip );
-
-	return EXPP_incr_ret( Py_None );
-}
-
-
-
 static PyObject *Method_BeginAlign( PyObject * self, PyObject * args )
 {
 	uiBlock *block = Get_uiBlock(  );
@@ -961,7 +1077,7 @@ static PyObject *Method_BeginAlign( PyObject * self, PyObject * args )
 	if (block)
 		uiBlockBeginAlign(block);
 	
-	return EXPP_incr_ret( Py_None );
+	Py_RETURN_NONE;
 }
 
 static PyObject *Method_EndAlign( PyObject * self, PyObject * args )
@@ -971,7 +1087,30 @@ static PyObject *Method_EndAlign( PyObject * self, PyObject * args )
 	if (block)
 		uiBlockEndAlign(block);
 	
-	return EXPP_incr_ret( Py_None );
+	Py_RETURN_NONE;
+}
+
+static PyObject *Method_Button( PyObject * self, PyObject * args )
+{
+	uiBlock *block;
+	char *name, *tip = NULL;
+	int event;
+	int x, y, w, h;
+	PyObject *callback=NULL;
+
+	if( !PyArg_ParseTuple( args, "siiiii|sO", &name, &event,
+			       &x, &y, &w, &h, &tip, &callback ) )
+		return EXPP_ReturnPyObjError( PyExc_TypeError,
+					      "expected a string, five ints and optionally string and callback arguments" );
+
+	UI_METHOD_ERRORCHECK;
+
+	block = Get_uiBlock(  );
+	if( block ) {
+		uiBut *ubut = uiDefBut( block, BUT, event, name, (short)x, (short)y, (short)w, (short)h, 0, 0, 0, 0, 0, tip );
+		set_pycallback(ubut, callback);
+	}
+	Py_RETURN_NONE;
 }
 
 static PyObject *Method_Menu( PyObject * self, PyObject * args )
@@ -981,25 +1120,25 @@ static PyObject *Method_Menu( PyObject * self, PyObject * args )
 	int event, def;
 	int x, y, w, h;
 	Button *but;
+	PyObject *callback=NULL;
 
-	if( !PyArg_ParseTuple( args, "siiiiii|s", &name, &event,
-			       &x, &y, &w, &h, &def, &tip ) )
+	if( !PyArg_ParseTuple( args, "siiiiii|sO", &name, &event,
+			       &x, &y, &w, &h, &def, &tip, &callback ) )
 		return EXPP_ReturnPyObjError( PyExc_TypeError,
-					      "expected a string, six ints and optionally another string as arguments" );
-
-	if (check_button_event(&event) == -1)
-		return EXPP_ReturnPyObjError( PyExc_AttributeError,
-			"button event argument must be in the range [0, 16382]");
+					      "expected a string, six ints and optionally string and callback arguments" );
 
+	UI_METHOD_ERRORCHECK;
+	
 	but = newbutton(  );
 	but->type = BINT_TYPE;
 	but->val.asint = def;
 
 	block = Get_uiBlock(  );
-	if( block )
-		uiDefButI( block, MENU, event, name, (short)x, (short)y, (short)w, (short)h,
+	if( block ) {
+		uiBut *ubut = uiDefButI( block, MENU, event, name, (short)x, (short)y, (short)w, (short)h,
 			   &but->val.asint, 0, 0, 0, 0, tip );
-
+		set_pycallback(ubut, callback);
+	}
 	return ( PyObject * ) but;
 }
 
@@ -1010,25 +1149,25 @@ static PyObject *Method_Toggle( PyObject * self, PyObject * args )
 	int event;
 	int x, y, w, h, def;
 	Button *but;
+	PyObject *callback=NULL;
 
-	if( !PyArg_ParseTuple( args, "siiiiii|s", &name, &event,
-			       &x, &y, &w, &h, &def, &tip ) )
+	if( !PyArg_ParseTuple( args, "siiiiii|sO", &name, &event,
+			       &x, &y, &w, &h, &def, &tip, &callback ) )
 		return EXPP_ReturnPyObjError( PyExc_TypeError,
-					      "expected a string, six ints and optionally another string as arguments" );
-
-	if (check_button_event(&event) == -1)
-		return EXPP_ReturnPyObjError( PyExc_AttributeError,
-			"button event argument must be in the range [0, 16382]");
+					      "expected a string, six ints and optionally string and callback arguments" );
 
+	UI_METHOD_ERRORCHECK;
+	
 	but = newbutton(  );
 	but->type = BINT_TYPE;
 	but->val.asint = def;
 
 	block = Get_uiBlock(  );
-	if( block )
-		uiDefButI( block, TOG, event, name, (short)x, (short)y, (short)w, (short)h,
+	if( block ) {
+		uiBut *ubut = uiDefButI( block, TOG, event, name, (short)x, (short)y, (short)w, (short)h,
 			   &but->val.asint, 0, 0, 0, 0, tip );
-
+		set_pycallback(ubut, callback);
+	}
 	return ( PyObject * ) but;
 }
 
@@ -1076,18 +1215,17 @@ static PyObject *Method_Slider( PyObject * self, PyObject * args )
 	int x, y, w, h, realtime = 1;
 	Button *but;
 	PyObject *mino, *maxo, *inio;
+	PyObject *callback=NULL;
 
-	if( !PyArg_ParseTuple( args, "siiiiiOOO|is", &name, &event,
+	if( !PyArg_ParseTuple( args, "siiiiiOOO|isO", &name, &event,
 			       &x, &y, &w, &h, &inio, &mino, &maxo, &realtime,
-			       &tip ) )
+			       &tip, &callback ) )
 		return EXPP_ReturnPyObjError( PyExc_TypeError,
 					      "expected a string, five ints, three PyObjects\n\
-			and optionally another int and string as arguments" );
-
-	if (check_button_event(&event) == -1)
-		return EXPP_ReturnPyObjError( PyExc_AttributeError,
-			"button event argument must be in the range [0, 16382]");
+			and optionally int, string and callback arguments" );
 
+	UI_METHOD_ERRORCHECK;
+	
 	but = newbutton(  );
 
 	if( PyFloat_Check( inio ) ) {
@@ -1107,8 +1245,9 @@ static PyObject *Method_Slider( PyObject * self, PyObject * args )
 					  (short)h, &but->val.asfloat, min, max, 0, 0,
 					  tip );
 			if( realtime )
-				uiButSetFunc( ubut, py_slider_update, ubut,
-					      NULL );
+				uiButSetFunc( ubut, py_slider_update, ubut, NULL );
+			else
+				set_pycallback(ubut, callback);
 		}
 	} else {
 		int ini, min, max;
@@ -1127,8 +1266,9 @@ static PyObject *Method_Slider( PyObject * self, PyObject * args )
 					  (short)h, &but->val.asint, (float)min, (float)max, 0, 0,
 					  tip );
 			if( realtime )
-				uiButSetFunc( ubut, py_slider_update, ubut,
-					      NULL );
+				uiButSetFunc( ubut, py_slider_update, ubut, NULL );
+			else
+				set_pycallback(ubut, callback);
 		}
 	}
 	return ( PyObject * ) but;
@@ -1143,8 +1283,9 @@ static PyObject *Method_Scrollbar( PyObject * self, PyObject * args )
 	Button *but;
 	PyObject *mino, *maxo, *inio;
 	float ini, min, max;
-
-	if( !PyArg_ParseTuple( args, "iiiiiOOO|is", &event, &x, &y, &w, &h,
+	uiBut *ubut;
+	
+	if( !PyArg_ParseTuple( args, "iiiiiOOO|isO", &event, &x, &y, &w, &h,
 			       &inio, &mino, &maxo, &realtime, &tip ) )
 		return EXPP_ReturnPyObjError( PyExc_TypeError,
 			"expected five ints, three PyObjects and optionally\n\
@@ -1156,8 +1297,8 @@ another int and string as arguments" );
 					      "expected numbers for initial, min, and max" );
 
 	if (check_button_event(&event) == -1)
-		return EXPP_ReturnPyObjError( PyExc_AttributeError,
-			"button event argument must be in the range [0, 16382]");
+	return EXPP_ReturnPyObjError( PyExc_AttributeError,
+		"button event argument must be in the range [0, 16382]");
 
 	but = newbutton(  );
 
@@ -1169,33 +1310,24 @@ another int and string as arguments" );
 	ini = (float)PyFloat_AsDouble( inio );
 	min = (float)PyFloat_AsDouble( mino );
 	max = (float)PyFloat_AsDouble( maxo );
+	
+	block = Get_uiBlock(  );
 
-	if( but->type == BFLOAT_TYPE ) {
-		but->val.asfloat = ini;
-		block = Get_uiBlock(  );
-		if( block ) {
-			uiBut *ubut;
+	if( block ) {
+		if( but->type == BFLOAT_TYPE ) {
+			but->val.asfloat = ini;
 			ubut = uiDefButF( block, SCROLL, event, "", (short)x, (short)y, (short)w, (short)h,
-					  &but->val.asfloat, min, max, 0, 0,
-					  tip );
+					  &but->val.asfloat, min, max, 0, 0, tip );
 			if( realtime )
-				uiButSetFunc( ubut, py_slider_update, ubut,
-					      NULL );
-		}
-	} else {
-		but->val.asint = (int)ini;
-		block = Get_uiBlock(  );
-		if( block ) {
-			uiBut *ubut;
+				uiButSetFunc( ubut, py_slider_update, ubut, NULL );
+		} else {
+			but->val.asint = (int)ini;
 			ubut = uiDefButI( block, SCROLL, event, "", (short)x, (short)y, (short)w, (short)h,
-					  &but->val.asint, min, max, 0, 0,
-					  tip );
+					  &but->val.asint, min, max, 0, 0, tip );
 			if( realtime )
-				uiButSetFunc( ubut, py_slider_update, ubut,
-					      NULL );
+				uiButSetFunc( ubut, py_slider_update, ubut, NULL );
 		}
 	}
-
 	return ( PyObject * ) but;
 }
 
@@ -1209,15 +1341,14 @@ static PyObject *Method_ColorPicker( PyObject * self, PyObject * args )
 	float col[3];
 	int event;
 	short x, y, w, h;
+	PyObject *callback=NULL;
 	
-	if( !PyArg_ParseTuple( args, "ihhhhO!|s", &event,
-			       &x, &y, &w, &h, &PyTuple_Type, &inio, &tip ) )
+	if( !PyArg_ParseTuple( args, "ihhhhO!|sO", &event,
+			       &x, &y, &w, &h, &PyTuple_Type, &inio, &tip, &callback ) )
  		return EXPP_ReturnPyObjError( PyExc_TypeError,
-					      "expected five ints, one tuple and optionally a string as arguments." );
+					      "expected five ints, one tuple and optionally string and callback arguments" );
  
- 	if (check_button_event(&event) == -1)
-		return EXPP_ReturnPyObjError( PyExc_ValueError,
- 			"button event argument must be in the range [0, 16382]");
+	UI_METHOD_ERRORCHECK;
  
 	if ( !PyArg_ParseTuple( inio, "fff", col, col+1, col+2 ) )
 		return EXPP_ReturnPyObjError( PyExc_ValueError, USAGE_ERROR);
@@ -1241,6 +1372,7 @@ static PyObject *Method_ColorPicker( PyObject * self, PyObject * args )
 	if( block ) {
 		uiBut *ubut;
 		ubut = uiDefButF( block, COL, event, "", x, y, w, h, but->val.asvec, 0, 0, 0, 0, tip);
+		set_pycallback(ubut, callback);
 	}
 
  	return ( PyObject * ) but;
@@ -1258,15 +1390,14 @@ static PyObject *Method_Normal( PyObject * self, PyObject * args )
 	float nor[3];
 	int event;
 	short x, y, w, h;
+	PyObject *callback=NULL;
 	
-	if( !PyArg_ParseTuple( args, "ihhhhO!|s", &event,
-			       &x, &y, &w, &h, &PyTuple_Type, &inio, &tip ) )
+	if( !PyArg_ParseTuple( args, "ihhhhO!|sO", &event,
+			       &x, &y, &w, &h, &PyTuple_Type, &inio, &tip, &callback ) )
  		return EXPP_ReturnPyObjError( PyExc_TypeError,
-					      "expected five ints, one tuple and optionally a string as arguments." );
+					      "expected five ints, one tuple and optionally string and callback arguments" );
  
- 	if (check_button_event(&event) == -1)
-		return EXPP_ReturnPyObjError( PyExc_ValueError,
- 			"button event argument must be in the range [0, 16382]");
+	UI_METHOD_ERRORCHECK;
  
 	if ( !PyArg_ParseTuple( inio, "fff", nor, nor+1, nor+2 ) )
 		return EXPP_ReturnPyObjError( PyExc_ValueError, USAGE_ERROR);
@@ -1285,6 +1416,7 @@ static PyObject *Method_Normal( PyObject * self, PyObject * args )
 	if( block ) {
 		uiBut *ubut;
 		ubut = uiDefButF( block, BUT_NORMAL, event, "", x, y, w, h, but->val.asvec, 0.0f, 1.0f, 0, 0, tip);
+		set_pycallback(ubut, callback);
 	}
 	
  	return ( PyObject * ) but;
@@ -1298,19 +1430,20 @@ static PyObject *Method_Number( PyObject * self, PyObject * args )
 	int x, y, w, h;
 	Button *but;
 	PyObject *mino, *maxo, *inio;
-
-	if( !PyArg_ParseTuple( args, "siiiiiOOO|s", &name, &event,
-			       &x, &y, &w, &h, &inio, &mino, &maxo, &tip ) )
+	PyObject *callback=NULL;
+	uiBut *ubut= NULL;
+	
+	if( !PyArg_ParseTuple( args, "siiiiiOOO|sO", &name, &event,
+			       &x, &y, &w, &h, &inio, &mino, &maxo, &tip, &callback ) )
 		return EXPP_ReturnPyObjError( PyExc_TypeError,
 					      "expected a string, five ints, three PyObjects and\n\
-			optionally another string as arguments" );
+			optionally string and callback arguments" );
 
-	if (check_button_event(&event) == -1)
-		return EXPP_ReturnPyObjError( PyExc_AttributeError,
-			"button event argument must be in the range [0, 16382]");
+	UI_METHOD_ERRORCHECK;
 
 	but = newbutton(  );
-
+	block = Get_uiBlock(  );
+	
 	if( PyFloat_Check( inio ) ) {
 		float ini, min, max, range, precission=0;
 
@@ -1318,7 +1451,7 @@ static PyObject *Method_Number( PyObject * self, PyObject * args )
 		min = (float)PyFloat_AsDouble( mino );
 		max = (float)PyFloat_AsDouble( maxo );
 		
-		range= fabs(max-min); /* Click step will be a 10th of the range. */
+		range= (float)fabs(max-min); /* Click step will be a 10th of the range. */
 		if (!range) range= 1.0f; /* avoid any odd errors */
 		
 		/* set the precission to display*/
@@ -1330,9 +1463,9 @@ static PyObject *Method_Number( PyObject * self, PyObject * args )
 		but->type = BFLOAT_TYPE;
 		but->val.asfloat = ini;
 
-		block = Get_uiBlock(  );
+		
 		if( block )
-			uiDefButF( block, NUM, event, name, (short)x, (short)y, (short)w, (short)h,
+			ubut= uiDefButF( block, NUM, event, name, (short)x, (short)y, (short)w, (short)h,
 				   &but->val.asfloat, min, max, 10*range, precission, tip );
 	} else {
 		int ini, min, max;
@@ -1344,12 +1477,13 @@ static PyObject *Method_Number( PyObject * self, PyObject * args )
 		but->type = BINT_TYPE;
 		but->val.asint = ini;
 
-		block = Get_uiBlock(  );
 		if( block )
-			uiDefButI( block, NUM, event, name, (short)x, (short)y, (short)w, (short)h,
+			ubut= uiDefButI( block, NUM, event, name, (short)x, (short)y, (short)w, (short)h,
 				   &but->val.asint, (float)min, (float)max, 0, 0, tip );
 	}
-
+	
+	if (ubut) set_pycallback(ubut, callback);
+	
 	return ( PyObject * ) but;
 }
 
@@ -1361,16 +1495,15 @@ static PyObject *Method_String( PyObject * self, PyObject * args )
 	int event;
 	int x, y, w, h, len, real_len = 0;
 	Button *but;
+	PyObject *callback=NULL;
 
-	if( !PyArg_ParseTuple( args, "siiiiisi|s", &info_arg, &event,
-			&x, &y, &w, &h, &newstr, &len, &tip ) )
+	if( !PyArg_ParseTuple( args, "siiiiisi|sO", &info_arg, &event,
+			&x, &y, &w, &h, &newstr, &len, &tip, &callback ) )
 		return EXPP_ReturnPyObjError( PyExc_TypeError,
 			"expected a string, five ints, a string, an int and\n\
-	optionally another string as arguments" );
+	optionally string and callback arguments" );
 
-	if (check_button_event(&event) == -1)
-		return EXPP_ReturnPyObjError( PyExc_AttributeError,
-			"button event argument must be in the range [0, 16382]");
+	UI_METHOD_ERRORCHECK;
 
 	if (len > (UI_MAX_DRAW_STR - 1))
 		return EXPP_ReturnPyObjError( PyExc_ValueError,
@@ -1391,10 +1524,11 @@ static PyObject *Method_String( PyObject * self, PyObject * args )
 	else info_str = info_arg;
 
 	block = Get_uiBlock(  );
-	if( block )
-		uiDefBut( block, TEX, event, info_str, (short)x, (short)y, (short)w, (short)h,
+	if( block ) {
+		uiBut *ubut = uiDefBut( block, TEX, event, info_str, (short)x, (short)y, (short)w, (short)h,
 			  but->val.asstr, 0, (float)len, 0, 0, tip );
-
+		set_pycallback(ubut, callback);
+	}
 	return ( PyObject * ) but;
 }
 
@@ -1459,6 +1593,23 @@ static PyObject *Method_Text( PyObject * self, PyObject * args )
 	return PyInt_FromLong( BMF_GetStringWidth( font, text ) );
 }
 
+static PyObject *Method_Label( PyObject * self, PyObject * args )
+{
+	uiBlock *block;
+	char *text;
+	int x, y, w, h;
+
+	if( !PyArg_ParseTuple( args, "siiii", &text, &x, &y, &w, &h ) )
+		return EXPP_ReturnPyObjError( PyExc_TypeError,
+			"expected a string and four ints" );
+
+	block = Get_uiBlock(  );
+	uiDefBut(block, LABEL, 0, text, x, y, w, h, 0, 0, 0, 0, 0, "");
+	
+	Py_RETURN_NONE;
+}
+
+
 static PyObject *Method_PupMenu( PyObject * self, PyObject * args )
 {
 	char *text;
@@ -1709,7 +1860,7 @@ static PyObject *Method_Image( PyObject * self, PyObject * args )
 			"expected a Blender.Image and 2 floats, and " \
 			"optionally 2 floats and 4 ints as arguments" );
 	/* check that the first PyObject is actually a Blender.Image */
-	if( !Image_CheckPyObject( pyObjImage ) )
+	if( !BPy_Image_Check( pyObjImage ) )
 		return EXPP_ReturnPyObjError( PyExc_TypeError,
 			"expected a Blender.Image and 2 floats, and " \
 			"optionally 2 floats and 4 ints as arguments" );
@@ -1813,7 +1964,8 @@ PyObject *Draw_Init( void )
 {
 	PyObject *submodule, *dict;
 
-	Button_Type.ob_type = &PyType_Type;
+	if( PyType_Ready( &Button_Type) < 0)
+		Py_RETURN_NONE;
 
 	submodule = Py_InitModule3( "Blender.Draw", Draw_methods, Draw_doc );
 
