@@ -1,5 +1,5 @@
 /*
- * $Id: Effect.c,v 1.26 2006/08/17 21:39:49 khughes Exp $
+ * $Id: Effect.c,v 1.28 2007/04/18 15:30:33 jiri Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -204,7 +204,6 @@ static PyObject *Effect_oldsetDefvec( BPy_Effect * self, PyObject * a );
 /*****************************************************************************/
 /* Python Effect_Type callback function prototypes:                           */
 /*****************************************************************************/
-static void Effect_dealloc( BPy_Effect * msh );
 static PyObject *Effect_repr( void );
 
 /*****************************************************************************/
@@ -476,7 +475,7 @@ PyTypeObject Effect_Type = {
 
 	/* Methods to implement standard operations */
 
-	( destructor ) Effect_dealloc,/* destructor tp_dealloc; */
+	NULL,						/* destructor tp_dealloc; */
 	NULL,                       /* printfunc tp_print; */
 	NULL,                       /* getattrfunc tp_getattr; */
 	NULL,                       /* setattrfunc tp_setattr; */
@@ -796,7 +795,7 @@ static int Effect_setStype( BPy_Effect * self, PyObject * args )
 	if( !PyArg_Parse( args, "h", &param ) )
 		return EXPP_ReturnIntError( PyExc_TypeError,
 						"expected an int as argument" );
-	self->effect->stype |= 1-param;
+	self->effect->stype = param;
 	return 0;
 }
 
@@ -1610,16 +1609,6 @@ static PyObject *Effect_getParticlesLoc( BPy_Effect * self )
 	return list;	
 }
 
-/*****************************************************************************/
-/* Function:    Effect_dealloc                                               */
-/* Description: This is a callback function for the BPy_Effect type. It is   */
-/*              the destructor function.                                     */
-/*****************************************************************************/
-static void Effect_dealloc( BPy_Effect * self )
-{
-	PyObject_DEL( self );
-}
-
 /*****************************************************************************/
 /* Function:    Effect_repr                                                  */
 /* Description: This is a callback function for the BPy_Effect type. It      */
