@@ -36,6 +36,7 @@
 #include "BKE_library.h" /* for rename_id() */
 #include "BLI_blenlib.h"
 #include "gen_utils.h"
+#include "gen_library.h"
 
 #include "BKE_main.h" /* so we can access G.main->vfont.first */
 #include "DNA_space_types.h" /* for FILE_MAXDIR only */
@@ -79,7 +80,6 @@ static PyMethodDef BPy_Font_methods[] = {
 };
 
 /*--------------- Python TypeFont callback function prototypes----------*/
-static void Font_dealloc( BPy_Font * font );
 static int Font_compare( BPy_Font * a1, BPy_Font * a2 );
 static PyObject *Font_repr( BPy_Font * font );
 
@@ -174,23 +174,10 @@ PyObject *M_Font_Load( PyObject * self, PyObject * args )
 	return ( PyObject * ) py_font;
 }
 
-/*--------------- Python BPy_Font methods---------------------------*/
+/*--------------- Python BPy_Font getsetattr funcs ---------------------*/
 
-/*--------------- BPy_Font.getName()--------------------------------*/
-static PyObject *Font_getName( BPy_Font * self )
-{
-	PyObject *attr = NULL;
 
-	if( self->font )
-		attr = PyString_FromString( self->font->id.name+2 );
-	if( attr )
-		return attr;
-
-	return ( EXPP_ReturnPyObjError( PyExc_RuntimeError,
-					"couldn't get Font.name attribute" ) );
-}
-
-/*--------------- BPy_Font.getFilename()--------------------------------*/
+/*--------------- BPy_Font.filename-------------------------------------*/
 static PyObject *Font_getFilename( BPy_Font * self )
 {
 	PyObject *attr = NULL;
@@ -204,26 +191,6 @@ static PyObject *Font_getFilename( BPy_Font * self )
 					"couldn't get Font.filename attribute" ) );
 }
 
-static int Font_setName( BPy_Font * self, PyObject * value )
-{
-	char *name = NULL;
-	
-	if( !(self->font) )
-		return EXPP_ReturnIntError( PyExc_RuntimeError,
-					      "Blender Font was deleted!" );
-	
-	name = PyString_AsString ( value );
-	if( !name )
-		return EXPP_ReturnIntError( PyExc_TypeError,
-					      "expected string argument" );
-
-	rename_id( &self->font->id, name );
-
-	return 0;
-}
-
-
-
 static int Font_setFilename( BPy_Font * self, PyObject * value )
 {
 	char *name = NULL;
@@ -241,7 +208,6 @@ static int Font_setFilename( BPy_Font * self, PyObject * value )
 	return 0;
 }
 
-
 /*--------------- BPy_Font.pack()---------------------------------*/
 static PyObject *Font_pack( BPy_Font * self ) 
 {
@@ -278,27 +244,15 @@ static PyObject *Font_getPacked( BPy_Font * self )
 		return EXPP_incr_ret_False();
 }
 
-static PyObject *Font_getUsers( BPy_Font* self )
-{
-	return PyInt_FromLong( self->font->id.us );
-}
-
 /*****************************************************************************/
 /* Python attributes get/set structure:                                      */
 /*****************************************************************************/
 static PyGetSetDef BPy_Font_getseters[] = {
-	{"name",
-	 (getter)Font_getName, (setter)Font_setName,
-	 "Font name",
-	 NULL},
+	GENERIC_LIB_GETSETATTR,
 	{"filename",
 	 (getter)Font_getFilename, (setter)Font_setFilename,
 	 "Font filepath",
 	 NULL},
-	{"users",
-	 (getter)Font_getUsers, (setter)NULL,
-	 "Number of font users",
-	 NULL},
 	{"packed",
 	 (getter)Font_getPacked, (setter)NULL,
 	 "Packed status",
@@ -319,7 +273,7 @@ PyTypeObject Font_Type = {
 
 	/* Methods to implement standard operations */
 
-	( destructor ) Font_dealloc,/* destructor tp_dealloc; */
+	NULL,						/* destructor tp_dealloc; */
 	NULL,                       /* printfunc tp_print; */
 	NULL,                       /* getattrfunc tp_getattr; */
 	NULL,                       /* setattrfunc tp_setattr; */
@@ -334,7 +288,7 @@ PyTypeObject Font_Type = {
 
 	/* More standard operations (here for binary compatibility) */
 
-	NULL,                       /* hashfunc tp_hash; */
+	( hashfunc ) GenericLib_hash,	/* hashfunc tp_hash; */
 	NULL,                       /* ternaryfunc tp_call; */
 	NULL,                       /* reprfunc tp_str; */
 	NULL,                       /* getattrofunc tp_getattro; */
@@ -413,12 +367,6 @@ PyObject *Font_Init( void )
 /*--------------- Font module internal callbacks-----------------*/
 /*---------------BPy_Font internal callbacks/methods-------------*/
 
-//--------------- dealloc------------------------------------------
-static void Font_dealloc( BPy_Font * self )
-{
-	PyObject_DEL( self );
-}
-
 /*--------------- repr---------------------------------------------*/
 static PyObject *Font_repr( BPy_Font * self )
 {
@@ -447,12 +395,6 @@ PyObject *Font_CreatePyObject( struct VFont * font )
 	return ( ( PyObject * ) blen_font );
 }
 
-/*--------------- Font_CheckPyObject--------------------------------*/
-int Font_CheckPyObject( PyObject * py_obj )
-{
-	return ( py_obj->ob_type == &Font_Type );
-}
-
 /*--------------- Font_FromPyObject---------------------------------*/
 struct VFont *Font_FromPyObject( PyObject * py_obj )
 {
