@@ -1,5 +1,5 @@
 /*
- * $Id: Font.h,v 1.3 2006/04/16 15:28:50 stiv Exp $
+ * $Id: Font.h,v 1.5 2007/03/15 01:09:11 campbellbarton Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -40,12 +40,11 @@ extern PyTypeObject Font_Type;
 
 typedef struct {
 	PyObject_HEAD		/* required py macro */
-	VFont * font;
+	VFont * font;		/* libdata must be second */
 } BPy_Font;
 
 /*------------------------------visible prototypes----------------------*/
 PyObject *Font_CreatePyObject( struct VFont * font );
-int Font_CheckPyObject( PyObject * py_obj );
 struct VFont *Font_FromPyObject( PyObject * py_obj );
 PyObject *Font_Init( void );
 
