@@ -1,5 +1,5 @@
 /* 
- * $Id: Geometry.c,v 1.7 2006/12/01 17:53:03 stiv Exp $
+ * $Id: Geometry.c,v 1.12 2007/04/27 20:48:23 campbellbarton Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -39,14 +39,14 @@
 
 /* Used for PolyFill */
 #include "BKE_displist.h"
-#include "MEM_guardedalloc.h" 
+#include "MEM_guardedalloc.h"
 #include "BLI_blenlib.h"
 
 /* needed for EXPP_ReturnPyObjError and EXPP_check_sequence_consistency */
 #include "gen_utils.h"
-
-//#include "util.h" /* MIN2 and MAX2 */ 
+ 
 #include "BKE_utildefines.h"
+#include "BLI_boxpack2d.h"
 
 #define SWAP_FLOAT(a,b,tmp) tmp=a; a=b; b=tmp
 #define eul 0.000001
@@ -54,15 +54,21 @@
 /*-- forward declarations -- */
 static PyObject *M_Geometry_PolyFill( PyObject * self, PyObject * args );
 static PyObject *M_Geometry_LineIntersect2D( PyObject * self, PyObject * args );
+static PyObject *M_Geometry_PointInTriangle2D( PyObject * self, PyObject * args );
+static PyObject *M_Geometry_BoxPack2D( PyObject * self, PyObject * args );
 
 /*-------------------------DOC STRINGS ---------------------------*/
 static char M_Geometry_doc[] = "The Blender Geometry module\n\n";
 static char M_Geometry_PolyFill_doc[] = "(veclist_list) - takes a list of polylines (each point a vector) and returns the point indicies for a polyline filled with triangles";
 static char M_Geometry_LineIntersect2D_doc[] = "(lineA_p1, lineA_p2, lineB_p1, lineB_p2) - takes 2 lines (as 4 vectors) and returns a vector for their point of intersection or None";
+static char M_Geometry_PointInTriangle2D_doc[] = "(pt, tri_p1, tri_p2, tri_p3) - takes 4 vectors, one is the point and the next 3 define the triabgle, only the x and y are used from the vectors";
+static char M_Geometry_BoxPack2D_doc[] = "";
 /*-----------------------METHOD DEFINITIONS ----------------------*/
 struct PyMethodDef M_Geometry_methods[] = {
 	{"PolyFill", ( PyCFunction ) M_Geometry_PolyFill, METH_VARARGS, M_Geometry_PolyFill_doc},
 	{"LineIntersect2D", ( PyCFunction ) M_Geometry_LineIntersect2D, METH_VARARGS, M_Geometry_LineIntersect2D_doc},
+	{"PointInTriangle2D", ( PyCFunction ) M_Geometry_PointInTriangle2D, METH_VARARGS, M_Geometry_PointInTriangle2D_doc},
+	{"BoxPack2D", ( PyCFunction ) M_Geometry_BoxPack2D, METH_VARARGS, M_Geometry_BoxPack2D_doc},
 	{NULL, NULL, 0, NULL}
 };
 /*----------------------------MODULE INIT-------------------------*/
@@ -225,7 +231,7 @@ static PyObject *M_Geometry_LineIntersect2D( PyObject * self, PyObject * args )
 			return newVectorObject(newvec, 2, Py_NEW);
 		}
 		
-		yi = ((b1y / fabs(b1x - b2x)) * fabs(b2x - a1x)) + ((b2y / fabs(b1x - b2x)) * fabs(b1x - a1x));
+		yi = (float)(((b1y / fabs(b1x - b2x)) * fabs(b2x - a1x)) + ((b2y / fabs(b1x - b2x)) * fabs(b1x - a1x)));
 		
 		if (yi > MAX2(a1y, a2y)) {/* New point above seg1's vert line */
 			Py_RETURN_NONE;
@@ -241,7 +247,7 @@ static PyObject *M_Geometry_LineIntersect2D( PyObject * self, PyObject * args )
 		}
 		
 		/* Can skip vert line check for seg 2 since its covered above. */
-		xi = ((b1x / fabs(b1y - b2y)) * fabs(b2y - a1y)) + ((b2x / fabs(b1y - b2y)) * fabs(b1y - a1y));
+		xi = (float)(((b1x / fabs(b1y - b2y)) * fabs(b2y - a1y)) + ((b2x / fabs(b1y - b2y)) * fabs(b1y - a1y)));
 		if (xi > MAX2(a1x, a2x)) { /* New point right of hoz line1's */
 			Py_RETURN_NONE;
 		} else if (xi < MIN2(a1x, a2x)) { /*New point left of seg1's hoz line */
@@ -270,3 +276,116 @@ static PyObject *M_Geometry_LineIntersect2D( PyObject * self, PyObject * args )
 	}
 	Py_RETURN_NONE;
 }
+
+#define SIDE_OF_LINE(pa,pb,pp)	((pa[0]-pp[0])*(pb[1]-pp[1]))-((pb[0]-pp[0])*(pa[1]-pp[1]))
+#define POINT_IN_TRI(p0,p1,p2,p3)	((SIDE_OF_LINE(p1,p2,p0)>=0) && (SIDE_OF_LINE(p2,p3,p0)>=0) && (SIDE_OF_LINE(p3,p1,p0)>=0))
+
+static PyObject *M_Geometry_PointInTriangle2D( PyObject * self, PyObject * args )
+{
+	VectorObject *pt_vec, *tri_p1, *tri_p2, *tri_p3;
+	
+	if( !PyArg_ParseTuple ( args, "O!O!O!O!",
+	  &vector_Type, &pt_vec,
+	  &vector_Type, &tri_p1,
+	  &vector_Type, &tri_p2,
+	  &vector_Type, &tri_p3)
+	)
+		return ( EXPP_ReturnPyObjError
+			 ( PyExc_TypeError, "expected 4 vector types\n" ) );
+	
+	if POINT_IN_TRI(pt_vec->vec, tri_p1->vec, tri_p2->vec, tri_p3->vec)
+		Py_RETURN_TRUE;
+	else
+		Py_RETURN_FALSE;
+}
+
+int boxPack_FromPyObject(PyObject * value, boxPack **boxarray )
+{
+	int len, i;
+	PyObject *list_item, *item_1, *item_2;
+	boxPack *box;
+	
+	
+	/* Error checking must alredy be done */
+	if( !PyList_Check( value ) )
+		return EXPP_ReturnIntError( PyExc_TypeError,
+				"can only back a list of [x,y,x,w]" );
+	
+	len = PyList_Size( value );
+	
+	(*boxarray) = MEM_mallocN( len*sizeof(boxPack), "boxPack box");
+	
+	
+	for( i = 0; i < len; i++ ) {
+		list_item = PyList_GET_ITEM( value, i );
+		if( !PyList_Check( list_item ) || PyList_Size( list_item ) < 4 ) {
+			MEM_freeN(*boxarray);
+			return EXPP_ReturnIntError( PyExc_TypeError,
+					"can only back a list of [x,y,x,w]" );
+		}
+		
+		box = (*boxarray)+i;
+		
+		item_1 = PyList_GET_ITEM(list_item, 2);
+		item_2 = PyList_GET_ITEM(list_item, 3);
+		
+		if (!PyNumber_Check(item_1) || !PyNumber_Check(item_2)) {
+			MEM_freeN(*boxarray);
+			return EXPP_ReturnIntError( PyExc_TypeError,
+					"can only back a list of 2d boxes [x,y,x,w]" );
+		}
+		
+		box->w =  (float)PyFloat_AsDouble( item_1 );
+		box->h =  (float)PyFloat_AsDouble( item_2 );
+		box->index = i;
+		/* verts will be added later */
+	}
+	return 0;
+}
+
+void boxPack_ToPyObject(PyObject * value, boxPack **boxarray)
+{
+	int len, i;
+	PyObject *list_item;
+	boxPack *box;
+	
+	len = PyList_Size( value );
+	
+	for( i = 0; i < len; i++ ) {
+		box = (*boxarray)+i;
+		list_item = PyList_GET_ITEM( value, box->index );
+		PyList_SET_ITEM( list_item, 0, PyFloat_FromDouble( box->x ));
+		PyList_SET_ITEM( list_item, 1, PyFloat_FromDouble( box->y ));
+	}
+	MEM_freeN(*boxarray);
+}
+
+
+static PyObject *M_Geometry_BoxPack2D( PyObject * self, PyObject * args )
+{
+	PyObject *boxlist; /*return this list of tri's */
+	boxPack *boxarray;
+	float tot_width, tot_height;
+	int len;
+	int error;
+	
+	if(!PyArg_ParseTuple ( args, "O", &boxlist) || !PyList_Check(boxlist)) {
+		return EXPP_ReturnPyObjError( PyExc_TypeError,
+					      "expected a sequence of boxes [[x,y,w,h], ... ]" );
+	}
+	
+	len = PyList_Size( boxlist );
+	
+	if (!len)
+		return Py_BuildValue( "ff", 0.0, 0.0);
+	
+	error = boxPack_FromPyObject(boxlist, &boxarray);
+	if (error!=0)	return NULL;
+	
+	/* Non Python function */
+	boxPack2D(boxarray, len, &tot_width, &tot_height);
+	
+	boxPack_ToPyObject(boxlist, &boxarray);
+	
+	return Py_BuildValue( "ff", tot_width, tot_height);
+}
