@@ -28,10 +28,10 @@
  * ***** END GPL/BL DUAL LICENSE BLOCK *****
  */
 
-#include "MEM_guardedalloc.h"
-
 #include "Group.h" /* This must come first */
 
+#include "MEM_guardedalloc.h"
+
 #include "DNA_group_types.h"
 #include "DNA_scene_types.h" /* for Base */
 
@@ -47,6 +47,7 @@
 #include "blendef.h"
 #include "Object.h"
 #include "gen_utils.h"
+#include "gen_library.h"
 
 /* checks for the group being removed */
 #define GROUP_DEL_CHECK_PY(bpy_group) if (!(bpy_group->group)) return ( EXPP_ReturnPyObjError( PyExc_RuntimeError, "Group has been removed" ) )
@@ -85,6 +86,8 @@ static PyMethodDef BPy_Group_methods[] = {
 	/* name, method, flags, doc */
 	{"__copy__", ( PyCFunction ) BPy_Group_copy, METH_VARARGS,
 	 "() - Return a copy of the group containing the same objects."},
+	{"copy", ( PyCFunction ) BPy_Group_copy, METH_VARARGS,
+	 "() - Return a copy of the group containing the same objects."},
 	{NULL, NULL, 0, NULL}
 };
 
@@ -97,7 +100,7 @@ static PyObject *BPy_Group_copy( BPy_Group * self )
 	
 	GROUP_DEL_CHECK_PY(self);
 	
-	bl_group= add_group();
+	bl_group= add_group( self->group->id.name + 2 );
 	
 	if( bl_group )		/* now create the wrapper grp in Python */
 		py_group = ( BPy_Group * ) Group_CreatePyObject( bl_group );
@@ -105,8 +108,6 @@ static PyObject *BPy_Group_copy( BPy_Group * self )
 		return ( EXPP_ReturnPyObjError( PyExc_RuntimeError,
 						"couldn't create Group Data in Blender" ) );
 	
-	rename_id( &bl_group->id, self->group->id.name + 2 );
-	
 	bl_group->id.us = 1;
 	
 	/* Now add the objects to the group */
@@ -207,69 +208,11 @@ static int Group_setObjects( BPy_Group * self, PyObject * args )
 /*****************************************************************************/
 /* PythonTypeObject callback function prototypes			 */
 /*****************************************************************************/
-static void Group_dealloc( BPy_Group * obj );
 static PyObject *Group_repr( BPy_Group * obj );
 static int Group_compare( BPy_Group * a, BPy_Group * b );
 
 /*****************************************************************************/
-/* Python BPy_Group methods:                                                  */
-/*****************************************************************************/
-static int Group_setName( BPy_Group * self, PyObject * value )
-{
-	char *name = NULL;
-	
-	GROUP_DEL_CHECK_INT(self);
-	
-	name = PyString_AsString ( value );
-	if( !name )
-		return EXPP_ReturnIntError( PyExc_TypeError,
-					      "expected string argument" );
-
-	rename_id( &self->group->id, name );
-
-	return 0;
-}
-
-
-static PyObject *Group_getName( BPy_Group * self )
-{
-	PyObject *attr;
-	GROUP_DEL_CHECK_PY(self);
-	
-	attr = PyString_FromString( self->group->id.name + 2 );
-
-	if( attr )
-		return attr;
-
-	return ( EXPP_ReturnPyObjError( PyExc_RuntimeError,
-					"couldn't get Group.name attribute" ) );
-}
-
-static PyObject *Group_getUsers( BPy_Group * self )
-{
-	GROUP_DEL_CHECK_PY(self);
-	return PyInt_FromLong( self->group->id.us );
-}
-
-static PyObject *Group_getFakeUser( BPy_Group * self )
-{
-	GROUP_DEL_CHECK_PY(self);
-	if (self->group->id.flag & LIB_FAKEUSER)
-		Py_RETURN_TRUE;
-	else
-		Py_RETURN_FALSE;
-}
-
-static int Group_setFakeUser( BPy_Group * self, PyObject * value )
-{
-	GROUP_DEL_CHECK_INT(self);
-	return SetIdFakeUser(&self->group->id, value);
-}
-
-
-
-/*****************************************************************************/
-/* Python BPy_Group methods:                                                  */
+/* Python BPy_Group getsetattr funcs:                                        */
 /*****************************************************************************/
 static int Group_setLayers( BPy_Group * self, PyObject * value )
 {
@@ -302,21 +245,10 @@ static PyObject *Group_getLayers( BPy_Group * self )
 /* Python attributes get/set structure:                                      */
 /*****************************************************************************/
 static PyGetSetDef BPy_Group_getseters[] = {
-	{"name",
-	 (getter)Group_getName, (setter)Group_setName,
-	 "Group name",
-	 NULL},
-	{"users",
-	 (getter)Group_getUsers, (setter)NULL,
-	 "Number of group users",
-	 NULL},
-	{"fakeUser",
-	 (getter)Group_getFakeUser, (setter)Group_setFakeUser,
-	 "Groups fake user state",
-	 NULL},
+	GENERIC_LIB_GETSETATTR,
 	{"layers",
 	 (getter)Group_getLayers, (setter)Group_setLayers,
-	 "Number of group users",
+	 "layer mask for this group",
 	 NULL},
 	{"objects",
 	 (getter)Group_getObjects, (setter)Group_setObjects,
@@ -338,7 +270,7 @@ PyTypeObject Group_Type = {
 
 	/* Methods to implement standard operations */
 
-	( destructor ) Group_dealloc,/* destructor tp_dealloc; */
+	NULL,						/* destructor tp_dealloc; */
 	NULL,                       /* printfunc tp_print; */
 	NULL,                       /* getattrfunc tp_getattr; */
 	NULL,                       /* setattrfunc tp_setattr; */
@@ -353,7 +285,7 @@ PyTypeObject Group_Type = {
 
 	/* More standard operations (here for binary compatibility) */
 
-	NULL,                       /* hashfunc tp_hash; */
+	( hashfunc ) GenericLib_hash,	/* hashfunc tp_hash; */
 	NULL,                       /* ternaryfunc tp_call; */
 	NULL,                       /* reprfunc tp_str; */
 	NULL,                       /* getattrofunc tp_getattro; */
@@ -421,7 +353,6 @@ PyTypeObject Group_Type = {
 PyObject *M_Group_New( PyObject * self, PyObject * args )
 {
 	char *name = "Group";
-	char buf[21];
 	BPy_Group *py_group;	/* for Group Data object wrapper in Python */
 	struct Group *bl_group;
 	
@@ -429,7 +360,7 @@ PyObject *M_Group_New( PyObject * self, PyObject * args )
 		return EXPP_ReturnPyObjError( PyExc_TypeError,
 				"string expected as argument or nothing" );
 	
-	bl_group= add_group();
+	bl_group= add_group( name );
 	
 	if( bl_group )		/* now create the wrapper grp in Python */
 		py_group = ( BPy_Group * ) Group_CreatePyObject( bl_group );
@@ -437,12 +368,6 @@ PyObject *M_Group_New( PyObject * self, PyObject * args )
 		return ( EXPP_ReturnPyObjError( PyExc_RuntimeError,
 						"couldn't create Group Data in Blender" ) );
 	
-	
-	if( strcmp( name, "Group" ) != 0 ) {
-		PyOS_snprintf( buf, sizeof( buf ), "%s", name );
-		rename_id( &bl_group->id, buf );
-	}
-	
 	bl_group->id.us = 1;
 	
 	return ( PyObject * ) py_group;
@@ -588,16 +513,6 @@ PyObject *Group_CreatePyObject( struct Group * grp )
 	return ( ( PyObject * ) pygrp );
 }
 
-/*****************************************************************************/
-/* Function:	Group_CheckPyObject					 */
-/* Description: This function returns true when the given PyObject is of the */
-/*		type Group. Otherwise it will return false.		 */
-/*****************************************************************************/
-int Group_CheckPyObject( PyObject * py_grp)
-{
-	return ( py_grp->ob_type == &Group_Type );
-}
-
 /*****************************************************************************/
 /* Function:	Group_FromPyObject					 */
 /* Description: This function returns the Blender group from the given	 */
@@ -611,40 +526,6 @@ Group *Group_FromPyObject( PyObject * py_grp )
 	return ( blen_grp->group );
 }
 
-/*****************************************************************************/
-/* Description: Returns the object with the name specified by the argument  */
-/*		name. Note that the calling function has to remove the first */
-/*		two characters of the object name. These two characters	   */
-/*		specify the type of the object (OB, ME, WO, ...)	 */
-/*		The function will return NULL when no object with the given  */
-/*		name is found.						 */
-/*****************************************************************************/
-Group *GetGroupByName( char *name )
-{
-	Group *grp_iter;
-
-	grp_iter = G.main->group.first;
-	while( grp_iter ) {
-		if( StringEqual( name, GetIdName( &( grp_iter->id ) ) ) ) {
-			return ( grp_iter );
-		}
-		grp_iter = grp_iter->id.next;
-	}
-
-	/* There is no object with the given name */
-	return ( NULL );
-}
-
-/*****************************************************************************/
-/* Function:	Group_dealloc						 */
-/* Description: This is a callback function for the BlenObject type. It is  */
-/*		the destructor function.				 */
-/*****************************************************************************/
-static void Group_dealloc( BPy_Group * grp )
-{
-	PyObject_DEL( grp );
-}
-
 /*****************************************************************************/
 /* Function:	Group_compare						 */
 /* Description: This is a callback function for the BPy_Group type. It	 */
@@ -803,13 +684,6 @@ static PyObject *GroupObSeq_link( BPy_GroupObSeq * self, PyObject *args )
 	Py_RETURN_NONE;
 }
 
-static PyObject *GroupObSeq_append( BPy_GroupObSeq * self, PyObject *args )
-{
-	printf("WARNING: grp.objects.append() has been deprecated, use grp.objects.link() instead.\nBlender releases after 2.43 will not support .append()");
-	return GroupObSeq_link(self, args);
-}
-
-
 static PyObject *GroupObSeq_unlink( BPy_GroupObSeq * self, PyObject *args )
 {
 	PyObject *pyobj;
@@ -838,25 +712,11 @@ static PyObject *GroupObSeq_unlink( BPy_GroupObSeq * self, PyObject *args )
 	Py_RETURN_NONE;
 }
 
-static PyObject *GroupObSeq_remove( BPy_GroupObSeq * self, PyObject *args )
-{
-	printf("WARNING: grp.objects.remove() has been deprecated, use grp.objects.unlink() instead.\nBlender releases after 2.43 will not support .remove()");
-	return GroupObSeq_unlink(self, args);
-}
-
-
 static struct PyMethodDef BPy_GroupObSeq_methods[] = {
 	{"link", (PyCFunction)GroupObSeq_link, METH_VARARGS,
 		"make the object a part of this group"},
 	{"unlink", (PyCFunction)GroupObSeq_unlink, METH_VARARGS,
 		"unlink an object from this group"},
-		
-	/* Deprecated! use the above functions */
-	{"append", (PyCFunction)GroupObSeq_append, METH_VARARGS,
-		"make the object a part of this group - deprecated, use link"},
-	{"remove", (PyCFunction)GroupObSeq_remove, METH_VARARGS,
-		"remove an object from this group - deprecated, use unlink"},
-		
 	{NULL, NULL, 0, NULL}	
 };
 
