@@ -1,5 +1,5 @@
 /* 
- * $Id: Group.h,v 1.4 2006/08/27 16:27:33 campbellbarton Exp $
+ * $Id: Group.h,v 1.8 2007/03/15 01:09:11 campbellbarton Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -47,8 +47,8 @@ extern PyTypeObject GroupObSeq_Type;
 /* Python BPy_Group structure definition.                                  */
 /*****************************************************************************/
 typedef struct {
-	PyObject_HEAD /* required python macro   */
-	struct Group *group; /* Blender group type, can be NULL when unlinked */
+	PyObject_HEAD
+	struct Group *group; 
 } BPy_Group;
 
 
@@ -62,7 +62,5 @@ typedef struct {
 PyObject *Group_Init( void );
 PyObject *Group_CreatePyObject( struct Group *group );
 Group *Group_FromPyObject( PyObject * py_obj );
-int Group_CheckPyObject( PyObject * py_obj );
-Group *GetGroupByName( char *name );
 
 #endif				/* EXPP_GROUP_H */
