@@ -1,5 +1,5 @@
 /* 
- * $Id: Image.c,v 1.62 2006/12/27 05:04:17 campbellbarton Exp $
+ * $Id: Image.c,v 1.76 2007/05/03 15:10:44 bebraw Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -44,6 +44,7 @@
 #include "IMB_imbuf_types.h"	/* for the IB_rect define */
 #include "BIF_gl.h"
 #include "gen_utils.h"
+#include "gen_library.h"
 #include "BKE_packedFile.h"
 #include "DNA_packedFile_types.h"
 #include "BKE_icons.h"
@@ -62,6 +63,16 @@ current image frame, some images change frame if they are a sequence */
 #define EXPP_IMAGE_REP_MAX 16
 
 
+enum img_consts {
+	EXPP_IMAGE_ATTR_XREP = 0,
+	EXPP_IMAGE_ATTR_YREP,
+	EXPP_IMAGE_ATTR_START,
+	EXPP_IMAGE_ATTR_END,
+	EXPP_IMAGE_ATTR_SPEED,
+	EXPP_IMAGE_ATTR_BINDCODE,
+	EXPP_IMAGE_ATTR_SOURCE,
+};
+
 /************************/
 /*** The Image Module ***/
 /************************/
@@ -78,7 +89,6 @@ static PyObject *M_Image_Load( PyObject * self, PyObject * args );
 /*****************************************************************************/
 /* Python BPy_Image methods declarations:	 */
 /*****************************************************************************/
-static PyObject *Image_getName( BPy_Image * self );
 static PyObject *Image_getFilename( BPy_Image * self );
 static PyObject *Image_getSize( BPy_Image * self );
 static PyObject *Image_getDepth( BPy_Image * self );
@@ -88,8 +98,8 @@ static PyObject *Image_getBindCode( BPy_Image * self );
 static PyObject *Image_getStart( BPy_Image * self );
 static PyObject *Image_getEnd( BPy_Image * self );
 static PyObject *Image_getSpeed( BPy_Image * self );
-static PyObject *Image_setName( BPy_Image * self, PyObject * args );
-static PyObject *Image_setFilename( BPy_Image * self, PyObject * args );
+static int Image_setFilename( BPy_Image * self, PyObject * args );
+static PyObject *Image_oldsetFilename( BPy_Image * self, PyObject * args );
 static PyObject *Image_setXRep( BPy_Image * self, PyObject * args );
 static PyObject *Image_setYRep( BPy_Image * self, PyObject * args );
 static PyObject *Image_setStart( BPy_Image * self, PyObject * args );
@@ -127,7 +137,7 @@ static PyMethodDef BPy_Image_methods[] = {
 	 "() - Get maximum x & y coordinates of current image as [x, y]"},
 	{"getMinXY", ( PyCFunction ) Image_getMinXY, METH_NOARGS,
 	 "() - Get minimun x & y coordinates of image as [x, y]"},
-	{"getName", ( PyCFunction ) Image_getName, METH_NOARGS,
+	{"getName", ( PyCFunction ) GenericLib_getName, METH_NOARGS,
 	 "() - Return Image object name"},
 	{"getFilename", ( PyCFunction ) Image_getFilename, METH_NOARGS,
 	 "() - Return Image object filename"},
@@ -155,9 +165,9 @@ static PyMethodDef BPy_Image_methods[] = {
 	{"glFree", ( PyCFunction ) Image_glFree, METH_NOARGS,
 	 "() - Free the image data from OpenGL texture memory only,\n\
 		see also image.glLoad()."},
-	{"setName", ( PyCFunction ) Image_setName, METH_VARARGS,
+	{"setName", ( PyCFunction ) GenericLib_setName_with_method, METH_VARARGS,
 	 "(str) - Change Image object name"},
-	{"setFilename", ( PyCFunction ) Image_setFilename, METH_VARARGS,
+	{"setFilename", ( PyCFunction ) Image_oldsetFilename, METH_VARARGS,
 	 "(str) - Change Image file name"},
 	{"setXRep", ( PyCFunction ) Image_setXRep, METH_VARARGS,
 	 "(int) - Change Image object x repetition value"},
@@ -225,6 +235,7 @@ static PyObject *M_Image_New( PyObject * self, PyObject * args)
 {
 	int width, height, depth;
 	char *name;
+	float color[] = {0, 0, 0, 1};
 	Image *image;
 	if( !PyArg_ParseTuple( args, "siii", &name, &width, &height, &depth ) )
 		return ( EXPP_ReturnPyObjError( PyExc_TypeError,
@@ -232,7 +243,7 @@ static PyObject *M_Image_New( PyObject * self, PyObject * args)
 	if (width > 5000 || height > 5000 || width < 1 || height < 1)
 		return ( EXPP_ReturnPyObjError( PyExc_TypeError,
 					"Image width and height must be between 1 and 5000" ) );
-	image = BKE_add_image_size(width, height, name, 0);
+	image = BKE_add_image_size(width, height, name, 0, color);
 	if( !image )
 		return ( EXPP_ReturnPyObjError( PyExc_MemoryError,
 						"couldn't create PyObject Image_Type" ) );
@@ -532,6 +543,7 @@ static PyObject *Image_setPixelF( BPy_Image * self, PyObject * args )
 	pixel[index + 2] = ( char ) ( p[2] * 255.0 );
 	pixel[index + 3] = ( char ) ( p[3] * 255.0 );
 
+	ibuf->userflags |= IB_BITMAPDIRTY;
 	Py_RETURN_NONE;
 }
 
@@ -587,7 +599,8 @@ static PyObject *Image_setPixelI( BPy_Image * self, PyObject * args )
 	pixel[index + 1] = ( char ) p[1];
 	pixel[index + 2] = ( char ) p[2];
 	pixel[index + 3] = ( char ) p[3];
-
+	
+	ibuf->userflags |= IB_BITMAPDIRTY;
 	Py_RETURN_NONE;
 }
 
@@ -661,24 +674,35 @@ static PyObject *Image_unpack( BPy_Image * self, PyObject * args )
 static PyObject *Image_pack( BPy_Image * self )
 {
 	Image *image = self->image;	
-	char expandpath[FILE_MAXDIR + FILE_MAXFILE];
-	BLI_strncpy(expandpath, image->name, FILE_MAXDIR+FILE_MAXFILE);
-	BLI_convertstringcode(expandpath, G.sce, 1);
-
-	if (image->packedfile )
+	ImBuf *ibuf = BKE_image_get_ibuf(image, NULL);
+	
+	if( !ibuf || !ibuf->rect )	/* didn't work */
 		return EXPP_ReturnPyObjError( PyExc_RuntimeError,
-					      "image alredy packed" );
-	if (!BLI_exists(expandpath))
-		return EXPP_ReturnPyObjError( PyExc_RuntimeError,
-					      "image path does not exist" );
-		
-	image->packedfile = newPackedFile(image->name);
+				      "couldn't load image data in Blender" );
+	
+	if (image->packedfile ) { /* RePack? */
+		if (ibuf->userflags & IB_BITMAPDIRTY)
+			BKE_image_memorypack(image);
+	} else { /* Pack for the first time */
+		if (ibuf && (ibuf->userflags & IB_BITMAPDIRTY))
+			BKE_image_memorypack(image);
+		else
+			image->packedfile = newPackedFile(image->name);
+	}
 	Py_RETURN_NONE;
 }
 
 
 static PyObject *Image_makeCurrent( BPy_Image * self )
 {
+#if 0	/* add back in when bpy becomes "official" */
+	static char warning = 1;
+	if( warning ) {
+		printf("image.makeCurrent() deprecated!\n\t use 'bpy.images.active = image instead'\n");
+		--warning;
+	}
+#endif
+	
 	if (!G.sima)
 		Py_RETURN_FALSE;
 	
@@ -700,69 +724,46 @@ static PyObject *Image_save( BPy_Image * self )
 	Py_RETURN_NONE;		/*  normal return, image saved */
 }
 
+static PyObject *M_Image_SourceDict( void )
+{
+	PyObject *Dict = PyConstant_New(  );
+	if( Dict ) {
+		BPy_constant *d = ( BPy_constant * ) Dict;
+		PyConstant_Insert(d, "STILL", PyInt_FromLong(IMA_SRC_FILE));
+		PyConstant_Insert(d, "MOVIE", PyInt_FromLong(IMA_SRC_MOVIE));
+		PyConstant_Insert(d, "SEQUENCE", PyInt_FromLong(IMA_SRC_SEQUENCE));
+		PyConstant_Insert(d, "GENERATED", PyInt_FromLong(IMA_SRC_GENERATED));
+	}
+	return Dict;
+}
+
 /*****************************************************************************/
 /* Function:		Image_Init	 */
 /*****************************************************************************/
 PyObject *Image_Init( void )
 {
 	PyObject *submodule;
+	PyObject *Sources = M_Image_SourceDict( );
 
-	Image_Type.ob_type = &PyType_Type;
+	if( PyType_Ready( &Image_Type ) < 0 )
+		return NULL;
 
 	submodule =
 		Py_InitModule3( "Blender.Image", M_Image_methods,
 				M_Image_doc );
 
-	return ( submodule );
+	if( Sources )
+		PyModule_AddObject( submodule, "Sources", Sources );
+
+	return submodule;
 }
 
 /*****************************************************************************/
 /* Python Image_Type callback function prototypes:	 */
 /*****************************************************************************/
-static void Image_dealloc( BPy_Image * self );
-static int Image_setAttr( BPy_Image * self, char *name, PyObject * v );
 static int Image_compare( BPy_Image * a, BPy_Image * b );
-static PyObject *Image_getAttr( BPy_Image * self, char *name );
 static PyObject *Image_repr( BPy_Image * self );
 
-/*****************************************************************************/
-/* Python Image_Type structure definition:   */
-/*****************************************************************************/
-PyTypeObject Image_Type = {
-	PyObject_HEAD_INIT( NULL ) /*     required macro. ( no comma needed )  */ 
-	0,	/* ob_size */
-	"Blender Image",	/* tp_name */
-	sizeof( BPy_Image ),	/* tp_basicsize */
-	0,			/* tp_itemsize */
-	/* methods */
-	( destructor ) Image_dealloc,	/* tp_dealloc */
-	0,			/* tp_print */
-	( getattrfunc ) Image_getAttr,	/* tp_getattr */
-	( setattrfunc ) Image_setAttr,	/* tp_setattr */
-	( cmpfunc ) Image_compare,	/* tp_compare */
-	( reprfunc ) Image_repr,	/* tp_repr */
-	0,			/* tp_as_number */
-	0,			/* tp_as_sequence */
-	0,			/* tp_as_mapping */
-	0,			/* tp_as_hash */
-	0, 0, 0, 0, 0, 0,
-	0,			/* tp_doc */
-	0, 0, 0, 0, 0, 0,
-	BPy_Image_methods,	/* tp_methods */
-	0,			/* tp_members */
-	0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,	/* up to tp_del, to avoid a warning */
-};
-
-/*****************************************************************************/
-/* Function:		Image_dealloc		 */
-/* Description: This is a callback function for the BPy_Image type. It is  */
-/*		the destructor function.	 */
-/*****************************************************************************/
-static void Image_dealloc( BPy_Image * self )
-{
-	PyObject_DEL( self );
-}
-
 /*****************************************************************************/
 /* Function:		Image_CreatePyObject	 */
 /* Description: This function will create a new BPy_Image from an existing  */
@@ -781,16 +782,6 @@ PyObject *Image_CreatePyObject( Image * image )
 	return ( PyObject * ) py_img;
 }
 
-/*****************************************************************************/
-/* Function:		Image_CheckPyObject	 */
-/* Description: This function returns true when the given PyObject is of the */
-/*		type Image. Otherwise it will return false.	 */
-/*****************************************************************************/
-int Image_CheckPyObject( PyObject * pyobj )
-{
-	return ( pyobj->ob_type == &Image_Type );
-}
-
 /*****************************************************************************/
 /* Function:	Image_FromPyObject	 */
 /* Description: Returns the Blender Image associated with this object  	 */
@@ -800,20 +791,6 @@ Image *Image_FromPyObject( PyObject * pyobj )
 	return ( ( BPy_Image * ) pyobj )->image;
 }
 
-/*****************************************************************************/
-/* Python BPy_Image methods:		 */
-/*****************************************************************************/
-static PyObject *Image_getName( BPy_Image * self )
-{
-	PyObject *attr = PyString_FromString( self->image->id.name + 2 );
-
-	if( attr )
-		return attr;
-
-	return ( EXPP_ReturnPyObjError( PyExc_RuntimeError,
-					"couldn't get Image.name attribute" ) );
-}
-
 static PyObject *Image_getFilename( BPy_Image * self )
 {
 	PyObject *attr = PyString_FromString( self->image->name );
@@ -839,9 +816,7 @@ static PyObject *Image_getSize( BPy_Image * self )
 	
 	PyList_SetItem( attr, 0, PyInt_FromLong(ibuf->x));
 	PyList_SetItem( attr, 1, PyInt_FromLong(ibuf->y));
-	return attr;	
-
-
+	return attr;
 }
 
 static PyObject *Image_getDepth( BPy_Image * self )
@@ -986,38 +961,28 @@ static PyObject *Image_glLoad( BPy_Image * self )
 	return PyLong_FromUnsignedLong( image->bindcode );
 }
 
-static PyObject *Image_setName( BPy_Image * self, PyObject * args )
+static int Image_setFilename( BPy_Image * self, PyObject * value )
 {
 	char *name;
 
-	if( !PyArg_ParseTuple( args, "s", &name ) )
-		return ( EXPP_ReturnPyObjError( PyExc_TypeError,
-						"expected string argument" ) );
+	name = PyString_AsString(value);
 
-	rename_id( &self->image->id, name );
-
-	Py_RETURN_NONE;
-}
-
-static PyObject *Image_setFilename( BPy_Image * self, PyObject * args )
-{
-	char *name;
-	int namelen = 0;
-
-	/* max len is FILE_MAXDIR == 160, FILE_MAXFILE == 80 chars like done in DNA_image_types.h */
-
-	if( !PyArg_ParseTuple( args, "s#", &name, &namelen ) )
-		return ( EXPP_ReturnPyObjError( PyExc_TypeError,
+	if( !name )
+		return ( EXPP_ReturnIntError( PyExc_TypeError,
 						"expected a string argument" ) );
 
-	if( namelen >= FILE_MAXDIR + FILE_MAXFILE )
-		return ( EXPP_ReturnPyObjError( PyExc_TypeError,
+	/* max len is FILE_MAXDIR == 160, FILE_MAXFILE == 80 chars like done in DNA_image_types.h */
+	if( strlen(name) >= FILE_MAXDIR + FILE_MAXFILE )
+		return ( EXPP_ReturnIntError( PyExc_TypeError,
 						"string argument is limited to 240 chars at most" ) );
 
-	PyOS_snprintf( self->image->name, (FILE_MAXDIR + FILE_MAXFILE) * sizeof( char ), "%s",
-		       name );
+	strcpy( self->image->name, name );
+	return 0;
+}
 
-	Py_RETURN_NONE;
+static PyObject *Image_oldsetFilename( BPy_Image * self, PyObject * args )
+{
+	return EXPP_setterWrapper( (void *)self, args, (setter)Image_setFilename );
 }
 
 static PyObject *Image_setXRep( BPy_Image * self, PyObject * args )
@@ -1107,162 +1072,6 @@ static PyObject *Image_setSpeed( BPy_Image * self, PyObject * args )
 	Py_RETURN_NONE;
 }
 
-/*****************************************************************************/
-/* Function:		Image_getAttr		 */
-/* Description: This is a callback function for the BPy_Image type. It is */
-/*		the function that accesses BPy_Image member variables and   */
-/*		methods.	 */
-/*****************************************************************************/
-static PyObject *Image_getAttr( BPy_Image * self, char *name )
-{
-	PyObject *attr = Py_None;
-
-	if( strcmp( name, "name" ) == 0 )
-		attr = PyString_FromString( self->image->id.name + 2 );
-	else if ( strcmp( name, "properties" ) == 0 )
-		return BPy_Wrap_IDProperty( (ID*)self->image, IDP_GetProperties((ID*)self->image, 1), NULL );
-	else if( strcmp( name, "filename" ) == 0 )
-		attr = PyString_FromString( self->image->name );
-	else if( strcmp( name, "size" ) == 0 )
-		return Image_getSize( self );
-	else if( strcmp( name, "depth" ) == 0 )
-		return Image_getDepth( self );
-	else if( strcmp( name, "xrep" ) == 0 )
-		attr = PyInt_FromLong( self->image->xrep );
-	else if( strcmp( name, "yrep" ) == 0 )
-		attr = PyInt_FromLong( self->image->yrep );
-	else if( strcmp( name, "start" ) == 0 )
-		attr = PyInt_FromLong( self->image->twsta );
-	else if( strcmp( name, "end" ) == 0 )
-		attr = PyInt_FromLong( self->image->twend );
-	else if( strcmp( name, "speed" ) == 0 )
-		attr = PyInt_FromLong( self->image->animspeed );
-	else if( strcmp( name, "packed" ) == 0 ) {
-		if (self->image->packedfile) attr = Py_True;
-		else attr = Py_False;
-		EXPP_incr_ret(attr);
-	} else if( strcmp( name, "has_data" ) == 0 ) {
-		if (self->image->ibufs.first) attr = Py_True;
-		else attr = Py_False;
-		EXPP_incr_ret(attr);
-	} else if( strcmp( name, "fields" ) == 0 ) {
-		if (self->image->flag & IMA_FIELDS) attr = Py_True;
-		else attr = Py_False;
-		EXPP_incr_ret(attr);
-	} else if( strcmp( name, "fields_odd" ) == 0 ) {
-		if (self->image->flag & IMA_STD_FIELD) attr = Py_True;
-		else attr = Py_False;
-		EXPP_incr_ret(attr);
-	} else if( strcmp( name, "antialias" ) == 0 ) {
-		if (self->image->flag & IMA_ANTIALI) attr = Py_True;
-		else attr = Py_False;
-		EXPP_incr_ret(attr);
-		
-	} else if( strcmp( name, "bindcode" ) == 0 )
-		attr = PyInt_FromLong( self->image->bindcode );
-	else if( strcmp( name, "users" ) == 0 )
-		attr = PyInt_FromLong( self->image->id.us );
-	else if( strcmp( name, "__members__" ) == 0 )
-		attr = Py_BuildValue( "[s,s,s,s,s,s,s,s,s,s,s,s,s,s,s,s]",
-				      "name", "filename", "size", "depth",
-				      "xrep", "yrep", "start", "end",
-				      "speed", "packed", "has_data"
-				      "fields", "odd", "antialias",
-					  "bindcode", "users" );
-
-	if( !attr )
-		return ( EXPP_ReturnPyObjError( PyExc_MemoryError,
-						"couldn't create PyObject" ) );
-
-	if( attr != Py_None )
-		return attr;	/* attribute found, return its value */
-
-	/* not an attribute, search the methods table */
-	return Py_FindMethod( BPy_Image_methods, ( PyObject * ) self, name );
-}
-
-/*****************************************************************************/
-/* Function:		Image_setAttr		 */
-/* Description: This is a callback function for the BPy_Image type. It is the*/
-/*		function that changes Image object members values. If this  */
-/*		data is linked to a Blender Image, it also gets updated.  */
-/*****************************************************************************/
-static int Image_setAttr( BPy_Image * self, char *name, PyObject * value )
-{
-	PyObject *valtuple;
-	PyObject *error = NULL;
-
-/* We're playing a trick on the Python API users here.	Even if they use
- * Image.member = val instead of Image.setMember(value), we end up using the
- * function anyway, since it already has error checking, clamps to the right
- * interval and updates the Blender Image structure when necessary. */
-
-	valtuple = Py_BuildValue( "(O)", value );	/*the set* functions expect a tuple */
-
-	if( !valtuple )
-		return EXPP_ReturnIntError( PyExc_MemoryError,
-					    "ImageSetAttr: couldn't create PyTuple" );
-
-	if( strcmp( name, "name" ) == 0 )
-		error = Image_setName( self, valtuple );
-	else if( strcmp( name, "filename" ) == 0 )
-		error = Image_setFilename( self, valtuple );
-	else if( strcmp( name, "xrep" ) == 0 )
-		error = Image_setXRep( self, valtuple );
-	else if( strcmp( name, "yrep" ) == 0 )
-		error = Image_setYRep( self, valtuple );
-	else if( strcmp( name, "start" ) == 0 )
-		error = Image_setStart( self, valtuple );
-	else if( strcmp( name, "end" ) == 0 )
-		error = Image_setEnd( self, valtuple );
-	else if( strcmp( name, "speed" ) == 0 )
-		error = Image_setSpeed( self, valtuple );
-	
-	else if( strcmp( name, "fields" ) == 0 ) {
-		int param = PyObject_IsTrue( value );
-		if( param == -1 )
-			return EXPP_ReturnIntError( PyExc_TypeError,
-				"expected true/false argument" );
-		
-		if (param) self->image->flag |= IMA_FIELDS;
-		else self->image->flag &= ~IMA_FIELDS;
-		Py_INCREF( Py_None );
-		error = Py_None;
-	} else if( strcmp( name, "fields_odd" ) == 0 ) {
-		int param = PyObject_IsTrue( value );
-		if( param == -1 )
-			return EXPP_ReturnIntError( PyExc_TypeError,
-				"expected true/false argument" );
-		
-		if (param) self->image->flag |= IMA_STD_FIELD;
-		else self->image->flag &= ~IMA_STD_FIELD;
-		Py_INCREF( Py_None );
-		error = Py_None;
-	} else if( strcmp( name, "antialias" ) == 0 ) {
-		int param = PyObject_IsTrue( value );
-		if( param == -1 )
-			return EXPP_ReturnIntError( PyExc_TypeError,
-				"expected true/false argument" );
-		
-		if (param) self->image->flag |= IMA_ANTIALI;
-		else self->image->flag &= ~IMA_ANTIALI;
-		Py_INCREF( Py_None );
-		error = Py_None;
-	} else {			/* Error: no such member in the Image object structure */
-		/*Py_DECREF( value ); borrowed ref, no need to decref */
-		Py_DECREF( valtuple );
-		return ( EXPP_ReturnIntError( PyExc_KeyError,
-					      "attribute not found or immutable" ) );
-	}
-
-	Py_DECREF( valtuple );
-
-	if( error != Py_None )
-		return -1;
-
-	Py_DECREF( Py_None );	/* incref'ed by the called set* function */
-	return 0;		/* normal exit */
-}
 
 /*****************************************************************************/
 /* Function:	Image_compare			 */
@@ -1287,3 +1096,299 @@ static PyObject *Image_repr( BPy_Image * self )
 	return PyString_FromFormat( "[Image \"%s\"]",
 				    self->image->id.name + 2 );
 }
+
+static PyObject *Image_getPacked(BPy_Image *self, void *closure)
+{
+	if (self->image->packedfile)
+		Py_RETURN_TRUE;
+	else
+		Py_RETURN_FALSE;
+}
+
+static PyObject *Image_hasData(BPy_Image *self, void *closure)
+{
+	if (self->image->ibufs.first)
+		Py_RETURN_TRUE;
+	else
+		Py_RETURN_FALSE;
+}
+
+static PyObject *Image_getFlag(BPy_Image *self, void *flag)
+{
+	if (self->image->flag & (int)flag)
+		Py_RETURN_TRUE;
+	else
+		Py_RETURN_FALSE;
+		
+}
+
+static PyObject *Image_getFlagTpage(BPy_Image *self, void *flag)
+{
+	if (self->image->tpageflag & (int)flag)
+		Py_RETURN_TRUE;
+	else
+		Py_RETURN_FALSE;
+		
+}
+
+static int Image_setSource( BPy_Image *self, PyObject *args)
+{
+    PyObject* integer = PyNumber_Int( args );
+	short value;
+
+	if( !integer )
+		return EXPP_ReturnIntError( PyExc_TypeError,
+				"expected integer argument" );
+
+	value = ( short )PyInt_AS_LONG( integer );
+	Py_DECREF( integer );
+
+	if( value < IMA_SRC_FILE || value > IMA_SRC_GENERATED )
+		return EXPP_ReturnIntError( PyExc_ValueError,
+				"expected integer argument in range 1-4" );
+
+	self->image->source = value;
+	return 0;
+}
+
+static int Image_setFlag(BPy_Image *self, PyObject *value, void *flag)
+{
+	if ( PyObject_IsTrue(value) )
+		self->image->flag |= (int)flag;
+	else
+		self->image->flag &= ~(int)flag;
+	return 0;
+}
+
+static int Image_setFlagTpage(BPy_Image *self, PyObject *value, void *flag)
+{
+	if ( PyObject_IsTrue(value) )
+		self->image->tpageflag |= (int)flag;
+	else
+		self->image->tpageflag &= ~(int)flag;
+	return 0;
+}
+
+/*
+ * get integer attributes
+ */
+static PyObject *getIntAttr( BPy_Image *self, void *type )
+{
+	PyObject *attr = NULL;
+	int param;
+	struct Image *image = self->image;
+
+	switch( (int)type ) {
+	case EXPP_IMAGE_ATTR_XREP:
+		param = image->xrep;
+		break;
+	case EXPP_IMAGE_ATTR_YREP:
+		param = image->xrep;
+		break;
+	case EXPP_IMAGE_ATTR_START:
+		param = image->twsta;
+		break;	
+	case EXPP_IMAGE_ATTR_END:
+		param = image->twend;
+		break;
+	case EXPP_IMAGE_ATTR_SPEED:
+		param = image->animspeed;
+		break;
+	case EXPP_IMAGE_ATTR_BINDCODE:
+		param = image->bindcode;
+		break;
+	case EXPP_IMAGE_ATTR_SOURCE:
+		param = image->source;
+		break;
+	default:
+		return EXPP_ReturnPyObjError( PyExc_RuntimeError,
+				"undefined type in getIntAttr" );
+	}
+
+	attr = PyInt_FromLong( param );
+	
+	if( attr )
+		return attr;
+
+	return EXPP_ReturnPyObjError( PyExc_MemoryError,
+				"PyInt_FromLong() failed!" );
+}
+
+
+/*
+ * set integer attributes which require clamping
+ */
+
+static int setIntAttrClamp( BPy_Image *self, PyObject *value, void *type )
+{
+	void *param;
+	struct Image *image = self->image;
+	int min, max, size;
+
+	switch( (int)type ) {
+	case EXPP_IMAGE_ATTR_XREP:
+		min = EXPP_IMAGE_REP_MIN;
+		max = EXPP_IMAGE_REP_MAX;
+		size = 'h';
+		param = (void *)&image->xrep;
+		break;
+	case EXPP_IMAGE_ATTR_YREP:
+		min = EXPP_IMAGE_REP_MIN;
+		max = EXPP_IMAGE_REP_MAX;
+		size = 'h';
+		param = (void *)&image->yrep;
+		break;
+	case EXPP_IMAGE_ATTR_START:
+		min = 0;
+		max = 128;
+		size = 'h';
+		param = (void *)&image->twsta;
+		break;
+	case EXPP_IMAGE_ATTR_END:
+		min = 0;
+		max = 128;
+		size = 'h';
+		param = (void *)&image->twend;
+		break;
+	case EXPP_IMAGE_ATTR_SPEED:
+		min = 0;
+		max = 100;
+		size = 'h';
+		param = (void *)&image->animspeed;
+		break;
+	default:
+		return EXPP_ReturnIntError( PyExc_RuntimeError,
+				"undefined type in setIntAttrClamp");
+	}
+	return EXPP_setIValueClamped( value, param, min, max, size );
+}
+
+/*****************************************************************************/
+/* Python attributes get/set structure:                                      */
+/*****************************************************************************/
+static PyGetSetDef BPy_Image_getseters[] = {
+	GENERIC_LIB_GETSETATTR,
+	{"filename", (getter)Image_getFilename, (setter)Image_setFilename,
+	 "image path", NULL},
+	/* readonly */
+	{"depth", (getter)Image_getDepth, (setter)NULL,
+	 "image depth", NULL},
+	{"size", (getter)Image_getSize, (setter)NULL,
+	 "image size", NULL},
+	{"packed", (getter)Image_getPacked, (setter)NULL,
+	 "image packed state", NULL },
+	{"has_data", (getter)Image_hasData, (setter)NULL,
+	 "is image data loaded", NULL },
+	/* ints */
+	{"xrep", (getter)getIntAttr, (setter)setIntAttrClamp,
+	 "image xrep", (void *)EXPP_IMAGE_ATTR_XREP },
+	{"yrep", (getter)getIntAttr, (setter)setIntAttrClamp,
+	 "image yrep", (void *)EXPP_IMAGE_ATTR_YREP },
+	{"start", (getter)getIntAttr, (setter)setIntAttrClamp,
+	 "image start frame", (void *)EXPP_IMAGE_ATTR_START },
+	{"end", (getter)getIntAttr, (setter)setIntAttrClamp,
+	 "image end frame", (void *)EXPP_IMAGE_ATTR_END },
+	{"speed", (getter)getIntAttr, (setter)setIntAttrClamp,
+	 "image end frame", (void *)EXPP_IMAGE_ATTR_SPEED },
+	{"bindcode", (getter)getIntAttr, (setter)NULL,
+	 "openGL bindcode", (void *)EXPP_IMAGE_ATTR_BINDCODE },
+	{"source", (getter)getIntAttr, (setter)Image_setSource,
+	 "image source type", (void *)EXPP_IMAGE_ATTR_SOURCE },
+	/* flags */
+	{"fields", (getter)Image_getFlag, (setter)Image_setFlag,
+	 "image fields toggle", (void *)IMA_FIELDS },
+	{"fields_odd", (getter)Image_getFlag, (setter)Image_setFlag,
+	 "image odd fields toggle", (void *)IMA_STD_FIELD },
+	{"antialias", (getter)Image_getFlag, (setter)Image_setFlag,
+	 "image antialiasing toggle", (void *)IMA_ANTIALI },
+	{"reflect", (getter)Image_getFlag, (setter)Image_setFlag,
+	 "image reflect toggle", (void *)IMA_REFLECT },
+	{"clampX", (getter)Image_getFlagTpage, (setter)Image_setFlagTpage,
+	 "disable tiling on the X axis", (void *)IMA_CLAMP_U },
+	{"clampY", (getter)Image_getFlagTpage, (setter)Image_setFlagTpage,
+	 "disable tiling on the Y axis", (void *)IMA_CLAMP_V },
+	{NULL,NULL,NULL,NULL,NULL}  /* Sentinel */
+};
+
+
+/*****************************************************************************/
+/* Python Image_Type structure definition:   */
+/*****************************************************************************/
+PyTypeObject Image_Type = {
+	PyObject_HEAD_INIT( NULL ) /*     required macro. ( no comma needed )  */ 
+	0,	/* ob_size */
+	"Blender Image",	/* tp_name */
+	sizeof( BPy_Image ),	/* tp_basicsize */
+	0,			/* tp_itemsize */
+	/* methods */
+	NULL,	/* tp_dealloc */
+	0,		/* tp_print */
+	NULL,	/* tp_getattr */
+	NULL,	/* tp_setattr */
+	( cmpfunc ) Image_compare,	/* tp_compare */
+	( reprfunc ) Image_repr,	/* tp_repr */
+
+	/* Method suites for standard classes */
+
+	NULL,                       /* PyNumberMethods *tp_as_number; */
+	NULL,                       /* PySequenceMethods *tp_as_sequence; */
+	NULL,                       /* PyMappingMethods *tp_as_mapping; */
+
+	/* More standard operations (here for binary compatibility) */
+
+	( hashfunc ) GenericLib_hash,	/* hashfunc tp_hash; */
+	NULL,                       /* ternaryfunc tp_call; */
+	NULL,                       /* reprfunc tp_str; */
+	NULL,                       /* getattrofunc tp_getattro; */
+	NULL,                       /* setattrofunc tp_setattro; */
+
+	/* Functions to access object as input/output buffer */
+	NULL,                       /* PyBufferProcs *tp_as_buffer; */
+
+  /*** Flags to define presence of optional/expanded features ***/
+	Py_TPFLAGS_DEFAULT,         /* long tp_flags; */
+
+	NULL,                       /*  char *tp_doc;  Documentation string */
+  /*** Assigned meaning in release 2.0 ***/
+	/* call function for all accessible objects */
+	NULL,                       /* traverseproc tp_traverse; */
+
+	/* delete references to contained objects */
+	NULL,                       /* inquiry tp_clear; */
+
+  /***  Assigned meaning in release 2.1 ***/
+  /*** rich comparisons ***/
+	NULL,                       /* richcmpfunc tp_richcompare; */
+
+  /***  weak reference enabler ***/
+	0,                          /* long tp_weaklistoffset; */
+
+  /*** Added in release 2.2 ***/
+	/*   Iterators */
+	NULL,                       /* getiterfunc tp_iter; */
+	NULL,                       /* iternextfunc tp_iternext; */
+
+  /*** Attribute descriptor and subclassing stuff ***/
+	BPy_Image_methods,           /* struct PyMethodDef *tp_methods; */
+	NULL,                       /* struct PyMemberDef *tp_members; */
+	BPy_Image_getseters,         /* struct PyGetSetDef *tp_getset; */
+	NULL,                       /* struct _typeobject *tp_base; */
+	NULL,                       /* PyObject *tp_dict; */
+	NULL,                       /* descrgetfunc tp_descr_get; */
+	NULL,                       /* descrsetfunc tp_descr_set; */
+	0,                          /* long tp_dictoffset; */
+	NULL,                       /* initproc tp_init; */
+	NULL,                       /* allocfunc tp_alloc; */
+	NULL,                       /* newfunc tp_new; */
+	/*  Low-level free-memory routine */
+	NULL,                       /* freefunc tp_free;  */
+	/* For PyObject_IS_GC */
+	NULL,                       /* inquiry tp_is_gc;  */
+	NULL,                       /* PyObject *tp_bases; */
+	/* method resolution order */
+	NULL,                       /* PyObject *tp_mro;  */
+	NULL,                       /* PyObject *tp_cache; */
+	NULL,                       /* PyObject *tp_subclasses; */
+	NULL,                       /* PyObject *tp_weaklist; */
+	NULL
+};
