@@ -1,5 +1,5 @@
 /*
- * $Id: Ipo.c,v 1.59 2006/12/27 05:04:17 campbellbarton Exp $
+ * $Id: Ipo.c,v 1.67 2007/03/26 02:10:21 campbellbarton Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -36,6 +36,7 @@
 #include "BKE_main.h"
 #include "BKE_global.h"
 #include "BKE_library.h"
+#include "BKE_object.h"
 #include "BKE_ipo.h"
 #include "BLI_blenlib.h"
 #include "BIF_space.h"
@@ -45,6 +46,7 @@
 #include "mydevice.h"
 #include "Ipocurve.h"
 #include "gen_utils.h"
+#include "gen_library.h"
 
 extern int ob_ar[];
 extern int la_ar[];
@@ -91,9 +93,6 @@ struct PyMethodDef M_Ipo_methods[] = {
 /*****************************************************************************/
 /* Python BPy_Ipo methods declarations:                                     */
 /*****************************************************************************/
-static PyObject *Ipo_getName( BPy_Ipo * self );
-static PyObject *Ipo_oldsetName( BPy_Ipo * self, PyObject * args );
-static int Ipo_setName( BPy_Ipo * self, PyObject * args );
 static PyObject *Ipo_getBlocktype( BPy_Ipo * self );
 static PyObject *Ipo_oldsetBlocktype( BPy_Ipo * self, PyObject * args );
 static int Ipo_setBlocktype( BPy_Ipo * self, PyObject * args );
@@ -117,6 +116,7 @@ static PyObject *Ipo_setCurveBeztriple( BPy_Ipo * self, PyObject * args );
 static PyObject *Ipo_getCurveBeztriple( BPy_Ipo * self, PyObject * args );
 
 static PyObject *Ipo_getChannel( BPy_Ipo * self );
+static PyObject *Ipo_copy( BPy_Ipo * self );
 static int Ipo_setChannel( BPy_Ipo * self, PyObject * args );
 
 static int Ipo_length( BPy_Ipo * inst );
@@ -130,9 +130,9 @@ static int Ipo_contains( BPy_Ipo * self, PyObject * key );
 /*****************************************************************************/
 static PyMethodDef BPy_Ipo_methods[] = {
 	/* name, method, flags, doc */
-	{"getName", ( PyCFunction ) Ipo_getName, METH_NOARGS,
+	{"getName", ( PyCFunction ) GenericLib_getName, METH_NOARGS,
 	 "() - Return Ipo Data name"},
-	{"setName", ( PyCFunction ) Ipo_oldsetName, METH_VARARGS,
+	{"setName", ( PyCFunction ) GenericLib_setName_with_method, METH_VARARGS,
 	 "(str) - Change Ipo Data name"},
 	{"getBlocktype", ( PyCFunction ) Ipo_getBlocktype, METH_NOARGS,
 	 "() - Return Ipo blocktype"},
@@ -166,6 +166,12 @@ static PyMethodDef BPy_Ipo_methods[] = {
 	 "(int,int) - deprecated: see ipocurve.bezierPoints[]"},
 	{"setCurveBeztriple", ( PyCFunction ) Ipo_setCurveBeztriple, METH_VARARGS,
 	 "(int,int,list) - set a BezTriple"},
+
+	{"__copy__", ( PyCFunction ) Ipo_copy, METH_NOARGS,
+	 "() - copy the ipo"},
+	{"copy", ( PyCFunction ) Ipo_copy, METH_NOARGS,
+	 "() - copy the ipo"},
+	 
 	{NULL, NULL, 0, NULL}
 };
 
@@ -173,10 +179,7 @@ static PyMethodDef BPy_Ipo_methods[] = {
 /* Python BPy_Ipo attributes get/set structure:                              */
 /*****************************************************************************/
 static PyGetSetDef BPy_Ipo_getseters[] = {
-	{"name",
-	 (getter)Ipo_getName, (setter)Ipo_setName,
-	 "Ipo data name",
-	 NULL},
+	GENERIC_LIB_GETSETATTR,
 	{"curves",
 	 (getter)Ipo_getCurves, (setter)NULL,
 	 "Ipo curves",
@@ -226,8 +229,7 @@ static PySequenceMethods Ipo_as_sequence = {
 /*****************************************************************************/
 /* Python Ipo_Type callback function prototypes:                             */
 /*****************************************************************************/
-static void Ipo_dealloc( BPy_Ipo * self );
-//static int IpoPrint (BPy_Ipo *self, FILE *fp, int flags);
+/*static int IpoPrint (BPy_Ipo *self, FILE *fp, int flags);*/
 static int Ipo_compare( BPy_Ipo * a, BPy_Ipo * b );
 static PyObject *Ipo_repr( BPy_Ipo * self );
 static PyObject *Ipo_getIter( BPy_Ipo * self );
@@ -362,7 +364,7 @@ PyTypeObject Ipo_Type = {
 
 	/* Methods to implement standard operations */
 
-	( destructor ) Ipo_dealloc,/* destructor tp_dealloc; */
+	NULL,						/* destructor tp_dealloc; */
 	NULL,                       /* printfunc tp_print; */
 	NULL,                       /* getattrfunc tp_getattr; */
 	NULL,                       /* setattrfunc tp_setattr; */
@@ -377,7 +379,7 @@ PyTypeObject Ipo_Type = {
 
 	/* More standard operations (here for binary compatibility) */
 
-	NULL,                       /* hashfunc tp_hash; */
+	( hashfunc ) GenericLib_hash,	/* hashfunc tp_hash; */
 	NULL,                       /* ternaryfunc tp_call; */
 	NULL,                       /* reprfunc tp_str; */
 #ifdef CURVEATTRS
@@ -804,32 +806,6 @@ static PyObject *M_Ipo_Recalc( PyObject * self_unused, PyObject * args )
 /*****************************************************************************/
 /* Python BPy_Ipo methods:                                                   */
 /*****************************************************************************/
-
-static PyObject *Ipo_getName( BPy_Ipo * self )
-{
-	PyObject *attr = PyString_FromString( self->ipo->id.name + 2 );
-
-	if( attr )
-		return attr;
-
-	return EXPP_ReturnPyObjError( PyExc_RuntimeError,
-			"couldn't get Ipo.name attribute" );
-}
-
-static int Ipo_setName( BPy_Ipo * self, PyObject * args )
-{
-	char *name;
-
-	name = PyString_AsString( args );
-	if( !name )
-		return EXPP_ReturnIntError( PyExc_TypeError,
-				"expected string argument" );
-
-	rename_id( &self->ipo->id, name );
-
-	return 0;
-}
-
 static PyObject *Ipo_getBlocktype( BPy_Ipo * self )
 {
 	PyObject *attr = PyInt_FromLong( self->ipo->blocktype );
@@ -1290,16 +1266,6 @@ static int Ipo_setChannel( BPy_Ipo * self, PyObject * value )
 	return 0;
 }
 
-/*****************************************************************************/
-/* Function:    Ipo_dealloc                                                  */
-/* Description: This is a callback function for the BPy_Ipo type. It is      */
-/*              the destructor function.                                     */
-/*****************************************************************************/
-static void Ipo_dealloc( BPy_Ipo * self )
-{
-	PyObject_DEL( self );
-}
-
 /*****************************************************************************/
 /* Function:    Ipo_compare			                                         */
 /* Description: This compares 2 ipo python types, == or != only.             */
@@ -1373,16 +1339,6 @@ PyObject *Ipo_CreatePyObject( Ipo * ipo )
 	return ( PyObject * ) pyipo;
 }
 
-/*****************************************************************************/
-/* Function:    Ipo_CheckPyObject                                            */
-/* Description: This function returns true when the given PyObject is of the */
-/*              type Ipo. Otherwise it will return false.                    */
-/*****************************************************************************/
-int Ipo_CheckPyObject( PyObject * pyobj )
-{
-	return ( pyobj->ob_type == &Ipo_Type );
-}
-
 /*****************************************************************************/
 /* Function:    Ipo_FromPyObject                                             */
 /* Description: This function returns the Blender ipo from the given         */
@@ -1588,8 +1544,13 @@ static int Ipo_contains( BPy_Ipo *self, PyObject *key )
 
 static PyObject *Ipo_getIter( BPy_Ipo * self )
 {
-	self->iter = 0;
-	return EXPP_incr_ret ( (PyObject *) self );
+	/* return a new IPO object if we are looping on the existing one
+	   This allows nested loops */
+	if (self->iter==0) {
+		return EXPP_incr_ret ( (PyObject *) self );
+	} else {
+		return Ipo_CreatePyObject(self->ipo);
+	}
 }
 
 /*
@@ -1621,6 +1582,7 @@ static PyObject *Ipo_nextIter( BPy_Ipo * self )
 		}
 	}
 
+	self->iter = 0; /* allow iter use again */
 	/* ran out of curves */
 	return EXPP_ReturnPyObjError( PyExc_StopIteration,
 			"iterator at end" );
@@ -1631,7 +1593,7 @@ static PyObject *Ipo_nextIter( BPy_Ipo * self )
 /*****************************************************************************/
 PyObject *Ipo_Init( void )
 {
-	// PyObject *submodule;
+	/* PyObject *submodule; */
 
 	if( PyType_Ready( &Ipo_Type ) < 0 )
 		return NULL;
@@ -1786,6 +1748,14 @@ static PyObject *Ipo_setCurveBeztriple( BPy_Ipo * self, PyObject * args )
 	return Py_None;
 }
 
+/* Ipo.__copy__ */
+static PyObject *Ipo_copy( BPy_Ipo * self )
+{
+	Ipo *ipo = copy_ipo(self->ipo );
+	ipo->id.us = 0;
+	return Ipo_CreatePyObject(ipo);
+}
+
 static PyObject *Ipo_EvaluateCurveOn( BPy_Ipo * self, PyObject * args )
 {
 	int num = 0, i;
@@ -1822,7 +1792,7 @@ static PyObject *Ipo_getCurvecurval( BPy_Ipo * self, PyObject * args )
 		return ( EXPP_ReturnPyObjError
 			 ( PyExc_TypeError, "No IPO curve" ) );
 
-	if( PyNumber_Check( PyTuple_GetItem( args, 0 ) ) )	// args is an integer
+	if( PyNumber_Check( PyTuple_GetItem( args, 0 ) ) )	/* args is an integer */
 	{
 		if( !PyArg_ParseTuple( args, "i", &numcurve ) )
 			return ( EXPP_ReturnPyObjError
@@ -1837,7 +1807,7 @@ static PyObject *Ipo_getCurvecurval( BPy_Ipo * self, PyObject * args )
 		}
 	}
 
-	else			// args is a string
+	else			/* args is a string */
 	{
 		if( !PyArg_ParseTuple( args, "s", &stringname ) )
 			return ( EXPP_ReturnPyObjError
@@ -1867,12 +1837,6 @@ static PyObject *Ipo_oldsetRctf( BPy_Ipo * self, PyObject * args )
 			(setter)Ipo_setRctf );
 }
 
-static PyObject *Ipo_oldsetName( BPy_Ipo * self, PyObject * args )
-{
-	return EXPP_setterWrapperTuple( (void *)self, args,
-			(setter)Ipo_setName );
-}
-
 static PyObject *Ipo_oldsetBlocktype( BPy_Ipo * self, PyObject * args )
 {
 	return EXPP_setterWrapperTuple( (void *)self, args,
