@@ -1,5 +1,5 @@
 /* 
- * $Id: Ipo.h,v 1.15 2006/04/21 20:27:52 khughes Exp $
+ * $Id: Ipo.h,v 1.17 2007/03/15 01:09:11 campbellbarton Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -41,7 +41,7 @@
 /*****************************************************************************/
 typedef struct {
 	PyObject_HEAD		/* required macro */
-	Ipo * ipo;
+	Ipo * ipo;		/* libdata must be second */
 	short iter;
 	short mtex; 
 } BPy_Ipo;
@@ -58,7 +58,6 @@ extern PyTypeObject Ipo_Type;
 PyObject *Ipo_Init( void );
 PyObject *Ipo_CreatePyObject( struct Ipo *ipo );
 Ipo *Ipo_FromPyObject( PyObject * py_obj );
-int Ipo_CheckPyObject( PyObject * py_obj );
 
 
 #endif				/* EXPP_IPO_H */
