@@ -1,5 +1,5 @@
 /*
- * $Id: Ipocurve.c,v 1.39 2007/01/13 04:53:40 campbellbarton Exp $
+ * $Id: Ipocurve.c,v 1.42 2007/04/05 04:34:41 khughes Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -192,7 +192,6 @@ static PyMappingMethods IpoCurve_as_mapping = {
 /*****************************************************************************/
 static int IpoCurve_compare( C_IpoCurve * a, C_IpoCurve * b );
 static PyObject *IpoCurve_repr( C_IpoCurve * self );
-static void IpoCurve_dealloc( C_IpoCurve * self );
 
 /*****************************************************************************/
 /* Python IpoCurve_Type structure definition:                                */
@@ -204,7 +203,7 @@ PyTypeObject IpoCurve_Type = {
 	sizeof( C_IpoCurve ),				/* tp_basicsize */
 	0,									/* tp_itemsize */
 	/* methods */
-	( destructor ) IpoCurve_dealloc,    /* tp_dealloc */
+	NULL,							    /* tp_dealloc */
 	0,									/* tp_print */
 	( getattrfunc ) NULL,	            /* tp_getattr */
 	( setattrfunc ) NULL,	            /* tp_setattr */
@@ -534,7 +533,9 @@ static PyObject *IpoCurve_append( C_IpoCurve * self, PyObject * args )
 				"expected tuple of floats" );
 
 		x = (float)PyFloat_AsDouble( xobj );
+		Py_DECREF( xobj );
 		y = (float)PyFloat_AsDouble( yobj );
+		Py_DECREF( yobj );
 		insert_vert_ipo( icu, x, y);
 	}
 
@@ -651,17 +652,6 @@ static PyObject *IpoCurve_getPoints( C_IpoCurve * self )
 	return list;
 }
 
-/*****************************************************************************/
-/* Function:    Ipo_dealloc                                                  */
-/* Description: This is a callback function for the C_IpoCurve type. It is   */
-/*              the destructor function.                                     */
-/*****************************************************************************/
-
-static void IpoCurve_dealloc( C_IpoCurve * self )
-{
-	PyObject_DEL( self );
-}
-
 /*****************************************************************************/
 /* Function:    IpoCurve_compare                                     		 */
 /* Description: This compares 2 python types, == or != only.			     */
@@ -704,16 +694,6 @@ PyObject *IpoCurve_CreatePyObject( IpoCurve * icu )
 	return ( PyObject * ) pyipo;
 }
 
-/*****************************************************************************/
-/* Function:    IpoCurve_CheckPyObject                                      */
-/* Description: This function returns true when the given PyObject is of the */
-/*              type IpoCurve. Otherwise it will return false.               */
-/*****************************************************************************/
-int IpoCurve_CheckPyObject( PyObject * pyobj )
-{
-	return ( pyobj->ob_type == &IpoCurve_Type );
-}
-
 /*****************************************************************************/
 /* Function:    IpoCurve_FromPyObject                                       */
 /* Description: This function returns the Blender ipo from the given         */
