@@ -1,5 +1,5 @@
 /* 
- * $Id: Key.c,v 1.12 2007/01/16 05:20:39 stiv Exp $
+ * $Id: Key.c,v 1.17 2007/03/29 03:47:50 khughes Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -31,6 +31,8 @@
  * ***** END GPL/BL DUAL LICENSE BLOCK *****
  */
 
+#include "Key.h" /*This must come first*/
+
 #include "DNA_scene_types.h"
 
 #include <BLI_blenlib.h>
@@ -41,7 +43,6 @@
 #include "BIF_space.h"
 
 #include "Ipocurve.h"
-#include "Key.h"
 #include "NMesh.h" /* we create NMesh.NMVert objects */
 #include "Ipo.h"
 #include "BezTriple.h"
@@ -52,6 +53,7 @@
 #include "blendef.h"
 #include "constant.h"
 #include "gen_utils.h"
+#include "gen_library.h"
 
 #define KEY_TYPE_MESH    0
 #define KEY_TYPE_CURVE   1
@@ -60,17 +62,15 @@
 /* macro from blenkernel/intern/key.c:98 */
 #define GS(a)	(*((short *)(a)))
 
-static void Key_dealloc( PyObject * self );
-static void KeyBlock_dealloc( PyObject * self );
 static int Key_compare( BPy_Key * a, BPy_Key * b );
 static PyObject *Key_repr( BPy_Key * self );
 
-static PyObject *Key_getBlocks( PyObject * self );
+static PyObject *Key_getBlocks( BPy_Key * self );
 static PyObject *Key_getType( BPy_Key * self );
 static PyObject *Key_getRelative( BPy_Key * self );
-static PyObject *Key_getIpo( PyObject * self );
-static int Key_setIpo( PyObject * self, PyObject * args );
-static PyObject *Key_getValue( PyObject * self );
+static PyObject *Key_getIpo( BPy_Key * self );
+static int Key_setIpo( BPy_Key * self, PyObject * args );
+static PyObject *Key_getValue( BPy_Key * self );
 static int Key_setRelative( BPy_Key * self, PyObject * value );
 
 static struct PyMethodDef Key_methods[] = {
@@ -136,7 +136,7 @@ PyTypeObject Key_Type = {
 	sizeof( BPy_Key ),				/*tp_basicsize */
 	0,								/*tp_itemsize */
 	/* methods */
-	( destructor ) Key_dealloc,		/*tp_dealloc */
+	NULL,							/*tp_dealloc */
 	( printfunc ) 0,				/*tp_print */
 	( getattrfunc ) 0,	/*tp_getattr */
 	( setattrfunc ) 0,			 	/*tp_setattr */
@@ -150,7 +150,7 @@ PyTypeObject Key_Type = {
 
 	/* More standard operations (here for binary compatibility) */
 
-	NULL,                       /* hashfunc tp_hash; */
+	( hashfunc ) GenericLib_hash,	/* hashfunc tp_hash; */
 	NULL,                       /* ternaryfunc tp_call; */
 	NULL,                       /* reprfunc tp_str; */
 	NULL,                       /* getattrofunc tp_getattro; */
@@ -213,7 +213,7 @@ PyTypeObject KeyBlock_Type = {
 	sizeof( BPy_KeyBlock ),	/*tp_basicsize */
 	0,			/*tp_itemsize */
 	/* methods */
-	( destructor ) KeyBlock_dealloc,	/*tp_dealloc */
+	NULL,				/*tp_dealloc */
 	NULL,				/*tp_print */
 	NULL, 				/*tp_getattr */
 	NULL,				/*tp_setattr */
@@ -284,11 +284,6 @@ PyTypeObject KeyBlock_Type = {
 	NULL
 };
 
-static void Key_dealloc( PyObject * self )
-{
-	PyObject_DEL( self );
-}
-
 static PyObject *new_Key(Key * oldkey)
 {
 	BPy_Key *k = PyObject_NEW( BPy_Key, &Key_Type );
@@ -319,69 +314,30 @@ static PyObject *Key_repr( BPy_Key * self )
 	return PyString_FromFormat( "[Key \"%s\"]", self->key->id.name + 2 );
 }
 
-static PyObject *Key_getIpo( PyObject * self )
+static PyObject *Key_getIpo( BPy_Key * self )
 {
-	BPy_Key *k = ( BPy_Key * ) self;
 	BPy_Ipo *new_ipo;
 
-	if (k->key->ipo) {
+	if (self->key->ipo) {
 		new_ipo = ( BPy_Ipo * ) PyObject_NEW( BPy_Ipo, &Ipo_Type );
-		new_ipo->ipo = k->key->ipo;
+		new_ipo->ipo = self->key->ipo;
 		return (PyObject *) new_ipo;
 	} else {
-		return EXPP_incr_ret( Py_None );
+		Py_RETURN_NONE;
 	}
 }
 
-static int Key_setIpo( PyObject * self, PyObject * value )
+static int Key_setIpo( BPy_Key * self, PyObject * value )
 {
-	Ipo *ipo = NULL;
-	Ipo *oldipo = (( BPy_Key * )self)->key->ipo;
-	ID *id;
-
-	/* if parameter is not None, check for valid Ipo */
-
-	if ( value != Py_None ) {
-		if ( !Ipo_CheckPyObject( value ) )
-			return EXPP_ReturnIntError( PyExc_RuntimeError,
-					      	"expected an Ipo object" );
-
-		ipo = Ipo_FromPyObject( value );
-
-		if( !ipo )
-			return EXPP_ReturnIntError( PyExc_RuntimeError,
-					      	"null ipo!" );
-
-		if( ipo->blocktype != ID_KE )
-			return EXPP_ReturnIntError( PyExc_TypeError,
-					      	"Ipo is not a key data Ipo" );
-	}
-
-	/* if already linked to Ipo, delete link */
-
-	if ( oldipo ) {
-		id = &oldipo->id;
-		if( id->us > 0 )
-			id->us--;
-	}
-
-	/* assign new Ipo and increment user count, or set to NULL if deleting */
-
-	(( BPy_Key * )self)->key->ipo = ipo;
-	if ( ipo ) {
-		id = &ipo->id;
-		id_us_plus(id);
-	}
-
-	return 0;
+	return GenericLib_assignData(value, (void **) &self->key->ipo, 0, 1, ID_IP, ID_KE);
 }
 
 static PyObject *Key_getRelative( BPy_Key * self )
 {
 	if( self->key->type == KEY_RELATIVE )
-		return EXPP_incr_ret(Py_True);
+		Py_RETURN_TRUE;
 	else
-		return EXPP_incr_ret(Py_False);
+		Py_RETURN_FALSE;
 }
 
 static int Key_setRelative( BPy_Key * self, PyObject * value )
@@ -418,16 +374,14 @@ static PyObject *Key_getType( BPy_Key * self )
 	return PyInt_FromLong( type );
 }
 
-static PyObject *Key_getBlocks( PyObject * self )
+static PyObject *Key_getBlocks( BPy_Key * self )
 {
-	BPy_Key *k = ( BPy_Key * ) self;
-	Key *key = k->key;
+	Key *key = self->key;
 	KeyBlock *kb;
 	PyObject *keyblock_object;
 	PyObject *l = PyList_New( 0 );
 
 	for (kb = key->block.first; kb; kb = kb->next) {
-		
 		keyblock_object =  KeyBlock_CreatePyObject( kb, key );
 		PyList_Append( l, keyblock_object );
 	}
@@ -435,7 +389,7 @@ static PyObject *Key_getBlocks( PyObject * self )
 	return l;
 }
 
-static PyObject *Key_getValue( PyObject * self )
+static PyObject *Key_getValue( BPy_Key * self )
 {
 	BPy_Key *k = ( BPy_Key * ) self;
 
@@ -444,11 +398,6 @@ static PyObject *Key_getValue( PyObject * self )
 
 /* ------------ Key Block Functions -------------- */
 
-static void KeyBlock_dealloc( PyObject * self )
-{
-	PyObject_DEL( self );
-}
-
 static PyObject *new_KeyBlock( KeyBlock * oldkeyBlock, Key *parentKey)
 {
 	BPy_KeyBlock *kb = PyObject_NEW( BPy_KeyBlock, &KeyBlock_Type );
