@@ -1,5 +1,5 @@
 /*
- * $Id: Key.h,v 1.3 2006/12/24 10:51:30 campbellbarton Exp $
+ * $Id: Key.h,v 1.4 2007/02/25 01:07:27 campbellbarton Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -47,7 +47,7 @@ extern PyTypeObject KeyBlock_Type;
 
 typedef struct {
 	PyObject_HEAD		/* required python macro   */
-	Key * key;
+	Key * key;			/* libdata must be second */
 	/* Object *object;*/		/* for vertex grouping info, since it's stored on the object */
 	/*PyObject *keyBlock;*/
 	PyObject *ipo;
