@@ -1,5 +1,5 @@
 /* 
- * $Id: Lamp.c,v 1.47 2006/12/27 05:04:17 campbellbarton Exp $
+ * $Id: Lamp.c,v 1.56 2007/03/26 02:10:21 campbellbarton Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -44,6 +44,7 @@
 #include "Ipo.h"
 #include "constant.h"
 #include "gen_utils.h"
+#include "gen_library.h"
 
 /*****************************************************************************/
 /* Python BPy_Lamp defaults:                                                 */
@@ -178,7 +179,6 @@ struct PyMethodDef M_Lamp_methods[] = {
 /*****************************************************************************/
 /* Python BPy_Lamp methods declarations:                                     */
 /*****************************************************************************/
-static PyObject *Lamp_getName( BPy_Lamp * self );
 static PyObject *Lamp_getType( BPy_Lamp * self );
 static PyObject *Lamp_getTypesConst( void );
 static PyObject *Lamp_getMode( BPy_Lamp * self );
@@ -204,11 +204,9 @@ static PyObject *Lamp_getQuad2( BPy_Lamp * self );
 static PyObject *Lamp_getCol( BPy_Lamp * self );
 static PyObject *Lamp_getIpo( BPy_Lamp * self );
 static PyObject *Lamp_getComponent( BPy_Lamp * self, void * closure );
-static PyObject *Lamp_getUsers( BPy_Lamp * self );
 static PyObject *Lamp_clearIpo( BPy_Lamp * self );
 static PyObject *Lamp_insertIpoKey( BPy_Lamp * self, PyObject * args );
 static PyObject *Lamp_oldsetIpo( BPy_Lamp * self, PyObject * args );
-static PyObject *Lamp_oldsetName( BPy_Lamp * self, PyObject * args );
 static PyObject *Lamp_oldsetType( BPy_Lamp * self, PyObject * args );
 static PyObject *Lamp_oldsetMode( BPy_Lamp * self, PyObject * args );
 static PyObject *Lamp_oldsetSamples( BPy_Lamp * self, PyObject * args );
@@ -230,8 +228,8 @@ static PyObject *Lamp_oldsetHaloInt( BPy_Lamp * self, PyObject * args );
 static PyObject *Lamp_oldsetQuad1( BPy_Lamp * self, PyObject * args );
 static PyObject *Lamp_oldsetQuad2( BPy_Lamp * self, PyObject * args );
 static PyObject *Lamp_oldsetCol( BPy_Lamp * self, PyObject * args );
+static PyObject *Lamp_copy( BPy_Lamp * self );
 static int Lamp_setIpo( BPy_Lamp * self, PyObject * args );
-static int Lamp_setName( BPy_Lamp * self, PyObject * args );
 static int Lamp_setType( BPy_Lamp * self, PyObject * args );
 static int Lamp_setMode( BPy_Lamp * self, PyObject * args );
 static int Lamp_setSamples( BPy_Lamp * self, PyObject * args );
@@ -263,8 +261,7 @@ static int Lamp_setComponent( BPy_Lamp * self, PyObject * value, void * closure
 /*****************************************************************************/
 static PyMethodDef BPy_Lamp_methods[] = {
 	/* name, method, flags, doc */
-	{"getName", ( PyCFunction ) Lamp_getName, METH_NOARGS,
-	 "() - return Lamp name"},
+	
 	{"getType", ( PyCFunction ) Lamp_getType, METH_NOARGS,
 	 "() - return Lamp type - 'Lamp':0, 'Sun':1, 'Spot':2, 'Hemi':3, 'Area':4, 'Photon':5"},
 	{"getMode", ( PyCFunction ) Lamp_getMode, METH_NOARGS,
@@ -307,7 +304,7 @@ static PyMethodDef BPy_Lamp_methods[] = {
 	 "() - return light intensity value #2 for a Quad Lamp"},
 	{"getCol", ( PyCFunction ) Lamp_getCol, METH_NOARGS,
 	 "() - return light rgb color triplet"},
-	{"setName", ( PyCFunction ) Lamp_oldsetName, METH_VARARGS,
+	{"setName", ( PyCFunction ) GenericLib_setName_with_method, METH_VARARGS,
 	 "(str) - rename Lamp"},
 	{"setType", ( PyCFunction ) Lamp_oldsetType, METH_VARARGS,
 	 "(str) - change Lamp type, which can be 'Lamp', 'Sun', 'Spot', 'Hemi', 'Area', 'Photon'"},
@@ -371,7 +368,10 @@ static PyMethodDef BPy_Lamp_methods[] = {
 	 "( lamp-ipo ) - link an IPO to this lamp"},
 	 {"insertIpoKey", ( PyCFunction ) Lamp_insertIpoKey, METH_VARARGS,
 	 "( Lamp IPO type ) - Inserts a key into IPO"},
-
+	{"__copy__", ( PyCFunction ) Lamp_copy, METH_NOARGS,
+	 "() - Makes a copy of this lamp."},
+	{"copy", ( PyCFunction ) Lamp_copy, METH_NOARGS,
+	 "() - Makes a copy of this lamp."},
 	{NULL, NULL, 0, NULL}
 };
 
@@ -379,6 +379,7 @@ static PyMethodDef BPy_Lamp_methods[] = {
 /* Python attributes get/set structure:                                      */
 /*****************************************************************************/
 static PyGetSetDef BPy_Lamp_getseters[] = {
+	GENERIC_LIB_GETSETATTR,
 	{"bias",
 	 (getter)Lamp_getBias, (setter)Lamp_setBias,
 	 "Lamp shadow map sampling bias",
@@ -423,10 +424,6 @@ static PyGetSetDef BPy_Lamp_getseters[] = {
 	 (getter)Lamp_getMode, (setter)Lamp_setMode,
 	 "Lamp mode bitmask",
 	 NULL},
-	{"name",
-	 (getter)Lamp_getName, (setter)Lamp_setName,
-	 "Lamp data name",
-	 NULL},
 	{"quad1",
 	 (getter)Lamp_getQuad1, (setter)Lamp_setQuad1,
 	 "Quad lamp linear distance attenuation",
@@ -495,10 +492,6 @@ static PyGetSetDef BPy_Lamp_getseters[] = {
 	 (getter)Lamp_getComponent, (setter)Lamp_setComponent,
 	 "Lamp color blue component",
 	 (void *)EXPP_LAMP_COMP_B},
-	{"users",
-	 (getter)Lamp_getUsers, (setter)NULL,
-	 "Number of lamp users",
-	 NULL},
 	{"Modes",
 	 (getter)Lamp_getModesConst, (setter)NULL,
 	 "Dictionary of values for 'mode' attribute",
@@ -545,7 +538,7 @@ PyTypeObject Lamp_Type = {
 
 	/* More standard operations (here for binary compatibility) */
 
-	NULL,                       /* hashfunc tp_hash; */
+	( hashfunc ) GenericLib_hash,	/* hashfunc tp_hash; */
 	NULL,                       /* ternaryfunc tp_call; */
 	NULL,                       /* reprfunc tp_str; */
 	NULL,                       /* getattrofunc tp_getattro; */
@@ -610,18 +603,18 @@ static PyObject *M_Lamp_New( PyObject * self, PyObject * args,
 			     PyObject * keywords )
 {
 	char *type_str = "Lamp";
-	char *name_str = "LampData";
+	char *name_str = "Lamp";
 	static char *kwlist[] = { "type_str", "name_str", NULL };
 	BPy_Lamp *py_lamp;	/* for Lamp Data object wrapper in Python */
 	Lamp *bl_lamp;		/* for actual Lamp Data we create in Blender */
-	char buf[21];
 
 	if( !PyArg_ParseTupleAndKeywords( args, keywords, "|ss", kwlist,
 					  &type_str, &name_str ) )
 		return ( EXPP_ReturnPyObjError( PyExc_AttributeError,
 						"expected string(s) or empty argument" ) );
-
-	bl_lamp = add_lamp(  );	/* first create in Blender */
+	
+	bl_lamp = add_lamp( name_str );	/* first create in Blender */
+	
 	if( bl_lamp )		/* now create the wrapper obj in Python */
 		py_lamp = ( BPy_Lamp * ) Lamp_CreatePyObject( bl_lamp );
 	else
@@ -651,11 +644,6 @@ static PyObject *M_Lamp_New( PyObject * self, PyObject * args,
 		return ( EXPP_ReturnPyObjError( PyExc_AttributeError,
 						"unknown lamp type" ) );
 
-	if( strcmp( name_str, "LampData" ) == 0 )
-		return ( PyObject * ) py_lamp;
-	else			/* user gave us a name for the lamp, use it */
-		rename_id( &bl_lamp->id, name_str );
-
 	return ( PyObject * ) py_lamp;
 }
 
@@ -845,20 +833,11 @@ PyObject *Lamp_CreatePyObject( Lamp * lamp )
 	rgb[2] = &lamp->b;
 
 	pylamp->color = ( BPy_rgbTuple * ) rgbTuple_New( rgb );
-
+	Py_INCREF(pylamp->color);
+	
 	return ( PyObject * ) pylamp;
 }
 
-/*****************************************************************************/
-/* Function:    Lamp_CheckPyObject                                           */
-/* Description: This function returns true when the given PyObject is of the */
-/*              type Lamp. Otherwise it will return false.                   */
-/*****************************************************************************/
-int Lamp_CheckPyObject( PyObject * pyobj )
-{
-	return ( pyobj->ob_type == &Lamp_Type );
-}
-
 /*****************************************************************************/
 /* Function:    Lamp_FromPyObject                                            */
 /* Description: This function returns the Blender lamp from the given        */
@@ -869,42 +848,16 @@ Lamp *Lamp_FromPyObject( PyObject * pyobj )
 	return ( ( BPy_Lamp * ) pyobj )->lamp;
 }
 
-/*****************************************************************************/
-/* Description: Returns the lamp with the name specified by the argument     */
-/*              name. Note that the calling function has to remove the first */
-/*              two characters of the lamp name. These two characters        */
-/*              specify the type of the object (OB, ME, WO, ...)             */
-/*              The function will return NULL when no lamp with the given    */
-/*              name is found.                                               */
-/*****************************************************************************/
-Lamp *GetLampByName( char *name )
-{
-	Lamp *lamp_iter;
-
-	lamp_iter = G.main->lamp.first;
-	while( lamp_iter ) {
-		if( StringEqual( name, GetIdName( &( lamp_iter->id ) ) ) ) {
-			return lamp_iter;
-		}
-		lamp_iter = lamp_iter->id.next;
-	}
-
-	/* There is no lamp with the given name */
-	return NULL;
-}
-
 /*****************************************************************************/
 /* Python BPy_Lamp methods:                                                  */
 /*****************************************************************************/
-static PyObject *Lamp_getName( BPy_Lamp * self )
+
+/* Lamp.__copy__ */
+static PyObject *Lamp_copy( BPy_Lamp * self )
 {
-	PyObject *attr = PyString_FromString( self->lamp->id.name + 2 );
-
-	if( attr )
-		return attr;
-
-	return ( EXPP_ReturnPyObjError( PyExc_RuntimeError,
-					"couldn't get Lamp.name attribute" ) );
+	Lamp *lamp = copy_lamp(self->lamp );
+	lamp->id.us = 0;
+	return Lamp_CreatePyObject(lamp);
 }
 
 static PyObject *Lamp_getType( BPy_Lamp * self )
@@ -1131,23 +1084,6 @@ static PyObject *Lamp_getCol( BPy_Lamp * self )
 	return rgbTuple_getCol( self->color );
 }
 
-static int Lamp_setName( BPy_Lamp * self, PyObject * value )
-{
-	char *name = NULL;
-	char buf[21];
-
-	name = PyString_AsString ( value );
-	if( !name )
-		return EXPP_ReturnIntError( PyExc_TypeError,
-					      "expected string argument" );
-
-	PyOS_snprintf( buf, sizeof( buf ), "%s", name );
-
-	rename_id( &self->lamp->id, buf );
-
-	return 0;
-}
-
 static int Lamp_setType( BPy_Lamp * self, PyObject * value )
 {
 	return EXPP_setIValueRange ( value, &self->lamp->type,
@@ -1459,45 +1395,7 @@ static PyObject *Lamp_getIpo( BPy_Lamp * self )
 
 static int Lamp_setIpo( BPy_Lamp * self, PyObject * value )
 {
-	Ipo *ipo = NULL;
-	Ipo *oldipo = self->lamp->ipo;
-	ID *id;
-
-	/* if parameter is not None, check for valid Ipo */
-
-	if ( value != Py_None ) {
-		if ( !Ipo_CheckPyObject( value ) )
-			return EXPP_ReturnIntError( PyExc_RuntimeError,
-					      	"expected an Ipo object" );
-
-		ipo = Ipo_FromPyObject( value );
-
-		if( !ipo )
-			return EXPP_ReturnIntError( PyExc_RuntimeError,
-					      	"null ipo!" );
-
-		if( ipo->blocktype != ID_LA )
-			return EXPP_ReturnIntError( PyExc_TypeError,
-					      	"Ipo is not a lamp data Ipo" );
-	}
-
-	/* if already linked to Ipo, delete link */
-
-	if ( oldipo ) {
-		id = &oldipo->id;
-		if( id->us > 0 )
-			id->us--;
-	}
-
-	/* assign new Ipo and increment user count, or set to NULL if deleting */
-
-	self->lamp->ipo = ipo;
-	if ( ipo ) {
-		id = &ipo->id;
-		id_us_plus(id);
-	}
-
-	return 0;
+	return GenericLib_assignData(value, (void **) &self->lamp->ipo, 0, 1, ID_IP, ID_LA);
 }
 
 /*
@@ -1585,18 +1483,8 @@ static PyObject *Lamp_getTypesConst( void )
 	return attr;
 }
 
-static PyObject *Lamp_getUsers( BPy_Lamp * self )
-{
-	return PyInt_FromLong( self->lamp->id.us );
-}
-
 /* #####DEPRECATED###### */
 
-static PyObject *Lamp_oldsetName( BPy_Lamp * self, PyObject * args )
-{
-	return EXPP_setterWrapper ( (void *)self, args, (setter)Lamp_setName );
-}
-
 static PyObject *Lamp_oldsetSamples( BPy_Lamp * self, PyObject * args )
 {
 	return EXPP_setterWrapper ( (void *)self, args, (setter)Lamp_setSamples );
