@@ -1,5 +1,5 @@
 /* 
- * $Id: Lattice.h,v 1.5 2005/07/18 03:50:37 ascotan Exp $
+ * $Id: Lattice.h,v 1.8 2007/03/15 01:09:11 campbellbarton Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -36,18 +36,19 @@
 #include <Python.h>
 #include "DNA_lattice_types.h"
 
+/* The Group PyTypeObject defined in Lattice.c */
+extern PyTypeObject Lattice_Type;
 
+#define BPy_Lattice_Check(v)       ((v)->ob_type == &Lattice_Type)
 
 /*****************************************************************************/
 /* Python BPy_Lattice structure definition:   */
 /*****************************************************************************/
 typedef struct {
 	PyObject_HEAD 
-	Lattice * Lattice;
+	Lattice * lattice; /* libdata must be second */
 } BPy_Lattice;
 
-
-
 /*
  * prototypes
  */
@@ -55,7 +56,5 @@ typedef struct {
 PyObject *Lattice_Init( void );
 PyObject *Lattice_CreatePyObject( Lattice * lt );
 Lattice *Lattice_FromPyObject( PyObject * pyobj );
-int Lattice_CheckPyObject( PyObject * pyobj );
-
 
 #endif				/* EXPP_LATTICE_H */
