@@ -1,5 +1,5 @@
 /* 
- * $Id: MTex.c,v 1.10 2007/01/13 05:06:27 campbellbarton Exp $
+ * $Id: MTex.c,v 1.16 2007/03/16 11:38:02 campbellbarton Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -32,9 +32,11 @@
 #include "MTex.h" /*This must come first*/
 
 #include "BKE_utildefines.h"
+#include "BLI_blenlib.h"
 #include "Texture.h"
 #include "Object.h"
 #include "gen_utils.h"
+#include "gen_library.h"
 
 #include <DNA_material_types.h>
 
@@ -63,7 +65,6 @@ static PyMethodDef BPy_MTex_methods[] = {
 /*****************************************************************************/
 /* Python MTex_Type callback function prototypes:                            */
 /*****************************************************************************/
-static void MTex_dealloc( BPy_MTex * self );
 static int MTex_compare( BPy_MTex * a, BPy_MTex * b );
 static PyObject *MTex_repr( BPy_MTex * self );
 
@@ -78,6 +79,7 @@ static PyObject *MTex_repr( BPy_MTex * self );
 MTEXGETSET(Tex)
 MTEXGETSET(TexCo)
 MTEXGETSET(Object)
+MTEXGETSET(UVLayer)
 MTEXGETSET(MapTo)
 MTEXGETSET(Col)
 MTEXGETSET(DVar)
@@ -107,6 +109,8 @@ static PyGetSetDef MTex_getseters[] = {
 		"Texture coordinate space (UV, Global, etc.)", NULL },
 	{ "object", (getter) MTex_getObject, (setter) MTex_setObject,
 		"Object whose space to use when texco is Object", NULL },
+	{ "uvlayer", (getter) MTex_getUVLayer, (setter) MTex_setUVLayer,
+		"Name of the UV layer to use", NULL },
 	{ "mapto", (getter) MTex_getMapTo, (setter) MTex_setMapTo,
 		"What values the texture affects", NULL },
 	{ "col", (getter) MTex_getCol, (setter) MTex_setCol,
@@ -191,7 +195,7 @@ PyTypeObject MTex_Type = {
 	sizeof( BPy_MTex ),	/* tp_basicsize */
 	0,			/* tp_itemsize */
 	/* methods */
-	( destructor ) MTex_dealloc,	/* tp_dealloc */
+	NULL,		/* tp_dealloc */
 	0,			/* tp_print */
 	0,	/* tp_getattr */
 	0,	/* tp_setattr */
@@ -238,10 +242,7 @@ PyObject *MTex_Init( void )
 
 	/* call PyType_Ready() to init dictionaries & such */
 	if( PyType_Ready( &MTex_Type) < 0)
-		return NULL;
-
-	/* So do we need this? */
-/*	MTex_Type.ob_type = &PyType_Type;*/
+		Py_RETURN_NONE;
 
 	submodule = Py_InitModule( "Blender.Texture.MTex", M_MTex_methods );
 
@@ -266,45 +267,13 @@ MTex *MTex_FromPyObject( PyObject * pyobj )
 	return ( ( BPy_MTex * ) pyobj )->mtex;
 }
 
-
-int MTex_CheckPyObject( PyObject * pyobj )
-{
-	return ( pyobj->ob_type == &MTex_Type );
-}
-
-
 /*****************************************************************************/
 /* Python BPy_MTex methods:                                                  */
 /*****************************************************************************/
 
 static PyObject *MTex_setTexMethod( BPy_MTex * self, PyObject * args )
 {
-	Tex *tex;
-
-	if( args == Py_None )
-	{
-		tex = NULL;
-	} else if( Texture_CheckPyObject( args ) ) {
-		tex = Texture_FromPyObject( args );
-	} else {
-		return EXPP_ReturnPyObjError( PyExc_TypeError,
-					      "expected Texture argument" );
-	}
-	
-	if( self->mtex->tex )
-		self->mtex->tex->id.us--;
-
-	self->mtex->tex = tex;
-
-	if( self->mtex->tex )
-		self->mtex->tex->id.us++;
-
-	Py_RETURN_NONE;
-}
-
-static void MTex_dealloc( BPy_MTex * self )
-{
-	PyObject_DEL( self );
+	return EXPP_setterWrapper( (void *)self, args, (setter)MTex_setTex );
 }
 
 static int MTex_compare( BPy_MTex * a, BPy_MTex * b )
@@ -332,27 +301,7 @@ static PyObject *MTex_getTex( BPy_MTex *self, void *closure )
 
 static int MTex_setTex( BPy_MTex *self, PyObject *value, void *closure)
 {
-	Tex *tex;
-
-	if( value == Py_None )
-	{
-		tex = NULL;
-	} else if( Texture_CheckPyObject( value ) ) {
-		tex = Texture_FromPyObject( value );
-	} else {
-		return EXPP_ReturnIntError( PyExc_TypeError,
-					      "expected Texture argument" );
-	}
-	
-	if( self->mtex->tex )
-		self->mtex->tex->id.us--;
-
-	self->mtex->tex = tex;
-
-	if( self->mtex->tex )
-		self->mtex->tex->id.us++;
-
-	return 0;
+	return GenericLib_assignData(value, (void **) &self->mtex->tex, 0, 1, ID_TE, 0);
 }
 
 static PyObject *MTex_getTexCo( BPy_MTex *self, void *closure )
@@ -393,26 +342,20 @@ static PyObject *MTex_getObject( BPy_MTex *self, void *closure )
 
 static int MTex_setObject( BPy_MTex *self, PyObject *value, void *closure)
 {
-	Object *obj;
+	return GenericLib_assignData(value, (void **) &self->mtex->object, 0, 1, ID_OB, 0);
+}
 
-	if( value == Py_None )
-	{
-		obj = NULL;
-	} else if( Object_CheckPyObject( value ) ) {
-		obj = Object_FromPyObject( value );
-	} else {
+static PyObject *MTex_getUVLayer( BPy_MTex *self, void *closure )
+{
+	return PyString_FromString(self->mtex->uvname);
+}
+
+static int MTex_setUVLayer( BPy_MTex *self, PyObject *value, void *closure)
+{
+	if ( !PyString_Check(value) )
 		return EXPP_ReturnIntError( PyExc_TypeError,
-					      "expected Object argument" );
-	}
-	
-	if( self->mtex->object )
-		self->mtex->object->id.us--;
-
-	self->mtex->object = obj;
-
-	if( self->mtex->object )
-		self->mtex->object->id.us++;
-
+					    "expected string value" );
+	BLI_strncpy(self->mtex->uvname, PyString_AsString(value), 31);
 	return 0;
 }
 
