@@ -1,5 +1,5 @@
 /* 
- * $Id: Mathutils.c,v 1.33 2006/10/03 16:07:48 campbellbarton Exp $
+ * $Id: Mathutils.c,v 1.36 2007/04/04 13:18:29 campbellbarton Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -121,6 +121,12 @@ PyObject *Mathutils_Init(void)
 	/* needed for getseters */
 	if( PyType_Ready( &vector_Type ) < 0 )
 		return NULL;
+	if( PyType_Ready( &matrix_Type ) < 0 )
+		return NULL;	
+	if( PyType_Ready( &euler_Type ) < 0 )
+		return NULL;
+	if( PyType_Ready( &quaternion_Type ) < 0 )
+		return NULL;
 	
 	submodule = Py_InitModule3("Blender.Mathutils",
 				    M_Mathutils_methods, M_Mathutils_doc);
@@ -1383,7 +1389,7 @@ PyObject *M_Mathutils_Intersect( PyObject * self, PyObject * args )
 	VECCOPY(v3, vec3->vec);
 
 	VECCOPY(dir, ray->vec);
-	Normalise(dir);
+	Normalize(dir);
 
 	VECCOPY(orig, ray_off->vec);
 
@@ -1479,9 +1485,9 @@ PyObject *M_Mathutils_LineIntersect( PyObject * self, PyObject * args )
 		VecSubf(b, v4, v3);
 
 		VECCOPY(dir1, a);
-		Normalise(dir1);
+		Normalize(dir1);
 		VECCOPY(dir2, b);
-		Normalise(dir2);
+		Normalize(dir2);
 		d = Inpf(dir1, dir2);
 		if (d == 1.0f || d == -1.0f) {
 			/* colinear */
@@ -1571,18 +1577,18 @@ PyObject *M_Mathutils_QuadNormal( PyObject * self, PyObject * args )
 	VecSubf(e2, v3, v2);
 
 	Crossf(n1, e2, e1);
-	Normalise(n1);
+	Normalize(n1);
 
 	/* find vectors for two edges sharing v4 */
 	VecSubf(e1, v3, v4);
 	VecSubf(e2, v1, v4);
 
 	Crossf(n2, e2, e1);
-	Normalise(n2);
+	Normalize(n2);
 
 	/* adding and averaging the normals of both triangles */
 	VecAddf(n1, n2, n1);
-	Normalise(n1);
+	Normalize(n1);
 
 	return newVectorObject(n1, 3, Py_NEW);
 }
@@ -1614,7 +1620,7 @@ PyObject *M_Mathutils_TriangleNormal( PyObject * self, PyObject * args )
 	VecSubf(e2, v3, v2);
 
 	Crossf(n, e2, e1);
-	Normalise(n);
+	Normalize(n);
 
 	return newVectorObject(n, 3, Py_NEW);
 }
@@ -1666,8 +1672,13 @@ PyObject *M_Mathutils_TriangleArea( PyObject * self, PyObject * args )
 PyObject *M_Mathutils_CopyMat(PyObject * self, PyObject * args)
 {
 	PyObject *matrix = NULL;
+	static char warning = 1;
+
+	if( warning ) {
+		printf("Mathutils.CopyMat(): deprecated :use Mathutils.Matrix() to copy matrices\n");
+		--warning;
+	}
 
-	printf("Mathutils.CopyMat(): Deprecated :use Mathutils.Matrix() to copy matrices\n");
 	matrix = M_Mathutils_Matrix(self, args);
 	if(matrix == NULL)
 		return NULL; //error string already set if we get here
@@ -1679,8 +1690,13 @@ PyObject *M_Mathutils_CopyMat(PyObject * self, PyObject * args)
 PyObject *M_Mathutils_CopyVec(PyObject * self, PyObject * args)
 {
 	PyObject *vec = NULL;
+	static char warning = 1;
+
+	if( warning ) {
+		printf("Mathutils.CopyVec(): Deprecated: use Mathutils.Vector() to copy vectors\n");
+		--warning;
+	}
 
-	printf("Mathutils.CopyVec(): Deprecated: use Mathutils.Vector() to copy vectors\n");
 	vec = M_Mathutils_Vector(self, args);
 	if(vec == NULL)
 		return NULL; //error string already set if we get here
@@ -1692,8 +1708,13 @@ PyObject *M_Mathutils_CopyVec(PyObject * self, PyObject * args)
 PyObject *M_Mathutils_CopyQuat(PyObject * self, PyObject * args)
 {
 	PyObject *quat = NULL;
+	static char warning = 1;
+
+	if( warning ) {
+		printf("Mathutils.CopyQuat(): Deprecated: use Mathutils.Quaternion() to copy vectors\n");
+		--warning;
+	}
 
-	printf("Mathutils.CopyQuat(): Deprecated:use Mathutils.Quaternion() to copy vectors\n");
 	quat = M_Mathutils_Quaternion(self, args);
 	if(quat == NULL)
 		return NULL; //error string already set if we get here
@@ -1705,8 +1726,13 @@ PyObject *M_Mathutils_CopyQuat(PyObject * self, PyObject * args)
 PyObject *M_Mathutils_CopyEuler(PyObject * self, PyObject * args)
 {
 	PyObject *eul = NULL;
+	static char warning = 1;
+
+	if( warning ) {
+		printf("Mathutils.CopyEuler(): deprecated:use Mathutils.Euler() to copy vectors\n");
+		--warning;
+	}
 
-	printf("Mathutils.CopyEuler(): Deprecated:use Mathutils.Euler() to copy vectors\n");
 	eul = M_Mathutils_Euler(self, args);
 	if(eul == NULL)
 		return NULL; //error string already set if we get here
@@ -1721,12 +1747,17 @@ PyObject *M_Mathutils_RotateEuler(PyObject * self, PyObject * args)
 	EulerObject *Eul = NULL;
 	float angle;
 	char *axis;
+	static char warning = 1;
+
+	if( warning ) {
+		printf("Mathutils.RotateEuler(): Deprecated:use Euler.rotate() to rotate a euler\n");
+		--warning;
+	}
 
 	if(!PyArg_ParseTuple(args, "O!fs", &euler_Type, &Eul, &angle, &axis))
 		return EXPP_ReturnPyObjError(PyExc_TypeError,
 			   "Mathutils.RotateEuler(): expected euler type & float & string");
 
-	printf("Mathutils.RotateEuler(): Deprecated:use Euler.rotate() to rotate a euler\n");
 	Euler_Rotate(Eul, Py_BuildValue("fs", angle, axis));
 	return EXPP_incr_ret(Py_None);
 }
@@ -1736,13 +1767,18 @@ PyObject *M_Mathutils_MatMultVec(PyObject * self, PyObject * args)
 {
 	MatrixObject *mat = NULL;
 	VectorObject *vec = NULL;
+	static char warning = 1;
+
+	if( warning ) {
+		printf("Mathutils.MatMultVec(): Deprecated: use matrix * vec to perform column vector multiplication\n");
+		--warning;
+	}
 
 	//get pyObjects
 	if(!PyArg_ParseTuple(args, "O!O!", &matrix_Type, &mat, &vector_Type, &vec))
 		return EXPP_ReturnPyObjError(PyExc_TypeError, 
 			"Mathutils.MatMultVec(): MatMultVec() expects a matrix and a vector object - in that order\n");
 
-	printf("Mathutils.MatMultVec(): Deprecated: use matrix * vec to perform column vector multiplication\n");
 	return column_vector_multiplication(mat, vec);
 }
 //----------------------------------Mathutils.VecMultMat() ---------------
@@ -1751,13 +1787,18 @@ PyObject *M_Mathutils_VecMultMat(PyObject * self, PyObject * args)
 {
 	MatrixObject *mat = NULL;
 	VectorObject *vec = NULL;
+	static char warning = 1;
+
+	if( warning ) {
+		printf("Mathutils.VecMultMat(): Deprecated: use vec * matrix to perform row vector multiplication\n");
+		--warning;
+	}
 
 	//get pyObjects
 	if(!PyArg_ParseTuple(args, "O!O!", &vector_Type, &vec, &matrix_Type, &mat))
 		return EXPP_ReturnPyObjError(PyExc_TypeError, 
 			"Mathutils.VecMultMat(): VecMultMat() expects a vector and matrix object - in that order\n");
 
-	printf("Mathutils.VecMultMat(): Deprecated: use vec * matrix to perform row vector multiplication\n");
 	return row_vector_multiplication(vec, mat);
 }
 //#######################################################################
