@@ -1,5 +1,5 @@
 /* 
- * $Id: Metaball.c,v 1.32 2007/01/28 04:58:22 campbellbarton Exp $
+ * $Id: Metaball.c,v 1.42 2007/03/26 02:10:21 campbellbarton Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -42,6 +42,7 @@
 #include "Mathutils.h"
 #include "Material.h"
 #include "gen_utils.h"
+#include "gen_library.h"
 
 /* for dealing with materials */
 #include "MEM_guardedalloc.h"
@@ -102,12 +103,25 @@ static PyObject *M_MetaElem_TypesDict( void )
 	return Types;
 }
 
+static PyObject *M_MetaElem_UpdateDict( void )
+{
+	PyObject *Update = PyConstant_New(  );
+
+	if( Update ) {
+		BPy_constant *d = ( BPy_constant * ) Update;
+		PyConstant_Insert( d, "ALWAYS",	PyInt_FromLong( MB_UPDATE_ALWAYS ) );
+		PyConstant_Insert( d, "HALFRES",PyInt_FromLong( MB_UPDATE_HALFRES ) );
+		PyConstant_Insert( d, "FAST",	PyInt_FromLong( MB_UPDATE_FAST ) );
+		PyConstant_Insert( d, "NEVER",	PyInt_FromLong( MB_UPDATE_NEVER ) );
+	}
+
+	return Update;
+}
+
 /*****************************************************************************/
 /* Python BPy_Metaball methods declarations:                                */
 /*****************************************************************************/
 static PyObject *Metaball_getElements( BPy_Metaball * self );
-static PyObject *Metaball_getName( BPy_Metaball * self );
-static int Metaball_setName( BPy_Metaball * self, PyObject * value );
 static PyObject *Metaball_getMaterials( BPy_Metaball * self );
 static int Metaball_setMaterials( BPy_Metaball * self, PyObject * value );
 static PyObject *Metaball_getWiresize( BPy_Metaball * self );
@@ -116,10 +130,9 @@ static PyObject *Metaball_getRendersize( BPy_Metaball * self );
 static int Metaball_setRendersize( BPy_Metaball * self, PyObject * value);
 static PyObject *Metaball_getThresh( BPy_Metaball * self );
 static int Metaball_setThresh( BPy_Metaball * self, PyObject * args );
+static PyObject *Metaball_getUpdate( BPy_Metaball * self );
+static int Metaball_setUpdate( BPy_Metaball * self, PyObject * args );
 static PyObject *Metaball_copy( BPy_Metaball * self );
-static PyObject *Metaball_getUsers( BPy_Metaball * self );
-static PyObject *Metaball_getFakeUser( BPy_Metaball * self );
-static int Metaball_setFakeUser( BPy_Metaball * self, PyObject * value );
 
 /*****************************************************************************/
 /* Python BPy_Metaball methods table:                                       */
@@ -128,6 +141,8 @@ static PyMethodDef BPy_Metaball_methods[] = {
 	/* name, method, flags, doc */
 	{"__copy__", ( PyCFunction ) Metaball_copy,
 	 METH_NOARGS, "() - Return a copy of this metaball"},
+	{"copy", ( PyCFunction ) Metaball_copy,
+	 METH_NOARGS, "() - Return a copy of this metaball"},
 	{NULL, NULL, 0, NULL}
 };
 
@@ -141,7 +156,6 @@ static PyMethodDef BPy_Metaelem_methods[] = {
 /*****************************************************************************/
 /* Python Metaball_Type callback function prototypes:                       */
 /*****************************************************************************/
-static void Metaball_dealloc( BPy_Metaball * self );
 static PyObject *Metaball_repr( BPy_Metaball * self );
 static int Metaball_compare( BPy_Metaball * a, BPy_Metaball * b );
 
@@ -172,18 +186,7 @@ static int Metaelem_setMFlagBits( BPy_Metaelem * self, PyObject * value, void *
 /* Python attributes get/set structure:                                      */
 /*****************************************************************************/
 static PyGetSetDef BPy_Metaball_getseters[] = {
-	{"name",
-	 (getter)Metaball_getName, (setter)Metaball_setName,
-	 "Metaball name",
-	 NULL},
-	{"users",
-	 (getter)Metaball_getUsers, (setter)NULL,
-	 "Number of metaball users",
-	 NULL},
-	{"fakeUser",
-	 (getter)Metaball_getFakeUser, (setter)Metaball_setFakeUser,
-	 "The fake user status of this object",
-	 NULL},
+	GENERIC_LIB_GETSETATTR,
 	{"materials",
 	 (getter)Metaball_getMaterials, (setter)Metaball_setMaterials,
 	 "Number of metaball users",
@@ -204,6 +207,10 @@ static PyGetSetDef BPy_Metaball_getseters[] = {
 	 (getter)Metaball_getThresh, (setter)Metaball_setThresh,
 	 "The density to render wire",
 	 NULL},
+	{"update",
+	 (getter)Metaball_getUpdate, (setter)Metaball_setUpdate,
+	 "The setting for updating this metaball data",
+	 NULL},
 	{NULL,NULL,NULL,NULL,NULL}  /* Sentinel */
 };
 
@@ -221,7 +228,7 @@ PyTypeObject Metaball_Type = {
 
 	/* Methods to implement standard operations */
 
-	( destructor ) Metaball_dealloc,/* destructor tp_dealloc; */
+	NULL,						/* destructor tp_dealloc; */
 	NULL,                       /* printfunc tp_print; */
 	NULL,                       /* getattrfunc tp_getattr; */
 	NULL,                       /* setattrfunc tp_setattr; */
@@ -236,7 +243,7 @@ PyTypeObject Metaball_Type = {
 
 	/* More standard operations (here for binary compatibility) */
 
-	NULL,                       /* hashfunc tp_hash; */
+	( hashfunc ) GenericLib_hash,	/* hashfunc tp_hash; */
 	NULL,                       /* ternaryfunc tp_call; */
 	NULL,                       /* reprfunc tp_str; */
 	NULL,                       /* getattrofunc tp_getattro; */
@@ -429,9 +436,13 @@ static PyObject *M_Metaball_New( PyObject * self, PyObject * args )
 	
 	if( !PyArg_ParseTuple( args, "|s", &name ) )
 		return ( EXPP_ReturnPyObjError( PyExc_TypeError,
-						"expected string argument (or nothing)" ) );
+						"Metaball.New() - expected string argument (or nothing)" ) );
 
-	blmball = add_mball(  );	/* first create the MetaBall Data in Blender */
+	/* first create the MetaBall Data in Blender */
+	if (name)
+		blmball = add_mball( name );
+	else
+		blmball = add_mball( "Meta" );
 
 	if( blmball ) {
 		/* return user count to zero since add_mball() incref'ed it */
@@ -442,7 +453,7 @@ static PyObject *M_Metaball_New( PyObject * self, PyObject * args )
 							 &Metaball_Type );
 	} else
 		return ( EXPP_ReturnPyObjError( PyExc_RuntimeError,
-						"couldn't create MetaBall Data in Blender" ) );
+						"Metaball.New() - couldn't create data in Blender" ) );
 
 	if( pymball == NULL )
 		return ( EXPP_ReturnPyObjError( PyExc_MemoryError,
@@ -450,8 +461,6 @@ static PyObject *M_Metaball_New( PyObject * self, PyObject * args )
 
 	pymball->metaball = blmball;
 	/*link Python mballer wrapper to Blender MetaBall */
-	if( name )		/* user gave us a name for the metaball, use it */
-		rename_id( &blmball->id, name );
 	
 	return ( PyObject * ) pymball;
 }
@@ -472,15 +481,15 @@ static PyObject *M_Metaball_Get( PyObject * self, PyObject * args )
 
 	if( !PyArg_ParseTuple( args, "|s", &name ) )
 		return ( EXPP_ReturnPyObjError( PyExc_TypeError,
-						"expected string argument (or nothing)" ) );
+						"Metaball.Get() - expected string argument (or nothing)" ) );
 
 	if( name ) {		/* (name) - Search mball by name */
 		mball_iter = ( MetaBall * ) GetIdFromList( &( G.main->mball ), name );
 		
 		if (!mball_iter) {
-			char error_msg[64];
+			char error_msg[128];
 			PyOS_snprintf( error_msg, sizeof( error_msg ),
-				       "MetaBall \"%s\" not found", name );
+				       "Metaball.Get(\"%s\") - not found", name );
 			return ( EXPP_ReturnPyObjError
 				 ( PyExc_NameError, error_msg ) );
 		}
@@ -493,7 +502,7 @@ static PyObject *M_Metaball_Get( PyObject * self, PyObject * args )
 		
 		if( mballlist == NULL )
 			return ( EXPP_ReturnPyObjError( PyExc_MemoryError,
-							"couldn't create PyList" ) );
+							"MetaBall.Get() - couldn't create PyList" ) );
 		
 		mball_iter = G.main->mball.first;
 		while( mball_iter ) {
@@ -513,7 +522,8 @@ static PyObject *M_Metaball_Get( PyObject * self, PyObject * args )
 PyObject *Metaball_Init( void )
 {
 	PyObject *submodule;
-	PyObject *Types= M_MetaElem_TypesDict( );
+	PyObject *Types=	M_MetaElem_TypesDict( );
+	PyObject *Update=	M_MetaElem_UpdateDict( );
 	
 	if( PyType_Ready( &Metaball_Type ) < 0 )
 		return NULL;
@@ -526,51 +536,18 @@ PyObject *Metaball_Init( void )
 
 	if( Types )
 		PyModule_AddObject( submodule, "Types", Types );
+		PyModule_AddObject( submodule, "Update", Update );
 	
 	/*Add SUBMODULES to the module*/
 	/*PyDict_SetItemString(dict, "Constraint", Constraint_Init()); */ /*creates a *new* module*/
 	return submodule;
 }
 
-
-
-int Metaball_CheckPyObject( PyObject * pyobj )
-{
-	return ( pyobj->ob_type == &Metaball_Type );
-}
-
-
 MetaBall *Metaball_FromPyObject( PyObject * pyobj )
 {
 	return ( ( BPy_Metaball * ) pyobj )->metaball;
 }
 
-static PyObject *Metaball_getName( BPy_Metaball * self )
-{
-
-	PyObject *attr = PyString_FromString( self->metaball->id.name + 2 );
-
-	if( attr )
-		return attr;
-
-	return ( EXPP_ReturnPyObjError( PyExc_RuntimeError,
-					"couldn't get Metaball.name attribute" ) );
-}
-
-
-
-static int Metaball_setName( BPy_Metaball * self, PyObject * value )
-{
-	char *name = NULL;
-	
-	name = PyString_AsString ( value );
-	if( !name )
-		return ( EXPP_ReturnIntError( PyExc_TypeError,
-						"expected string argument" ) );
-	rename_id( &self->metaball->id, name );
-	return 0;
-}
-
 static PyObject *Metaball_getMaterials( BPy_Metaball *self )
 {
 	return EXPP_PyList_fromMaterialList( self->metaball->mat,
@@ -584,12 +561,12 @@ static int Metaball_setMaterials( BPy_Metaball *self, PyObject * value )
     if( !PySequence_Check( value ) ||
 			!EXPP_check_sequence_consistency( value, &Material_Type ) )
         return EXPP_ReturnIntError( PyExc_TypeError,
-                  "list should only contain materials or None)" );
+                  "metaball.materials - list should only contain materials or None)" );
 
     len = PyList_Size( value );
     if( len > 16 )
         return EXPP_ReturnIntError( PyExc_TypeError,
-                          "list can't have more than 16 materials" );
+                          "metaball.materials - list can't have more than 16 materials" );
 
 	/* free old material list (if it exists) and adjust user counts */
 	if( self->metaball->mat ) {
@@ -629,11 +606,11 @@ static int Metaball_setWiresize( BPy_Metaball * self, PyObject * value )
 	float param;
 	if( !PyNumber_Check( value ) )
 		return EXPP_ReturnIntError( PyExc_TypeError,
-					"expected float argument" );
+					"metaball.wiresize - expected float argument" );
 
-	param = PyFloat_AsDouble( value );
+	param = (float)PyFloat_AsDouble( value );
 
-	self->metaball->wiresize = EXPP_ClampFloat(param, 0.05, 1.0);
+	self->metaball->wiresize = EXPP_ClampFloat(param, 0.05f, 1.0);
 	return 0;
 
 }
@@ -648,11 +625,11 @@ static int Metaball_setRendersize( BPy_Metaball * self, PyObject * value )
 	float param;
 	if( !PyNumber_Check( value ) )
 		return EXPP_ReturnIntError( PyExc_TypeError,
-					"expected float argument" );
+					"metaball.rendersize - expected float argument" );
 
-	param = PyFloat_AsDouble( value );
+	param = (float)PyFloat_AsDouble( value );
 
-	self->metaball->rendersize = EXPP_ClampFloat(param, 0.05, 1.0);
+	self->metaball->rendersize = EXPP_ClampFloat(param, 0.05f, 1.0);
 	return 0;
 }
 
@@ -667,35 +644,33 @@ static int Metaball_setThresh( BPy_Metaball * self, PyObject * value )
 	float param;
 	if( !PyNumber_Check( value ) )
 		return EXPP_ReturnIntError( PyExc_TypeError,
-					"expected float argument" );
+					"metaball.thresh - expected float argument" );
 
-	param = PyFloat_AsDouble( value );
+	param = (float)PyFloat_AsDouble( value );
 
 	self->metaball->thresh = EXPP_ClampFloat(param, 0.0, 5.0);
 	return 0;
 }
 
-
-static PyObject *Metaball_getUsers( BPy_Metaball * self )
+static PyObject *Metaball_getUpdate( BPy_Metaball * self )
 {
-	return PyInt_FromLong( self->metaball->id.us );
+	return PyInt_FromLong( (long)self->metaball->flag );
 }
 
-static PyObject *Metaball_getFakeUser( BPy_Metaball * self )
+static int Metaball_setUpdate( BPy_Metaball * self, PyObject * value )
 {
-	if (self->metaball->id.flag & LIB_FAKEUSER)
-		Py_RETURN_TRUE;
-	else
-		Py_RETURN_FALSE;
+
+	int param;
+	if( !PyInt_CheckExact( value ) )
+		return EXPP_ReturnIntError( PyExc_TypeError,
+					"metaball.update - expected an int argument" );
+
+	param = (int)PyInt_AS_LONG( value );
+
+	self->metaball->flag = EXPP_ClampInt( param, 0, 3 );
+	return 0;
 }
 
-static int Metaball_setFakeUser( BPy_Metaball * self, PyObject * value )
-{
-	return SetIdFakeUser(&self->metaball->id, value);
-}
-
-
-
 static PyObject *Metaball_copy( BPy_Metaball * self )
 {
 	BPy_Metaball *pymball;	/* for Data object wrapper in Python */
@@ -712,11 +687,11 @@ static PyObject *Metaball_copy( BPy_Metaball * self )
 							 &Metaball_Type );
 	} else
 		return ( EXPP_ReturnPyObjError( PyExc_RuntimeError,
-						"couldn't create MetaBall Data in Blender" ) );
+						"metaball.__copy__() - couldn't create data in Blender" ) );
 
 	if( pymball == NULL )
 		return ( EXPP_ReturnPyObjError( PyExc_MemoryError,
-						"couldn't create MetaBall Data object" ) );
+						"metaball.__copy__() - couldn't create data in Blender" ) );
 
 	pymball->metaball = blmball;
 	
@@ -755,14 +730,6 @@ static PyObject *Metaball_getElements( BPy_Metaball * self )
 	return MetaElemSeq_CreatePyObject(self, NULL);
 }
 
-/*
- * Metaball dealloc - free from memory
- */
-static void Metaball_dealloc( BPy_Metaball * self )
-{
-	PyObject_DEL( self );
-}
-
 /*
  * Metaelem dealloc - free from memory
  */
@@ -788,7 +755,7 @@ static PyObject *Metaelem_getType( BPy_Metaelem *self )
 		return attr;
 
 	return EXPP_ReturnPyObjError( PyExc_MemoryError,
-				"PyInt_FromLong() failed!" );
+				"metaelem.type - PyInt_FromLong() failed!" );
 }
 static int Metaelem_setType( BPy_Metaelem * self,  PyObject * value )
 {
@@ -796,7 +763,7 @@ static int Metaelem_setType( BPy_Metaelem * self,  PyObject * value )
 	int type;
 	if( !PyInt_CheckExact( value ) )
 		return EXPP_ReturnIntError( PyExc_TypeError,
-			"expected an integer (bitmask) as argument" );
+			"metaelem.type - expected an integer (bitmask) as argument" );
 	
 	METAELEM_DEL_CHECK_INT(self);
 	
@@ -804,7 +771,7 @@ static int Metaelem_setType( BPy_Metaelem * self,  PyObject * value )
 	
 	if( (type < 0) || ( type > ( MB_BALL | MB_TUBEX | MB_TUBEY | MB_TUBEZ | MB_TUBE | MB_PLANE | MB_ELIPSOID | MB_CUBE ) ))
 		return EXPP_ReturnIntError( PyExc_ValueError,
-					      "value out of range" );
+					      "metaelem.type - value out of range" );
 	
 	self->metaelem->type= type;
 	return 0;
@@ -830,7 +797,7 @@ static int Metaelem_setCoord( BPy_Metaelem * self,  VectorObject * value )
 
 	if( !VectorObject_Check( value ) || value->size != 3 )
 		return EXPP_ReturnIntError( PyExc_TypeError,
-				"expected vector argument of size 3" );
+				"metaelem.co - expected vector argument of size 3" );
 	
 	METAELEM_DEL_CHECK_INT(self);
 	
@@ -858,7 +825,7 @@ static int Metaelem_setDims( BPy_Metaelem * self,  VectorObject * value )
 {
 	if( !VectorObject_Check( value ) || value->size != 3 )
 		return EXPP_ReturnIntError( PyExc_TypeError,
-				"expected vector argument of size 3" );
+				"metaelem.dims - expected vector argument of size 3" );
 
 	METAELEM_DEL_CHECK_INT(self);
 	
@@ -882,7 +849,7 @@ static int Metaelem_setQuat( BPy_Metaelem * self,  QuaternionObject * value )
 	int i;
 	if( !QuaternionObject_Check( value ) )
 		return EXPP_ReturnIntError( PyExc_TypeError,
-				"expected quat argument" );
+				"metaelem.quat - expected quat argument" );
 	
 	METAELEM_DEL_CHECK_INT(self);
 	
@@ -923,9 +890,9 @@ static int Metaelem_setStiffness( BPy_Metaelem *self, PyObject *value)
 {
 	if( !PyNumber_Check( value ) )
 		return EXPP_ReturnIntError( PyExc_TypeError,
-					"expected float argument" );
+					"metaelem.stiffness - expected float argument" );
 
-	self->metaelem->s = EXPP_ClampFloat(PyFloat_AsDouble( value ), 0.0, 10.0);
+	self->metaelem->s = EXPP_ClampFloat((float)PyFloat_AsDouble( value ), 0.0, 10.0);
 	return 0;
 }
 
@@ -941,10 +908,10 @@ static int Metaelem_setRadius( BPy_Metaelem *self, PyObject *value)
 {
 	if( !PyNumber_Check( value ) )
 		return EXPP_ReturnIntError( PyExc_TypeError,
-					"expected float argument" );
+					"metaelem.radius - expected float argument" );
 
 	self->metaelem->rad = /* is 5000 too small? */
-			EXPP_ClampFloat(PyFloat_AsDouble( value ), 0.0, 5000.0);
+			EXPP_ClampFloat((float)PyFloat_AsDouble( value ), 0.0, 5000.0);
 	
 	return 0;
 }
@@ -1106,7 +1073,7 @@ static PyObject *MetaElemSeq_remove( BPy_MetaElemSeq * self, PyObject *args )
 	
 	if( !PyArg_ParseTuple( args, "O!", &Metaelem_Type, &elem) )
 		return EXPP_ReturnPyObjError( PyExc_TypeError,
-			"expected a Metaball element" );
+			"elements.remove(metaelem) - expected a Metaball element" );
 	
 	METAELEM_DEL_CHECK_PY(elem);
 	
