@@ -1,5 +1,5 @@
 /* 
- * $Id: Metaball.h,v 1.18 2006/09/30 14:14:49 campbellbarton Exp $
+ * $Id: Metaball.h,v 1.20 2007/03/15 01:09:12 campbellbarton Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -45,7 +45,7 @@ extern PyTypeObject Metaball_Type;
 /* Python BPy_Metaball structure definition */
 typedef struct {
 	PyObject_HEAD		/* required py macro */
-	MetaBall * metaball;
+	MetaBall * metaball; /* libdata must be second */
 } BPy_Metaball;
 
 
@@ -77,6 +77,5 @@ typedef struct {
 PyObject *Metaball_Init( void );
 PyObject *Metaball_CreatePyObject( MetaBall * mball );
 MetaBall *Metaball_FromPyObject( PyObject * py_obj );
-int Metaball_CheckPyObject( PyObject * py_obj );
 
 #endif				/* EXPP_METABALL_H */
