@@ -1,5 +1,5 @@
 /*
- * $Id: Modifier.c,v 1.17 2007/02/05 05:09:15 stiv Exp $
+ * $Id: Modifier.c,v 1.26 2007/04/30 16:49:12 artificer Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -42,14 +42,17 @@
 #include "BKE_modifier.h"
 #include "BKE_library.h"
 #include "BLI_blenlib.h"
+#include "BLI_arithb.h"
 #include "MEM_guardedalloc.h"
 #include "butspace.h"
 #include "blendef.h"
 #include "mydevice.h"
 
 #include "Object.h"
+#include "Texture.h"
 #include "Mathutils.h"
 #include "gen_utils.h"
+#include "gen_library.h"
 
 /* checks for the scene being removed */
 #define MODIFIER_DEL_CHECK_PY(bpy_modifier) if (!(bpy_modifier->md)) return ( EXPP_ReturnPyObjError( PyExc_RuntimeError, "Modifier has been removed" ) )
@@ -64,14 +67,18 @@ enum mod_constants {
 	
 	/*GENERIC*/
 	EXPP_MOD_OBJECT, /*ARMATURE, LATTICE, CURVE, BOOLEAN, ARRAY*/
-	EXPP_MOD_VERTGROUP, /*ARMATURE, LATTICE, CURVE*/
+	EXPP_MOD_VERTGROUP, /*ARMATURE, LATTICE, CURVE, SMOOTH, CAST*/
 	EXPP_MOD_LIMIT, /*ARRAY, MIRROR*/
 	EXPP_MOD_FLAG, /*MIRROR, WAVE*/
 	EXPP_MOD_COUNT, /*DECIMATOR, ARRAY*/
 	EXPP_MOD_LENGTH, /*BUILD, ARRAY*/
+	EXPP_MOD_FACTOR, /*SMOOTH, CAST*/
+	EXPP_MOD_ENABLE_X, /*SMOOTH, CAST*/
+	EXPP_MOD_ENABLE_Y, /*SMOOTH, CAST*/
+	EXPP_MOD_ENABLE_Z, /*SMOOTH, CAST*/
+	EXPP_MOD_TYPES, /*SUBSURF, CAST*/
 	
 	/*SUBSURF SPESIFIC*/
-	EXPP_MOD_TYPES,
 	EXPP_MOD_LEVELS,
 	EXPP_MOD_RENDLEVELS,
 	EXPP_MOD_OPTIMAL,
@@ -115,8 +122,25 @@ enum mod_constants {
 	/*EDGE SPLIT SPESIFIC */
 	EXPP_MOD_EDGESPLIT_ANGLE,
 	EXPP_MOD_EDGESPLIT_FROM_ANGLE,
-	EXPP_MOD_EDGESPLIT_FROM_SHARP
+	EXPP_MOD_EDGESPLIT_FROM_SHARP,
+	
+	/* DISPLACE */
+	EXPP_MOD_UVLAYER,
+	EXPP_MOD_MID_LEVEL,
+	EXPP_MOD_STRENGTH,
+	EXPP_MOD_TEXTURE,
+	EXPP_MOD_MAPPING,
+	EXPP_MOD_DIRECTION,
 
+	/* SMOOTH */
+	EXPP_MOD_REPEAT,
+
+	/* CAST */
+	EXPP_MOD_RADIUS,
+	EXPP_MOD_SIZE,
+	EXPP_MOD_USE_OB_TRANSFORM,
+	EXPP_MOD_SIZE_FROM_RADIUS
+	
 	/* yet to be implemented */
 	/* EXPP_MOD_HOOK_,*/
 	/* , */
@@ -128,6 +152,7 @@ enum mod_constants {
 static PyObject *Modifier_getName( BPy_Modifier * self );
 static int Modifier_setName( BPy_Modifier * self, PyObject *arg );
 static PyObject *Modifier_getType( BPy_Modifier * self );
+static PyObject *Modifier_reset( BPy_Modifier * self );
 
 static PyObject *Modifier_getData( BPy_Modifier * self, PyObject * key );
 static int Modifier_setData( BPy_Modifier * self, PyObject * key, 
@@ -138,6 +163,8 @@ static int Modifier_setData( BPy_Modifier * self, PyObject * key,
 /*****************************************************************************/
 static PyMethodDef BPy_Modifier_methods[] = {
 	/* name, method, flags, doc */
+	{"reset", (PyCFunction)Modifier_reset, METH_NOARGS,
+		"resets a hook modifier location"},
 	{NULL, NULL, 0, NULL}
 };
 
@@ -166,7 +193,6 @@ static PyMappingMethods Modifier_as_mapping = {
 /*****************************************************************************/
 /* Python Modifier_Type callback function prototypes:                        */
 /*****************************************************************************/
-static void Modifier_dealloc( BPy_Modifier * self );
 static PyObject *Modifier_repr( BPy_Modifier * self );
 
 /*****************************************************************************/
@@ -182,7 +208,7 @@ PyTypeObject Modifier_Type = {
 
 	/* Methods to implement standard operations */
 
-	( destructor ) Modifier_dealloc,/* destructor tp_dealloc; */
+	NULL,						/* destructor tp_dealloc; */
 	NULL,                       /* printfunc tp_print; */
 	NULL,                       /* getattrfunc tp_getattr; */
 	NULL,                       /* setattrfunc tp_setattr; */
@@ -363,17 +389,8 @@ static int armature_setter( BPy_Modifier *self, int type, PyObject *value )
 	ArmatureModifierData *md = (ArmatureModifierData *)(self->md);
 
 	switch( type ) {
-	case EXPP_MOD_OBJECT: {
-		Object *ob = (( BPy_Object * )value)->object;
-		if( !BPy_Object_Check( value ) || ob->type != OB_ARMATURE )
-			return EXPP_ReturnIntError( PyExc_TypeError, 
-					"expected BPy armature object argument" );
-		if(ob == self->object )
-			return EXPP_ReturnIntError( PyExc_TypeError,
-					"Cannot lattice deform an object with its self" );
-		md->object = ob;
-		return 0;
-		}
+	case EXPP_MOD_OBJECT: 
+		return GenericLib_assignData(value, (void **) &md->object, 0, 0, ID_OB, OB_ARMATURE);
 	case EXPP_MOD_VERTGROUP:
 		return EXPP_setBitfield( value, &md->deformflag, 1, 'h' );
 	case EXPP_MOD_ENVELOPES:
@@ -402,17 +419,8 @@ static int lattice_setter( BPy_Modifier *self, int type, PyObject *value )
 	LatticeModifierData *md = (LatticeModifierData *)(self->md);
 
 	switch( type ) {
-	case EXPP_MOD_OBJECT: {
-		Object *ob = (( BPy_Object * )value)->object;
-		if( !BPy_Object_Check( value ) || ob->type != OB_LATTICE )
-			return EXPP_ReturnIntError( PyExc_TypeError, 
-					"expected BPy lattice object argument" );
-		if(ob == self->object )
-			return EXPP_ReturnIntError( PyExc_TypeError,
-					"Cannot curve deform an object with its self" );
-		md->object = ob;
-		break;
-		}
+	case EXPP_MOD_OBJECT:
+		return GenericLib_assignData(value, (void **) &md->object, (void **) &self->object, 0, ID_OB, OB_LATTICE);
 	case EXPP_MOD_VERTGROUP: {
 		char *name = PyString_AsString( value );
 		if( !name )
@@ -446,17 +454,8 @@ static int curve_setter( BPy_Modifier *self, int type, PyObject *value )
 	CurveModifierData *md = (CurveModifierData *)(self->md);
 
 	switch( type ) {
-	case EXPP_MOD_OBJECT: {
-		Object *ob = (( BPy_Object * )value)->object;
-		if( !BPy_Object_Check( value ) || ob->type != OB_CURVE )
-			return EXPP_ReturnIntError( PyExc_TypeError,
-					"expected BPy lattice object argument" );
-		if(ob == self->object )
-			return EXPP_ReturnIntError( PyExc_TypeError,
-					"Cannot curve deform an object with its self" );
-		md->object = ob;
-		break;
-		}
+	case EXPP_MOD_OBJECT:
+		return GenericLib_assignData(value, (void **) &md->object, (void **) &self->object, 0, ID_OB, OB_CURVE);
 	case EXPP_MOD_VERTGROUP: {
 		char *name = PyString_AsString( value );
 		if( !name )
@@ -562,6 +561,133 @@ static int decimate_setter( BPy_Modifier *self, int type, PyObject *value )
 	return EXPP_ReturnIntError( PyExc_KeyError, "key not found" );
 }
 
+static PyObject *smooth_getter( BPy_Modifier * self, int type )
+{
+	SmoothModifierData *md = (SmoothModifierData *)(self->md);
+
+	switch( type ) {
+	case EXPP_MOD_FACTOR:
+		return PyFloat_FromDouble( (double)md->fac );
+	case EXPP_MOD_REPEAT:
+		return PyInt_FromLong( (long)md->repeat );
+	case EXPP_MOD_VERTGROUP:
+		return PyString_FromString( md->defgrp_name ) ;
+	case EXPP_MOD_ENABLE_X:
+		return EXPP_getBitfield( &md->flag, MOD_SMOOTH_X, 'h' );
+	case EXPP_MOD_ENABLE_Y:
+		return EXPP_getBitfield( &md->flag, MOD_SMOOTH_Y, 'h' );
+	case EXPP_MOD_ENABLE_Z:
+		return EXPP_getBitfield( &md->flag, MOD_SMOOTH_Z, 'h' );
+	default:
+		return EXPP_ReturnPyObjError( PyExc_KeyError, "key not found" );
+	}
+}
+
+static int smooth_setter( BPy_Modifier *self, int type, PyObject *value )
+{
+	SmoothModifierData *md = (SmoothModifierData *)(self->md);
+
+	switch( type ) {
+	case EXPP_MOD_FACTOR:
+		return EXPP_setFloatClamped( value, &md->fac, -10.0, 10.0 );
+	case EXPP_MOD_REPEAT:
+		return EXPP_setIValueRange( value, &md->repeat, 0, 30, 'h' );
+	case EXPP_MOD_VERTGROUP: {
+		char *name = PyString_AsString( value );
+		if( !name ) return EXPP_ReturnIntError( PyExc_TypeError, "expected string arg" );
+		BLI_strncpy( md->defgrp_name, name, sizeof( md->defgrp_name ) );
+		return 0;
+	}
+	case EXPP_MOD_ENABLE_X:
+		return EXPP_setBitfield( value, &md->flag, MOD_SMOOTH_X, 'h' ); 
+	case EXPP_MOD_ENABLE_Y:
+		return EXPP_setBitfield( value, &md->flag, MOD_SMOOTH_Y, 'h' ); 
+	case EXPP_MOD_ENABLE_Z:
+		return EXPP_setBitfield( value, &md->flag, MOD_SMOOTH_Z, 'h' ); 
+	default:
+		return EXPP_ReturnIntError( PyExc_KeyError, "key not found" );
+	}
+}
+
+static PyObject *cast_getter( BPy_Modifier * self, int type )
+{
+	CastModifierData *md = (CastModifierData *)(self->md);
+
+	switch( type ) {
+	case EXPP_MOD_TYPES:
+		return PyInt_FromLong( (long)md->type );
+	case EXPP_MOD_FACTOR:
+		return PyFloat_FromDouble( (double)md->fac );
+	case EXPP_MOD_RADIUS:
+		return PyFloat_FromDouble( (double)md->radius );
+	case EXPP_MOD_SIZE:
+		return PyFloat_FromDouble( (double)md->size );
+	case EXPP_MOD_OBJECT:
+		return Object_CreatePyObject( md->object );
+	case EXPP_MOD_VERTGROUP:
+		return PyString_FromString( md->defgrp_name ) ;
+	case EXPP_MOD_ENABLE_X:
+		return EXPP_getBitfield( &md->flag, MOD_CAST_X, 'h' );
+	case EXPP_MOD_ENABLE_Y:
+		return EXPP_getBitfield( &md->flag, MOD_CAST_Y, 'h' );
+	case EXPP_MOD_ENABLE_Z:
+		return EXPP_getBitfield( &md->flag, MOD_CAST_Z, 'h' );
+	case EXPP_MOD_USE_OB_TRANSFORM:
+		return EXPP_getBitfield( &md->flag, MOD_CAST_USE_OB_TRANSFORM, 'h' );
+	case EXPP_MOD_SIZE_FROM_RADIUS:
+		return EXPP_getBitfield( &md->flag, MOD_CAST_SIZE_FROM_RADIUS, 'h' );
+	default:
+		return EXPP_ReturnPyObjError( PyExc_KeyError, "key not found" );
+	}
+}
+
+static int cast_setter( BPy_Modifier *self, int type, PyObject *value )
+{
+	CastModifierData *md = (CastModifierData *)(self->md);
+
+	switch( type ) {
+	case EXPP_MOD_TYPES:
+		return EXPP_setIValueRange( value, &md->type, 0, MOD_CAST_TYPE_CUBOID, 'h' );
+	case EXPP_MOD_FACTOR:
+		return EXPP_setFloatClamped( value, &md->fac, -10.0, 10.0 );
+	case EXPP_MOD_RADIUS:
+		return EXPP_setFloatClamped( value, &md->radius, 0.0, 100.0 );
+	case EXPP_MOD_SIZE:
+		return EXPP_setFloatClamped( value, &md->size, 0.0, 100.0 );
+	case EXPP_MOD_OBJECT: {
+		Object *ob_new=NULL;
+		if (value == Py_None) {
+			md->object = NULL;
+		} else if (BPy_Object_Check( value )) {
+			ob_new = ((( BPy_Object * )value)->object);
+			md->object = ob_new;
+		} else {
+			return EXPP_ReturnIntError( PyExc_TypeError,
+				"Expected an Object or None value" );
+		}
+		return 0;
+	}
+	case EXPP_MOD_VERTGROUP: {
+		char *name = PyString_AsString( value );
+		if( !name ) return EXPP_ReturnIntError( PyExc_TypeError, "expected string arg" );
+		BLI_strncpy( md->defgrp_name, name, sizeof( md->defgrp_name ) );
+		return 0;
+	}
+	case EXPP_MOD_ENABLE_X:
+		return EXPP_setBitfield( value, &md->flag, MOD_CAST_X, 'h' ); 
+	case EXPP_MOD_ENABLE_Y:
+		return EXPP_setBitfield( value, &md->flag, MOD_CAST_Y, 'h' ); 
+	case EXPP_MOD_ENABLE_Z:
+		return EXPP_setBitfield( value, &md->flag, MOD_CAST_Z, 'h' ); 
+	case EXPP_MOD_USE_OB_TRANSFORM:
+		return EXPP_setBitfield( value, &md->flag, MOD_CAST_USE_OB_TRANSFORM, 'h' ); 
+	case EXPP_MOD_SIZE_FROM_RADIUS:
+		return EXPP_setBitfield( value, &md->flag, MOD_CAST_SIZE_FROM_RADIUS, 'h' ); 
+	default:
+		return EXPP_ReturnIntError( PyExc_KeyError, "key not found" );
+	}
+}
+
 static PyObject *wave_getter( BPy_Modifier * self, int type )
 {
 	WaveModifierData *md = (WaveModifierData *)(self->md);
@@ -617,7 +743,7 @@ static int wave_setter( BPy_Modifier *self, int type, PyObject *value )
 		return EXPP_setFloatClamped( value, &md->timeoffs, -1000.0, 1000.0 );
 	case EXPP_MOD_FLAG:
 		return EXPP_setIValueRange( value, &md->flag, 0, 
-				WAV_X+WAV_Y+WAV_CYCL, 'h' );
+				MOD_WAVE_X | MOD_WAVE_Y | MOD_WAVE_CYCL, 'h' );
 	default:
 		return EXPP_ReturnIntError( PyExc_KeyError, "key not found" );
 	}
@@ -651,28 +777,10 @@ static int array_setter( BPy_Modifier *self, int type, PyObject *value )
 {
 	ArrayModifierData *md = (ArrayModifierData *)(self->md);
 	switch( type ) {
-	case EXPP_MOD_OBJECT_OFFSET: {
-		Object *ob = (( BPy_Object * )value)->object;
-		if( !BPy_Object_Check( value ) )
-			return EXPP_ReturnIntError( PyExc_TypeError, 
-					"expected BPy object argument" );
-		if(ob == self->object )
-			return EXPP_ReturnIntError( PyExc_TypeError,
-					"Cannot lattice deform an object with its self" );
-		md->offset_ob = ob;
-		return 0;
-	}
-	case EXPP_MOD_OBJECT_CURVE: {
-		Object *ob = (( BPy_Object * )value)->object;
-		if( !BPy_Object_Check( value ) || ob->type != OB_CURVE )
-			return EXPP_ReturnIntError( PyExc_TypeError, 
-					"expected BPy object argument" );
-		if(ob == self->object )
-			return EXPP_ReturnIntError( PyExc_TypeError,
-					"Cannot lattice deform an object with its self" );
-		md->curve_ob = ob;
-		return 0;
-	}
+	case EXPP_MOD_OBJECT_OFFSET:
+		return GenericLib_assignData(value, (void **) &md->offset_ob, (void **) &self->object, 0, ID_OB, 0);
+	case EXPP_MOD_OBJECT_CURVE:
+		return GenericLib_assignData(value, (void **) &md->curve_ob, 0, 0, ID_OB, OB_CURVE);
 	case EXPP_MOD_COUNT:
 		return EXPP_setIValueClamped( value, &md->count, 1, 1000, 'i' );
 	case EXPP_MOD_LENGTH:
@@ -704,17 +812,9 @@ static int boolean_setter( BPy_Modifier *self, int type, PyObject *value )
 {
 	BooleanModifierData *md = (BooleanModifierData *)(self->md);
 
-	if( type == EXPP_MOD_OBJECT ) {
-		Object *ob = (( BPy_Object * )value)->object;
-		if( !BPy_Object_Check( value ) || ob->type != OB_MESH )
-			return EXPP_ReturnIntError( PyExc_TypeError,
-					"expected BPy mesh object argument" );
-		if(ob == self->object )
-			return EXPP_ReturnIntError( PyExc_TypeError,
-					"Cannot boolean an object with its self" );
-		md->object = ob;
-		return 0;
-	} else if( type == EXPP_MOD_OPERATION )
+	if( type == EXPP_MOD_OBJECT )
+		return GenericLib_assignData(value, (void **) &md->object, (void **) &self->object, 0, ID_OB, OB_MESH);
+	else if( type == EXPP_MOD_OPERATION )
 		return EXPP_setIValueRange( value, &md->operation, 
 				eBooleanModifierOp_Intersect, eBooleanModifierOp_Difference,
 				'h' );
@@ -761,7 +861,109 @@ static int edgesplit_setter( BPy_Modifier *self, int type, PyObject *value )
 	}
 }
 
+static PyObject *displace_getter( BPy_Modifier * self, int type )
+{
+	DisplaceModifierData *md = (DisplaceModifierData *)(self->md);
 
+	switch( type ) {
+	case EXPP_MOD_TEXTURE:
+		if (md->texture)	Texture_CreatePyObject( md->texture );
+		else				Py_RETURN_NONE;
+	case EXPP_MOD_STRENGTH:
+		return PyFloat_FromDouble( (double)md->strength );
+	case EXPP_MOD_DIRECTION:
+		PyInt_FromLong( md->direction );
+	case EXPP_MOD_VERTGROUP:
+		return PyString_FromString( md->defgrp_name ) ;
+	case EXPP_MOD_MID_LEVEL:
+		return PyFloat_FromDouble( (double)md->midlevel );
+	case EXPP_MOD_MAPPING:
+		PyInt_FromLong( md->texmapping );
+	case EXPP_MOD_OBJECT:
+		return Object_CreatePyObject( md->map_object );
+	case EXPP_MOD_UVLAYER:
+		return PyString_FromString( md->uvlayer_name );
+	default:
+		return EXPP_ReturnPyObjError( PyExc_KeyError, "key not found" );
+	}
+}
+
+static int displace_setter( BPy_Modifier *self, int type, PyObject *value )
+{
+	DisplaceModifierData *md = (DisplaceModifierData *)(self->md);
+
+	switch( type ) {
+	case EXPP_MOD_TEXTURE:
+		return GenericLib_assignData(value, (void **) &md->texture, 0, 1, ID_TE, 0);
+	case EXPP_MOD_STRENGTH:
+		return EXPP_setFloatClamped( value, &md->strength, -1000.0, 1000.0 );
+	
+	case EXPP_MOD_DIRECTION:
+		return EXPP_setIValueClamped( value, &md->direction,
+				MOD_DISP_DIR_X, MOD_DISP_DIR_RGB_XYZ, 'i' );
+	
+	case EXPP_MOD_VERTGROUP: {
+		char *name = PyString_AsString( value );
+		if( !name ) return EXPP_ReturnIntError( PyExc_TypeError, "expected string arg" );
+		BLI_strncpy( md->defgrp_name, name, sizeof( md->defgrp_name ) );
+		return 0;
+	}
+	case EXPP_MOD_MID_LEVEL:
+		return EXPP_setFloatClamped( value, &md->midlevel, 0.0, 1.0 );
+	
+	case EXPP_MOD_MAPPING:
+		return EXPP_setIValueClamped( value, &md->texmapping,
+				MOD_DISP_MAP_LOCAL, MOD_DISP_MAP_UV, 'i' );
+	
+	case EXPP_MOD_OBJECT: {
+		Object *ob_new=NULL;
+		if (value == Py_None) {
+			md->map_object = NULL;
+		} else if (BPy_Object_Check( value )) {
+			ob_new = ((( BPy_Object * )value)->object);
+			md->map_object = ob_new;
+		} else {
+			return EXPP_ReturnIntError( PyExc_TypeError,
+				"Expected an Object or None value" );
+		}
+		return 0;
+	}
+	
+	case EXPP_MOD_UVLAYER: {
+		char *name = PyString_AsString( value );
+		if( !name ) return EXPP_ReturnIntError( PyExc_TypeError, "expected string arg" );
+		BLI_strncpy( md->uvlayer_name, name, sizeof( md->uvlayer_name ) );
+		return 0;
+	}
+	default:
+		return EXPP_ReturnIntError( PyExc_KeyError, "key not found" );
+	}
+}
+
+
+/* static PyObject *uvproject_getter( BPy_Modifier * self, int type )
+{
+	DisplaceModifierData *md = (DisplaceModifierData *)(self->md);
+
+	switch( type ) {
+	case EXPP_MOD_MID_LEVEL:
+		return PyFloat_FromDouble( (double)md->midlevel );
+	default:
+		return EXPP_ReturnPyObjError( PyExc_KeyError, "key not found" );
+	}
+}
+
+static int uvproject_setter( BPy_Modifier *self, int type, PyObject *value )
+{
+	DisplaceModifierData *md = (DisplaceModifierData *)(self->md);
+
+	switch( type ) {
+	case EXPP_MOD_TEXTURE:
+		return 0;
+	default:
+		return EXPP_ReturnIntError( PyExc_KeyError, "key not found" );
+	}
+} */
 
 
 /*
@@ -804,6 +1006,10 @@ static PyObject *Modifier_getData( BPy_Modifier * self, PyObject * key )
 				return mirror_getter( self, setting );
 			case eModifierType_Decimate:
 				return decimate_getter( self, setting );
+			case eModifierType_Smooth:
+				return smooth_getter( self, setting );
+			case eModifierType_Cast:
+				return cast_getter( self, setting );
 			case eModifierType_Wave:
 				return wave_getter( self, setting );
 			case eModifierType_Boolean:
@@ -812,6 +1018,10 @@ static PyObject *Modifier_getData( BPy_Modifier * self, PyObject * key )
 				return array_getter( self, setting );
 			case eModifierType_EdgeSplit:
 				return edgesplit_getter( self, setting );
+			case eModifierType_Displace:
+				return displace_getter( self, setting );
+			/*case eModifierType_UVProject:
+				return uvproject_getter( self, setting );*/
 			case eModifierType_Hook:
 			case eModifierType_Softbody:
 			case eModifierType_None:
@@ -868,12 +1078,20 @@ static int Modifier_setData( BPy_Modifier * self, PyObject * key,
 			return array_setter( self, key_int, arg );
 		case eModifierType_Decimate:
 			return decimate_setter( self, key_int, arg );
+		case eModifierType_Smooth:
+			return smooth_setter( self, key_int, arg );
+		case eModifierType_Cast:
+			return cast_setter( self, key_int, arg );
 		case eModifierType_Wave:
 			return wave_setter( self, key_int, arg );
 		case eModifierType_Boolean:
 			return boolean_setter( self, key_int, arg );
 		case eModifierType_EdgeSplit:
 			return edgesplit_setter( self, key_int, arg );
+		case eModifierType_Displace:
+			return displace_setter( self, key_int, arg );
+		/*case eModifierType_UVProject:
+			return uvproject_setter( self, key_int, arg );*/
 		case eModifierType_Hook:
 		case eModifierType_Softbody:
 		case eModifierType_None:
@@ -883,14 +1101,24 @@ static int Modifier_setData( BPy_Modifier * self, PyObject * key,
 			"unsupported modifier setting" );
 }
 
-/*****************************************************************************/
-/* Function:    Modifier_dealloc                                             */
-/* Description: This is a callback function for the BPy_Modifier type. It    */
-/*              destroys data when the object is deleted.                    */
-/*****************************************************************************/
-static void Modifier_dealloc( BPy_Modifier * self )
+
+static PyObject *Modifier_reset( BPy_Modifier * self )
 {
-	PyObject_DEL( self );
+	Object *ob = self->object;
+	ModifierData *md = self->md;
+	HookModifierData *hmd = (HookModifierData*) md;
+	
+	MODIFIER_DEL_CHECK_PY(self);
+	
+	if (md->type != eModifierType_Hook)
+		return EXPP_ReturnPyObjError( PyExc_TypeError,
+			"can only reset hooks" );
+	
+	if (hmd->object) {
+		Mat4Invert(hmd->object->imat, hmd->object->obmat);
+		Mat4MulSerie(hmd->parentinv, hmd->object->imat, ob->obmat, NULL, NULL, NULL, NULL, NULL, NULL);
+	}
+	Py_RETURN_NONE;
 }
 
 /*****************************************************************************/
@@ -927,16 +1155,6 @@ PyObject *Modifier_CreatePyObject( Object *ob, ModifierData * md )
 	return ( PyObject * ) pymod;
 }
 
-/*****************************************************************************/
-/* Function:    Modifier_CheckPyObject                                       */
-/* Description: This function returns true when the given PyObject is of the */
-/*              type Modifier. Otherwise it will return false.               */
-/*****************************************************************************/
-int Modifier_CheckPyObject( PyObject * pyobj )
-{
-	return ( pyobj->ob_type == &Modifier_Type );
-}
-
 /*****************************************************************************/
 /* Function:    Modifier_FromPyObject                                        */
 /* Description: This function returns the Blender modifier from the given    */
@@ -1130,17 +1348,6 @@ static PyObject *ModSeq_moveDown( BPy_ModSeq * self, PyObject *args )
 	Py_RETURN_NONE;
 }
 
-
-/*****************************************************************************/
-/* Function:    ModSeq_dealloc                                            */
-/* Description: This is a callback function for the BPy_Modifier type. It    */
-/*              destroys data when the object is deleted.                    */
-/*****************************************************************************/
-static void ModSeq_dealloc( BPy_Modifier * self )
-{
-	PyObject_DEL( self );
-}
-
 /*****************************************************************************/
 /* Python BPy_ModSeq methods table:                                       */
 /*****************************************************************************/
@@ -1170,7 +1377,7 @@ PyTypeObject ModSeq_Type = {
 
 	/* Methods to implement standard operations */
 
-	( destructor ) ModSeq_dealloc,/* destructor tp_dealloc; */
+	NULL,						/* destructor tp_dealloc; */
 	NULL,                       /* printfunc tp_print; */
 	NULL,                       /* getattrfunc tp_getattr; */
 	NULL,                       /* setattrfunc tp_setattr; */
@@ -1288,6 +1495,10 @@ static PyObject *M_Modifier_TypeDict( void )
 				PyInt_FromLong( eModifierType_Array ) );
 		PyConstant_Insert( d, "EDGESPLIT",
 				PyInt_FromLong( eModifierType_EdgeSplit ) );
+		PyConstant_Insert( d, "SMOOTH",
+				PyInt_FromLong( eModifierType_Smooth ) );
+		PyConstant_Insert( d, "CAST",
+				PyInt_FromLong( eModifierType_Cast ) );
 	}
 	return S;
 }
@@ -1348,6 +1559,14 @@ for var in st.replace(',','').split('\n'):
 				PyInt_FromLong( EXPP_MOD_COUNT ) );
 			PyConstant_Insert( d, "LENGTH", 
 				PyInt_FromLong( EXPP_MOD_LENGTH ) );
+			PyConstant_Insert( d, "FACTOR", 
+				PyInt_FromLong( EXPP_MOD_FACTOR ) );
+			PyConstant_Insert( d, "ENABLE_X", 
+				PyInt_FromLong( EXPP_MOD_ENABLE_X ) );
+			PyConstant_Insert( d, "ENABLE_Y", 
+				PyInt_FromLong( EXPP_MOD_ENABLE_Y ) );
+			PyConstant_Insert( d, "ENABLE_Z", 
+				PyInt_FromLong( EXPP_MOD_ENABLE_Z ) );
 			PyConstant_Insert( d, "TYPES", 
 				PyInt_FromLong( EXPP_MOD_TYPES ) );
 			PyConstant_Insert( d, "LEVELS", 
@@ -1406,6 +1625,28 @@ for var in st.replace(',','').split('\n'):
 				PyInt_FromLong( EXPP_MOD_EDGESPLIT_FROM_ANGLE ) );
 			PyConstant_Insert( d, "EDGESPLIT_FROM_SHARP", 
 				PyInt_FromLong( EXPP_MOD_EDGESPLIT_FROM_SHARP ) );
+			PyConstant_Insert( d, "UVLAYER",
+				PyInt_FromLong( EXPP_MOD_UVLAYER ) );
+			PyConstant_Insert( d, "MID_LEVEL",
+				PyInt_FromLong( EXPP_MOD_MID_LEVEL ) );
+			PyConstant_Insert( d, "STRENGTH",
+				PyInt_FromLong( EXPP_MOD_STRENGTH ) );
+			PyConstant_Insert( d, "TEXTURE", 
+				PyInt_FromLong( EXPP_MOD_TEXTURE ) );
+			PyConstant_Insert( d, "MAPPING", 
+				PyInt_FromLong( EXPP_MOD_MAPPING ) );
+			PyConstant_Insert( d, "DIRECTION", 
+				PyInt_FromLong( EXPP_MOD_DIRECTION ) );
+			PyConstant_Insert( d, "REPEAT", 
+				PyInt_FromLong( EXPP_MOD_REPEAT ) );
+			PyConstant_Insert( d, "RADIUS", 
+				PyInt_FromLong( EXPP_MOD_RADIUS ) );
+			PyConstant_Insert( d, "SIZE", 
+				PyInt_FromLong( EXPP_MOD_SIZE ) );
+			PyConstant_Insert( d, "USE_OB_TRANSFORM", 
+				PyInt_FromLong( EXPP_MOD_USE_OB_TRANSFORM ) );
+			PyConstant_Insert( d, "SIZE_FROM_RADIUS", 
+				PyInt_FromLong( EXPP_MOD_SIZE_FROM_RADIUS ) );
 			/*End Auto generated code*/
 	}
 	return S;
